# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from azure.core.exceptions import HttpResponseError
import msrest.serialization


class CollectionOfUser(msrest.serialization.Model):
    """Collection of user.

    :param value:
    :type value: list[~users_user.models.MicrosoftGraphUser]
    :param odata_next_link:
    :type odata_next_link: str
    """

    _attribute_map = {
        'value': {'key': 'value', 'type': '[MicrosoftGraphUser]'},
        'odata_next_link': {'key': '@odata\\.nextLink', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(CollectionOfUser, self).__init__(**kwargs)
        self.value = kwargs.get('value', None)
        self.odata_next_link = kwargs.get('odata_next_link', None)


class MicrosoftGraphEntity(msrest.serialization.Model):
    """entity.

    :param id: Read-only.
    :type id: str
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphEntity, self).__init__(**kwargs)
        self.id = kwargs.get('id', None)


class MicrosoftGraphActivityHistoryItem(MicrosoftGraphEntity):
    """activityHistoryItem.

    :param id: Read-only.
    :type id: str
    :param status: status. Possible values include: "active", "updated", "deleted", "ignored",
     "unknownFutureValue".
    :type status: str or ~users_user.models.MicrosoftGraphStatus
    :param active_duration_seconds: Optional. The duration of active user engagement. if not
     supplied, this is calculated from the startedDateTime and lastActiveDateTime.
    :type active_duration_seconds: int
    :param created_date_time: Set by the server. DateTime in UTC when the object was created on the
     server.
    :type created_date_time: ~datetime.datetime
    :param last_active_date_time: Optional. UTC DateTime when the historyItem (activity session)
     was last understood as active or finished - if null, historyItem status should be Ongoing.
    :type last_active_date_time: ~datetime.datetime
    :param last_modified_date_time: Set by the server. DateTime in UTC when the object was modified
     on the server.
    :type last_modified_date_time: ~datetime.datetime
    :param expiration_date_time: Optional. UTC DateTime when the historyItem will undergo hard-
     delete. Can be set by the client.
    :type expiration_date_time: ~datetime.datetime
    :param started_date_time: Required. UTC DateTime when the historyItem (activity session) was
     started. Required for timeline history.
    :type started_date_time: ~datetime.datetime
    :param user_timezone: Optional. The timezone in which the user's device used to generate the
     activity was located at activity creation time. Values supplied as Olson IDs in order to
     support cross-platform representation.
    :type user_timezone: str
    :param activity: userActivity.
    :type activity: ~users_user.models.MicrosoftGraphUserActivity
    """

    _validation = {
        'active_duration_seconds': {'maximum': 2147483647, 'minimum': -2147483648},
    }

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'status': {'key': 'status', 'type': 'str'},
        'active_duration_seconds': {'key': 'activeDurationSeconds', 'type': 'int'},
        'created_date_time': {'key': 'createdDateTime', 'type': 'iso-8601'},
        'last_active_date_time': {'key': 'lastActiveDateTime', 'type': 'iso-8601'},
        'last_modified_date_time': {'key': 'lastModifiedDateTime', 'type': 'iso-8601'},
        'expiration_date_time': {'key': 'expirationDateTime', 'type': 'iso-8601'},
        'started_date_time': {'key': 'startedDateTime', 'type': 'iso-8601'},
        'user_timezone': {'key': 'userTimezone', 'type': 'str'},
        'activity': {'key': 'activity', 'type': 'MicrosoftGraphUserActivity'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphActivityHistoryItem, self).__init__(**kwargs)
        self.status = kwargs.get('status', None)
        self.active_duration_seconds = kwargs.get('active_duration_seconds', None)
        self.created_date_time = kwargs.get('created_date_time', None)
        self.last_active_date_time = kwargs.get('last_active_date_time', None)
        self.last_modified_date_time = kwargs.get('last_modified_date_time', None)
        self.expiration_date_time = kwargs.get('expiration_date_time', None)
        self.started_date_time = kwargs.get('started_date_time', None)
        self.user_timezone = kwargs.get('user_timezone', None)
        self.activity = kwargs.get('activity', None)


class MicrosoftGraphActivityStatistics(MicrosoftGraphEntity):
    """activityStatistics.

    :param id: Read-only.
    :type id: str
    :param activity: analyticsActivityType. Possible values include: "Email", "Meeting", "Focus",
     "Chat", "Call".
    :type activity: str or ~users_user.models.MicrosoftGraphAnalyticsActivityType
    :param start_date:
    :type start_date: ~datetime.date
    :param end_date:
    :type end_date: ~datetime.date
    :param time_zone_used:
    :type time_zone_used: str
    :param duration:
    :type duration: ~datetime.timedelta
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'activity': {'key': 'activity', 'type': 'str'},
        'start_date': {'key': 'startDate', 'type': 'date'},
        'end_date': {'key': 'endDate', 'type': 'date'},
        'time_zone_used': {'key': 'timeZoneUsed', 'type': 'str'},
        'duration': {'key': 'duration', 'type': 'duration'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphActivityStatistics, self).__init__(**kwargs)
        self.activity = kwargs.get('activity', None)
        self.start_date = kwargs.get('start_date', None)
        self.end_date = kwargs.get('end_date', None)
        self.time_zone_used = kwargs.get('time_zone_used', None)
        self.duration = kwargs.get('duration', None)


class MicrosoftGraphAgreementAcceptance(MicrosoftGraphEntity):
    """agreementAcceptance.

    :param id: Read-only.
    :type id: str
    :param agreement_id:
    :type agreement_id: str
    :param user_id:
    :type user_id: str
    :param agreement_file_id:
    :type agreement_file_id: str
    :param recorded_date_time:
    :type recorded_date_time: ~datetime.datetime
    :param user_display_name:
    :type user_display_name: str
    :param user_principal_name:
    :type user_principal_name: str
    :param user_email:
    :type user_email: str
    :param state: agreementAcceptanceState. Possible values include: "accepted", "declined".
    :type state: str or ~users_user.models.MicrosoftGraphAgreementAcceptanceState
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'agreement_id': {'key': 'agreementId', 'type': 'str'},
        'user_id': {'key': 'userId', 'type': 'str'},
        'agreement_file_id': {'key': 'agreementFileId', 'type': 'str'},
        'recorded_date_time': {'key': 'recordedDateTime', 'type': 'iso-8601'},
        'user_display_name': {'key': 'userDisplayName', 'type': 'str'},
        'user_principal_name': {'key': 'userPrincipalName', 'type': 'str'},
        'user_email': {'key': 'userEmail', 'type': 'str'},
        'state': {'key': 'state', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphAgreementAcceptance, self).__init__(**kwargs)
        self.agreement_id = kwargs.get('agreement_id', None)
        self.user_id = kwargs.get('user_id', None)
        self.agreement_file_id = kwargs.get('agreement_file_id', None)
        self.recorded_date_time = kwargs.get('recorded_date_time', None)
        self.user_display_name = kwargs.get('user_display_name', None)
        self.user_principal_name = kwargs.get('user_principal_name', None)
        self.user_email = kwargs.get('user_email', None)
        self.state = kwargs.get('state', None)


class MicrosoftGraphAlbum(msrest.serialization.Model):
    """album.

    :param cover_image_item_id:
    :type cover_image_item_id: str
    """

    _attribute_map = {
        'cover_image_item_id': {'key': 'coverImageItemId', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphAlbum, self).__init__(**kwargs)
        self.cover_image_item_id = kwargs.get('cover_image_item_id', None)


class MicrosoftGraphAlternativeSecurityId(msrest.serialization.Model):
    """alternativeSecurityId.

    :param type: For internal use only.
    :type type: int
    :param identity_provider: For internal use only.
    :type identity_provider: str
    :param key: For internal use only.
    :type key: bytes
    """

    _validation = {
        'type': {'maximum': 2147483647, 'minimum': -2147483648},
    }

    _attribute_map = {
        'type': {'key': 'type', 'type': 'int'},
        'identity_provider': {'key': 'identityProvider', 'type': 'str'},
        'key': {'key': 'key', 'type': 'base64'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphAlternativeSecurityId, self).__init__(**kwargs)
        self.type = kwargs.get('type', None)
        self.identity_provider = kwargs.get('identity_provider', None)
        self.key = kwargs.get('key', None)


class MicrosoftGraphAppConsentRequest(MicrosoftGraphEntity):
    """appConsentRequest.

    :param id: Read-only.
    :type id: str
    :param app_id:
    :type app_id: str
    :param app_display_name:
    :type app_display_name: str
    :param consent_type:
    :type consent_type: str
    :param pending_scopes:
    :type pending_scopes: list[~users_user.models.MicrosoftGraphAppConsentRequestScope]
    :param user_consent_requests:
    :type user_consent_requests: list[~users_user.models.MicrosoftGraphUserConsentRequest]
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'app_id': {'key': 'appId', 'type': 'str'},
        'app_display_name': {'key': 'appDisplayName', 'type': 'str'},
        'consent_type': {'key': 'consentType', 'type': 'str'},
        'pending_scopes': {'key': 'pendingScopes', 'type': '[MicrosoftGraphAppConsentRequestScope]'},
        'user_consent_requests': {'key': 'userConsentRequests', 'type': '[MicrosoftGraphUserConsentRequest]'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphAppConsentRequest, self).__init__(**kwargs)
        self.app_id = kwargs.get('app_id', None)
        self.app_display_name = kwargs.get('app_display_name', None)
        self.consent_type = kwargs.get('consent_type', None)
        self.pending_scopes = kwargs.get('pending_scopes', None)
        self.user_consent_requests = kwargs.get('user_consent_requests', None)


class MicrosoftGraphAppConsentRequestScope(msrest.serialization.Model):
    """appConsentRequestScope.

    :param display_name:
    :type display_name: str
    """

    _attribute_map = {
        'display_name': {'key': 'displayName', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphAppConsentRequestScope, self).__init__(**kwargs)
        self.display_name = kwargs.get('display_name', None)


class MicrosoftGraphAppLogCollectionRequest(MicrosoftGraphEntity):
    """AppLogCollectionRequest Entity.

    :param id: Read-only.
    :type id: str
    :param status: appLogUploadState. Possible values include: "pending", "completed", "failed".
    :type status: str or ~users_user.models.MicrosoftGraphAppLogUploadState
    :param error_message: Error message if any during the upload process.
    :type error_message: str
    :param custom_log_folders: List of log folders.
    :type custom_log_folders: list[str]
    :param completed_date_time: Time at which the upload log request reached a terminal state.
    :type completed_date_time: ~datetime.datetime
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'status': {'key': 'status', 'type': 'str'},
        'error_message': {'key': 'errorMessage', 'type': 'str'},
        'custom_log_folders': {'key': 'customLogFolders', 'type': '[str]'},
        'completed_date_time': {'key': 'completedDateTime', 'type': 'iso-8601'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphAppLogCollectionRequest, self).__init__(**kwargs)
        self.status = kwargs.get('status', None)
        self.error_message = kwargs.get('error_message', None)
        self.custom_log_folders = kwargs.get('custom_log_folders', None)
        self.completed_date_time = kwargs.get('completed_date_time', None)


class MicrosoftGraphAppRoleAssignment(MicrosoftGraphEntity):
    """appRoleAssignment.

    :param id: Read-only.
    :type id: str
    :param app_role_id:
    :type app_role_id: str
    :param creation_timestamp:
    :type creation_timestamp: ~datetime.datetime
    :param principal_display_name:
    :type principal_display_name: str
    :param principal_id:
    :type principal_id: str
    :param principal_type:
    :type principal_type: str
    :param resource_display_name:
    :type resource_display_name: str
    :param resource_id:
    :type resource_id: str
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'app_role_id': {'key': 'appRoleId', 'type': 'str'},
        'creation_timestamp': {'key': 'creationTimestamp', 'type': 'iso-8601'},
        'principal_display_name': {'key': 'principalDisplayName', 'type': 'str'},
        'principal_id': {'key': 'principalId', 'type': 'str'},
        'principal_type': {'key': 'principalType', 'type': 'str'},
        'resource_display_name': {'key': 'resourceDisplayName', 'type': 'str'},
        'resource_id': {'key': 'resourceId', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphAppRoleAssignment, self).__init__(**kwargs)
        self.app_role_id = kwargs.get('app_role_id', None)
        self.creation_timestamp = kwargs.get('creation_timestamp', None)
        self.principal_display_name = kwargs.get('principal_display_name', None)
        self.principal_id = kwargs.get('principal_id', None)
        self.principal_type = kwargs.get('principal_type', None)
        self.resource_display_name = kwargs.get('resource_display_name', None)
        self.resource_id = kwargs.get('resource_id', None)


class MicrosoftGraphApproval(MicrosoftGraphEntity):
    """approval.

    :param id: Read-only.
    :type id: str
    :param pending_steps:
    :type pending_steps: list[~users_user.models.MicrosoftGraphApprovalStep]
    :param completed_steps:
    :type completed_steps: list[~users_user.models.MicrosoftGraphApprovalStep]
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'pending_steps': {'key': 'pendingSteps', 'type': '[MicrosoftGraphApprovalStep]'},
        'completed_steps': {'key': 'completedSteps', 'type': '[MicrosoftGraphApprovalStep]'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphApproval, self).__init__(**kwargs)
        self.pending_steps = kwargs.get('pending_steps', None)
        self.completed_steps = kwargs.get('completed_steps', None)


class MicrosoftGraphApprovalStep(MicrosoftGraphEntity):
    """approvalStep.

    :param id: Read-only.
    :type id: str
    :param display_name:
    :type display_name: str
    :param reviewed_by: identity.
    :type reviewed_by: ~users_user.models.MicrosoftGraphIdentity
    :param reviewed_date_time:
    :type reviewed_date_time: ~datetime.datetime
    :param review_result:
    :type review_result: str
    :param justification:
    :type justification: str
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'display_name': {'key': 'displayName', 'type': 'str'},
        'reviewed_by': {'key': 'reviewedBy', 'type': 'MicrosoftGraphIdentity'},
        'reviewed_date_time': {'key': 'reviewedDateTime', 'type': 'iso-8601'},
        'review_result': {'key': 'reviewResult', 'type': 'str'},
        'justification': {'key': 'justification', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphApprovalStep, self).__init__(**kwargs)
        self.display_name = kwargs.get('display_name', None)
        self.reviewed_by = kwargs.get('reviewed_by', None)
        self.reviewed_date_time = kwargs.get('reviewed_date_time', None)
        self.review_result = kwargs.get('review_result', None)
        self.justification = kwargs.get('justification', None)


class MicrosoftGraphAssignedLabel(msrest.serialization.Model):
    """assignedLabel.

    :param label_id:
    :type label_id: str
    :param display_name:
    :type display_name: str
    """

    _attribute_map = {
        'label_id': {'key': 'labelId', 'type': 'str'},
        'display_name': {'key': 'displayName', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphAssignedLabel, self).__init__(**kwargs)
        self.label_id = kwargs.get('label_id', None)
        self.display_name = kwargs.get('display_name', None)


class MicrosoftGraphAssignedLicense(msrest.serialization.Model):
    """assignedLicense.

    :param disabled_plans: A collection of the unique identifiers for plans that have been
     disabled.
    :type disabled_plans: list[str]
    :param sku_id: The unique identifier for the SKU.
    :type sku_id: str
    """

    _attribute_map = {
        'disabled_plans': {'key': 'disabledPlans', 'type': '[str]'},
        'sku_id': {'key': 'skuId', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphAssignedLicense, self).__init__(**kwargs)
        self.disabled_plans = kwargs.get('disabled_plans', None)
        self.sku_id = kwargs.get('sku_id', None)


class MicrosoftGraphAssignedPlan(msrest.serialization.Model):
    """assignedPlan.

    :param assigned_date_time: The date and time at which the plan was assigned; for example:
     2013-01-02T19:32:30Z. The Timestamp type represents date and time information using ISO 8601
     format and is always in UTC time. For example, midnight UTC on Jan 1, 2014 would look like
     this: '2014-01-01T00:00:00Z'.
    :type assigned_date_time: ~datetime.datetime
    :param capability_status: For example, 'Enabled'.
    :type capability_status: str
    :param service: The name of the service; for example, 'Exchange'.
    :type service: str
    :param service_plan_id: A GUID that identifies the service plan.
    :type service_plan_id: str
    """

    _attribute_map = {
        'assigned_date_time': {'key': 'assignedDateTime', 'type': 'iso-8601'},
        'capability_status': {'key': 'capabilityStatus', 'type': 'str'},
        'service': {'key': 'service', 'type': 'str'},
        'service_plan_id': {'key': 'servicePlanId', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphAssignedPlan, self).__init__(**kwargs)
        self.assigned_date_time = kwargs.get('assigned_date_time', None)
        self.capability_status = kwargs.get('capability_status', None)
        self.service = kwargs.get('service', None)
        self.service_plan_id = kwargs.get('service_plan_id', None)


class MicrosoftGraphAttachment(MicrosoftGraphEntity):
    """attachment.

    :param id: Read-only.
    :type id: str
    :param last_modified_date_time: The Timestamp type represents date and time information using
     ISO 8601 format and is always in UTC time. For example, midnight UTC on Jan 1, 2014 would look
     like this: '2014-01-01T00:00:00Z'.
    :type last_modified_date_time: ~datetime.datetime
    :param name: The attachment's file name.
    :type name: str
    :param content_type: The MIME type.
    :type content_type: str
    :param size: The length of the attachment in bytes.
    :type size: int
    :param is_inline: true if the attachment is an inline attachment; otherwise, false.
    :type is_inline: bool
    """

    _validation = {
        'size': {'maximum': 2147483647, 'minimum': -2147483648},
    }

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'last_modified_date_time': {'key': 'lastModifiedDateTime', 'type': 'iso-8601'},
        'name': {'key': 'name', 'type': 'str'},
        'content_type': {'key': 'contentType', 'type': 'str'},
        'size': {'key': 'size', 'type': 'int'},
        'is_inline': {'key': 'isInline', 'type': 'bool'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphAttachment, self).__init__(**kwargs)
        self.last_modified_date_time = kwargs.get('last_modified_date_time', None)
        self.name = kwargs.get('name', None)
        self.content_type = kwargs.get('content_type', None)
        self.size = kwargs.get('size', None)
        self.is_inline = kwargs.get('is_inline', None)


class MicrosoftGraphRecipient(msrest.serialization.Model):
    """recipient.

    :param email_address: emailAddress.
    :type email_address: ~users_user.models.MicrosoftGraphEmailAddress
    """

    _attribute_map = {
        'email_address': {'key': 'emailAddress', 'type': 'MicrosoftGraphEmailAddress'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphRecipient, self).__init__(**kwargs)
        self.email_address = kwargs.get('email_address', None)


class MicrosoftGraphAttendeeBase(MicrosoftGraphRecipient):
    """attendeeBase.

    :param email_address: emailAddress.
    :type email_address: ~users_user.models.MicrosoftGraphEmailAddress
    :param type: attendeeType. Possible values include: "required", "optional", "resource".
    :type type: str or ~users_user.models.MicrosoftGraphAttendeeType
    """

    _attribute_map = {
        'email_address': {'key': 'emailAddress', 'type': 'MicrosoftGraphEmailAddress'},
        'type': {'key': 'type', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphAttendeeBase, self).__init__(**kwargs)
        self.type = kwargs.get('type', None)


class MicrosoftGraphAttendee(MicrosoftGraphAttendeeBase):
    """attendee.

    :param email_address: emailAddress.
    :type email_address: ~users_user.models.MicrosoftGraphEmailAddress
    :param type: attendeeType. Possible values include: "required", "optional", "resource".
    :type type: str or ~users_user.models.MicrosoftGraphAttendeeType
    :param status: responseStatus.
    :type status: ~users_user.models.MicrosoftGraphResponseStatus
    :param proposed_new_time: timeSlot.
    :type proposed_new_time: ~users_user.models.MicrosoftGraphTimeSlot
    """

    _attribute_map = {
        'email_address': {'key': 'emailAddress', 'type': 'MicrosoftGraphEmailAddress'},
        'type': {'key': 'type', 'type': 'str'},
        'status': {'key': 'status', 'type': 'MicrosoftGraphResponseStatus'},
        'proposed_new_time': {'key': 'proposedNewTime', 'type': 'MicrosoftGraphTimeSlot'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphAttendee, self).__init__(**kwargs)
        self.status = kwargs.get('status', None)
        self.proposed_new_time = kwargs.get('proposed_new_time', None)


class MicrosoftGraphAudio(msrest.serialization.Model):
    """audio.

    :param album: The title of the album for this audio file.
    :type album: str
    :param album_artist: The artist named on the album for the audio file.
    :type album_artist: str
    :param artist: The performing artist for the audio file.
    :type artist: str
    :param bitrate: Bitrate expressed in kbps.
    :type bitrate: long
    :param composers: The name of the composer of the audio file.
    :type composers: str
    :param copyright: Copyright information for the audio file.
    :type copyright: str
    :param disc: The number of the disc this audio file came from.
    :type disc: int
    :param disc_count: The total number of discs in this album.
    :type disc_count: int
    :param duration: Duration of the audio file, expressed in milliseconds.
    :type duration: long
    :param genre: The genre of this audio file.
    :type genre: str
    :param has_drm: Indicates if the file is protected with digital rights management.
    :type has_drm: bool
    :param is_variable_bitrate: Indicates if the file is encoded with a variable bitrate.
    :type is_variable_bitrate: bool
    :param title: The title of the audio file.
    :type title: str
    :param track: The number of the track on the original disc for this audio file.
    :type track: int
    :param track_count: The total number of tracks on the original disc for this audio file.
    :type track_count: int
    :param year: The year the audio file was recorded.
    :type year: int
    """

    _validation = {
        'disc': {'maximum': 32767, 'minimum': -32768},
        'disc_count': {'maximum': 32767, 'minimum': -32768},
        'track': {'maximum': 2147483647, 'minimum': -2147483648},
        'track_count': {'maximum': 2147483647, 'minimum': -2147483648},
        'year': {'maximum': 2147483647, 'minimum': -2147483648},
    }

    _attribute_map = {
        'album': {'key': 'album', 'type': 'str'},
        'album_artist': {'key': 'albumArtist', 'type': 'str'},
        'artist': {'key': 'artist', 'type': 'str'},
        'bitrate': {'key': 'bitrate', 'type': 'long'},
        'composers': {'key': 'composers', 'type': 'str'},
        'copyright': {'key': 'copyright', 'type': 'str'},
        'disc': {'key': 'disc', 'type': 'int'},
        'disc_count': {'key': 'discCount', 'type': 'int'},
        'duration': {'key': 'duration', 'type': 'long'},
        'genre': {'key': 'genre', 'type': 'str'},
        'has_drm': {'key': 'hasDrm', 'type': 'bool'},
        'is_variable_bitrate': {'key': 'isVariableBitrate', 'type': 'bool'},
        'title': {'key': 'title', 'type': 'str'},
        'track': {'key': 'track', 'type': 'int'},
        'track_count': {'key': 'trackCount', 'type': 'int'},
        'year': {'key': 'year', 'type': 'int'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphAudio, self).__init__(**kwargs)
        self.album = kwargs.get('album', None)
        self.album_artist = kwargs.get('album_artist', None)
        self.artist = kwargs.get('artist', None)
        self.bitrate = kwargs.get('bitrate', None)
        self.composers = kwargs.get('composers', None)
        self.copyright = kwargs.get('copyright', None)
        self.disc = kwargs.get('disc', None)
        self.disc_count = kwargs.get('disc_count', None)
        self.duration = kwargs.get('duration', None)
        self.genre = kwargs.get('genre', None)
        self.has_drm = kwargs.get('has_drm', None)
        self.is_variable_bitrate = kwargs.get('is_variable_bitrate', None)
        self.title = kwargs.get('title', None)
        self.track = kwargs.get('track', None)
        self.track_count = kwargs.get('track_count', None)
        self.year = kwargs.get('year', None)


class MicrosoftGraphAudioConferencing(msrest.serialization.Model):
    """audioConferencing.

    :param conference_id:
    :type conference_id: str
    :param toll_number: The toll number that connects to the Audio Conference Provider.
    :type toll_number: str
    :param toll_free_number: The toll-free number that connects to the Audio Conference Provider.
    :type toll_free_number: str
    :param dialin_url: A URL to the externally-accessible web page that contains dial-in
     information.
    :type dialin_url: str
    """

    _attribute_map = {
        'conference_id': {'key': 'conferenceId', 'type': 'str'},
        'toll_number': {'key': 'tollNumber', 'type': 'str'},
        'toll_free_number': {'key': 'tollFreeNumber', 'type': 'str'},
        'dialin_url': {'key': 'dialinUrl', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphAudioConferencing, self).__init__(**kwargs)
        self.conference_id = kwargs.get('conference_id', None)
        self.toll_number = kwargs.get('toll_number', None)
        self.toll_free_number = kwargs.get('toll_free_number', None)
        self.dialin_url = kwargs.get('dialin_url', None)


class MicrosoftGraphAuthentication(MicrosoftGraphEntity):
    """authentication.

    :param id: Read-only.
    :type id: str
    :param methods:
    :type methods: list[~users_user.models.MicrosoftGraphEntity]
    :param phone_methods:
    :type phone_methods: list[~users_user.models.MicrosoftGraphPhoneAuthenticationMethod]
    :param password_methods:
    :type password_methods: list[~users_user.models.MicrosoftGraphPasswordAuthenticationMethod]
    :param operations:
    :type operations: list[~users_user.models.MicrosoftGraphLongRunningOperation]
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'methods': {'key': 'methods', 'type': '[MicrosoftGraphEntity]'},
        'phone_methods': {'key': 'phoneMethods', 'type': '[MicrosoftGraphPhoneAuthenticationMethod]'},
        'password_methods': {'key': 'passwordMethods', 'type': '[MicrosoftGraphPasswordAuthenticationMethod]'},
        'operations': {'key': 'operations', 'type': '[MicrosoftGraphLongRunningOperation]'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphAuthentication, self).__init__(**kwargs)
        self.methods = kwargs.get('methods', None)
        self.phone_methods = kwargs.get('phone_methods', None)
        self.password_methods = kwargs.get('password_methods', None)
        self.operations = kwargs.get('operations', None)


class MicrosoftGraphAuthenticationMethod(MicrosoftGraphEntity):
    """authenticationMethod.

    :param id: Read-only.
    :type id: str
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphAuthenticationMethod, self).__init__(**kwargs)


class MicrosoftGraphAutoLabeling(msrest.serialization.Model):
    """autoLabeling.

    :param sensitive_type_ids:
    :type sensitive_type_ids: list[str]
    :param message:
    :type message: str
    """

    _attribute_map = {
        'sensitive_type_ids': {'key': 'sensitiveTypeIds', 'type': '[str]'},
        'message': {'key': 'message', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphAutoLabeling, self).__init__(**kwargs)
        self.sensitive_type_ids = kwargs.get('sensitive_type_ids', None)
        self.message = kwargs.get('message', None)


class MicrosoftGraphAutomaticRepliesSetting(msrest.serialization.Model):
    """automaticRepliesSetting.

    :param status: automaticRepliesStatus. Possible values include: "disabled", "alwaysEnabled",
     "scheduled".
    :type status: str or ~users_user.models.MicrosoftGraphAutomaticRepliesStatus
    :param external_audience: externalAudienceScope. Possible values include: "none",
     "contactsOnly", "all".
    :type external_audience: str or ~users_user.models.MicrosoftGraphExternalAudienceScope
    :param scheduled_start_date_time: dateTimeTimeZone.
    :type scheduled_start_date_time: ~users_user.models.MicrosoftGraphDateTimeZone
    :param scheduled_end_date_time: dateTimeTimeZone.
    :type scheduled_end_date_time: ~users_user.models.MicrosoftGraphDateTimeZone
    :param internal_reply_message: The automatic reply to send to the audience internal to the
     signed-in user's organization, if Status is AlwaysEnabled or Scheduled.
    :type internal_reply_message: str
    :param external_reply_message: The automatic reply to send to the specified external audience,
     if Status is AlwaysEnabled or Scheduled.
    :type external_reply_message: str
    """

    _attribute_map = {
        'status': {'key': 'status', 'type': 'str'},
        'external_audience': {'key': 'externalAudience', 'type': 'str'},
        'scheduled_start_date_time': {'key': 'scheduledStartDateTime', 'type': 'MicrosoftGraphDateTimeZone'},
        'scheduled_end_date_time': {'key': 'scheduledEndDateTime', 'type': 'MicrosoftGraphDateTimeZone'},
        'internal_reply_message': {'key': 'internalReplyMessage', 'type': 'str'},
        'external_reply_message': {'key': 'externalReplyMessage', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphAutomaticRepliesSetting, self).__init__(**kwargs)
        self.status = kwargs.get('status', None)
        self.external_audience = kwargs.get('external_audience', None)
        self.scheduled_start_date_time = kwargs.get('scheduled_start_date_time', None)
        self.scheduled_end_date_time = kwargs.get('scheduled_end_date_time', None)
        self.internal_reply_message = kwargs.get('internal_reply_message', None)
        self.external_reply_message = kwargs.get('external_reply_message', None)


class MicrosoftGraphBaseItem(MicrosoftGraphEntity):
    """baseItem.

    :param id: Read-only.
    :type id: str
    :param created_by: identitySet.
    :type created_by: ~users_user.models.MicrosoftGraphIdentitySet
    :param created_date_time: Date and time of item creation. Read-only.
    :type created_date_time: ~datetime.datetime
    :param description: Provides a user-visible description of the item. Optional.
    :type description: str
    :param e_tag: ETag for the item. Read-only.
    :type e_tag: str
    :param last_modified_by: identitySet.
    :type last_modified_by: ~users_user.models.MicrosoftGraphIdentitySet
    :param last_modified_date_time: Date and time the item was last modified. Read-only.
    :type last_modified_date_time: ~datetime.datetime
    :param name: The name of the item. Read-write.
    :type name: str
    :param parent_reference: itemReference.
    :type parent_reference: ~users_user.models.MicrosoftGraphItemReference
    :param web_url: URL that displays the resource in the browser. Read-only.
    :type web_url: str
    :param created_by_user: Represents an Azure Active Directory user object.
    :type created_by_user: ~users_user.models.MicrosoftGraphUser
    :param last_modified_by_user: Represents an Azure Active Directory user object.
    :type last_modified_by_user: ~users_user.models.MicrosoftGraphUser
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'created_by': {'key': 'createdBy', 'type': 'MicrosoftGraphIdentitySet'},
        'created_date_time': {'key': 'createdDateTime', 'type': 'iso-8601'},
        'description': {'key': 'description', 'type': 'str'},
        'e_tag': {'key': 'eTag', 'type': 'str'},
        'last_modified_by': {'key': 'lastModifiedBy', 'type': 'MicrosoftGraphIdentitySet'},
        'last_modified_date_time': {'key': 'lastModifiedDateTime', 'type': 'iso-8601'},
        'name': {'key': 'name', 'type': 'str'},
        'parent_reference': {'key': 'parentReference', 'type': 'MicrosoftGraphItemReference'},
        'web_url': {'key': 'webUrl', 'type': 'str'},
        'created_by_user': {'key': 'createdByUser', 'type': 'MicrosoftGraphUser'},
        'last_modified_by_user': {'key': 'lastModifiedByUser', 'type': 'MicrosoftGraphUser'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphBaseItem, self).__init__(**kwargs)
        self.created_by = kwargs.get('created_by', None)
        self.created_date_time = kwargs.get('created_date_time', None)
        self.description = kwargs.get('description', None)
        self.e_tag = kwargs.get('e_tag', None)
        self.last_modified_by = kwargs.get('last_modified_by', None)
        self.last_modified_date_time = kwargs.get('last_modified_date_time', None)
        self.name = kwargs.get('name', None)
        self.parent_reference = kwargs.get('parent_reference', None)
        self.web_url = kwargs.get('web_url', None)
        self.created_by_user = kwargs.get('created_by_user', None)
        self.last_modified_by_user = kwargs.get('last_modified_by_user', None)


class MicrosoftGraphBaseItemVersion(MicrosoftGraphEntity):
    """baseItemVersion.

    :param id: Read-only.
    :type id: str
    :param last_modified_by: identitySet.
    :type last_modified_by: ~users_user.models.MicrosoftGraphIdentitySet
    :param last_modified_date_time: Date and time the version was last modified. Read-only.
    :type last_modified_date_time: ~datetime.datetime
    :param publication: publicationFacet.
    :type publication: ~users_user.models.MicrosoftGraphPublicationFacet
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'last_modified_by': {'key': 'lastModifiedBy', 'type': 'MicrosoftGraphIdentitySet'},
        'last_modified_date_time': {'key': 'lastModifiedDateTime', 'type': 'iso-8601'},
        'publication': {'key': 'publication', 'type': 'MicrosoftGraphPublicationFacet'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphBaseItemVersion, self).__init__(**kwargs)
        self.last_modified_by = kwargs.get('last_modified_by', None)
        self.last_modified_date_time = kwargs.get('last_modified_date_time', None)
        self.publication = kwargs.get('publication', None)


class MicrosoftGraphBundle(msrest.serialization.Model):
    """bundle.

    :param child_count:
    :type child_count: int
    :param album: album.
    :type album: ~users_user.models.MicrosoftGraphAlbum
    """

    _validation = {
        'child_count': {'maximum': 2147483647, 'minimum': -2147483648},
    }

    _attribute_map = {
        'child_count': {'key': 'childCount', 'type': 'int'},
        'album': {'key': 'album', 'type': 'MicrosoftGraphAlbum'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphBundle, self).__init__(**kwargs)
        self.child_count = kwargs.get('child_count', None)
        self.album = kwargs.get('album', None)


class MicrosoftGraphCalculatedColumn(msrest.serialization.Model):
    """calculatedColumn.

    :param format: For dateTime output types, the format of the value. Must be one of dateOnly or
     dateTime.
    :type format: str
    :param formula: The formula used to compute the value for this column.
    :type formula: str
    :param output_type: The output type used to format values in this column. Must be one of
     boolean, currency, dateTime, number, or text.
    :type output_type: str
    """

    _attribute_map = {
        'format': {'key': 'format', 'type': 'str'},
        'formula': {'key': 'formula', 'type': 'str'},
        'output_type': {'key': 'outputType', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphCalculatedColumn, self).__init__(**kwargs)
        self.format = kwargs.get('format', None)
        self.formula = kwargs.get('formula', None)
        self.output_type = kwargs.get('output_type', None)


class MicrosoftGraphCalendar(MicrosoftGraphEntity):
    """calendar.

    :param id: Read-only.
    :type id: str
    :param name: The calendar name.
    :type name: str
    :param color: calendarColor. Possible values include: "lightBlue", "lightGreen", "lightOrange",
     "lightGray", "lightYellow", "lightTeal", "lightPink", "lightBrown", "lightRed", "maxColor",
     "auto".
    :type color: str or ~users_user.models.MicrosoftGraphCalendarColor
    :param hex_color:
    :type hex_color: str
    :param is_default_calendar: True if this is the default calendar where new events are created
     by default, false otherwise.
    :type is_default_calendar: bool
    :param change_key: Identifies the version of the calendar object. Every time the calendar is
     changed, changeKey changes as well. This allows Exchange to apply changes to the correct
     version of the object. Read-only.
    :type change_key: str
    :param can_share: True if the user has the permission to share the calendar, false otherwise.
     Only the user who created the calendar can share it.
    :type can_share: bool
    :param can_view_private_items: True if the user can read calendar items that have been marked
     private, false otherwise.
    :type can_view_private_items: bool
    :param is_shared:
    :type is_shared: bool
    :param is_shared_with_me:
    :type is_shared_with_me: bool
    :param can_edit: True if the user can write to the calendar, false otherwise. This property is
     true for the user who created the calendar. This property is also true for a user who has been
     shared a calendar and granted write access.
    :type can_edit: bool
    :param owner: emailAddress.
    :type owner: ~users_user.models.MicrosoftGraphEmailAddress
    :param calendar_group_id:
    :type calendar_group_id: str
    :param allowed_online_meeting_providers: Represent the online meeting service providers that
     can be used to create online meetings in this calendar. Possible values are: unknown,
     skypeForBusiness, skypeForConsumer, teamsForBusiness.
    :type allowed_online_meeting_providers: list[str or
     ~users_user.models.MicrosoftGraphOnlineMeetingProviderType]
    :param default_online_meeting_provider: onlineMeetingProviderType. Possible values include:
     "unknown", "skypeForBusiness", "skypeForConsumer", "teamsForBusiness".
    :type default_online_meeting_provider: str or
     ~users_user.models.MicrosoftGraphOnlineMeetingProviderType
    :param is_tallying_responses: Indicates whether this user calendar supports tracking of meeting
     responses. Only meeting invites sent from users' primary calendars support tracking of meeting
     responses.
    :type is_tallying_responses: bool
    :param is_removable: Indicates whether this user calendar can be deleted from the user mailbox.
    :type is_removable: bool
    :param single_value_extended_properties: The collection of single-value extended properties
     defined for the calendar. Read-only. Nullable.
    :type single_value_extended_properties:
     list[~users_user.models.MicrosoftGraphSingleValueLegacyExtendedProperty]
    :param multi_value_extended_properties: The collection of multi-value extended properties
     defined for the calendar. Read-only. Nullable.
    :type multi_value_extended_properties:
     list[~users_user.models.MicrosoftGraphMultiValueLegacyExtendedProperty]
    :param calendar_permissions: The permissions of the users with whom the calendar is shared.
    :type calendar_permissions: list[~users_user.models.MicrosoftGraphCalendarPermission]
    :param events: The events in the calendar. Navigation property. Read-only.
    :type events: list[~users_user.models.MicrosoftGraphEvent]
    :param calendar_view: The calendar view for the calendar. Navigation property. Read-only.
    :type calendar_view: list[~users_user.models.MicrosoftGraphEvent]
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'name': {'key': 'name', 'type': 'str'},
        'color': {'key': 'color', 'type': 'str'},
        'hex_color': {'key': 'hexColor', 'type': 'str'},
        'is_default_calendar': {'key': 'isDefaultCalendar', 'type': 'bool'},
        'change_key': {'key': 'changeKey', 'type': 'str'},
        'can_share': {'key': 'canShare', 'type': 'bool'},
        'can_view_private_items': {'key': 'canViewPrivateItems', 'type': 'bool'},
        'is_shared': {'key': 'isShared', 'type': 'bool'},
        'is_shared_with_me': {'key': 'isSharedWithMe', 'type': 'bool'},
        'can_edit': {'key': 'canEdit', 'type': 'bool'},
        'owner': {'key': 'owner', 'type': 'MicrosoftGraphEmailAddress'},
        'calendar_group_id': {'key': 'calendarGroupId', 'type': 'str'},
        'allowed_online_meeting_providers': {'key': 'allowedOnlineMeetingProviders', 'type': '[str]'},
        'default_online_meeting_provider': {'key': 'defaultOnlineMeetingProvider', 'type': 'str'},
        'is_tallying_responses': {'key': 'isTallyingResponses', 'type': 'bool'},
        'is_removable': {'key': 'isRemovable', 'type': 'bool'},
        'single_value_extended_properties': {'key': 'singleValueExtendedProperties', 'type': '[MicrosoftGraphSingleValueLegacyExtendedProperty]'},
        'multi_value_extended_properties': {'key': 'multiValueExtendedProperties', 'type': '[MicrosoftGraphMultiValueLegacyExtendedProperty]'},
        'calendar_permissions': {'key': 'calendarPermissions', 'type': '[MicrosoftGraphCalendarPermission]'},
        'events': {'key': 'events', 'type': '[MicrosoftGraphEvent]'},
        'calendar_view': {'key': 'calendarView', 'type': '[MicrosoftGraphEvent]'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphCalendar, self).__init__(**kwargs)
        self.name = kwargs.get('name', None)
        self.color = kwargs.get('color', None)
        self.hex_color = kwargs.get('hex_color', None)
        self.is_default_calendar = kwargs.get('is_default_calendar', None)
        self.change_key = kwargs.get('change_key', None)
        self.can_share = kwargs.get('can_share', None)
        self.can_view_private_items = kwargs.get('can_view_private_items', None)
        self.is_shared = kwargs.get('is_shared', None)
        self.is_shared_with_me = kwargs.get('is_shared_with_me', None)
        self.can_edit = kwargs.get('can_edit', None)
        self.owner = kwargs.get('owner', None)
        self.calendar_group_id = kwargs.get('calendar_group_id', None)
        self.allowed_online_meeting_providers = kwargs.get('allowed_online_meeting_providers', None)
        self.default_online_meeting_provider = kwargs.get('default_online_meeting_provider', None)
        self.is_tallying_responses = kwargs.get('is_tallying_responses', None)
        self.is_removable = kwargs.get('is_removable', None)
        self.single_value_extended_properties = kwargs.get('single_value_extended_properties', None)
        self.multi_value_extended_properties = kwargs.get('multi_value_extended_properties', None)
        self.calendar_permissions = kwargs.get('calendar_permissions', None)
        self.events = kwargs.get('events', None)
        self.calendar_view = kwargs.get('calendar_view', None)


class MicrosoftGraphCalendarGroup(MicrosoftGraphEntity):
    """calendarGroup.

    :param id: Read-only.
    :type id: str
    :param name: The group name.
    :type name: str
    :param class_id: The class identifier. Read-only.
    :type class_id: str
    :param change_key: Identifies the version of the calendar group. Every time the calendar group
     is changed, ChangeKey changes as well. This allows Exchange to apply changes to the correct
     version of the object. Read-only.
    :type change_key: str
    :param calendars: The calendars in the calendar group. Navigation property. Read-only.
     Nullable.
    :type calendars: list[~users_user.models.MicrosoftGraphCalendar]
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'name': {'key': 'name', 'type': 'str'},
        'class_id': {'key': 'classId', 'type': 'str'},
        'change_key': {'key': 'changeKey', 'type': 'str'},
        'calendars': {'key': 'calendars', 'type': '[MicrosoftGraphCalendar]'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphCalendarGroup, self).__init__(**kwargs)
        self.name = kwargs.get('name', None)
        self.class_id = kwargs.get('class_id', None)
        self.change_key = kwargs.get('change_key', None)
        self.calendars = kwargs.get('calendars', None)


class MicrosoftGraphCalendarPermission(MicrosoftGraphEntity):
    """calendarPermission.

    :param id: Read-only.
    :type id: str
    :param email_address: emailAddress.
    :type email_address: ~users_user.models.MicrosoftGraphEmailAddress
    :param is_removable: True if the user can be removed from the list of sharees or delegates for
     the specified calendar, false otherwise. The 'My organization' user determines the permissions
     other people within your organization have to the given calendar. You cannot remove 'My
     organization' as a sharee to a calendar.
    :type is_removable: bool
    :param is_inside_organization: True if the user in context (sharee or delegate) is inside the
     same organization as the calendar owner.
    :type is_inside_organization: bool
    :param role: calendarRoleType. Possible values include: "none", "freeBusyRead", "limitedRead",
     "read", "write", "delegateWithoutPrivateEventAccess", "delegateWithPrivateEventAccess",
     "custom".
    :type role: str or ~users_user.models.MicrosoftGraphCalendarRoleType
    :param allowed_roles: List of allowed sharing or delegating permission levels for the calendar.
     Possible values are: none, freeBusyRead, limitedRead, read, write,
     delegateWithoutPrivateEventAccess, delegateWithPrivateEventAccess, custom.
    :type allowed_roles: list[str or ~users_user.models.MicrosoftGraphCalendarRoleType]
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'email_address': {'key': 'emailAddress', 'type': 'MicrosoftGraphEmailAddress'},
        'is_removable': {'key': 'isRemovable', 'type': 'bool'},
        'is_inside_organization': {'key': 'isInsideOrganization', 'type': 'bool'},
        'role': {'key': 'role', 'type': 'str'},
        'allowed_roles': {'key': 'allowedRoles', 'type': '[str]'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphCalendarPermission, self).__init__(**kwargs)
        self.email_address = kwargs.get('email_address', None)
        self.is_removable = kwargs.get('is_removable', None)
        self.is_inside_organization = kwargs.get('is_inside_organization', None)
        self.role = kwargs.get('role', None)
        self.allowed_roles = kwargs.get('allowed_roles', None)


class MicrosoftGraphChangeTrackedEntity(MicrosoftGraphEntity):
    """changeTrackedEntity.

    :param id: Read-only.
    :type id: str
    :param created_date_time:
    :type created_date_time: ~datetime.datetime
    :param last_modified_date_time:
    :type last_modified_date_time: ~datetime.datetime
    :param last_modified_by: identitySet.
    :type last_modified_by: ~users_user.models.MicrosoftGraphIdentitySet
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'created_date_time': {'key': 'createdDateTime', 'type': 'iso-8601'},
        'last_modified_date_time': {'key': 'lastModifiedDateTime', 'type': 'iso-8601'},
        'last_modified_by': {'key': 'lastModifiedBy', 'type': 'MicrosoftGraphIdentitySet'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphChangeTrackedEntity, self).__init__(**kwargs)
        self.created_date_time = kwargs.get('created_date_time', None)
        self.last_modified_date_time = kwargs.get('last_modified_date_time', None)
        self.last_modified_by = kwargs.get('last_modified_by', None)


class MicrosoftGraphChannel(MicrosoftGraphEntity):
    """channel.

    :param id: Read-only.
    :type id: str
    :param display_name: Channel name as it will appear to the user in Microsoft Teams.
    :type display_name: str
    :param description: Optional textual description for the channel.
    :type description: str
    :param is_favorite_by_default:
    :type is_favorite_by_default: bool
    :param email: The email address for sending messages to the channel. Read-only.
    :type email: str
    :param web_url: A hyperlink that will navigate to the channel in Microsoft Teams. This is the
     URL that you get when you right-click a channel in Microsoft Teams and select Get link to
     channel. This URL should be treated as an opaque blob, and not parsed. Read-only.
    :type web_url: str
    :param membership_type: channelMembershipType. Possible values include: "standard", "private",
     "unknownFutureValue".
    :type membership_type: str or ~users_user.models.MicrosoftGraphChannelMembershipType
    :param messages:
    :type messages: list[~users_user.models.MicrosoftGraphChatMessage]
    :param tabs: A collection of all the tabs in the channel. A navigation property.
    :type tabs: list[~users_user.models.MicrosoftGraphTeamsTab]
    :param members:
    :type members: list[~users_user.models.MicrosoftGraphConversationMember]
    :param files_folder: driveItem.
    :type files_folder: ~users_user.models.MicrosoftGraphDriveItem
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'display_name': {'key': 'displayName', 'type': 'str'},
        'description': {'key': 'description', 'type': 'str'},
        'is_favorite_by_default': {'key': 'isFavoriteByDefault', 'type': 'bool'},
        'email': {'key': 'email', 'type': 'str'},
        'web_url': {'key': 'webUrl', 'type': 'str'},
        'membership_type': {'key': 'membershipType', 'type': 'str'},
        'messages': {'key': 'messages', 'type': '[MicrosoftGraphChatMessage]'},
        'tabs': {'key': 'tabs', 'type': '[MicrosoftGraphTeamsTab]'},
        'members': {'key': 'members', 'type': '[MicrosoftGraphConversationMember]'},
        'files_folder': {'key': 'filesFolder', 'type': 'MicrosoftGraphDriveItem'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphChannel, self).__init__(**kwargs)
        self.display_name = kwargs.get('display_name', None)
        self.description = kwargs.get('description', None)
        self.is_favorite_by_default = kwargs.get('is_favorite_by_default', None)
        self.email = kwargs.get('email', None)
        self.web_url = kwargs.get('web_url', None)
        self.membership_type = kwargs.get('membership_type', None)
        self.messages = kwargs.get('messages', None)
        self.tabs = kwargs.get('tabs', None)
        self.members = kwargs.get('members', None)
        self.files_folder = kwargs.get('files_folder', None)


class MicrosoftGraphChat(MicrosoftGraphEntity):
    """chat.

    :param id: Read-only.
    :type id: str
    :param topic:
    :type topic: str
    :param created_date_time:
    :type created_date_time: ~datetime.datetime
    :param last_updated_date_time:
    :type last_updated_date_time: ~datetime.datetime
    :param members:
    :type members: list[~users_user.models.MicrosoftGraphConversationMember]
    :param messages:
    :type messages: list[~users_user.models.MicrosoftGraphChatMessage]
    :param installed_apps:
    :type installed_apps: list[~users_user.models.MicrosoftGraphTeamsAppInstallation]
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'topic': {'key': 'topic', 'type': 'str'},
        'created_date_time': {'key': 'createdDateTime', 'type': 'iso-8601'},
        'last_updated_date_time': {'key': 'lastUpdatedDateTime', 'type': 'iso-8601'},
        'members': {'key': 'members', 'type': '[MicrosoftGraphConversationMember]'},
        'messages': {'key': 'messages', 'type': '[MicrosoftGraphChatMessage]'},
        'installed_apps': {'key': 'installedApps', 'type': '[MicrosoftGraphTeamsAppInstallation]'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphChat, self).__init__(**kwargs)
        self.topic = kwargs.get('topic', None)
        self.created_date_time = kwargs.get('created_date_time', None)
        self.last_updated_date_time = kwargs.get('last_updated_date_time', None)
        self.members = kwargs.get('members', None)
        self.messages = kwargs.get('messages', None)
        self.installed_apps = kwargs.get('installed_apps', None)


class MicrosoftGraphChatInfo(msrest.serialization.Model):
    """chatInfo.

    :param thread_id: The unique identifier for a thread in Microsoft Teams.
    :type thread_id: str
    :param message_id: The unique identifier of a message in a Microsoft Teams channel.
    :type message_id: str
    :param reply_chain_message_id: The ID of the reply message.
    :type reply_chain_message_id: str
    """

    _attribute_map = {
        'thread_id': {'key': 'threadId', 'type': 'str'},
        'message_id': {'key': 'messageId', 'type': 'str'},
        'reply_chain_message_id': {'key': 'replyChainMessageId', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphChatInfo, self).__init__(**kwargs)
        self.thread_id = kwargs.get('thread_id', None)
        self.message_id = kwargs.get('message_id', None)
        self.reply_chain_message_id = kwargs.get('reply_chain_message_id', None)


class MicrosoftGraphChatMessage(MicrosoftGraphEntity):
    """chatMessage.

    :param id: Read-only.
    :type id: str
    :param reply_to_id:
    :type reply_to_id: str
    :param from_property: identitySet.
    :type from_property: ~users_user.models.MicrosoftGraphIdentitySet
    :param etag:
    :type etag: str
    :param message_type: chatMessageType. Possible values include: "message", "chatEvent",
     "typing".
    :type message_type: str or ~users_user.models.MicrosoftGraphChatMessageType
    :param created_date_time:
    :type created_date_time: ~datetime.datetime
    :param last_modified_date_time:
    :type last_modified_date_time: ~datetime.datetime
    :param deleted_date_time:
    :type deleted_date_time: ~datetime.datetime
    :param subject:
    :type subject: str
    :param body: itemBody.
    :type body: ~users_user.models.MicrosoftGraphItemBody
    :param summary:
    :type summary: str
    :param attachments:
    :type attachments: list[~users_user.models.MicrosoftGraphChatMessageAttachment]
    :param mentions:
    :type mentions: list[~users_user.models.MicrosoftGraphChatMessageMention]
    :param importance: chatMessageImportance. Possible values include: "normal", "high", "urgent".
    :type importance: str or ~users_user.models.MicrosoftGraphChatMessageImportance
    :param policy_violation: chatMessagePolicyViolation.
    :type policy_violation: ~users_user.models.MicrosoftGraphChatMessagePolicyViolation
    :param reactions:
    :type reactions: list[~users_user.models.MicrosoftGraphChatMessageReaction]
    :param locale:
    :type locale: str
    :param web_url:
    :type web_url: str
    :param replies:
    :type replies: list[~users_user.models.MicrosoftGraphChatMessage]
    :param hosted_contents:
    :type hosted_contents: list[~users_user.models.MicrosoftGraphEntity]
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'reply_to_id': {'key': 'replyToId', 'type': 'str'},
        'from_property': {'key': 'from', 'type': 'MicrosoftGraphIdentitySet'},
        'etag': {'key': 'etag', 'type': 'str'},
        'message_type': {'key': 'messageType', 'type': 'str'},
        'created_date_time': {'key': 'createdDateTime', 'type': 'iso-8601'},
        'last_modified_date_time': {'key': 'lastModifiedDateTime', 'type': 'iso-8601'},
        'deleted_date_time': {'key': 'deletedDateTime', 'type': 'iso-8601'},
        'subject': {'key': 'subject', 'type': 'str'},
        'body': {'key': 'body', 'type': 'MicrosoftGraphItemBody'},
        'summary': {'key': 'summary', 'type': 'str'},
        'attachments': {'key': 'attachments', 'type': '[MicrosoftGraphChatMessageAttachment]'},
        'mentions': {'key': 'mentions', 'type': '[MicrosoftGraphChatMessageMention]'},
        'importance': {'key': 'importance', 'type': 'str'},
        'policy_violation': {'key': 'policyViolation', 'type': 'MicrosoftGraphChatMessagePolicyViolation'},
        'reactions': {'key': 'reactions', 'type': '[MicrosoftGraphChatMessageReaction]'},
        'locale': {'key': 'locale', 'type': 'str'},
        'web_url': {'key': 'webUrl', 'type': 'str'},
        'replies': {'key': 'replies', 'type': '[MicrosoftGraphChatMessage]'},
        'hosted_contents': {'key': 'hostedContents', 'type': '[MicrosoftGraphEntity]'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphChatMessage, self).__init__(**kwargs)
        self.reply_to_id = kwargs.get('reply_to_id', None)
        self.from_property = kwargs.get('from_property', None)
        self.etag = kwargs.get('etag', None)
        self.message_type = kwargs.get('message_type', None)
        self.created_date_time = kwargs.get('created_date_time', None)
        self.last_modified_date_time = kwargs.get('last_modified_date_time', None)
        self.deleted_date_time = kwargs.get('deleted_date_time', None)
        self.subject = kwargs.get('subject', None)
        self.body = kwargs.get('body', None)
        self.summary = kwargs.get('summary', None)
        self.attachments = kwargs.get('attachments', None)
        self.mentions = kwargs.get('mentions', None)
        self.importance = kwargs.get('importance', None)
        self.policy_violation = kwargs.get('policy_violation', None)
        self.reactions = kwargs.get('reactions', None)
        self.locale = kwargs.get('locale', None)
        self.web_url = kwargs.get('web_url', None)
        self.replies = kwargs.get('replies', None)
        self.hosted_contents = kwargs.get('hosted_contents', None)


class MicrosoftGraphChatMessageAttachment(msrest.serialization.Model):
    """chatMessageAttachment.

    :param id:
    :type id: str
    :param content_type:
    :type content_type: str
    :param content_url:
    :type content_url: str
    :param content:
    :type content: str
    :param name:
    :type name: str
    :param thumbnail_url:
    :type thumbnail_url: str
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'content_type': {'key': 'contentType', 'type': 'str'},
        'content_url': {'key': 'contentUrl', 'type': 'str'},
        'content': {'key': 'content', 'type': 'str'},
        'name': {'key': 'name', 'type': 'str'},
        'thumbnail_url': {'key': 'thumbnailUrl', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphChatMessageAttachment, self).__init__(**kwargs)
        self.id = kwargs.get('id', None)
        self.content_type = kwargs.get('content_type', None)
        self.content_url = kwargs.get('content_url', None)
        self.content = kwargs.get('content', None)
        self.name = kwargs.get('name', None)
        self.thumbnail_url = kwargs.get('thumbnail_url', None)


class MicrosoftGraphChatMessageHostedContent(MicrosoftGraphEntity):
    """chatMessageHostedContent.

    :param id: Read-only.
    :type id: str
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphChatMessageHostedContent, self).__init__(**kwargs)


class MicrosoftGraphChatMessageMention(msrest.serialization.Model):
    """chatMessageMention.

    :param id:
    :type id: int
    :param mention_text:
    :type mention_text: str
    :param mentioned: identitySet.
    :type mentioned: ~users_user.models.MicrosoftGraphIdentitySet
    """

    _validation = {
        'id': {'maximum': 2147483647, 'minimum': -2147483648},
    }

    _attribute_map = {
        'id': {'key': 'id', 'type': 'int'},
        'mention_text': {'key': 'mentionText', 'type': 'str'},
        'mentioned': {'key': 'mentioned', 'type': 'MicrosoftGraphIdentitySet'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphChatMessageMention, self).__init__(**kwargs)
        self.id = kwargs.get('id', None)
        self.mention_text = kwargs.get('mention_text', None)
        self.mentioned = kwargs.get('mentioned', None)


class MicrosoftGraphChatMessagePolicyViolation(msrest.serialization.Model):
    """chatMessagePolicyViolation.

    :param dlp_action: chatMessagePolicyViolationDlpActionTypes. Possible values include: "none",
     "notifySender", "blockAccess", "blockAccessExternal".
    :type dlp_action: str or
     ~users_user.models.MicrosoftGraphChatMessagePolicyViolationDlpActionTypes
    :param justification_text:
    :type justification_text: str
    :param policy_tip: chatMessagePolicyViolationPolicyTip.
    :type policy_tip: ~users_user.models.MicrosoftGraphChatMessagePolicyViolationPolicyTip
    :param user_action: chatMessagePolicyViolationUserActionTypes. Possible values include: "none",
     "override", "reportFalsePositive".
    :type user_action: str or
     ~users_user.models.MicrosoftGraphChatMessagePolicyViolationUserActionTypes
    :param verdict_details: chatMessagePolicyViolationVerdictDetailsTypes. Possible values include:
     "none", "allowFalsePositiveOverride", "allowOverrideWithoutJustification",
     "allowOverrideWithJustification".
    :type verdict_details: str or
     ~users_user.models.MicrosoftGraphChatMessagePolicyViolationVerdictDetailsTypes
    """

    _attribute_map = {
        'dlp_action': {'key': 'dlpAction', 'type': 'str'},
        'justification_text': {'key': 'justificationText', 'type': 'str'},
        'policy_tip': {'key': 'policyTip', 'type': 'MicrosoftGraphChatMessagePolicyViolationPolicyTip'},
        'user_action': {'key': 'userAction', 'type': 'str'},
        'verdict_details': {'key': 'verdictDetails', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphChatMessagePolicyViolation, self).__init__(**kwargs)
        self.dlp_action = kwargs.get('dlp_action', None)
        self.justification_text = kwargs.get('justification_text', None)
        self.policy_tip = kwargs.get('policy_tip', None)
        self.user_action = kwargs.get('user_action', None)
        self.verdict_details = kwargs.get('verdict_details', None)


class MicrosoftGraphChatMessagePolicyViolationPolicyTip(msrest.serialization.Model):
    """chatMessagePolicyViolationPolicyTip.

    :param general_text:
    :type general_text: str
    :param compliance_url:
    :type compliance_url: str
    :param matched_condition_descriptions:
    :type matched_condition_descriptions: list[str]
    """

    _attribute_map = {
        'general_text': {'key': 'generalText', 'type': 'str'},
        'compliance_url': {'key': 'complianceUrl', 'type': 'str'},
        'matched_condition_descriptions': {'key': 'matchedConditionDescriptions', 'type': '[str]'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphChatMessagePolicyViolationPolicyTip, self).__init__(**kwargs)
        self.general_text = kwargs.get('general_text', None)
        self.compliance_url = kwargs.get('compliance_url', None)
        self.matched_condition_descriptions = kwargs.get('matched_condition_descriptions', None)


class MicrosoftGraphChatMessageReaction(msrest.serialization.Model):
    """chatMessageReaction.

    :param reaction_type:
    :type reaction_type: str
    :param created_date_time:
    :type created_date_time: ~datetime.datetime
    :param user: identitySet.
    :type user: ~users_user.models.MicrosoftGraphIdentitySet
    """

    _attribute_map = {
        'reaction_type': {'key': 'reactionType', 'type': 'str'},
        'created_date_time': {'key': 'createdDateTime', 'type': 'iso-8601'},
        'user': {'key': 'user', 'type': 'MicrosoftGraphIdentitySet'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphChatMessageReaction, self).__init__(**kwargs)
        self.reaction_type = kwargs.get('reaction_type', None)
        self.created_date_time = kwargs.get('created_date_time', None)
        self.user = kwargs.get('user', None)


class MicrosoftGraphChoiceColumn(msrest.serialization.Model):
    """choiceColumn.

    :param allow_text_entry: If true, allows custom values that aren't in the configured choices.
    :type allow_text_entry: bool
    :param choices: The list of values available for this column.
    :type choices: list[str]
    :param display_as: How the choices are to be presented in the UX. Must be one of checkBoxes,
     dropDownMenu, or radioButtons.
    :type display_as: str
    """

    _attribute_map = {
        'allow_text_entry': {'key': 'allowTextEntry', 'type': 'bool'},
        'choices': {'key': 'choices', 'type': '[str]'},
        'display_as': {'key': 'displayAs', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphChoiceColumn, self).__init__(**kwargs)
        self.allow_text_entry = kwargs.get('allow_text_entry', None)
        self.choices = kwargs.get('choices', None)
        self.display_as = kwargs.get('display_as', None)


class MicrosoftGraphColumnDefinition(MicrosoftGraphEntity):
    """columnDefinition.

    :param id: Read-only.
    :type id: str
    :param boolean: Any object.
    :type boolean: object
    :param calculated: calculatedColumn.
    :type calculated: ~users_user.models.MicrosoftGraphCalculatedColumn
    :param choice: choiceColumn.
    :type choice: ~users_user.models.MicrosoftGraphChoiceColumn
    :param column_group: For site columns, the name of the group this column belongs to. Helps
     organize related columns.
    :type column_group: str
    :param currency: currencyColumn.
    :type currency: ~users_user.models.MicrosoftGraphCurrencyColumn
    :param date_time: dateTimeColumn.
    :type date_time: ~users_user.models.MicrosoftGraphDateTimeColumn
    :param default_value: defaultColumnValue.
    :type default_value: ~users_user.models.MicrosoftGraphDefaultColumnValue
    :param description: The user-facing description of the column.
    :type description: str
    :param display_name: The user-facing name of the column.
    :type display_name: str
    :param enforce_unique_values: If true, no two list items may have the same value for this
     column.
    :type enforce_unique_values: bool
    :param geolocation: Any object.
    :type geolocation: object
    :param hidden: Specifies whether the column is displayed in the user interface.
    :type hidden: bool
    :param indexed: Specifies whether the column values can used for sorting and searching.
    :type indexed: bool
    :param lookup: lookupColumn.
    :type lookup: ~users_user.models.MicrosoftGraphLookupColumn
    :param name: The API-facing name of the column as it appears in the [fields][] on a
     [listItem][]. For the user-facing name, see displayName.
    :type name: str
    :param number: numberColumn.
    :type number: ~users_user.models.MicrosoftGraphNumberColumn
    :param person_or_group: personOrGroupColumn.
    :type person_or_group: ~users_user.models.MicrosoftGraphPersonOrGroupColumn
    :param read_only: Specifies whether the column values can be modified.
    :type read_only: bool
    :param required: Specifies whether the column value is not optional.
    :type required: bool
    :param text: textColumn.
    :type text: ~users_user.models.MicrosoftGraphTextColumn
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'boolean': {'key': 'boolean', 'type': 'object'},
        'calculated': {'key': 'calculated', 'type': 'MicrosoftGraphCalculatedColumn'},
        'choice': {'key': 'choice', 'type': 'MicrosoftGraphChoiceColumn'},
        'column_group': {'key': 'columnGroup', 'type': 'str'},
        'currency': {'key': 'currency', 'type': 'MicrosoftGraphCurrencyColumn'},
        'date_time': {'key': 'dateTime', 'type': 'MicrosoftGraphDateTimeColumn'},
        'default_value': {'key': 'defaultValue', 'type': 'MicrosoftGraphDefaultColumnValue'},
        'description': {'key': 'description', 'type': 'str'},
        'display_name': {'key': 'displayName', 'type': 'str'},
        'enforce_unique_values': {'key': 'enforceUniqueValues', 'type': 'bool'},
        'geolocation': {'key': 'geolocation', 'type': 'object'},
        'hidden': {'key': 'hidden', 'type': 'bool'},
        'indexed': {'key': 'indexed', 'type': 'bool'},
        'lookup': {'key': 'lookup', 'type': 'MicrosoftGraphLookupColumn'},
        'name': {'key': 'name', 'type': 'str'},
        'number': {'key': 'number', 'type': 'MicrosoftGraphNumberColumn'},
        'person_or_group': {'key': 'personOrGroup', 'type': 'MicrosoftGraphPersonOrGroupColumn'},
        'read_only': {'key': 'readOnly', 'type': 'bool'},
        'required': {'key': 'required', 'type': 'bool'},
        'text': {'key': 'text', 'type': 'MicrosoftGraphTextColumn'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphColumnDefinition, self).__init__(**kwargs)
        self.boolean = kwargs.get('boolean', None)
        self.calculated = kwargs.get('calculated', None)
        self.choice = kwargs.get('choice', None)
        self.column_group = kwargs.get('column_group', None)
        self.currency = kwargs.get('currency', None)
        self.date_time = kwargs.get('date_time', None)
        self.default_value = kwargs.get('default_value', None)
        self.description = kwargs.get('description', None)
        self.display_name = kwargs.get('display_name', None)
        self.enforce_unique_values = kwargs.get('enforce_unique_values', None)
        self.geolocation = kwargs.get('geolocation', None)
        self.hidden = kwargs.get('hidden', None)
        self.indexed = kwargs.get('indexed', None)
        self.lookup = kwargs.get('lookup', None)
        self.name = kwargs.get('name', None)
        self.number = kwargs.get('number', None)
        self.person_or_group = kwargs.get('person_or_group', None)
        self.read_only = kwargs.get('read_only', None)
        self.required = kwargs.get('required', None)
        self.text = kwargs.get('text', None)


class MicrosoftGraphColumnLink(MicrosoftGraphEntity):
    """columnLink.

    :param id: Read-only.
    :type id: str
    :param name: The name of the column  in this content type.
    :type name: str
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'name': {'key': 'name', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphColumnLink, self).__init__(**kwargs)
        self.name = kwargs.get('name', None)


class MicrosoftGraphCommand(MicrosoftGraphEntity):
    """command.

    :param id: Read-only.
    :type id: str
    :param status:
    :type status: str
    :param type:
    :type type: str
    :param app_service_name:
    :type app_service_name: str
    :param package_family_name:
    :type package_family_name: str
    :param error:
    :type error: str
    :param payload: Any object.
    :type payload: object
    :param permission_ticket:
    :type permission_ticket: str
    :param post_back_uri:
    :type post_back_uri: str
    :param responsepayload: entity.
    :type responsepayload: ~users_user.models.MicrosoftGraphEntity
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'status': {'key': 'Status', 'type': 'str'},
        'type': {'key': 'Type', 'type': 'str'},
        'app_service_name': {'key': 'AppServiceName', 'type': 'str'},
        'package_family_name': {'key': 'PackageFamilyName', 'type': 'str'},
        'error': {'key': 'Error', 'type': 'str'},
        'payload': {'key': 'Payload', 'type': 'object'},
        'permission_ticket': {'key': 'PermissionTicket', 'type': 'str'},
        'post_back_uri': {'key': 'PostBackUri', 'type': 'str'},
        'responsepayload': {'key': 'responsepayload', 'type': 'MicrosoftGraphEntity'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphCommand, self).__init__(**kwargs)
        self.status = kwargs.get('status', None)
        self.type = kwargs.get('type', None)
        self.app_service_name = kwargs.get('app_service_name', None)
        self.package_family_name = kwargs.get('package_family_name', None)
        self.error = kwargs.get('error', None)
        self.payload = kwargs.get('payload', None)
        self.permission_ticket = kwargs.get('permission_ticket', None)
        self.post_back_uri = kwargs.get('post_back_uri', None)
        self.responsepayload = kwargs.get('responsepayload', None)


class MicrosoftGraphCommentAction(msrest.serialization.Model):
    """commentAction.

    :param is_reply: If true, this activity was a reply to an existing comment thread.
    :type is_reply: bool
    :param parent_author: identitySet.
    :type parent_author: ~users_user.models.MicrosoftGraphIdentitySet
    :param participants: The identities of the users participating in this comment thread.
    :type participants: list[~users_user.models.MicrosoftGraphIdentitySet]
    """

    _attribute_map = {
        'is_reply': {'key': 'isReply', 'type': 'bool'},
        'parent_author': {'key': 'parentAuthor', 'type': 'MicrosoftGraphIdentitySet'},
        'participants': {'key': 'participants', 'type': '[MicrosoftGraphIdentitySet]'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphCommentAction, self).__init__(**kwargs)
        self.is_reply = kwargs.get('is_reply', None)
        self.parent_author = kwargs.get('parent_author', None)
        self.participants = kwargs.get('participants', None)


class MicrosoftGraphCompanyDetail(msrest.serialization.Model):
    """companyDetail.

    :param display_name:
    :type display_name: str
    :param pronunciation:
    :type pronunciation: str
    :param department:
    :type department: str
    :param office_location:
    :type office_location: str
    :param address: physicalAddress.
    :type address: ~users_user.models.MicrosoftGraphPhysicalAddress
    :param web_url:
    :type web_url: str
    """

    _attribute_map = {
        'display_name': {'key': 'displayName', 'type': 'str'},
        'pronunciation': {'key': 'pronunciation', 'type': 'str'},
        'department': {'key': 'department', 'type': 'str'},
        'office_location': {'key': 'officeLocation', 'type': 'str'},
        'address': {'key': 'address', 'type': 'MicrosoftGraphPhysicalAddress'},
        'web_url': {'key': 'webUrl', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphCompanyDetail, self).__init__(**kwargs)
        self.display_name = kwargs.get('display_name', None)
        self.pronunciation = kwargs.get('pronunciation', None)
        self.department = kwargs.get('department', None)
        self.office_location = kwargs.get('office_location', None)
        self.address = kwargs.get('address', None)
        self.web_url = kwargs.get('web_url', None)


class MicrosoftGraphConfigurationManagerClientEnabledFeatures(msrest.serialization.Model):
    """configurationManagerClientEnabledFeatures.

    :param inventory: Whether inventory is managed by Intune.
    :type inventory: bool
    :param modern_apps: Whether modern application is managed by Intune.
    :type modern_apps: bool
    :param resource_access: Whether resource access is managed by Intune.
    :type resource_access: bool
    :param device_configuration: Whether device configuration is managed by Intune.
    :type device_configuration: bool
    :param compliance_policy: Whether compliance policy is managed by Intune.
    :type compliance_policy: bool
    :param windows_update_for_business: Whether Windows Update for Business is managed by Intune.
    :type windows_update_for_business: bool
    :param endpoint_protection: Whether Endpoint Protection is managed by Intune.
    :type endpoint_protection: bool
    :param office_apps: Whether Office application is managed by Intune.
    :type office_apps: bool
    """

    _attribute_map = {
        'inventory': {'key': 'inventory', 'type': 'bool'},
        'modern_apps': {'key': 'modernApps', 'type': 'bool'},
        'resource_access': {'key': 'resourceAccess', 'type': 'bool'},
        'device_configuration': {'key': 'deviceConfiguration', 'type': 'bool'},
        'compliance_policy': {'key': 'compliancePolicy', 'type': 'bool'},
        'windows_update_for_business': {'key': 'windowsUpdateForBusiness', 'type': 'bool'},
        'endpoint_protection': {'key': 'endpointProtection', 'type': 'bool'},
        'office_apps': {'key': 'officeApps', 'type': 'bool'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphConfigurationManagerClientEnabledFeatures, self).__init__(**kwargs)
        self.inventory = kwargs.get('inventory', None)
        self.modern_apps = kwargs.get('modern_apps', None)
        self.resource_access = kwargs.get('resource_access', None)
        self.device_configuration = kwargs.get('device_configuration', None)
        self.compliance_policy = kwargs.get('compliance_policy', None)
        self.windows_update_for_business = kwargs.get('windows_update_for_business', None)
        self.endpoint_protection = kwargs.get('endpoint_protection', None)
        self.office_apps = kwargs.get('office_apps', None)


class MicrosoftGraphConfigurationManagerClientHealthState(msrest.serialization.Model):
    """configurationManagerClientHealthState.

    :param state: configurationManagerClientState. Possible values include: "unknown", "installed",
     "healthy", "installFailed", "updateFailed", "communicationError".
    :type state: str or ~users_user.models.MicrosoftGraphConfigurationManagerClientState
    :param error_code: Error code for failed state.
    :type error_code: int
    :param last_sync_date_time: Datetime for last sync with configuration manager management point.
    :type last_sync_date_time: ~datetime.datetime
    """

    _validation = {
        'error_code': {'maximum': 2147483647, 'minimum': -2147483648},
    }

    _attribute_map = {
        'state': {'key': 'state', 'type': 'str'},
        'error_code': {'key': 'errorCode', 'type': 'int'},
        'last_sync_date_time': {'key': 'lastSyncDateTime', 'type': 'iso-8601'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphConfigurationManagerClientHealthState, self).__init__(**kwargs)
        self.state = kwargs.get('state', None)
        self.error_code = kwargs.get('error_code', None)
        self.last_sync_date_time = kwargs.get('last_sync_date_time', None)


class MicrosoftGraphConfigurationManagerClientInformation(msrest.serialization.Model):
    """configurationManagerClientInformation.

    :param client_identifier: Configuration Manager Client Id from SCCM.
    :type client_identifier: str
    :param is_blocked: Configuration Manager Client blocked status from SCCM.
    :type is_blocked: bool
    """

    _attribute_map = {
        'client_identifier': {'key': 'clientIdentifier', 'type': 'str'},
        'is_blocked': {'key': 'isBlocked', 'type': 'bool'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphConfigurationManagerClientInformation, self).__init__(**kwargs)
        self.client_identifier = kwargs.get('client_identifier', None)
        self.is_blocked = kwargs.get('is_blocked', None)


class MicrosoftGraphOutlookItem(MicrosoftGraphEntity):
    """outlookItem.

    :param id: Read-only.
    :type id: str
    :param created_date_time: The Timestamp type represents date and time information using ISO
     8601 format and is always in UTC time. For example, midnight UTC on Jan 1, 2014 would look like
     this: '2014-01-01T00:00:00Z'.
    :type created_date_time: ~datetime.datetime
    :param last_modified_date_time: The Timestamp type represents date and time information using
     ISO 8601 format and is always in UTC time. For example, midnight UTC on Jan 1, 2014 would look
     like this: '2014-01-01T00:00:00Z'.
    :type last_modified_date_time: ~datetime.datetime
    :param change_key: Identifies the version of the item. Every time the item is changed,
     changeKey changes as well. This allows Exchange to apply changes to the correct version of the
     object. Read-only.
    :type change_key: str
    :param categories: The categories associated with the item.
    :type categories: list[str]
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'created_date_time': {'key': 'createdDateTime', 'type': 'iso-8601'},
        'last_modified_date_time': {'key': 'lastModifiedDateTime', 'type': 'iso-8601'},
        'change_key': {'key': 'changeKey', 'type': 'str'},
        'categories': {'key': 'categories', 'type': '[str]'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphOutlookItem, self).__init__(**kwargs)
        self.created_date_time = kwargs.get('created_date_time', None)
        self.last_modified_date_time = kwargs.get('last_modified_date_time', None)
        self.change_key = kwargs.get('change_key', None)
        self.categories = kwargs.get('categories', None)


class MicrosoftGraphContact(MicrosoftGraphOutlookItem):
    """contact.

    :param id: Read-only.
    :type id: str
    :param created_date_time: The Timestamp type represents date and time information using ISO
     8601 format and is always in UTC time. For example, midnight UTC on Jan 1, 2014 would look like
     this: '2014-01-01T00:00:00Z'.
    :type created_date_time: ~datetime.datetime
    :param last_modified_date_time: The Timestamp type represents date and time information using
     ISO 8601 format and is always in UTC time. For example, midnight UTC on Jan 1, 2014 would look
     like this: '2014-01-01T00:00:00Z'.
    :type last_modified_date_time: ~datetime.datetime
    :param change_key: Identifies the version of the item. Every time the item is changed,
     changeKey changes as well. This allows Exchange to apply changes to the correct version of the
     object. Read-only.
    :type change_key: str
    :param categories: The categories associated with the item.
    :type categories: list[str]
    :param parent_folder_id: The ID of the contact's parent folder.
    :type parent_folder_id: str
    :param birthday: The contact's birthday. The Timestamp type represents date and time
     information using ISO 8601 format and is always in UTC time. For example, midnight UTC on Jan
     1, 2014 would look like this: '2014-01-01T00:00:00Z'.
    :type birthday: ~datetime.datetime
    :param file_as: The name the contact is filed under.
    :type file_as: str
    :param display_name: The contact's display name. You can specify the display name in a create
     or update operation. Note that later updates to other properties may cause an automatically
     generated value to overwrite the displayName value you have specified. To preserve a pre-
     existing value, always include it as displayName in an update operation.
    :type display_name: str
    :param given_name: The contact's given name.
    :type given_name: str
    :param initials: The contact's initials.
    :type initials: str
    :param middle_name: The contact's middle name.
    :type middle_name: str
    :param nick_name: The contact's nickname.
    :type nick_name: str
    :param surname: The contact's surname.
    :type surname: str
    :param title: The contact's title.
    :type title: str
    :param yomi_given_name: The phonetic Japanese given name (first name) of the contact.
    :type yomi_given_name: str
    :param yomi_surname: The phonetic Japanese surname (last name)  of the contact.
    :type yomi_surname: str
    :param yomi_company_name: The phonetic Japanese company name of the contact.
    :type yomi_company_name: str
    :param generation: The contact's generation.
    :type generation: str
    :param email_addresses: The contact's email addresses.
    :type email_addresses: list[~users_user.models.MicrosoftGraphTypedEmailAddress]
    :param websites:
    :type websites: list[~users_user.models.MicrosoftGraphWebsite]
    :param im_addresses: The contact's instant messaging (IM) addresses.
    :type im_addresses: list[str]
    :param job_title: The contact’s job title.
    :type job_title: str
    :param company_name: The name of the contact's company.
    :type company_name: str
    :param department: The contact's department.
    :type department: str
    :param office_location: The location of the contact's office.
    :type office_location: str
    :param profession: The contact's profession.
    :type profession: str
    :param assistant_name: The name of the contact's assistant.
    :type assistant_name: str
    :param manager: The name of the contact's manager.
    :type manager: str
    :param phones:
    :type phones: list[~users_user.models.MicrosoftGraphPhone]
    :param postal_addresses:
    :type postal_addresses: list[~users_user.models.MicrosoftGraphPhysicalAddress]
    :param spouse_name: The name of the contact's spouse/partner.
    :type spouse_name: str
    :param personal_notes: The user's notes about the contact.
    :type personal_notes: str
    :param children: The names of the contact's children.
    :type children: list[str]
    :param wedding_anniversary:
    :type wedding_anniversary: ~datetime.date
    :param gender:
    :type gender: str
    :param is_favorite:
    :type is_favorite: bool
    :param flag: followupFlag.
    :type flag: ~users_user.models.MicrosoftGraphFollowupFlag
    :param single_value_extended_properties: The collection of single-value extended properties
     defined for the contact. Read-only. Nullable.
    :type single_value_extended_properties:
     list[~users_user.models.MicrosoftGraphSingleValueLegacyExtendedProperty]
    :param multi_value_extended_properties: The collection of multi-value extended properties
     defined for the contact. Read-only. Nullable.
    :type multi_value_extended_properties:
     list[~users_user.models.MicrosoftGraphMultiValueLegacyExtendedProperty]
    :param photo: profilePhoto.
    :type photo: ~users_user.models.MicrosoftGraphProfilePhoto
    :param extensions: The collection of open extensions defined for the contact. Read-only.
     Nullable.
    :type extensions: list[~users_user.models.MicrosoftGraphEntity]
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'created_date_time': {'key': 'createdDateTime', 'type': 'iso-8601'},
        'last_modified_date_time': {'key': 'lastModifiedDateTime', 'type': 'iso-8601'},
        'change_key': {'key': 'changeKey', 'type': 'str'},
        'categories': {'key': 'categories', 'type': '[str]'},
        'parent_folder_id': {'key': 'parentFolderId', 'type': 'str'},
        'birthday': {'key': 'birthday', 'type': 'iso-8601'},
        'file_as': {'key': 'fileAs', 'type': 'str'},
        'display_name': {'key': 'displayName', 'type': 'str'},
        'given_name': {'key': 'givenName', 'type': 'str'},
        'initials': {'key': 'initials', 'type': 'str'},
        'middle_name': {'key': 'middleName', 'type': 'str'},
        'nick_name': {'key': 'nickName', 'type': 'str'},
        'surname': {'key': 'surname', 'type': 'str'},
        'title': {'key': 'title', 'type': 'str'},
        'yomi_given_name': {'key': 'yomiGivenName', 'type': 'str'},
        'yomi_surname': {'key': 'yomiSurname', 'type': 'str'},
        'yomi_company_name': {'key': 'yomiCompanyName', 'type': 'str'},
        'generation': {'key': 'generation', 'type': 'str'},
        'email_addresses': {'key': 'emailAddresses', 'type': '[MicrosoftGraphTypedEmailAddress]'},
        'websites': {'key': 'websites', 'type': '[MicrosoftGraphWebsite]'},
        'im_addresses': {'key': 'imAddresses', 'type': '[str]'},
        'job_title': {'key': 'jobTitle', 'type': 'str'},
        'company_name': {'key': 'companyName', 'type': 'str'},
        'department': {'key': 'department', 'type': 'str'},
        'office_location': {'key': 'officeLocation', 'type': 'str'},
        'profession': {'key': 'profession', 'type': 'str'},
        'assistant_name': {'key': 'assistantName', 'type': 'str'},
        'manager': {'key': 'manager', 'type': 'str'},
        'phones': {'key': 'phones', 'type': '[MicrosoftGraphPhone]'},
        'postal_addresses': {'key': 'postalAddresses', 'type': '[MicrosoftGraphPhysicalAddress]'},
        'spouse_name': {'key': 'spouseName', 'type': 'str'},
        'personal_notes': {'key': 'personalNotes', 'type': 'str'},
        'children': {'key': 'children', 'type': '[str]'},
        'wedding_anniversary': {'key': 'weddingAnniversary', 'type': 'date'},
        'gender': {'key': 'gender', 'type': 'str'},
        'is_favorite': {'key': 'isFavorite', 'type': 'bool'},
        'flag': {'key': 'flag', 'type': 'MicrosoftGraphFollowupFlag'},
        'single_value_extended_properties': {'key': 'singleValueExtendedProperties', 'type': '[MicrosoftGraphSingleValueLegacyExtendedProperty]'},
        'multi_value_extended_properties': {'key': 'multiValueExtendedProperties', 'type': '[MicrosoftGraphMultiValueLegacyExtendedProperty]'},
        'photo': {'key': 'photo', 'type': 'MicrosoftGraphProfilePhoto'},
        'extensions': {'key': 'extensions', 'type': '[MicrosoftGraphEntity]'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphContact, self).__init__(**kwargs)
        self.parent_folder_id = kwargs.get('parent_folder_id', None)
        self.birthday = kwargs.get('birthday', None)
        self.file_as = kwargs.get('file_as', None)
        self.display_name = kwargs.get('display_name', None)
        self.given_name = kwargs.get('given_name', None)
        self.initials = kwargs.get('initials', None)
        self.middle_name = kwargs.get('middle_name', None)
        self.nick_name = kwargs.get('nick_name', None)
        self.surname = kwargs.get('surname', None)
        self.title = kwargs.get('title', None)
        self.yomi_given_name = kwargs.get('yomi_given_name', None)
        self.yomi_surname = kwargs.get('yomi_surname', None)
        self.yomi_company_name = kwargs.get('yomi_company_name', None)
        self.generation = kwargs.get('generation', None)
        self.email_addresses = kwargs.get('email_addresses', None)
        self.websites = kwargs.get('websites', None)
        self.im_addresses = kwargs.get('im_addresses', None)
        self.job_title = kwargs.get('job_title', None)
        self.company_name = kwargs.get('company_name', None)
        self.department = kwargs.get('department', None)
        self.office_location = kwargs.get('office_location', None)
        self.profession = kwargs.get('profession', None)
        self.assistant_name = kwargs.get('assistant_name', None)
        self.manager = kwargs.get('manager', None)
        self.phones = kwargs.get('phones', None)
        self.postal_addresses = kwargs.get('postal_addresses', None)
        self.spouse_name = kwargs.get('spouse_name', None)
        self.personal_notes = kwargs.get('personal_notes', None)
        self.children = kwargs.get('children', None)
        self.wedding_anniversary = kwargs.get('wedding_anniversary', None)
        self.gender = kwargs.get('gender', None)
        self.is_favorite = kwargs.get('is_favorite', None)
        self.flag = kwargs.get('flag', None)
        self.single_value_extended_properties = kwargs.get('single_value_extended_properties', None)
        self.multi_value_extended_properties = kwargs.get('multi_value_extended_properties', None)
        self.photo = kwargs.get('photo', None)
        self.extensions = kwargs.get('extensions', None)


class MicrosoftGraphContactFolder(MicrosoftGraphEntity):
    """contactFolder.

    :param id: Read-only.
    :type id: str
    :param parent_folder_id: The ID of the folder's parent folder.
    :type parent_folder_id: str
    :param display_name: The folder's display name.
    :type display_name: str
    :param well_known_name:
    :type well_known_name: str
    :param single_value_extended_properties: The collection of single-value extended properties
     defined for the contactFolder. Read-only. Nullable.
    :type single_value_extended_properties:
     list[~users_user.models.MicrosoftGraphSingleValueLegacyExtendedProperty]
    :param multi_value_extended_properties: The collection of multi-value extended properties
     defined for the contactFolder. Read-only. Nullable.
    :type multi_value_extended_properties:
     list[~users_user.models.MicrosoftGraphMultiValueLegacyExtendedProperty]
    :param contacts: The contacts in the folder. Navigation property. Read-only. Nullable.
    :type contacts: list[~users_user.models.MicrosoftGraphContact]
    :param child_folders: The collection of child folders in the folder. Navigation property. Read-
     only. Nullable.
    :type child_folders: list[~users_user.models.MicrosoftGraphContactFolder]
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'parent_folder_id': {'key': 'parentFolderId', 'type': 'str'},
        'display_name': {'key': 'displayName', 'type': 'str'},
        'well_known_name': {'key': 'wellKnownName', 'type': 'str'},
        'single_value_extended_properties': {'key': 'singleValueExtendedProperties', 'type': '[MicrosoftGraphSingleValueLegacyExtendedProperty]'},
        'multi_value_extended_properties': {'key': 'multiValueExtendedProperties', 'type': '[MicrosoftGraphMultiValueLegacyExtendedProperty]'},
        'contacts': {'key': 'contacts', 'type': '[MicrosoftGraphContact]'},
        'child_folders': {'key': 'childFolders', 'type': '[MicrosoftGraphContactFolder]'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphContactFolder, self).__init__(**kwargs)
        self.parent_folder_id = kwargs.get('parent_folder_id', None)
        self.display_name = kwargs.get('display_name', None)
        self.well_known_name = kwargs.get('well_known_name', None)
        self.single_value_extended_properties = kwargs.get('single_value_extended_properties', None)
        self.multi_value_extended_properties = kwargs.get('multi_value_extended_properties', None)
        self.contacts = kwargs.get('contacts', None)
        self.child_folders = kwargs.get('child_folders', None)


class MicrosoftGraphContentType(MicrosoftGraphEntity):
    """contentType.

    :param id: Read-only.
    :type id: str
    :param description: The descriptive text for the item.
    :type description: str
    :param group: The name of the group this content type belongs to. Helps organize related
     content types.
    :type group: str
    :param hidden: Indicates whether the content type is hidden in the list's 'New' menu.
    :type hidden: bool
    :param inherited_from: itemReference.
    :type inherited_from: ~users_user.models.MicrosoftGraphItemReference
    :param name: The name of the content type.
    :type name: str
    :param order: contentTypeOrder.
    :type order: ~users_user.models.MicrosoftGraphContentTypeOrder
    :param parent_id: The unique identifier of the content type.
    :type parent_id: str
    :param read_only: If true, the content type cannot be modified unless this value is first set
     to false.
    :type read_only: bool
    :param sealed: If true, the content type cannot be modified by users or through push-down
     operations. Only site collection administrators can seal or unseal content types.
    :type sealed: bool
    :param column_links: The collection of columns that are required by this content type.
    :type column_links: list[~users_user.models.MicrosoftGraphColumnLink]
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'description': {'key': 'description', 'type': 'str'},
        'group': {'key': 'group', 'type': 'str'},
        'hidden': {'key': 'hidden', 'type': 'bool'},
        'inherited_from': {'key': 'inheritedFrom', 'type': 'MicrosoftGraphItemReference'},
        'name': {'key': 'name', 'type': 'str'},
        'order': {'key': 'order', 'type': 'MicrosoftGraphContentTypeOrder'},
        'parent_id': {'key': 'parentId', 'type': 'str'},
        'read_only': {'key': 'readOnly', 'type': 'bool'},
        'sealed': {'key': 'sealed', 'type': 'bool'},
        'column_links': {'key': 'columnLinks', 'type': '[MicrosoftGraphColumnLink]'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphContentType, self).__init__(**kwargs)
        self.description = kwargs.get('description', None)
        self.group = kwargs.get('group', None)
        self.hidden = kwargs.get('hidden', None)
        self.inherited_from = kwargs.get('inherited_from', None)
        self.name = kwargs.get('name', None)
        self.order = kwargs.get('order', None)
        self.parent_id = kwargs.get('parent_id', None)
        self.read_only = kwargs.get('read_only', None)
        self.sealed = kwargs.get('sealed', None)
        self.column_links = kwargs.get('column_links', None)


class MicrosoftGraphContentTypeInfo(msrest.serialization.Model):
    """contentTypeInfo.

    :param id: The id of the content type.
    :type id: str
    :param name:
    :type name: str
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'name': {'key': 'name', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphContentTypeInfo, self).__init__(**kwargs)
        self.id = kwargs.get('id', None)
        self.name = kwargs.get('name', None)


class MicrosoftGraphContentTypeOrder(msrest.serialization.Model):
    """contentTypeOrder.

    :param default: Whether this is the default Content Type.
    :type default: bool
    :param position: Specifies the position in which the Content Type appears in the selection UI.
    :type position: int
    """

    _validation = {
        'position': {'maximum': 2147483647, 'minimum': -2147483648},
    }

    _attribute_map = {
        'default': {'key': 'default', 'type': 'bool'},
        'position': {'key': 'position', 'type': 'int'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphContentTypeOrder, self).__init__(**kwargs)
        self.default = kwargs.get('default', None)
        self.position = kwargs.get('position', None)


class MicrosoftGraphConversation(MicrosoftGraphEntity):
    """conversation.

    :param id: Read-only.
    :type id: str
    :param topic: The topic of the conversation. This property can be set when the conversation is
     created, but it cannot be updated.
    :type topic: str
    :param has_attachments: Indicates whether any of the posts within this Conversation has at
     least one attachment.
    :type has_attachments: bool
    :param last_delivered_date_time: The Timestamp type represents date and time information using
     ISO 8601 format and is always in UTC time. For example, midnight UTC on Jan 1, 2014 would look
     like this: '2014-01-01T00:00:00Z'.
    :type last_delivered_date_time: ~datetime.datetime
    :param unique_senders: All the users that sent a message to this Conversation.
    :type unique_senders: list[str]
    :param preview: A short summary from the body of the latest post in this converstaion.
    :type preview: str
    :param threads: A collection of all the conversation threads in the conversation. A navigation
     property. Read-only. Nullable.
    :type threads: list[~users_user.models.MicrosoftGraphConversationThread]
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'topic': {'key': 'topic', 'type': 'str'},
        'has_attachments': {'key': 'hasAttachments', 'type': 'bool'},
        'last_delivered_date_time': {'key': 'lastDeliveredDateTime', 'type': 'iso-8601'},
        'unique_senders': {'key': 'uniqueSenders', 'type': '[str]'},
        'preview': {'key': 'preview', 'type': 'str'},
        'threads': {'key': 'threads', 'type': '[MicrosoftGraphConversationThread]'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphConversation, self).__init__(**kwargs)
        self.topic = kwargs.get('topic', None)
        self.has_attachments = kwargs.get('has_attachments', None)
        self.last_delivered_date_time = kwargs.get('last_delivered_date_time', None)
        self.unique_senders = kwargs.get('unique_senders', None)
        self.preview = kwargs.get('preview', None)
        self.threads = kwargs.get('threads', None)


class MicrosoftGraphConversationMember(MicrosoftGraphEntity):
    """conversationMember.

    :param id: Read-only.
    :type id: str
    :param roles:
    :type roles: list[str]
    :param display_name:
    :type display_name: str
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'roles': {'key': 'roles', 'type': '[str]'},
        'display_name': {'key': 'displayName', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphConversationMember, self).__init__(**kwargs)
        self.roles = kwargs.get('roles', None)
        self.display_name = kwargs.get('display_name', None)


class MicrosoftGraphConversationThread(MicrosoftGraphEntity):
    """conversationThread.

    :param id: Read-only.
    :type id: str
    :param to_recipients: The To: recipients for the thread.
    :type to_recipients: list[~users_user.models.MicrosoftGraphRecipient]
    :param topic: The topic of the conversation. This property can be set when the conversation is
     created, but it cannot be updated.
    :type topic: str
    :param has_attachments: Indicates whether any of the posts within this thread has at least one
     attachment.
    :type has_attachments: bool
    :param last_delivered_date_time: The Timestamp type represents date and time information using
     ISO 8601 format and is always in UTC time. For example, midnight UTC on Jan 1, 2014 would look
     like this: '2014-01-01T00:00:00Z'.
    :type last_delivered_date_time: ~datetime.datetime
    :param unique_senders: All the users that sent a message to this thread.
    :type unique_senders: list[str]
    :param cc_recipients: The Cc: recipients for the thread.
    :type cc_recipients: list[~users_user.models.MicrosoftGraphRecipient]
    :param preview: A short summary from the body of the latest post in this converstaion.
    :type preview: str
    :param is_locked: Indicates if the thread is locked.
    :type is_locked: bool
    :param posts: Read-only. Nullable.
    :type posts: list[~users_user.models.MicrosoftGraphPost]
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'to_recipients': {'key': 'toRecipients', 'type': '[MicrosoftGraphRecipient]'},
        'topic': {'key': 'topic', 'type': 'str'},
        'has_attachments': {'key': 'hasAttachments', 'type': 'bool'},
        'last_delivered_date_time': {'key': 'lastDeliveredDateTime', 'type': 'iso-8601'},
        'unique_senders': {'key': 'uniqueSenders', 'type': '[str]'},
        'cc_recipients': {'key': 'ccRecipients', 'type': '[MicrosoftGraphRecipient]'},
        'preview': {'key': 'preview', 'type': 'str'},
        'is_locked': {'key': 'isLocked', 'type': 'bool'},
        'posts': {'key': 'posts', 'type': '[MicrosoftGraphPost]'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphConversationThread, self).__init__(**kwargs)
        self.to_recipients = kwargs.get('to_recipients', None)
        self.topic = kwargs.get('topic', None)
        self.has_attachments = kwargs.get('has_attachments', None)
        self.last_delivered_date_time = kwargs.get('last_delivered_date_time', None)
        self.unique_senders = kwargs.get('unique_senders', None)
        self.cc_recipients = kwargs.get('cc_recipients', None)
        self.preview = kwargs.get('preview', None)
        self.is_locked = kwargs.get('is_locked', None)
        self.posts = kwargs.get('posts', None)


class MicrosoftGraphCurrencyColumn(msrest.serialization.Model):
    """currencyColumn.

    :param locale: Specifies the locale from which to infer the currency symbol.
    :type locale: str
    """

    _attribute_map = {
        'locale': {'key': 'locale', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphCurrencyColumn, self).__init__(**kwargs)
        self.locale = kwargs.get('locale', None)


class MicrosoftGraphDataLossPreventionPolicy(MicrosoftGraphEntity):
    """dataLossPreventionPolicy.

    :param id: Read-only.
    :type id: str
    :param name:
    :type name: str
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'name': {'key': 'name', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphDataLossPreventionPolicy, self).__init__(**kwargs)
        self.name = kwargs.get('name', None)


class MicrosoftGraphDateTimeColumn(msrest.serialization.Model):
    """dateTimeColumn.

    :param display_as: How the value should be presented in the UX. Must be one of default,
     friendly, or standard. See below for more details. If unspecified, treated as default.
    :type display_as: str
    :param format: Indicates whether the value should be presented as a date only or a date and
     time. Must be one of dateOnly or dateTime.
    :type format: str
    """

    _attribute_map = {
        'display_as': {'key': 'displayAs', 'type': 'str'},
        'format': {'key': 'format', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphDateTimeColumn, self).__init__(**kwargs)
        self.display_as = kwargs.get('display_as', None)
        self.format = kwargs.get('format', None)


class MicrosoftGraphDateTimeZone(msrest.serialization.Model):
    """dateTimeTimeZone.

    :param date_time: A single point of time in a combined date and time representation
     ({date}T{time}; for example, 2017-08-29T04:00:00.0000000).
    :type date_time: str
    :param time_zone: Represents a time zone, for example, 'Pacific Standard Time'. See below for
     more possible values.
    :type time_zone: str
    """

    _attribute_map = {
        'date_time': {'key': 'dateTime', 'type': 'str'},
        'time_zone': {'key': 'timeZone', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphDateTimeZone, self).__init__(**kwargs)
        self.date_time = kwargs.get('date_time', None)
        self.time_zone = kwargs.get('time_zone', None)


class MicrosoftGraphDefaultColumnValue(msrest.serialization.Model):
    """defaultColumnValue.

    :param formula: The formula used to compute the default value for this column.
    :type formula: str
    :param value: The direct value to use as the default value for this column.
    :type value: str
    """

    _attribute_map = {
        'formula': {'key': 'formula', 'type': 'str'},
        'value': {'key': 'value', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphDefaultColumnValue, self).__init__(**kwargs)
        self.formula = kwargs.get('formula', None)
        self.value = kwargs.get('value', None)


class MicrosoftGraphDeleteAction(msrest.serialization.Model):
    """deleteAction.

    :param name: The name of the item that was deleted.
    :type name: str
    :param object_type: File or Folder, depending on the type of the deleted item.
    :type object_type: str
    """

    _attribute_map = {
        'name': {'key': 'name', 'type': 'str'},
        'object_type': {'key': 'objectType', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphDeleteAction, self).__init__(**kwargs)
        self.name = kwargs.get('name', None)
        self.object_type = kwargs.get('object_type', None)


class MicrosoftGraphDeleted(msrest.serialization.Model):
    """deleted.

    :param state: Represents the state of the deleted item.
    :type state: str
    """

    _attribute_map = {
        'state': {'key': 'state', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphDeleted, self).__init__(**kwargs)
        self.state = kwargs.get('state', None)


class MicrosoftGraphDetectedApp(MicrosoftGraphEntity):
    """A managed or unmanaged app that is installed on a managed device. Unmanaged apps will only appear for devices marked as corporate owned.

    :param id: Read-only.
    :type id: str
    :param display_name: Name of the discovered application. Read-only.
    :type display_name: str
    :param version: Version of the discovered application. Read-only.
    :type version: str
    :param size_in_byte: Discovered application size in bytes. Read-only.
    :type size_in_byte: long
    :param device_count: The number of devices that have installed this application.
    :type device_count: int
    :param managed_devices: The devices that have the discovered application installed.
    :type managed_devices: list[~users_user.models.MicrosoftGraphManagedDevice]
    """

    _validation = {
        'device_count': {'maximum': 2147483647, 'minimum': -2147483648},
    }

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'display_name': {'key': 'displayName', 'type': 'str'},
        'version': {'key': 'version', 'type': 'str'},
        'size_in_byte': {'key': 'sizeInByte', 'type': 'long'},
        'device_count': {'key': 'deviceCount', 'type': 'int'},
        'managed_devices': {'key': 'managedDevices', 'type': '[MicrosoftGraphManagedDevice]'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphDetectedApp, self).__init__(**kwargs)
        self.display_name = kwargs.get('display_name', None)
        self.version = kwargs.get('version', None)
        self.size_in_byte = kwargs.get('size_in_byte', None)
        self.device_count = kwargs.get('device_count', None)
        self.managed_devices = kwargs.get('managed_devices', None)


class MicrosoftGraphDirectoryObject(MicrosoftGraphEntity):
    """Represents an Azure Active Directory object. The directoryObject type is the base type for many other directory entity types.

    :param id: Read-only.
    :type id: str
    :param deleted_date_time:
    :type deleted_date_time: ~datetime.datetime
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'deleted_date_time': {'key': 'deletedDateTime', 'type': 'iso-8601'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphDirectoryObject, self).__init__(**kwargs)
        self.deleted_date_time = kwargs.get('deleted_date_time', None)


class MicrosoftGraphDevice(MicrosoftGraphDirectoryObject):
    """Represents an Azure Active Directory object. The directoryObject type is the base type for many other directory entity types.

    :param id: Read-only.
    :type id: str
    :param deleted_date_time:
    :type deleted_date_time: ~datetime.datetime
    :param account_enabled: true if the account is enabled; otherwise, false. Required.
    :type account_enabled: bool
    :param alternative_security_ids: For internal use only. Not nullable.
    :type alternative_security_ids: list[~users_user.models.MicrosoftGraphAlternativeSecurityId]
    :param approximate_last_sign_in_date_time: The timestamp type represents date and time
     information using ISO 8601 format and is always in UTC time. For example, midnight UTC on Jan
     1, 2014 would look like this: '2014-01-01T00:00:00Z'. Read-only.
    :type approximate_last_sign_in_date_time: ~datetime.datetime
    :param compliance_expiration_date_time: The timestamp when the device is no longer deemed
     compliant. The timestamp type represents date and time information using ISO 8601 format and is
     always in UTC time. For example, midnight UTC on Jan 1, 2014 would look like this:
     '2014-01-01T00:00:00Z'. Read-only.
    :type compliance_expiration_date_time: ~datetime.datetime
    :param device_id: Unique identifier set by Azure Device Registration Service at the time of
     registration.
    :type device_id: str
    :param device_metadata: For interal use only. Set to null.
    :type device_metadata: str
    :param device_version: For interal use only.
    :type device_version: int
    :param display_name: The display name for the device. Required.
    :type display_name: str
    :param is_compliant: true if the device complies with Mobile Device Management (MDM) policies;
     otherwise, false. Read-only. This can only be updated by Intune for any device OS type or by an
     approved MDM app for Windows OS devices.
    :type is_compliant: bool
    :param is_managed: true if the device is managed by a Mobile Device Management (MDM) app;
     otherwise, false. This can only be updated by Intune for any device OS type or by an approved
     MDM app for Windows OS devices.
    :type is_managed: bool
    :param on_premises_last_sync_date_time: The last time at which the object was synced with the
     on-premises directory.The Timestamp type represents date and time information using ISO 8601
     format and is always in UTC time. For example, midnight UTC on Jan 1, 2014 would look like
     this: '2014-01-01T00:00:00Z' Read-only.
    :type on_premises_last_sync_date_time: ~datetime.datetime
    :param on_premises_sync_enabled: true if this object is synced from an on-premises directory;
     false if this object was originally synced from an on-premises directory but is no longer
     synced; null if this object has never been synced from an on-premises directory (default).
     Read-only.
    :type on_premises_sync_enabled: bool
    :param operating_system: The type of operating system on the device. Required.
    :type operating_system: str
    :param operating_system_version: The version of the operating system on the device. Required.
    :type operating_system_version: str
    :param physical_ids: For interal use only. Not nullable.
    :type physical_ids: list[str]
    :param profile_type: The profile type of the device. Possible values:RegisteredDevice
     (default)SecureVMPrinterSharedIoT.
    :type profile_type: str
    :param system_labels: List of labels applied to the device by the system.
    :type system_labels: list[str]
    :param trust_type: Type of trust for the joined device. Read-only. Possible values: Workplace -
     indicates bring your own personal devicesAzureAd - Cloud only joined devicesServerAd - on-
     premises domain joined devices joined to Azure AD. For more details, see Introduction to device
     management in Azure Active Directory.
    :type trust_type: str
    :param name:
    :type name: str
    :param manufacturer:
    :type manufacturer: str
    :param model:
    :type model: str
    :param kind:
    :type kind: str
    :param status:
    :type status: str
    :param platform:
    :type platform: str
    :param member_of: Groups that this group is a member of. HTTP Methods: GET (supported for all
     groups). Read-only. Nullable.
    :type member_of: list[~users_user.models.MicrosoftGraphDirectoryObject]
    :param registered_owners: The user that cloud joined the device or registered their personal
     device. The registered owner is set at the time of registration. Currently, there can be only
     one owner. Read-only. Nullable.
    :type registered_owners: list[~users_user.models.MicrosoftGraphDirectoryObject]
    :param registered_users: Collection of registered users of the device. For cloud joined devices
     and registered personal devices, registered users are set to the same value as registered
     owners at the time of registration. Read-only. Nullable.
    :type registered_users: list[~users_user.models.MicrosoftGraphDirectoryObject]
    :param transitive_member_of:
    :type transitive_member_of: list[~users_user.models.MicrosoftGraphDirectoryObject]
    :param extensions: The collection of open extensions defined for the device. Read-only.
     Nullable.
    :type extensions: list[~users_user.models.MicrosoftGraphEntity]
    :param commands:
    :type commands: list[~users_user.models.MicrosoftGraphCommand]
    """

    _validation = {
        'device_version': {'maximum': 2147483647, 'minimum': -2147483648},
    }

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'deleted_date_time': {'key': 'deletedDateTime', 'type': 'iso-8601'},
        'account_enabled': {'key': 'accountEnabled', 'type': 'bool'},
        'alternative_security_ids': {'key': 'alternativeSecurityIds', 'type': '[MicrosoftGraphAlternativeSecurityId]'},
        'approximate_last_sign_in_date_time': {'key': 'approximateLastSignInDateTime', 'type': 'iso-8601'},
        'compliance_expiration_date_time': {'key': 'complianceExpirationDateTime', 'type': 'iso-8601'},
        'device_id': {'key': 'deviceId', 'type': 'str'},
        'device_metadata': {'key': 'deviceMetadata', 'type': 'str'},
        'device_version': {'key': 'deviceVersion', 'type': 'int'},
        'display_name': {'key': 'displayName', 'type': 'str'},
        'is_compliant': {'key': 'isCompliant', 'type': 'bool'},
        'is_managed': {'key': 'isManaged', 'type': 'bool'},
        'on_premises_last_sync_date_time': {'key': 'onPremisesLastSyncDateTime', 'type': 'iso-8601'},
        'on_premises_sync_enabled': {'key': 'onPremisesSyncEnabled', 'type': 'bool'},
        'operating_system': {'key': 'operatingSystem', 'type': 'str'},
        'operating_system_version': {'key': 'operatingSystemVersion', 'type': 'str'},
        'physical_ids': {'key': 'physicalIds', 'type': '[str]'},
        'profile_type': {'key': 'profileType', 'type': 'str'},
        'system_labels': {'key': 'systemLabels', 'type': '[str]'},
        'trust_type': {'key': 'trustType', 'type': 'str'},
        'name': {'key': 'Name', 'type': 'str'},
        'manufacturer': {'key': 'Manufacturer', 'type': 'str'},
        'model': {'key': 'Model', 'type': 'str'},
        'kind': {'key': 'Kind', 'type': 'str'},
        'status': {'key': 'Status', 'type': 'str'},
        'platform': {'key': 'Platform', 'type': 'str'},
        'member_of': {'key': 'memberOf', 'type': '[MicrosoftGraphDirectoryObject]'},
        'registered_owners': {'key': 'registeredOwners', 'type': '[MicrosoftGraphDirectoryObject]'},
        'registered_users': {'key': 'registeredUsers', 'type': '[MicrosoftGraphDirectoryObject]'},
        'transitive_member_of': {'key': 'transitiveMemberOf', 'type': '[MicrosoftGraphDirectoryObject]'},
        'extensions': {'key': 'extensions', 'type': '[MicrosoftGraphEntity]'},
        'commands': {'key': 'commands', 'type': '[MicrosoftGraphCommand]'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphDevice, self).__init__(**kwargs)
        self.account_enabled = kwargs.get('account_enabled', None)
        self.alternative_security_ids = kwargs.get('alternative_security_ids', None)
        self.approximate_last_sign_in_date_time = kwargs.get('approximate_last_sign_in_date_time', None)
        self.compliance_expiration_date_time = kwargs.get('compliance_expiration_date_time', None)
        self.device_id = kwargs.get('device_id', None)
        self.device_metadata = kwargs.get('device_metadata', None)
        self.device_version = kwargs.get('device_version', None)
        self.display_name = kwargs.get('display_name', None)
        self.is_compliant = kwargs.get('is_compliant', None)
        self.is_managed = kwargs.get('is_managed', None)
        self.on_premises_last_sync_date_time = kwargs.get('on_premises_last_sync_date_time', None)
        self.on_premises_sync_enabled = kwargs.get('on_premises_sync_enabled', None)
        self.operating_system = kwargs.get('operating_system', None)
        self.operating_system_version = kwargs.get('operating_system_version', None)
        self.physical_ids = kwargs.get('physical_ids', None)
        self.profile_type = kwargs.get('profile_type', None)
        self.system_labels = kwargs.get('system_labels', None)
        self.trust_type = kwargs.get('trust_type', None)
        self.name = kwargs.get('name', None)
        self.manufacturer = kwargs.get('manufacturer', None)
        self.model = kwargs.get('model', None)
        self.kind = kwargs.get('kind', None)
        self.status = kwargs.get('status', None)
        self.platform = kwargs.get('platform', None)
        self.member_of = kwargs.get('member_of', None)
        self.registered_owners = kwargs.get('registered_owners', None)
        self.registered_users = kwargs.get('registered_users', None)
        self.transitive_member_of = kwargs.get('transitive_member_of', None)
        self.extensions = kwargs.get('extensions', None)
        self.commands = kwargs.get('commands', None)


class MicrosoftGraphDeviceActionResult(msrest.serialization.Model):
    """deviceActionResult.

    :param action_name: Action name.
    :type action_name: str
    :param action_state: actionState. Possible values include: "none", "pending", "canceled",
     "active", "done", "failed", "notSupported".
    :type action_state: str or ~users_user.models.MicrosoftGraphActionState
    :param start_date_time: Time the action was initiated.
    :type start_date_time: ~datetime.datetime
    :param last_updated_date_time: Time the action state was last updated.
    :type last_updated_date_time: ~datetime.datetime
    """

    _attribute_map = {
        'action_name': {'key': 'actionName', 'type': 'str'},
        'action_state': {'key': 'actionState', 'type': 'str'},
        'start_date_time': {'key': 'startDateTime', 'type': 'iso-8601'},
        'last_updated_date_time': {'key': 'lastUpdatedDateTime', 'type': 'iso-8601'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphDeviceActionResult, self).__init__(**kwargs)
        self.action_name = kwargs.get('action_name', None)
        self.action_state = kwargs.get('action_state', None)
        self.start_date_time = kwargs.get('start_date_time', None)
        self.last_updated_date_time = kwargs.get('last_updated_date_time', None)


class MicrosoftGraphDeviceCategory(MicrosoftGraphEntity):
    """Device categories provides a way to organize your devices. Using device categories, company administrators can define their own categories that make sense to their company.??These categories can then be applied to a device in the Intune Azure console or selected by a user during device enrollment. You can filter reports and create dynamic Azure Active Directory device groups based on device categories.

    :param id: Read-only.
    :type id: str
    :param display_name: Display name for the device category.
    :type display_name: str
    :param description: Optional description for the device category.
    :type description: str
    :param role_scope_tag_ids: Optional role scope tags for the device category.
    :type role_scope_tag_ids: list[str]
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'display_name': {'key': 'displayName', 'type': 'str'},
        'description': {'key': 'description', 'type': 'str'},
        'role_scope_tag_ids': {'key': 'roleScopeTagIds', 'type': '[str]'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphDeviceCategory, self).__init__(**kwargs)
        self.display_name = kwargs.get('display_name', None)
        self.description = kwargs.get('description', None)
        self.role_scope_tag_ids = kwargs.get('role_scope_tag_ids', None)


class MicrosoftGraphDeviceCompliancePolicySettingState(msrest.serialization.Model):
    """deviceCompliancePolicySettingState.

    :param setting: The setting that is being reported.
    :type setting: str
    :param setting_name: Localized/user friendly setting name that is being reported.
    :type setting_name: str
    :param instance_display_name: Name of setting instance that is being reported.
    :type instance_display_name: str
    :param state: complianceStatus. Possible values include: "unknown", "notApplicable",
     "compliant", "remediated", "nonCompliant", "error", "conflict", "notAssigned".
    :type state: str or ~users_user.models.MicrosoftGraphComplianceStatus
    :param error_code: Error code for the setting.
    :type error_code: long
    :param error_description: Error description.
    :type error_description: str
    :param user_id: UserId.
    :type user_id: str
    :param user_name: UserName.
    :type user_name: str
    :param user_email: UserEmail.
    :type user_email: str
    :param user_principal_name: UserPrincipalName.
    :type user_principal_name: str
    :param sources: Contributing policies.
    :type sources: list[~users_user.models.MicrosoftGraphSettingSource]
    :param current_value: Current value of setting on device.
    :type current_value: str
    :param setting_instance_id: SettingInstanceId.
    :type setting_instance_id: str
    """

    _attribute_map = {
        'setting': {'key': 'setting', 'type': 'str'},
        'setting_name': {'key': 'settingName', 'type': 'str'},
        'instance_display_name': {'key': 'instanceDisplayName', 'type': 'str'},
        'state': {'key': 'state', 'type': 'str'},
        'error_code': {'key': 'errorCode', 'type': 'long'},
        'error_description': {'key': 'errorDescription', 'type': 'str'},
        'user_id': {'key': 'userId', 'type': 'str'},
        'user_name': {'key': 'userName', 'type': 'str'},
        'user_email': {'key': 'userEmail', 'type': 'str'},
        'user_principal_name': {'key': 'userPrincipalName', 'type': 'str'},
        'sources': {'key': 'sources', 'type': '[MicrosoftGraphSettingSource]'},
        'current_value': {'key': 'currentValue', 'type': 'str'},
        'setting_instance_id': {'key': 'settingInstanceId', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphDeviceCompliancePolicySettingState, self).__init__(**kwargs)
        self.setting = kwargs.get('setting', None)
        self.setting_name = kwargs.get('setting_name', None)
        self.instance_display_name = kwargs.get('instance_display_name', None)
        self.state = kwargs.get('state', None)
        self.error_code = kwargs.get('error_code', None)
        self.error_description = kwargs.get('error_description', None)
        self.user_id = kwargs.get('user_id', None)
        self.user_name = kwargs.get('user_name', None)
        self.user_email = kwargs.get('user_email', None)
        self.user_principal_name = kwargs.get('user_principal_name', None)
        self.sources = kwargs.get('sources', None)
        self.current_value = kwargs.get('current_value', None)
        self.setting_instance_id = kwargs.get('setting_instance_id', None)


class MicrosoftGraphDeviceCompliancePolicyState(MicrosoftGraphEntity):
    """Device Compliance Policy State for a given device.

    :param id: Read-only.
    :type id: str
    :param setting_states:
    :type setting_states: list[~users_user.models.MicrosoftGraphDeviceCompliancePolicySettingState]
    :param display_name: The name of the policy for this policyBase.
    :type display_name: str
    :param version: The version of the policy.
    :type version: int
    :param platform_type: policyPlatformType. Possible values include: "android", "androidForWork",
     "iOS", "macOS", "windowsPhone81", "windows81AndLater", "windows10AndLater",
     "androidWorkProfile", "windows10XProfile", "all".
    :type platform_type: str or ~users_user.models.MicrosoftGraphPolicyPlatformType
    :param state: complianceStatus. Possible values include: "unknown", "notApplicable",
     "compliant", "remediated", "nonCompliant", "error", "conflict", "notAssigned".
    :type state: str or ~users_user.models.MicrosoftGraphComplianceStatus
    :param setting_count: Count of how many setting a policy holds.
    :type setting_count: int
    :param user_id: User unique identifier, must be Guid.
    :type user_id: str
    :param user_principal_name: User Principal Name.
    :type user_principal_name: str
    """

    _validation = {
        'version': {'maximum': 2147483647, 'minimum': -2147483648},
        'setting_count': {'maximum': 2147483647, 'minimum': -2147483648},
    }

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'setting_states': {'key': 'settingStates', 'type': '[MicrosoftGraphDeviceCompliancePolicySettingState]'},
        'display_name': {'key': 'displayName', 'type': 'str'},
        'version': {'key': 'version', 'type': 'int'},
        'platform_type': {'key': 'platformType', 'type': 'str'},
        'state': {'key': 'state', 'type': 'str'},
        'setting_count': {'key': 'settingCount', 'type': 'int'},
        'user_id': {'key': 'userId', 'type': 'str'},
        'user_principal_name': {'key': 'userPrincipalName', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphDeviceCompliancePolicyState, self).__init__(**kwargs)
        self.setting_states = kwargs.get('setting_states', None)
        self.display_name = kwargs.get('display_name', None)
        self.version = kwargs.get('version', None)
        self.platform_type = kwargs.get('platform_type', None)
        self.state = kwargs.get('state', None)
        self.setting_count = kwargs.get('setting_count', None)
        self.user_id = kwargs.get('user_id', None)
        self.user_principal_name = kwargs.get('user_principal_name', None)


class MicrosoftGraphDeviceConfigurationSettingState(msrest.serialization.Model):
    """deviceConfigurationSettingState.

    :param setting: The setting that is being reported.
    :type setting: str
    :param setting_name: Localized/user friendly setting name that is being reported.
    :type setting_name: str
    :param instance_display_name: Name of setting instance that is being reported.
    :type instance_display_name: str
    :param state: complianceStatus. Possible values include: "unknown", "notApplicable",
     "compliant", "remediated", "nonCompliant", "error", "conflict", "notAssigned".
    :type state: str or ~users_user.models.MicrosoftGraphComplianceStatus
    :param error_code: Error code for the setting.
    :type error_code: long
    :param error_description: Error description.
    :type error_description: str
    :param user_id: UserId.
    :type user_id: str
    :param user_name: UserName.
    :type user_name: str
    :param user_email: UserEmail.
    :type user_email: str
    :param user_principal_name: UserPrincipalName.
    :type user_principal_name: str
    :param sources: Contributing policies.
    :type sources: list[~users_user.models.MicrosoftGraphSettingSource]
    :param current_value: Current value of setting on device.
    :type current_value: str
    :param setting_instance_id: SettingInstanceId.
    :type setting_instance_id: str
    """

    _attribute_map = {
        'setting': {'key': 'setting', 'type': 'str'},
        'setting_name': {'key': 'settingName', 'type': 'str'},
        'instance_display_name': {'key': 'instanceDisplayName', 'type': 'str'},
        'state': {'key': 'state', 'type': 'str'},
        'error_code': {'key': 'errorCode', 'type': 'long'},
        'error_description': {'key': 'errorDescription', 'type': 'str'},
        'user_id': {'key': 'userId', 'type': 'str'},
        'user_name': {'key': 'userName', 'type': 'str'},
        'user_email': {'key': 'userEmail', 'type': 'str'},
        'user_principal_name': {'key': 'userPrincipalName', 'type': 'str'},
        'sources': {'key': 'sources', 'type': '[MicrosoftGraphSettingSource]'},
        'current_value': {'key': 'currentValue', 'type': 'str'},
        'setting_instance_id': {'key': 'settingInstanceId', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphDeviceConfigurationSettingState, self).__init__(**kwargs)
        self.setting = kwargs.get('setting', None)
        self.setting_name = kwargs.get('setting_name', None)
        self.instance_display_name = kwargs.get('instance_display_name', None)
        self.state = kwargs.get('state', None)
        self.error_code = kwargs.get('error_code', None)
        self.error_description = kwargs.get('error_description', None)
        self.user_id = kwargs.get('user_id', None)
        self.user_name = kwargs.get('user_name', None)
        self.user_email = kwargs.get('user_email', None)
        self.user_principal_name = kwargs.get('user_principal_name', None)
        self.sources = kwargs.get('sources', None)
        self.current_value = kwargs.get('current_value', None)
        self.setting_instance_id = kwargs.get('setting_instance_id', None)


class MicrosoftGraphDeviceConfigurationState(MicrosoftGraphEntity):
    """Device Configuration State for a given device.

    :param id: Read-only.
    :type id: str
    :param setting_states:
    :type setting_states: list[~users_user.models.MicrosoftGraphDeviceConfigurationSettingState]
    :param display_name: The name of the policy for this policyBase.
    :type display_name: str
    :param version: The version of the policy.
    :type version: int
    :param platform_type: policyPlatformType. Possible values include: "android", "androidForWork",
     "iOS", "macOS", "windowsPhone81", "windows81AndLater", "windows10AndLater",
     "androidWorkProfile", "windows10XProfile", "all".
    :type platform_type: str or ~users_user.models.MicrosoftGraphPolicyPlatformType
    :param state: complianceStatus. Possible values include: "unknown", "notApplicable",
     "compliant", "remediated", "nonCompliant", "error", "conflict", "notAssigned".
    :type state: str or ~users_user.models.MicrosoftGraphComplianceStatus
    :param setting_count: Count of how many setting a policy holds.
    :type setting_count: int
    :param user_id: User unique identifier, must be Guid.
    :type user_id: str
    :param user_principal_name: User Principal Name.
    :type user_principal_name: str
    """

    _validation = {
        'version': {'maximum': 2147483647, 'minimum': -2147483648},
        'setting_count': {'maximum': 2147483647, 'minimum': -2147483648},
    }

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'setting_states': {'key': 'settingStates', 'type': '[MicrosoftGraphDeviceConfigurationSettingState]'},
        'display_name': {'key': 'displayName', 'type': 'str'},
        'version': {'key': 'version', 'type': 'int'},
        'platform_type': {'key': 'platformType', 'type': 'str'},
        'state': {'key': 'state', 'type': 'str'},
        'setting_count': {'key': 'settingCount', 'type': 'int'},
        'user_id': {'key': 'userId', 'type': 'str'},
        'user_principal_name': {'key': 'userPrincipalName', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphDeviceConfigurationState, self).__init__(**kwargs)
        self.setting_states = kwargs.get('setting_states', None)
        self.display_name = kwargs.get('display_name', None)
        self.version = kwargs.get('version', None)
        self.platform_type = kwargs.get('platform_type', None)
        self.state = kwargs.get('state', None)
        self.setting_count = kwargs.get('setting_count', None)
        self.user_id = kwargs.get('user_id', None)
        self.user_principal_name = kwargs.get('user_principal_name', None)


class MicrosoftGraphDeviceEnrollmentConfiguration(MicrosoftGraphEntity):
    """The Base Class of Device Enrollment Configuration.

    :param id: Read-only.
    :type id: str
    :param display_name: Not yet documented.
    :type display_name: str
    :param description: Not yet documented.
    :type description: str
    :param priority: Not yet documented.
    :type priority: int
    :param created_date_time: Not yet documented.
    :type created_date_time: ~datetime.datetime
    :param last_modified_date_time: Not yet documented.
    :type last_modified_date_time: ~datetime.datetime
    :param version: Not yet documented.
    :type version: int
    :param assignments: The list of group assignments for the device configuration profile.
    :type assignments: list[~users_user.models.MicrosoftGraphEnrollmentConfigurationAssignment]
    """

    _validation = {
        'priority': {'maximum': 2147483647, 'minimum': -2147483648},
        'version': {'maximum': 2147483647, 'minimum': -2147483648},
    }

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'display_name': {'key': 'displayName', 'type': 'str'},
        'description': {'key': 'description', 'type': 'str'},
        'priority': {'key': 'priority', 'type': 'int'},
        'created_date_time': {'key': 'createdDateTime', 'type': 'iso-8601'},
        'last_modified_date_time': {'key': 'lastModifiedDateTime', 'type': 'iso-8601'},
        'version': {'key': 'version', 'type': 'int'},
        'assignments': {'key': 'assignments', 'type': '[MicrosoftGraphEnrollmentConfigurationAssignment]'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphDeviceEnrollmentConfiguration, self).__init__(**kwargs)
        self.display_name = kwargs.get('display_name', None)
        self.description = kwargs.get('description', None)
        self.priority = kwargs.get('priority', None)
        self.created_date_time = kwargs.get('created_date_time', None)
        self.last_modified_date_time = kwargs.get('last_modified_date_time', None)
        self.version = kwargs.get('version', None)
        self.assignments = kwargs.get('assignments', None)


class MicrosoftGraphDeviceHealthAttestationState(msrest.serialization.Model):
    """deviceHealthAttestationState.

    :param last_update_date_time: The Timestamp of the last update.
    :type last_update_date_time: str
    :param content_namespace_url: The DHA report version. (Namespace version).
    :type content_namespace_url: str
    :param device_health_attestation_status: The DHA report version. (Namespace version).
    :type device_health_attestation_status: str
    :param content_version: The HealthAttestation state schema version.
    :type content_version: str
    :param issued_date_time: The DateTime when device was evaluated or issued to MDM.
    :type issued_date_time: ~datetime.datetime
    :param attestation_identity_key: TWhen an Attestation Identity Key (AIK) is present on a
     device, it indicates that the device has an endorsement key (EK) certificate.
    :type attestation_identity_key: str
    :param reset_count: The number of times a PC device has hibernated or resumed.
    :type reset_count: long
    :param restart_count: The number of times a PC device has rebooted.
    :type restart_count: long
    :param data_excution_policy: DEP Policy defines a set of hardware and software technologies
     that perform additional checks on memory.
    :type data_excution_policy: str
    :param bit_locker_status: On or Off of BitLocker Drive Encryption.
    :type bit_locker_status: str
    :param boot_manager_version: The version of the Boot Manager.
    :type boot_manager_version: str
    :param code_integrity_check_version: The version of the Boot Manager.
    :type code_integrity_check_version: str
    :param secure_boot: When Secure Boot is enabled, the core components must have the correct
     cryptographic signatures.
    :type secure_boot: str
    :param boot_debugging: When bootDebugging is enabled, the device is used in development and
     testing.
    :type boot_debugging: str
    :param operating_system_kernel_debugging: When operatingSystemKernelDebugging is enabled, the
     device is used in development and testing.
    :type operating_system_kernel_debugging: str
    :param code_integrity: When code integrity is enabled, code execution is restricted to
     integrity verified code.
    :type code_integrity: str
    :param test_signing: When test signing is allowed, the device does not enforce signature
     validation during boot.
    :type test_signing: str
    :param safe_mode: Safe mode is a troubleshooting option for Windows that starts your computer
     in a limited state.
    :type safe_mode: str
    :param windows_pe: Operating system running with limited services that is used to prepare a
     computer for Windows.
    :type windows_pe: str
    :param early_launch_anti_malware_driver_protection: ELAM provides protection for the computers
     in your network when they start up.
    :type early_launch_anti_malware_driver_protection: str
    :param virtual_secure_mode: VSM is a container that protects high value assets from a
     compromised kernel.
    :type virtual_secure_mode: str
    :param pcr_hash_algorithm: Informational attribute that identifies the HASH algorithm that was
     used by TPM.
    :type pcr_hash_algorithm: str
    :param boot_app_security_version: The security version number of the Boot Application.
    :type boot_app_security_version: str
    :param boot_manager_security_version: The security version number of the Boot Application.
    :type boot_manager_security_version: str
    :param tpm_version: The security version number of the Boot Application.
    :type tpm_version: str
    :param pcr0: The measurement that is captured in PCR[0].
    :type pcr0: str
    :param secure_boot_configuration_policy_finger_print: Fingerprint of the Custom Secure Boot
     Configuration Policy.
    :type secure_boot_configuration_policy_finger_print: str
    :param code_integrity_policy: The Code Integrity policy that is controlling the security of the
     boot environment.
    :type code_integrity_policy: str
    :param boot_revision_list_info: The Boot Revision List that was loaded during initial boot on
     the attested device.
    :type boot_revision_list_info: str
    :param operating_system_rev_list_info: The Operating System Revision List that was loaded
     during initial boot on the attested device.
    :type operating_system_rev_list_info: str
    :param health_status_mismatch_info: This attribute appears if DHA-Service detects an integrity
     issue.
    :type health_status_mismatch_info: str
    :param health_attestation_supported_status: This attribute indicates if DHA is supported for
     the device.
    :type health_attestation_supported_status: str
    """

    _attribute_map = {
        'last_update_date_time': {'key': 'lastUpdateDateTime', 'type': 'str'},
        'content_namespace_url': {'key': 'contentNamespaceUrl', 'type': 'str'},
        'device_health_attestation_status': {'key': 'deviceHealthAttestationStatus', 'type': 'str'},
        'content_version': {'key': 'contentVersion', 'type': 'str'},
        'issued_date_time': {'key': 'issuedDateTime', 'type': 'iso-8601'},
        'attestation_identity_key': {'key': 'attestationIdentityKey', 'type': 'str'},
        'reset_count': {'key': 'resetCount', 'type': 'long'},
        'restart_count': {'key': 'restartCount', 'type': 'long'},
        'data_excution_policy': {'key': 'dataExcutionPolicy', 'type': 'str'},
        'bit_locker_status': {'key': 'bitLockerStatus', 'type': 'str'},
        'boot_manager_version': {'key': 'bootManagerVersion', 'type': 'str'},
        'code_integrity_check_version': {'key': 'codeIntegrityCheckVersion', 'type': 'str'},
        'secure_boot': {'key': 'secureBoot', 'type': 'str'},
        'boot_debugging': {'key': 'bootDebugging', 'type': 'str'},
        'operating_system_kernel_debugging': {'key': 'operatingSystemKernelDebugging', 'type': 'str'},
        'code_integrity': {'key': 'codeIntegrity', 'type': 'str'},
        'test_signing': {'key': 'testSigning', 'type': 'str'},
        'safe_mode': {'key': 'safeMode', 'type': 'str'},
        'windows_pe': {'key': 'windowsPE', 'type': 'str'},
        'early_launch_anti_malware_driver_protection': {'key': 'earlyLaunchAntiMalwareDriverProtection', 'type': 'str'},
        'virtual_secure_mode': {'key': 'virtualSecureMode', 'type': 'str'},
        'pcr_hash_algorithm': {'key': 'pcrHashAlgorithm', 'type': 'str'},
        'boot_app_security_version': {'key': 'bootAppSecurityVersion', 'type': 'str'},
        'boot_manager_security_version': {'key': 'bootManagerSecurityVersion', 'type': 'str'},
        'tpm_version': {'key': 'tpmVersion', 'type': 'str'},
        'pcr0': {'key': 'pcr0', 'type': 'str'},
        'secure_boot_configuration_policy_finger_print': {'key': 'secureBootConfigurationPolicyFingerPrint', 'type': 'str'},
        'code_integrity_policy': {'key': 'codeIntegrityPolicy', 'type': 'str'},
        'boot_revision_list_info': {'key': 'bootRevisionListInfo', 'type': 'str'},
        'operating_system_rev_list_info': {'key': 'operatingSystemRevListInfo', 'type': 'str'},
        'health_status_mismatch_info': {'key': 'healthStatusMismatchInfo', 'type': 'str'},
        'health_attestation_supported_status': {'key': 'healthAttestationSupportedStatus', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphDeviceHealthAttestationState, self).__init__(**kwargs)
        self.last_update_date_time = kwargs.get('last_update_date_time', None)
        self.content_namespace_url = kwargs.get('content_namespace_url', None)
        self.device_health_attestation_status = kwargs.get('device_health_attestation_status', None)
        self.content_version = kwargs.get('content_version', None)
        self.issued_date_time = kwargs.get('issued_date_time', None)
        self.attestation_identity_key = kwargs.get('attestation_identity_key', None)
        self.reset_count = kwargs.get('reset_count', None)
        self.restart_count = kwargs.get('restart_count', None)
        self.data_excution_policy = kwargs.get('data_excution_policy', None)
        self.bit_locker_status = kwargs.get('bit_locker_status', None)
        self.boot_manager_version = kwargs.get('boot_manager_version', None)
        self.code_integrity_check_version = kwargs.get('code_integrity_check_version', None)
        self.secure_boot = kwargs.get('secure_boot', None)
        self.boot_debugging = kwargs.get('boot_debugging', None)
        self.operating_system_kernel_debugging = kwargs.get('operating_system_kernel_debugging', None)
        self.code_integrity = kwargs.get('code_integrity', None)
        self.test_signing = kwargs.get('test_signing', None)
        self.safe_mode = kwargs.get('safe_mode', None)
        self.windows_pe = kwargs.get('windows_pe', None)
        self.early_launch_anti_malware_driver_protection = kwargs.get('early_launch_anti_malware_driver_protection', None)
        self.virtual_secure_mode = kwargs.get('virtual_secure_mode', None)
        self.pcr_hash_algorithm = kwargs.get('pcr_hash_algorithm', None)
        self.boot_app_security_version = kwargs.get('boot_app_security_version', None)
        self.boot_manager_security_version = kwargs.get('boot_manager_security_version', None)
        self.tpm_version = kwargs.get('tpm_version', None)
        self.pcr0 = kwargs.get('pcr0', None)
        self.secure_boot_configuration_policy_finger_print = kwargs.get('secure_boot_configuration_policy_finger_print', None)
        self.code_integrity_policy = kwargs.get('code_integrity_policy', None)
        self.boot_revision_list_info = kwargs.get('boot_revision_list_info', None)
        self.operating_system_rev_list_info = kwargs.get('operating_system_rev_list_info', None)
        self.health_status_mismatch_info = kwargs.get('health_status_mismatch_info', None)
        self.health_attestation_supported_status = kwargs.get('health_attestation_supported_status', None)


class MicrosoftGraphDeviceKey(msrest.serialization.Model):
    """deviceKey.

    :param key_type:
    :type key_type: str
    :param key_material:
    :type key_material: bytes
    :param device_id:
    :type device_id: str
    """

    _attribute_map = {
        'key_type': {'key': 'keyType', 'type': 'str'},
        'key_material': {'key': 'keyMaterial', 'type': 'base64'},
        'device_id': {'key': 'deviceId', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphDeviceKey, self).__init__(**kwargs)
        self.key_type = kwargs.get('key_type', None)
        self.key_material = kwargs.get('key_material', None)
        self.device_id = kwargs.get('device_id', None)


class MicrosoftGraphDeviceManagementTroubleshootingErrorDetails(msrest.serialization.Model):
    """deviceManagementTroubleshootingErrorDetails.

    :param context:
    :type context: str
    :param failure:
    :type failure: str
    :param failure_details: The detailed description of what went wrong.
    :type failure_details: str
    :param remediation: The detailed description of how to remediate this issue.
    :type remediation: str
    :param resources: Links to helpful documentation about this failure.
    :type resources:
     list[~users_user.models.MicrosoftGraphDeviceManagementTroubleshootingErrorResource]
    """

    _attribute_map = {
        'context': {'key': 'context', 'type': 'str'},
        'failure': {'key': 'failure', 'type': 'str'},
        'failure_details': {'key': 'failureDetails', 'type': 'str'},
        'remediation': {'key': 'remediation', 'type': 'str'},
        'resources': {'key': 'resources', 'type': '[MicrosoftGraphDeviceManagementTroubleshootingErrorResource]'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphDeviceManagementTroubleshootingErrorDetails, self).__init__(**kwargs)
        self.context = kwargs.get('context', None)
        self.failure = kwargs.get('failure', None)
        self.failure_details = kwargs.get('failure_details', None)
        self.remediation = kwargs.get('remediation', None)
        self.resources = kwargs.get('resources', None)


class MicrosoftGraphDeviceManagementTroubleshootingErrorResource(msrest.serialization.Model):
    """deviceManagementTroubleshootingErrorResource.

    :param text:
    :type text: str
    :param link: The link to the web resource. Can contain any of the following formatters:
     {{UPN}}, {{DeviceGUID}}, {{UserGUID}}.
    :type link: str
    """

    _attribute_map = {
        'text': {'key': 'text', 'type': 'str'},
        'link': {'key': 'link', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphDeviceManagementTroubleshootingErrorResource, self).__init__(**kwargs)
        self.text = kwargs.get('text', None)
        self.link = kwargs.get('link', None)


class MicrosoftGraphDeviceManagementTroubleshootingEvent(MicrosoftGraphEntity):
    """Event representing an general failure.

    :param id: Read-only.
    :type id: str
    :param event_date_time: Time when the event occurred .
    :type event_date_time: ~datetime.datetime
    :param correlation_id: Id used for tracing the failure in the service.
    :type correlation_id: str
    :param troubleshooting_error_details: deviceManagementTroubleshootingErrorDetails.
    :type troubleshooting_error_details:
     ~users_user.models.MicrosoftGraphDeviceManagementTroubleshootingErrorDetails
    :param event_name: Event Name corresponding to the Troubleshooting Event. It is an Optional
     field.
    :type event_name: str
    :param additional_information: A set of string key and string value pairs which provides
     additional information on the Troubleshooting event.
    :type additional_information: list[~users_user.models.MicrosoftGraphKeyValuePair]
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'event_date_time': {'key': 'eventDateTime', 'type': 'iso-8601'},
        'correlation_id': {'key': 'correlationId', 'type': 'str'},
        'troubleshooting_error_details': {'key': 'troubleshootingErrorDetails', 'type': 'MicrosoftGraphDeviceManagementTroubleshootingErrorDetails'},
        'event_name': {'key': 'eventName', 'type': 'str'},
        'additional_information': {'key': 'additionalInformation', 'type': '[MicrosoftGraphKeyValuePair]'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphDeviceManagementTroubleshootingEvent, self).__init__(**kwargs)
        self.event_date_time = kwargs.get('event_date_time', None)
        self.correlation_id = kwargs.get('correlation_id', None)
        self.troubleshooting_error_details = kwargs.get('troubleshooting_error_details', None)
        self.event_name = kwargs.get('event_name', None)
        self.additional_information = kwargs.get('additional_information', None)


class MicrosoftGraphDirectorySetting(MicrosoftGraphEntity):
    """directorySetting.

    :param id: Read-only.
    :type id: str
    :param display_name:
    :type display_name: str
    :param template_id:
    :type template_id: str
    :param values:
    :type values: list[~users_user.models.MicrosoftGraphSettingValue]
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'display_name': {'key': 'displayName', 'type': 'str'},
        'template_id': {'key': 'templateId', 'type': 'str'},
        'values': {'key': 'values', 'type': '[MicrosoftGraphSettingValue]'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphDirectorySetting, self).__init__(**kwargs)
        self.display_name = kwargs.get('display_name', None)
        self.template_id = kwargs.get('template_id', None)
        self.values = kwargs.get('values', None)


class MicrosoftGraphDrive(MicrosoftGraphBaseItem):
    """drive.

    :param id: Read-only.
    :type id: str
    :param created_by: identitySet.
    :type created_by: ~users_user.models.MicrosoftGraphIdentitySet
    :param created_date_time: Date and time of item creation. Read-only.
    :type created_date_time: ~datetime.datetime
    :param description: Provides a user-visible description of the item. Optional.
    :type description: str
    :param e_tag: ETag for the item. Read-only.
    :type e_tag: str
    :param last_modified_by: identitySet.
    :type last_modified_by: ~users_user.models.MicrosoftGraphIdentitySet
    :param last_modified_date_time: Date and time the item was last modified. Read-only.
    :type last_modified_date_time: ~datetime.datetime
    :param name: The name of the item. Read-write.
    :type name: str
    :param parent_reference: itemReference.
    :type parent_reference: ~users_user.models.MicrosoftGraphItemReference
    :param web_url: URL that displays the resource in the browser. Read-only.
    :type web_url: str
    :param created_by_user: Represents an Azure Active Directory user object.
    :type created_by_user: ~users_user.models.MicrosoftGraphUser
    :param last_modified_by_user: Represents an Azure Active Directory user object.
    :type last_modified_by_user: ~users_user.models.MicrosoftGraphUser
    :param drive_type: Describes the type of drive represented by this resource. OneDrive personal
     drives will return personal. OneDrive for Business will return business. SharePoint document
     libraries will return documentLibrary. Read-only.
    :type drive_type: str
    :param owner: identitySet.
    :type owner: ~users_user.models.MicrosoftGraphIdentitySet
    :param quota: quota.
    :type quota: ~users_user.models.MicrosoftGraphQuota
    :param share_point_ids: sharepointIds.
    :type share_point_ids: ~users_user.models.MicrosoftGraphSharepointIds
    :param system: Any object.
    :type system: object
    :param activities:
    :type activities: list[~users_user.models.MicrosoftGraphItemActivityOld]
    :param bundles:
    :type bundles: list[~users_user.models.MicrosoftGraphDriveItem]
    :param following: The list of items the user is following. Only in OneDrive for Business.
    :type following: list[~users_user.models.MicrosoftGraphDriveItem]
    :param items: All items contained in the drive. Read-only. Nullable.
    :type items: list[~users_user.models.MicrosoftGraphDriveItem]
    :param list: list.
    :type list: ~users_user.models.MicrosoftGraphList
    :param root: driveItem.
    :type root: ~users_user.models.MicrosoftGraphDriveItem
    :param special: Collection of common folders available in OneDrive. Read-only. Nullable.
    :type special: list[~users_user.models.MicrosoftGraphDriveItem]
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'created_by': {'key': 'createdBy', 'type': 'MicrosoftGraphIdentitySet'},
        'created_date_time': {'key': 'createdDateTime', 'type': 'iso-8601'},
        'description': {'key': 'description', 'type': 'str'},
        'e_tag': {'key': 'eTag', 'type': 'str'},
        'last_modified_by': {'key': 'lastModifiedBy', 'type': 'MicrosoftGraphIdentitySet'},
        'last_modified_date_time': {'key': 'lastModifiedDateTime', 'type': 'iso-8601'},
        'name': {'key': 'name', 'type': 'str'},
        'parent_reference': {'key': 'parentReference', 'type': 'MicrosoftGraphItemReference'},
        'web_url': {'key': 'webUrl', 'type': 'str'},
        'created_by_user': {'key': 'createdByUser', 'type': 'MicrosoftGraphUser'},
        'last_modified_by_user': {'key': 'lastModifiedByUser', 'type': 'MicrosoftGraphUser'},
        'drive_type': {'key': 'driveType', 'type': 'str'},
        'owner': {'key': 'owner', 'type': 'MicrosoftGraphIdentitySet'},
        'quota': {'key': 'quota', 'type': 'MicrosoftGraphQuota'},
        'share_point_ids': {'key': 'sharePointIds', 'type': 'MicrosoftGraphSharepointIds'},
        'system': {'key': 'system', 'type': 'object'},
        'activities': {'key': 'activities', 'type': '[MicrosoftGraphItemActivityOld]'},
        'bundles': {'key': 'bundles', 'type': '[MicrosoftGraphDriveItem]'},
        'following': {'key': 'following', 'type': '[MicrosoftGraphDriveItem]'},
        'items': {'key': 'items', 'type': '[MicrosoftGraphDriveItem]'},
        'list': {'key': 'list', 'type': 'MicrosoftGraphList'},
        'root': {'key': 'root', 'type': 'MicrosoftGraphDriveItem'},
        'special': {'key': 'special', 'type': '[MicrosoftGraphDriveItem]'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphDrive, self).__init__(**kwargs)
        self.drive_type = kwargs.get('drive_type', None)
        self.owner = kwargs.get('owner', None)
        self.quota = kwargs.get('quota', None)
        self.share_point_ids = kwargs.get('share_point_ids', None)
        self.system = kwargs.get('system', None)
        self.activities = kwargs.get('activities', None)
        self.bundles = kwargs.get('bundles', None)
        self.following = kwargs.get('following', None)
        self.items = kwargs.get('items', None)
        self.list = kwargs.get('list', None)
        self.root = kwargs.get('root', None)
        self.special = kwargs.get('special', None)


class MicrosoftGraphDriveItem(MicrosoftGraphBaseItem):
    """driveItem.

    :param id: Read-only.
    :type id: str
    :param created_by: identitySet.
    :type created_by: ~users_user.models.MicrosoftGraphIdentitySet
    :param created_date_time: Date and time of item creation. Read-only.
    :type created_date_time: ~datetime.datetime
    :param description: Provides a user-visible description of the item. Optional.
    :type description: str
    :param e_tag: ETag for the item. Read-only.
    :type e_tag: str
    :param last_modified_by: identitySet.
    :type last_modified_by: ~users_user.models.MicrosoftGraphIdentitySet
    :param last_modified_date_time: Date and time the item was last modified. Read-only.
    :type last_modified_date_time: ~datetime.datetime
    :param name: The name of the item. Read-write.
    :type name: str
    :param parent_reference: itemReference.
    :type parent_reference: ~users_user.models.MicrosoftGraphItemReference
    :param web_url: URL that displays the resource in the browser. Read-only.
    :type web_url: str
    :param created_by_user: Represents an Azure Active Directory user object.
    :type created_by_user: ~users_user.models.MicrosoftGraphUser
    :param last_modified_by_user: Represents an Azure Active Directory user object.
    :type last_modified_by_user: ~users_user.models.MicrosoftGraphUser
    :param audio: audio.
    :type audio: ~users_user.models.MicrosoftGraphAudio
    :param bundle: bundle.
    :type bundle: ~users_user.models.MicrosoftGraphBundle
    :param content: The content stream, if the item represents a file.
    :type content: bytes
    :param c_tag: An eTag for the content of the item. This eTag is not changed if only the
     metadata is changed. Note This property is not returned if the item is a folder. Read-only.
    :type c_tag: str
    :param deleted: deleted.
    :type deleted: ~users_user.models.MicrosoftGraphDeleted
    :param file: file.
    :type file: ~users_user.models.MicrosoftGraphFile
    :param file_system_info: fileSystemInfo.
    :type file_system_info: ~users_user.models.MicrosoftGraphFileSystemInfo
    :param folder: folder.
    :type folder: ~users_user.models.MicrosoftGraphFolder
    :param image: image.
    :type image: ~users_user.models.MicrosoftGraphImage
    :param location: geoCoordinates.
    :type location: ~users_user.models.MicrosoftGraphGeoCoordinates
    :param package: package.
    :type package: ~users_user.models.MicrosoftGraphPackage
    :param pending_operations: pendingOperations.
    :type pending_operations: ~users_user.models.MicrosoftGraphPendingOperations
    :param photo: photo.
    :type photo: ~users_user.models.MicrosoftGraphPhoto
    :param publication: publicationFacet.
    :type publication: ~users_user.models.MicrosoftGraphPublicationFacet
    :param remote_item: remoteItem.
    :type remote_item: ~users_user.models.MicrosoftGraphRemoteItem
    :param root: Any object.
    :type root: object
    :param search_result: searchResult.
    :type search_result: ~users_user.models.MicrosoftGraphSearchResult
    :param shared: shared.
    :type shared: ~users_user.models.MicrosoftGraphShared
    :param sharepoint_ids: sharepointIds.
    :type sharepoint_ids: ~users_user.models.MicrosoftGraphSharepointIds
    :param size: Size of the item in bytes. Read-only.
    :type size: long
    :param special_folder: specialFolder.
    :type special_folder: ~users_user.models.MicrosoftGraphSpecialFolder
    :param video: video.
    :type video: ~users_user.models.MicrosoftGraphVideo
    :param web_dav_url: WebDAV compatible URL for the item.
    :type web_dav_url: str
    :param workbook: workbook.
    :type workbook: ~users_user.models.MicrosoftGraphWorkbook
    :param activities: The list of recent activities that took place on this item.
    :type activities: list[~users_user.models.MicrosoftGraphItemActivityOld]
    :param analytics: itemAnalytics.
    :type analytics: ~users_user.models.MicrosoftGraphItemAnalytics
    :param children: Collection containing Item objects for the immediate children of Item. Only
     items representing folders have children. Read-only. Nullable.
    :type children: list[~users_user.models.MicrosoftGraphDriveItem]
    :param list_item: listItem.
    :type list_item: ~users_user.models.MicrosoftGraphListItem
    :param permissions: The set of permissions for the item. Read-only. Nullable.
    :type permissions: list[~users_user.models.MicrosoftGraphPermission]
    :param subscriptions: The set of subscriptions on the item. Only supported on the root of a
     drive.
    :type subscriptions: list[~users_user.models.MicrosoftGraphSubscription]
    :param thumbnails: Collection containing [ThumbnailSet][] objects associated with the item. For
     more info, see [getting thumbnails][]. Read-only. Nullable.
    :type thumbnails: list[~users_user.models.MicrosoftGraphThumbnailSet]
    :param versions: The list of previous versions of the item. For more info, see [getting
     previous versions][]. Read-only. Nullable.
    :type versions: list[~users_user.models.MicrosoftGraphDriveItemVersion]
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'created_by': {'key': 'createdBy', 'type': 'MicrosoftGraphIdentitySet'},
        'created_date_time': {'key': 'createdDateTime', 'type': 'iso-8601'},
        'description': {'key': 'description', 'type': 'str'},
        'e_tag': {'key': 'eTag', 'type': 'str'},
        'last_modified_by': {'key': 'lastModifiedBy', 'type': 'MicrosoftGraphIdentitySet'},
        'last_modified_date_time': {'key': 'lastModifiedDateTime', 'type': 'iso-8601'},
        'name': {'key': 'name', 'type': 'str'},
        'parent_reference': {'key': 'parentReference', 'type': 'MicrosoftGraphItemReference'},
        'web_url': {'key': 'webUrl', 'type': 'str'},
        'created_by_user': {'key': 'createdByUser', 'type': 'MicrosoftGraphUser'},
        'last_modified_by_user': {'key': 'lastModifiedByUser', 'type': 'MicrosoftGraphUser'},
        'audio': {'key': 'audio', 'type': 'MicrosoftGraphAudio'},
        'bundle': {'key': 'bundle', 'type': 'MicrosoftGraphBundle'},
        'content': {'key': 'content', 'type': 'base64'},
        'c_tag': {'key': 'cTag', 'type': 'str'},
        'deleted': {'key': 'deleted', 'type': 'MicrosoftGraphDeleted'},
        'file': {'key': 'file', 'type': 'MicrosoftGraphFile'},
        'file_system_info': {'key': 'fileSystemInfo', 'type': 'MicrosoftGraphFileSystemInfo'},
        'folder': {'key': 'folder', 'type': 'MicrosoftGraphFolder'},
        'image': {'key': 'image', 'type': 'MicrosoftGraphImage'},
        'location': {'key': 'location', 'type': 'MicrosoftGraphGeoCoordinates'},
        'package': {'key': 'package', 'type': 'MicrosoftGraphPackage'},
        'pending_operations': {'key': 'pendingOperations', 'type': 'MicrosoftGraphPendingOperations'},
        'photo': {'key': 'photo', 'type': 'MicrosoftGraphPhoto'},
        'publication': {'key': 'publication', 'type': 'MicrosoftGraphPublicationFacet'},
        'remote_item': {'key': 'remoteItem', 'type': 'MicrosoftGraphRemoteItem'},
        'root': {'key': 'root', 'type': 'object'},
        'search_result': {'key': 'searchResult', 'type': 'MicrosoftGraphSearchResult'},
        'shared': {'key': 'shared', 'type': 'MicrosoftGraphShared'},
        'sharepoint_ids': {'key': 'sharepointIds', 'type': 'MicrosoftGraphSharepointIds'},
        'size': {'key': 'size', 'type': 'long'},
        'special_folder': {'key': 'specialFolder', 'type': 'MicrosoftGraphSpecialFolder'},
        'video': {'key': 'video', 'type': 'MicrosoftGraphVideo'},
        'web_dav_url': {'key': 'webDavUrl', 'type': 'str'},
        'workbook': {'key': 'workbook', 'type': 'MicrosoftGraphWorkbook'},
        'activities': {'key': 'activities', 'type': '[MicrosoftGraphItemActivityOld]'},
        'analytics': {'key': 'analytics', 'type': 'MicrosoftGraphItemAnalytics'},
        'children': {'key': 'children', 'type': '[MicrosoftGraphDriveItem]'},
        'list_item': {'key': 'listItem', 'type': 'MicrosoftGraphListItem'},
        'permissions': {'key': 'permissions', 'type': '[MicrosoftGraphPermission]'},
        'subscriptions': {'key': 'subscriptions', 'type': '[MicrosoftGraphSubscription]'},
        'thumbnails': {'key': 'thumbnails', 'type': '[MicrosoftGraphThumbnailSet]'},
        'versions': {'key': 'versions', 'type': '[MicrosoftGraphDriveItemVersion]'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphDriveItem, self).__init__(**kwargs)
        self.audio = kwargs.get('audio', None)
        self.bundle = kwargs.get('bundle', None)
        self.content = kwargs.get('content', None)
        self.c_tag = kwargs.get('c_tag', None)
        self.deleted = kwargs.get('deleted', None)
        self.file = kwargs.get('file', None)
        self.file_system_info = kwargs.get('file_system_info', None)
        self.folder = kwargs.get('folder', None)
        self.image = kwargs.get('image', None)
        self.location = kwargs.get('location', None)
        self.package = kwargs.get('package', None)
        self.pending_operations = kwargs.get('pending_operations', None)
        self.photo = kwargs.get('photo', None)
        self.publication = kwargs.get('publication', None)
        self.remote_item = kwargs.get('remote_item', None)
        self.root = kwargs.get('root', None)
        self.search_result = kwargs.get('search_result', None)
        self.shared = kwargs.get('shared', None)
        self.sharepoint_ids = kwargs.get('sharepoint_ids', None)
        self.size = kwargs.get('size', None)
        self.special_folder = kwargs.get('special_folder', None)
        self.video = kwargs.get('video', None)
        self.web_dav_url = kwargs.get('web_dav_url', None)
        self.workbook = kwargs.get('workbook', None)
        self.activities = kwargs.get('activities', None)
        self.analytics = kwargs.get('analytics', None)
        self.children = kwargs.get('children', None)
        self.list_item = kwargs.get('list_item', None)
        self.permissions = kwargs.get('permissions', None)
        self.subscriptions = kwargs.get('subscriptions', None)
        self.thumbnails = kwargs.get('thumbnails', None)
        self.versions = kwargs.get('versions', None)


class MicrosoftGraphDriveItemVersion(MicrosoftGraphBaseItemVersion):
    """driveItemVersion.

    :param id: Read-only.
    :type id: str
    :param last_modified_by: identitySet.
    :type last_modified_by: ~users_user.models.MicrosoftGraphIdentitySet
    :param last_modified_date_time: Date and time the version was last modified. Read-only.
    :type last_modified_date_time: ~datetime.datetime
    :param publication: publicationFacet.
    :type publication: ~users_user.models.MicrosoftGraphPublicationFacet
    :param content: The content stream for this version of the item.
    :type content: bytes
    :param size: Indicates the size of the content stream for this version of the item.
    :type size: long
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'last_modified_by': {'key': 'lastModifiedBy', 'type': 'MicrosoftGraphIdentitySet'},
        'last_modified_date_time': {'key': 'lastModifiedDateTime', 'type': 'iso-8601'},
        'publication': {'key': 'publication', 'type': 'MicrosoftGraphPublicationFacet'},
        'content': {'key': 'content', 'type': 'base64'},
        'size': {'key': 'size', 'type': 'long'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphDriveItemVersion, self).__init__(**kwargs)
        self.content = kwargs.get('content', None)
        self.size = kwargs.get('size', None)


class MicrosoftGraphItemFacet(MicrosoftGraphEntity):
    """itemFacet.

    :param id: Read-only.
    :type id: str
    :param allowed_audiences: allowedAudiences. Possible values include: "me", "family",
     "contacts", "groupMembers", "organization", "federatedOrganizations", "everyone",
     "unknownFutureValue".
    :type allowed_audiences: str or ~users_user.models.MicrosoftGraphAllowedAudiences
    :param inference: inferenceData.
    :type inference: ~users_user.models.MicrosoftGraphInferenceData
    :param created_date_time:
    :type created_date_time: ~datetime.datetime
    :param created_by: identitySet.
    :type created_by: ~users_user.models.MicrosoftGraphIdentitySet
    :param last_modified_date_time:
    :type last_modified_date_time: ~datetime.datetime
    :param last_modified_by: identitySet.
    :type last_modified_by: ~users_user.models.MicrosoftGraphIdentitySet
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'allowed_audiences': {'key': 'allowedAudiences', 'type': 'str'},
        'inference': {'key': 'inference', 'type': 'MicrosoftGraphInferenceData'},
        'created_date_time': {'key': 'createdDateTime', 'type': 'iso-8601'},
        'created_by': {'key': 'createdBy', 'type': 'MicrosoftGraphIdentitySet'},
        'last_modified_date_time': {'key': 'lastModifiedDateTime', 'type': 'iso-8601'},
        'last_modified_by': {'key': 'lastModifiedBy', 'type': 'MicrosoftGraphIdentitySet'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphItemFacet, self).__init__(**kwargs)
        self.allowed_audiences = kwargs.get('allowed_audiences', None)
        self.inference = kwargs.get('inference', None)
        self.created_date_time = kwargs.get('created_date_time', None)
        self.created_by = kwargs.get('created_by', None)
        self.last_modified_date_time = kwargs.get('last_modified_date_time', None)
        self.last_modified_by = kwargs.get('last_modified_by', None)


class MicrosoftGraphEducationalActivity(MicrosoftGraphItemFacet):
    """educationalActivity.

    :param id: Read-only.
    :type id: str
    :param allowed_audiences: allowedAudiences. Possible values include: "me", "family",
     "contacts", "groupMembers", "organization", "federatedOrganizations", "everyone",
     "unknownFutureValue".
    :type allowed_audiences: str or ~users_user.models.MicrosoftGraphAllowedAudiences
    :param inference: inferenceData.
    :type inference: ~users_user.models.MicrosoftGraphInferenceData
    :param created_date_time:
    :type created_date_time: ~datetime.datetime
    :param created_by: identitySet.
    :type created_by: ~users_user.models.MicrosoftGraphIdentitySet
    :param last_modified_date_time:
    :type last_modified_date_time: ~datetime.datetime
    :param last_modified_by: identitySet.
    :type last_modified_by: ~users_user.models.MicrosoftGraphIdentitySet
    :param completion_month_year:
    :type completion_month_year: ~datetime.date
    :param end_month_year:
    :type end_month_year: ~datetime.date
    :param institution: institutionData.
    :type institution: ~users_user.models.MicrosoftGraphInstitutionData
    :param program: educationalActivityDetail.
    :type program: ~users_user.models.MicrosoftGraphEducationalActivityDetail
    :param start_month_year:
    :type start_month_year: ~datetime.date
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'allowed_audiences': {'key': 'allowedAudiences', 'type': 'str'},
        'inference': {'key': 'inference', 'type': 'MicrosoftGraphInferenceData'},
        'created_date_time': {'key': 'createdDateTime', 'type': 'iso-8601'},
        'created_by': {'key': 'createdBy', 'type': 'MicrosoftGraphIdentitySet'},
        'last_modified_date_time': {'key': 'lastModifiedDateTime', 'type': 'iso-8601'},
        'last_modified_by': {'key': 'lastModifiedBy', 'type': 'MicrosoftGraphIdentitySet'},
        'completion_month_year': {'key': 'completionMonthYear', 'type': 'date'},
        'end_month_year': {'key': 'endMonthYear', 'type': 'date'},
        'institution': {'key': 'institution', 'type': 'MicrosoftGraphInstitutionData'},
        'program': {'key': 'program', 'type': 'MicrosoftGraphEducationalActivityDetail'},
        'start_month_year': {'key': 'startMonthYear', 'type': 'date'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphEducationalActivity, self).__init__(**kwargs)
        self.completion_month_year = kwargs.get('completion_month_year', None)
        self.end_month_year = kwargs.get('end_month_year', None)
        self.institution = kwargs.get('institution', None)
        self.program = kwargs.get('program', None)
        self.start_month_year = kwargs.get('start_month_year', None)


class MicrosoftGraphEducationalActivityDetail(msrest.serialization.Model):
    """educationalActivityDetail.

    :param abbreviation:
    :type abbreviation: str
    :param activities:
    :type activities: str
    :param awards:
    :type awards: str
    :param description:
    :type description: str
    :param display_name:
    :type display_name: str
    :param fields_of_study:
    :type fields_of_study: str
    :param grade:
    :type grade: str
    :param notes:
    :type notes: str
    :param web_url:
    :type web_url: str
    """

    _attribute_map = {
        'abbreviation': {'key': 'abbreviation', 'type': 'str'},
        'activities': {'key': 'activities', 'type': 'str'},
        'awards': {'key': 'awards', 'type': 'str'},
        'description': {'key': 'description', 'type': 'str'},
        'display_name': {'key': 'displayName', 'type': 'str'},
        'fields_of_study': {'key': 'fieldsOfStudy', 'type': 'str'},
        'grade': {'key': 'grade', 'type': 'str'},
        'notes': {'key': 'notes', 'type': 'str'},
        'web_url': {'key': 'webUrl', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphEducationalActivityDetail, self).__init__(**kwargs)
        self.abbreviation = kwargs.get('abbreviation', None)
        self.activities = kwargs.get('activities', None)
        self.awards = kwargs.get('awards', None)
        self.description = kwargs.get('description', None)
        self.display_name = kwargs.get('display_name', None)
        self.fields_of_study = kwargs.get('fields_of_study', None)
        self.grade = kwargs.get('grade', None)
        self.notes = kwargs.get('notes', None)
        self.web_url = kwargs.get('web_url', None)


class MicrosoftGraphEmailAddress(msrest.serialization.Model):
    """emailAddress.

    :param name: The display name of the person or entity.
    :type name: str
    :param address: The email address of the person or entity.
    :type address: str
    """

    _attribute_map = {
        'name': {'key': 'name', 'type': 'str'},
        'address': {'key': 'address', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphEmailAddress, self).__init__(**kwargs)
        self.name = kwargs.get('name', None)
        self.address = kwargs.get('address', None)


class MicrosoftGraphEndpoint(MicrosoftGraphDirectoryObject):
    """Represents an Azure Active Directory object. The directoryObject type is the base type for many other directory entity types.

    :param id: Read-only.
    :type id: str
    :param deleted_date_time:
    :type deleted_date_time: ~datetime.datetime
    :param capability:
    :type capability: str
    :param provider_id:
    :type provider_id: str
    :param provider_name:
    :type provider_name: str
    :param uri:
    :type uri: str
    :param provider_resource_id:
    :type provider_resource_id: str
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'deleted_date_time': {'key': 'deletedDateTime', 'type': 'iso-8601'},
        'capability': {'key': 'capability', 'type': 'str'},
        'provider_id': {'key': 'providerId', 'type': 'str'},
        'provider_name': {'key': 'providerName', 'type': 'str'},
        'uri': {'key': 'uri', 'type': 'str'},
        'provider_resource_id': {'key': 'providerResourceId', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphEndpoint, self).__init__(**kwargs)
        self.capability = kwargs.get('capability', None)
        self.provider_id = kwargs.get('provider_id', None)
        self.provider_name = kwargs.get('provider_name', None)
        self.uri = kwargs.get('uri', None)
        self.provider_resource_id = kwargs.get('provider_resource_id', None)


class MicrosoftGraphEnrollmentConfigurationAssignment(MicrosoftGraphEntity):
    """Enrollment Configuration Assignment.

    :param id: Read-only.
    :type id: str
    :param target: Any object.
    :type target: object
    :param source: deviceAndAppManagementAssignmentSource. Possible values include: "direct",
     "policySets".
    :type source: str or ~users_user.models.MicrosoftGraphDeviceAndAppManagementAssignmentSource
    :param source_id: Identifier for resource used for deployment to a group.
    :type source_id: str
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'target': {'key': 'target', 'type': 'object'},
        'source': {'key': 'source', 'type': 'str'},
        'source_id': {'key': 'sourceId', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphEnrollmentConfigurationAssignment, self).__init__(**kwargs)
        self.target = kwargs.get('target', None)
        self.source = kwargs.get('source', None)
        self.source_id = kwargs.get('source_id', None)


class MicrosoftGraphEvent(MicrosoftGraphOutlookItem):
    """event.

    :param id: Read-only.
    :type id: str
    :param created_date_time: The Timestamp type represents date and time information using ISO
     8601 format and is always in UTC time. For example, midnight UTC on Jan 1, 2014 would look like
     this: '2014-01-01T00:00:00Z'.
    :type created_date_time: ~datetime.datetime
    :param last_modified_date_time: The Timestamp type represents date and time information using
     ISO 8601 format and is always in UTC time. For example, midnight UTC on Jan 1, 2014 would look
     like this: '2014-01-01T00:00:00Z'.
    :type last_modified_date_time: ~datetime.datetime
    :param change_key: Identifies the version of the item. Every time the item is changed,
     changeKey changes as well. This allows Exchange to apply changes to the correct version of the
     object. Read-only.
    :type change_key: str
    :param categories: The categories associated with the item.
    :type categories: list[str]
    :param transaction_id:
    :type transaction_id: str
    :param original_start_time_zone: The start time zone that was set when the event was created. A
     value of tzone://Microsoft/Custom indicates that a legacy custom time zone was set in desktop
     Outlook.
    :type original_start_time_zone: str
    :param original_end_time_zone: The end time zone that was set when the event was created. A
     value of tzone://Microsoft/Custom indicates that a legacy custom time zone was set in desktop
     Outlook.
    :type original_end_time_zone: str
    :param response_status: responseStatus.
    :type response_status: ~users_user.models.MicrosoftGraphResponseStatus
    :param uid:
    :type uid: str
    :param reminder_minutes_before_start: The number of minutes before the event start time that
     the reminder alert occurs.
    :type reminder_minutes_before_start: int
    :param is_reminder_on: Set to true if an alert is set to remind the user of the event.
    :type is_reminder_on: bool
    :param has_attachments: Set to true if the event has attachments.
    :type has_attachments: bool
    :param subject: The text of the event's subject line.
    :type subject: str
    :param body: itemBody.
    :type body: ~users_user.models.MicrosoftGraphItemBody
    :param body_preview: The preview of the message associated with the event. It is in text
     format.
    :type body_preview: str
    :param importance: importance. Possible values include: "low", "normal", "high".
    :type importance: str or ~users_user.models.MicrosoftGraphImportance
    :param sensitivity: sensitivity. Possible values include: "normal", "personal", "private",
     "confidential".
    :type sensitivity: str or ~users_user.models.MicrosoftGraphSensitivity
    :param start: dateTimeTimeZone.
    :type start: ~users_user.models.MicrosoftGraphDateTimeZone
    :param original_start: The Timestamp type represents date and time information using ISO 8601
     format and is always in UTC time. For example, midnight UTC on Jan 1, 2014 would look like
     this: '2014-01-01T00:00:00Z'.
    :type original_start: ~datetime.datetime
    :param end: dateTimeTimeZone.
    :type end: ~users_user.models.MicrosoftGraphDateTimeZone
    :param location: location.
    :type location: ~users_user.models.MicrosoftGraphLocation
    :param locations: The locations where the event is held or attended from. The location and
     locations properties always correspond with each other. If you update the location property,
     any prior locations in the locations collection would be removed and replaced by the new
     location value.
    :type locations: list[~users_user.models.MicrosoftGraphLocation]
    :param is_all_day: Set to true if the event lasts all day.
    :type is_all_day: bool
    :param is_cancelled: Set to true if the event has been canceled.
    :type is_cancelled: bool
    :param is_organizer: Set to true if the calendar owner (specified by the owner property of the
     calendar) is the organizer of the event (specified by the organizer property of the event).
     This also applies if a delegate organized the event on behalf of the owner.
    :type is_organizer: bool
    :param recurrence: patternedRecurrence.
    :type recurrence: ~users_user.models.MicrosoftGraphPatternedRecurrence
    :param response_requested: Set to true if the sender would like a response when the event is
     accepted or declined.
    :type response_requested: bool
    :param series_master_id: The ID for the recurring series master item, if this event is part of
     a recurring series.
    :type series_master_id: str
    :param show_as: freeBusyStatus. Possible values include: "free", "tentative", "busy", "oof",
     "workingElsewhere", "unknown".
    :type show_as: str or ~users_user.models.MicrosoftGraphFreeBusyStatus
    :param type: eventType. Possible values include: "singleInstance", "occurrence", "exception",
     "seriesMaster".
    :type type: str or ~users_user.models.MicrosoftGraphEventType
    :param attendees: The collection of attendees for the event.
    :type attendees: list[~users_user.models.MicrosoftGraphAttendee]
    :param organizer: recipient.
    :type organizer: ~users_user.models.MicrosoftGraphRecipient
    :param web_link: The URL to open the event in Outlook on the web.Outlook on the web opens the
     event in the browser if you are signed in to your mailbox. Otherwise, Outlook on the web
     prompts you to sign in.This URL can be accessed from within an iFrame.
    :type web_link: str
    :param online_meeting_url: A URL for an online meeting. The property is set only when an
     organizer specifies an event as an online meeting such as a Skype meeting. Read-only.
    :type online_meeting_url: str
    :param is_online_meeting: True if this event has online meeting information, false otherwise.
     Default is false. Optional.
    :type is_online_meeting: bool
    :param online_meeting_provider: onlineMeetingProviderType. Possible values include: "unknown",
     "skypeForBusiness", "skypeForConsumer", "teamsForBusiness".
    :type online_meeting_provider: str or
     ~users_user.models.MicrosoftGraphOnlineMeetingProviderType
    :param online_meeting: onlineMeetingInfo.
    :type online_meeting: ~users_user.models.MicrosoftGraphOnlineMeetingInfo
    :param allow_new_time_proposals:
    :type allow_new_time_proposals: bool
    :param is_draft:
    :type is_draft: bool
    :param attachments: The collection of fileAttachment and itemAttachment attachments for the
     event. Navigation property. Read-only. Nullable.
    :type attachments: list[~users_user.models.MicrosoftGraphAttachment]
    :param single_value_extended_properties: The collection of single-value extended properties
     defined for the event. Read-only. Nullable.
    :type single_value_extended_properties:
     list[~users_user.models.MicrosoftGraphSingleValueLegacyExtendedProperty]
    :param multi_value_extended_properties: The collection of multi-value extended properties
     defined for the event. Read-only. Nullable.
    :type multi_value_extended_properties:
     list[~users_user.models.MicrosoftGraphMultiValueLegacyExtendedProperty]
    :param calendar: calendar.
    :type calendar: ~users_user.models.MicrosoftGraphCalendar
    :param instances: The instances of the event. Navigation property. Read-only. Nullable.
    :type instances: list[~users_user.models.MicrosoftGraphEvent]
    :param extensions: The collection of open extensions defined for the event. Read-only.
     Nullable.
    :type extensions: list[~users_user.models.MicrosoftGraphEntity]
    """

    _validation = {
        'reminder_minutes_before_start': {'maximum': 2147483647, 'minimum': -2147483648},
    }

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'created_date_time': {'key': 'createdDateTime', 'type': 'iso-8601'},
        'last_modified_date_time': {'key': 'lastModifiedDateTime', 'type': 'iso-8601'},
        'change_key': {'key': 'changeKey', 'type': 'str'},
        'categories': {'key': 'categories', 'type': '[str]'},
        'transaction_id': {'key': 'transactionId', 'type': 'str'},
        'original_start_time_zone': {'key': 'originalStartTimeZone', 'type': 'str'},
        'original_end_time_zone': {'key': 'originalEndTimeZone', 'type': 'str'},
        'response_status': {'key': 'responseStatus', 'type': 'MicrosoftGraphResponseStatus'},
        'uid': {'key': 'uid', 'type': 'str'},
        'reminder_minutes_before_start': {'key': 'reminderMinutesBeforeStart', 'type': 'int'},
        'is_reminder_on': {'key': 'isReminderOn', 'type': 'bool'},
        'has_attachments': {'key': 'hasAttachments', 'type': 'bool'},
        'subject': {'key': 'subject', 'type': 'str'},
        'body': {'key': 'body', 'type': 'MicrosoftGraphItemBody'},
        'body_preview': {'key': 'bodyPreview', 'type': 'str'},
        'importance': {'key': 'importance', 'type': 'str'},
        'sensitivity': {'key': 'sensitivity', 'type': 'str'},
        'start': {'key': 'start', 'type': 'MicrosoftGraphDateTimeZone'},
        'original_start': {'key': 'originalStart', 'type': 'iso-8601'},
        'end': {'key': 'end', 'type': 'MicrosoftGraphDateTimeZone'},
        'location': {'key': 'location', 'type': 'MicrosoftGraphLocation'},
        'locations': {'key': 'locations', 'type': '[MicrosoftGraphLocation]'},
        'is_all_day': {'key': 'isAllDay', 'type': 'bool'},
        'is_cancelled': {'key': 'isCancelled', 'type': 'bool'},
        'is_organizer': {'key': 'isOrganizer', 'type': 'bool'},
        'recurrence': {'key': 'recurrence', 'type': 'MicrosoftGraphPatternedRecurrence'},
        'response_requested': {'key': 'responseRequested', 'type': 'bool'},
        'series_master_id': {'key': 'seriesMasterId', 'type': 'str'},
        'show_as': {'key': 'showAs', 'type': 'str'},
        'type': {'key': 'type', 'type': 'str'},
        'attendees': {'key': 'attendees', 'type': '[MicrosoftGraphAttendee]'},
        'organizer': {'key': 'organizer', 'type': 'MicrosoftGraphRecipient'},
        'web_link': {'key': 'webLink', 'type': 'str'},
        'online_meeting_url': {'key': 'onlineMeetingUrl', 'type': 'str'},
        'is_online_meeting': {'key': 'isOnlineMeeting', 'type': 'bool'},
        'online_meeting_provider': {'key': 'onlineMeetingProvider', 'type': 'str'},
        'online_meeting': {'key': 'onlineMeeting', 'type': 'MicrosoftGraphOnlineMeetingInfo'},
        'allow_new_time_proposals': {'key': 'allowNewTimeProposals', 'type': 'bool'},
        'is_draft': {'key': 'isDraft', 'type': 'bool'},
        'attachments': {'key': 'attachments', 'type': '[MicrosoftGraphAttachment]'},
        'single_value_extended_properties': {'key': 'singleValueExtendedProperties', 'type': '[MicrosoftGraphSingleValueLegacyExtendedProperty]'},
        'multi_value_extended_properties': {'key': 'multiValueExtendedProperties', 'type': '[MicrosoftGraphMultiValueLegacyExtendedProperty]'},
        'calendar': {'key': 'calendar', 'type': 'MicrosoftGraphCalendar'},
        'instances': {'key': 'instances', 'type': '[MicrosoftGraphEvent]'},
        'extensions': {'key': 'extensions', 'type': '[MicrosoftGraphEntity]'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphEvent, self).__init__(**kwargs)
        self.transaction_id = kwargs.get('transaction_id', None)
        self.original_start_time_zone = kwargs.get('original_start_time_zone', None)
        self.original_end_time_zone = kwargs.get('original_end_time_zone', None)
        self.response_status = kwargs.get('response_status', None)
        self.uid = kwargs.get('uid', None)
        self.reminder_minutes_before_start = kwargs.get('reminder_minutes_before_start', None)
        self.is_reminder_on = kwargs.get('is_reminder_on', None)
        self.has_attachments = kwargs.get('has_attachments', None)
        self.subject = kwargs.get('subject', None)
        self.body = kwargs.get('body', None)
        self.body_preview = kwargs.get('body_preview', None)
        self.importance = kwargs.get('importance', None)
        self.sensitivity = kwargs.get('sensitivity', None)
        self.start = kwargs.get('start', None)
        self.original_start = kwargs.get('original_start', None)
        self.end = kwargs.get('end', None)
        self.location = kwargs.get('location', None)
        self.locations = kwargs.get('locations', None)
        self.is_all_day = kwargs.get('is_all_day', None)
        self.is_cancelled = kwargs.get('is_cancelled', None)
        self.is_organizer = kwargs.get('is_organizer', None)
        self.recurrence = kwargs.get('recurrence', None)
        self.response_requested = kwargs.get('response_requested', None)
        self.series_master_id = kwargs.get('series_master_id', None)
        self.show_as = kwargs.get('show_as', None)
        self.type = kwargs.get('type', None)
        self.attendees = kwargs.get('attendees', None)
        self.organizer = kwargs.get('organizer', None)
        self.web_link = kwargs.get('web_link', None)
        self.online_meeting_url = kwargs.get('online_meeting_url', None)
        self.is_online_meeting = kwargs.get('is_online_meeting', None)
        self.online_meeting_provider = kwargs.get('online_meeting_provider', None)
        self.online_meeting = kwargs.get('online_meeting', None)
        self.allow_new_time_proposals = kwargs.get('allow_new_time_proposals', None)
        self.is_draft = kwargs.get('is_draft', None)
        self.attachments = kwargs.get('attachments', None)
        self.single_value_extended_properties = kwargs.get('single_value_extended_properties', None)
        self.multi_value_extended_properties = kwargs.get('multi_value_extended_properties', None)
        self.calendar = kwargs.get('calendar', None)
        self.instances = kwargs.get('instances', None)
        self.extensions = kwargs.get('extensions', None)


class MicrosoftGraphExtension(MicrosoftGraphEntity):
    """extension.

    :param id: Read-only.
    :type id: str
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphExtension, self).__init__(**kwargs)


class MicrosoftGraphExternalLink(msrest.serialization.Model):
    """externalLink.

    :param href: The url of the link.
    :type href: str
    """

    _attribute_map = {
        'href': {'key': 'href', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphExternalLink, self).__init__(**kwargs)
        self.href = kwargs.get('href', None)


class MicrosoftGraphFieldValueSet(MicrosoftGraphEntity):
    """fieldValueSet.

    :param id: Read-only.
    :type id: str
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphFieldValueSet, self).__init__(**kwargs)


class MicrosoftGraphFile(msrest.serialization.Model):
    """file.

    :param hashes: hashes.
    :type hashes: ~users_user.models.MicrosoftGraphHashes
    :param mime_type: The MIME type for the file. This is determined by logic on the server and
     might not be the value provided when the file was uploaded. Read-only.
    :type mime_type: str
    :param processing_metadata:
    :type processing_metadata: bool
    """

    _attribute_map = {
        'hashes': {'key': 'hashes', 'type': 'MicrosoftGraphHashes'},
        'mime_type': {'key': 'mimeType', 'type': 'str'},
        'processing_metadata': {'key': 'processingMetadata', 'type': 'bool'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphFile, self).__init__(**kwargs)
        self.hashes = kwargs.get('hashes', None)
        self.mime_type = kwargs.get('mime_type', None)
        self.processing_metadata = kwargs.get('processing_metadata', None)


class MicrosoftGraphFileSystemInfo(msrest.serialization.Model):
    """fileSystemInfo.

    :param created_date_time: The UTC date and time the file was created on a client.
    :type created_date_time: ~datetime.datetime
    :param last_accessed_date_time: The UTC date and time the file was last accessed. Available for
     the recent file list only.
    :type last_accessed_date_time: ~datetime.datetime
    :param last_modified_date_time: The UTC date and time the file was last modified on a client.
    :type last_modified_date_time: ~datetime.datetime
    """

    _attribute_map = {
        'created_date_time': {'key': 'createdDateTime', 'type': 'iso-8601'},
        'last_accessed_date_time': {'key': 'lastAccessedDateTime', 'type': 'iso-8601'},
        'last_modified_date_time': {'key': 'lastModifiedDateTime', 'type': 'iso-8601'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphFileSystemInfo, self).__init__(**kwargs)
        self.created_date_time = kwargs.get('created_date_time', None)
        self.last_accessed_date_time = kwargs.get('last_accessed_date_time', None)
        self.last_modified_date_time = kwargs.get('last_modified_date_time', None)


class MicrosoftGraphFolder(msrest.serialization.Model):
    """folder.

    :param child_count: Number of children contained immediately within this container.
    :type child_count: int
    :param view: folderView.
    :type view: ~users_user.models.MicrosoftGraphFolderView
    """

    _validation = {
        'child_count': {'maximum': 2147483647, 'minimum': -2147483648},
    }

    _attribute_map = {
        'child_count': {'key': 'childCount', 'type': 'int'},
        'view': {'key': 'view', 'type': 'MicrosoftGraphFolderView'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphFolder, self).__init__(**kwargs)
        self.child_count = kwargs.get('child_count', None)
        self.view = kwargs.get('view', None)


class MicrosoftGraphFolderView(msrest.serialization.Model):
    """folderView.

    :param sort_by: The method by which the folder should be sorted.
    :type sort_by: str
    :param sort_order: If true, indicates that items should be sorted in descending order.
     Otherwise, items should be sorted ascending.
    :type sort_order: str
    :param view_type: The type of view that should be used to represent the folder.
    :type view_type: str
    """

    _attribute_map = {
        'sort_by': {'key': 'sortBy', 'type': 'str'},
        'sort_order': {'key': 'sortOrder', 'type': 'str'},
        'view_type': {'key': 'viewType', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphFolderView, self).__init__(**kwargs)
        self.sort_by = kwargs.get('sort_by', None)
        self.sort_order = kwargs.get('sort_order', None)
        self.view_type = kwargs.get('view_type', None)


class MicrosoftGraphFollowupFlag(msrest.serialization.Model):
    """followupFlag.

    :param completed_date_time: dateTimeTimeZone.
    :type completed_date_time: ~users_user.models.MicrosoftGraphDateTimeZone
    :param due_date_time: dateTimeTimeZone.
    :type due_date_time: ~users_user.models.MicrosoftGraphDateTimeZone
    :param start_date_time: dateTimeTimeZone.
    :type start_date_time: ~users_user.models.MicrosoftGraphDateTimeZone
    :param flag_status: followupFlagStatus. Possible values include: "notFlagged", "complete",
     "flagged".
    :type flag_status: str or ~users_user.models.MicrosoftGraphFollowupFlagStatus
    """

    _attribute_map = {
        'completed_date_time': {'key': 'completedDateTime', 'type': 'MicrosoftGraphDateTimeZone'},
        'due_date_time': {'key': 'dueDateTime', 'type': 'MicrosoftGraphDateTimeZone'},
        'start_date_time': {'key': 'startDateTime', 'type': 'MicrosoftGraphDateTimeZone'},
        'flag_status': {'key': 'flagStatus', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphFollowupFlag, self).__init__(**kwargs)
        self.completed_date_time = kwargs.get('completed_date_time', None)
        self.due_date_time = kwargs.get('due_date_time', None)
        self.start_date_time = kwargs.get('start_date_time', None)
        self.flag_status = kwargs.get('flag_status', None)


class MicrosoftGraphGeoCoordinates(msrest.serialization.Model):
    """geoCoordinates.

    :param altitude: Optional. The altitude (height), in feet,  above sea level for the item. Read-
     only.
    :type altitude: float
    :param latitude: Optional. The latitude, in decimal, for the item. Read-only.
    :type latitude: float
    :param longitude: Optional. The longitude, in decimal, for the item. Read-only.
    :type longitude: float
    """

    _attribute_map = {
        'altitude': {'key': 'altitude', 'type': 'float'},
        'latitude': {'key': 'latitude', 'type': 'float'},
        'longitude': {'key': 'longitude', 'type': 'float'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphGeoCoordinates, self).__init__(**kwargs)
        self.altitude = kwargs.get('altitude', None)
        self.latitude = kwargs.get('latitude', None)
        self.longitude = kwargs.get('longitude', None)


class MicrosoftGraphGroup(MicrosoftGraphDirectoryObject):
    """Represents an Azure Active Directory object. The directoryObject type is the base type for many other directory entity types.

    :param id: Read-only.
    :type id: str
    :param deleted_date_time:
    :type deleted_date_time: ~datetime.datetime
    :param assigned_labels:
    :type assigned_labels: list[~users_user.models.MicrosoftGraphAssignedLabel]
    :param assigned_licenses: The licenses that are assigned to the group. Returned only on
     $select. Read-only.
    :type assigned_licenses: list[~users_user.models.MicrosoftGraphAssignedLicense]
    :param classification: Describes a classification for the group (such as low, medium or high
     business impact). Valid values for this property are defined by creating a ClassificationList
     setting value, based on the template definition.Returned by default.
    :type classification: str
    :param created_by_app_id:
    :type created_by_app_id: str
    :param created_date_time: Timestamp of when the group was created. The value cannot be modified
     and is automatically populated when the group is created. The Timestamp type represents date
     and time information using ISO 8601 format and is always in UTC time. For example, midnight UTC
     on Jan 1, 2014 would look like this: '2014-01-01T00:00:00Z'. Returned by default. Read-only.
    :type created_date_time: ~datetime.datetime
    :param description: An optional description for the group. Returned by default.
    :type description: str
    :param display_name: The display name for the group. This property is required when a group is
     created and cannot be cleared during updates. Returned by default. Supports $filter and
     $orderby.
    :type display_name: str
    :param expiration_date_time:
    :type expiration_date_time: ~datetime.datetime
    :param group_types: Specifies the group type and its membership.  If the collection contains
     Unified then the group is an Office 365 group; otherwise it's a security group.  If the
     collection includes DynamicMembership, the group has dynamic membership; otherwise, membership
     is static.  Returned by default. Supports $filter.
    :type group_types: list[str]
    :param has_members_with_license_errors: Indicates whether there are members in this group that
     have license errors from its group-based license assignment. This property is never returned on
     a GET operation. You can use it as a $filter argument to get groups that have members with
     license errors (that is, filter for this property being true). See an example.
    :type has_members_with_license_errors: bool
    :param is_assignable_to_role:
    :type is_assignable_to_role: bool
    :param license_processing_state: licenseProcessingState.
    :type license_processing_state: ~users_user.models.MicrosoftGraphLicenseProcessingState
    :param mail: The SMTP address for the group, for example,
     'serviceadmins@contoso.onmicrosoft.com'. Returned by default. Read-only. Supports $filter.
    :type mail: str
    :param mail_enabled: Specifies whether the group is mail-enabled. Returned by default.
    :type mail_enabled: bool
    :param mail_nickname: The mail alias for the group, unique in the organization. This property
     must be specified when a group is created. Returned by default. Supports $filter.
    :type mail_nickname: str
    :param mdm_app_id:
    :type mdm_app_id: str
    :param membership_rule:
    :type membership_rule: str
    :param membership_rule_processing_state:
    :type membership_rule_processing_state: str
    :param on_premises_domain_name:
    :type on_premises_domain_name: str
    :param on_premises_last_sync_date_time: Indicates the last time at which the group was synced
     with the on-premises directory.The Timestamp type represents date and time information using
     ISO 8601 format and is always in UTC time. For example, midnight UTC on Jan 1, 2014 would look
     like this: '2014-01-01T00:00:00Z'. Returned by default. Read-only. Supports $filter.
    :type on_premises_last_sync_date_time: ~datetime.datetime
    :param on_premises_net_bios_name:
    :type on_premises_net_bios_name: str
    :param on_premises_provisioning_errors: Errors when using Microsoft synchronization product
     during provisioning. Returned by default.
    :type on_premises_provisioning_errors:
     list[~users_user.models.MicrosoftGraphOnPremisesProvisioningError]
    :param on_premises_sam_account_name:
    :type on_premises_sam_account_name: str
    :param on_premises_security_identifier: Contains the on-premises security identifier (SID) for
     the group that was synchronized from on-premises to the cloud. Returned by default. Read-only.
    :type on_premises_security_identifier: str
    :param on_premises_sync_enabled: true if this group is synced from an on-premises directory;
     false if this group was originally synced from an on-premises directory but is no longer
     synced; null if this object has never been synced from an on-premises directory (default).
     Returned by default. Read-only. Supports $filter.
    :type on_premises_sync_enabled: bool
    :param preferred_data_location: The preferred data location for the group. For more
     information, see  OneDrive Online Multi-Geo. Returned by default.
    :type preferred_data_location: str
    :param preferred_language:
    :type preferred_language: str
    :param proxy_addresses: Email addresses for the group that direct to the same group mailbox.
     For example: ['SMTP: bob@contoso.com', 'smtp: bob@sales.contoso.com']. The any operator is
     required to filter expressions on multi-valued properties. Returned by default. Read-only. Not
     nullable. Supports $filter.
    :type proxy_addresses: list[str]
    :param renewed_date_time: Timestamp of when the group was last renewed. This cannot be modified
     directly and is only updated via the renew service action. The Timestamp type represents date
     and time information using ISO 8601 format and is always in UTC time. For example, midnight UTC
     on Jan 1, 2014 would look like this: '2014-01-01T00:00:00Z'. Returned by default. Read-only.
    :type renewed_date_time: ~datetime.datetime
    :param resource_behavior_options:
    :type resource_behavior_options: list[str]
    :param resource_provisioning_options:
    :type resource_provisioning_options: list[str]
    :param security_enabled: Specifies whether the group is a security group. Returned by default.
     Supports $filter.
    :type security_enabled: bool
    :param security_identifier: Security identifier of the group, used in Windows scenarios.
     Returned by default.
    :type security_identifier: str
    :param theme:
    :type theme: str
    :param visibility: Specifies the visibility of an Office 365 group. Possible values are:
     Private, Public, or Hiddenmembership; blank values are treated as public.  See group visibility
     options to learn more.Visibility can be set only when a group is created; it is not
     editable.Visibility is supported only for unified groups; it is not supported for security
     groups. Returned by default.
    :type visibility: str
    :param access_type: groupAccessType. Possible values include: "none", "private", "secret",
     "public".
    :type access_type: str or ~users_user.models.MicrosoftGraphGroupAccessType
    :param allow_external_senders: Indicates if people external to the organization can send
     messages to the group. Default value is false. Returned only on $select.
    :type allow_external_senders: bool
    :param auto_subscribe_new_members: Indicates if new members added to the group will be auto-
     subscribed to receive email notifications. You can set this property in a PATCH request for the
     group; do not set it in the initial POST request that creates the group. Default value is
     false. Returned only on $select.
    :type auto_subscribe_new_members: bool
    :param is_favorite:
    :type is_favorite: bool
    :param is_subscribed_by_mail: Indicates whether the signed-in user is subscribed to receive
     email conversations. Default value is true. Returned only on $select.
    :type is_subscribed_by_mail: bool
    :param unseen_count: Count of conversations that have received new posts since the signed-in
     user last visited the group. Returned only on $select.
    :type unseen_count: int
    :param unseen_conversations_count:
    :type unseen_conversations_count: int
    :param unseen_messages_count:
    :type unseen_messages_count: int
    :param hide_from_outlook_clients: True if the group is not displayed in Outlook clients, such
     as Outlook for Windows and Outlook on the web; otherwise, false. Default value is false.
     Returned only on $select.
    :type hide_from_outlook_clients: bool
    :param hide_from_address_lists: True if the group is not displayed in certain parts of the
     Outlook UI: the Address Book, address lists for selecting message recipients, and the Browse
     Groups dialog for searching groups; otherwise, false. Default value is false. Returned only on
     $select.
    :type hide_from_address_lists: bool
    :param is_archived:
    :type is_archived: bool
    :param app_role_assignments:
    :type app_role_assignments: list[~users_user.models.MicrosoftGraphAppRoleAssignment]
    :param members: Users and groups that are members of this group. HTTP Methods: GET (supported
     for all groups), POST (supported for Office 365 groups, security groups and mail-enabled
     security groups), DELETE (supported for Office 365 groups and security groups) Nullable.
    :type members: list[~users_user.models.MicrosoftGraphDirectoryObject]
    :param members_with_license_errors: A list of group members with license errors from this
     group-based license assignment. Read-only.
    :type members_with_license_errors: list[~users_user.models.MicrosoftGraphDirectoryObject]
    :param member_of: Groups that this group is a member of. HTTP Methods: GET (supported for all
     groups). Read-only. Nullable.
    :type member_of: list[~users_user.models.MicrosoftGraphDirectoryObject]
    :param transitive_members:
    :type transitive_members: list[~users_user.models.MicrosoftGraphDirectoryObject]
    :param transitive_member_of:
    :type transitive_member_of: list[~users_user.models.MicrosoftGraphDirectoryObject]
    :param created_on_behalf_of: Represents an Azure Active Directory object. The directoryObject
     type is the base type for many other directory entity types.
    :type created_on_behalf_of: ~users_user.models.MicrosoftGraphDirectoryObject
    :param owners: The owners of the group. The owners are a set of non-admin users who are allowed
     to modify this object. Limited to 100 owners. HTTP Methods: GET (supported for all groups),
     POST (supported for Office 365 groups, security groups and mail-enabled security groups),
     DELETE (supported for Office 365 groups and security groups). Nullable.
    :type owners: list[~users_user.models.MicrosoftGraphDirectoryObject]
    :param settings: Read-only. Nullable.
    :type settings: list[~users_user.models.MicrosoftGraphDirectorySetting]
    :param endpoints:
    :type endpoints: list[~users_user.models.MicrosoftGraphEndpoint]
    :param permission_grants:
    :type permission_grants: list[~users_user.models.MicrosoftGraphResourceSpecificPermissionGrant]
    :param conversations: The group's conversations.
    :type conversations: list[~users_user.models.MicrosoftGraphConversation]
    :param photos: The profile photos owned by the group. Read-only. Nullable.
    :type photos: list[~users_user.models.MicrosoftGraphProfilePhoto]
    :param accepted_senders: The list of users or groups that are allowed to create post's or
     calendar events in this group. If this list is non-empty then only users or groups listed here
     are allowed to post.
    :type accepted_senders: list[~users_user.models.MicrosoftGraphDirectoryObject]
    :param rejected_senders: The list of users or groups that are not allowed to create posts or
     calendar events in this group. Nullable.
    :type rejected_senders: list[~users_user.models.MicrosoftGraphDirectoryObject]
    :param threads: The group's conversation threads. Nullable.
    :type threads: list[~users_user.models.MicrosoftGraphConversationThread]
    :param calendar: calendar.
    :type calendar: ~users_user.models.MicrosoftGraphCalendar
    :param calendar_view: The calendar view for the calendar. Read-only.
    :type calendar_view: list[~users_user.models.MicrosoftGraphEvent]
    :param events: The group's calendar events.
    :type events: list[~users_user.models.MicrosoftGraphEvent]
    :param photo: profilePhoto.
    :type photo: ~users_user.models.MicrosoftGraphProfilePhoto
    :param drive: drive.
    :type drive: ~users_user.models.MicrosoftGraphDrive
    :param drives: The group's drives. Read-only.
    :type drives: list[~users_user.models.MicrosoftGraphDrive]
    :param sites: The list of SharePoint sites in this group. Access the default site with
     /sites/root.
    :type sites: list[~users_user.models.MicrosoftGraphSite]
    :param extensions: The collection of open extensions defined for the group. Read-only.
     Nullable.
    :type extensions: list[~users_user.models.MicrosoftGraphEntity]
    :param group_lifecycle_policies: The collection of lifecycle policies for this group. Read-
     only. Nullable.
    :type group_lifecycle_policies: list[~users_user.models.MicrosoftGraphGroupLifecyclePolicy]
    :param planner: plannerGroup.
    :type planner: ~users_user.models.MicrosoftGraphPlannerGroup
    :param onenote: onenote.
    :type onenote: ~users_user.models.MicrosoftGraphOnenote
    :param team: team.
    :type team: ~users_user.models.MicrosoftGraphTeam
    """

    _validation = {
        'unseen_count': {'maximum': 2147483647, 'minimum': -2147483648},
        'unseen_conversations_count': {'maximum': 2147483647, 'minimum': -2147483648},
        'unseen_messages_count': {'maximum': 2147483647, 'minimum': -2147483648},
    }

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'deleted_date_time': {'key': 'deletedDateTime', 'type': 'iso-8601'},
        'assigned_labels': {'key': 'assignedLabels', 'type': '[MicrosoftGraphAssignedLabel]'},
        'assigned_licenses': {'key': 'assignedLicenses', 'type': '[MicrosoftGraphAssignedLicense]'},
        'classification': {'key': 'classification', 'type': 'str'},
        'created_by_app_id': {'key': 'createdByAppId', 'type': 'str'},
        'created_date_time': {'key': 'createdDateTime', 'type': 'iso-8601'},
        'description': {'key': 'description', 'type': 'str'},
        'display_name': {'key': 'displayName', 'type': 'str'},
        'expiration_date_time': {'key': 'expirationDateTime', 'type': 'iso-8601'},
        'group_types': {'key': 'groupTypes', 'type': '[str]'},
        'has_members_with_license_errors': {'key': 'hasMembersWithLicenseErrors', 'type': 'bool'},
        'is_assignable_to_role': {'key': 'isAssignableToRole', 'type': 'bool'},
        'license_processing_state': {'key': 'licenseProcessingState', 'type': 'MicrosoftGraphLicenseProcessingState'},
        'mail': {'key': 'mail', 'type': 'str'},
        'mail_enabled': {'key': 'mailEnabled', 'type': 'bool'},
        'mail_nickname': {'key': 'mailNickname', 'type': 'str'},
        'mdm_app_id': {'key': 'mdmAppId', 'type': 'str'},
        'membership_rule': {'key': 'membershipRule', 'type': 'str'},
        'membership_rule_processing_state': {'key': 'membershipRuleProcessingState', 'type': 'str'},
        'on_premises_domain_name': {'key': 'onPremisesDomainName', 'type': 'str'},
        'on_premises_last_sync_date_time': {'key': 'onPremisesLastSyncDateTime', 'type': 'iso-8601'},
        'on_premises_net_bios_name': {'key': 'onPremisesNetBiosName', 'type': 'str'},
        'on_premises_provisioning_errors': {'key': 'onPremisesProvisioningErrors', 'type': '[MicrosoftGraphOnPremisesProvisioningError]'},
        'on_premises_sam_account_name': {'key': 'onPremisesSamAccountName', 'type': 'str'},
        'on_premises_security_identifier': {'key': 'onPremisesSecurityIdentifier', 'type': 'str'},
        'on_premises_sync_enabled': {'key': 'onPremisesSyncEnabled', 'type': 'bool'},
        'preferred_data_location': {'key': 'preferredDataLocation', 'type': 'str'},
        'preferred_language': {'key': 'preferredLanguage', 'type': 'str'},
        'proxy_addresses': {'key': 'proxyAddresses', 'type': '[str]'},
        'renewed_date_time': {'key': 'renewedDateTime', 'type': 'iso-8601'},
        'resource_behavior_options': {'key': 'resourceBehaviorOptions', 'type': '[str]'},
        'resource_provisioning_options': {'key': 'resourceProvisioningOptions', 'type': '[str]'},
        'security_enabled': {'key': 'securityEnabled', 'type': 'bool'},
        'security_identifier': {'key': 'securityIdentifier', 'type': 'str'},
        'theme': {'key': 'theme', 'type': 'str'},
        'visibility': {'key': 'visibility', 'type': 'str'},
        'access_type': {'key': 'accessType', 'type': 'str'},
        'allow_external_senders': {'key': 'allowExternalSenders', 'type': 'bool'},
        'auto_subscribe_new_members': {'key': 'autoSubscribeNewMembers', 'type': 'bool'},
        'is_favorite': {'key': 'isFavorite', 'type': 'bool'},
        'is_subscribed_by_mail': {'key': 'isSubscribedByMail', 'type': 'bool'},
        'unseen_count': {'key': 'unseenCount', 'type': 'int'},
        'unseen_conversations_count': {'key': 'unseenConversationsCount', 'type': 'int'},
        'unseen_messages_count': {'key': 'unseenMessagesCount', 'type': 'int'},
        'hide_from_outlook_clients': {'key': 'hideFromOutlookClients', 'type': 'bool'},
        'hide_from_address_lists': {'key': 'hideFromAddressLists', 'type': 'bool'},
        'is_archived': {'key': 'isArchived', 'type': 'bool'},
        'app_role_assignments': {'key': 'appRoleAssignments', 'type': '[MicrosoftGraphAppRoleAssignment]'},
        'members': {'key': 'members', 'type': '[MicrosoftGraphDirectoryObject]'},
        'members_with_license_errors': {'key': 'membersWithLicenseErrors', 'type': '[MicrosoftGraphDirectoryObject]'},
        'member_of': {'key': 'memberOf', 'type': '[MicrosoftGraphDirectoryObject]'},
        'transitive_members': {'key': 'transitiveMembers', 'type': '[MicrosoftGraphDirectoryObject]'},
        'transitive_member_of': {'key': 'transitiveMemberOf', 'type': '[MicrosoftGraphDirectoryObject]'},
        'created_on_behalf_of': {'key': 'createdOnBehalfOf', 'type': 'MicrosoftGraphDirectoryObject'},
        'owners': {'key': 'owners', 'type': '[MicrosoftGraphDirectoryObject]'},
        'settings': {'key': 'settings', 'type': '[MicrosoftGraphDirectorySetting]'},
        'endpoints': {'key': 'endpoints', 'type': '[MicrosoftGraphEndpoint]'},
        'permission_grants': {'key': 'permissionGrants', 'type': '[MicrosoftGraphResourceSpecificPermissionGrant]'},
        'conversations': {'key': 'conversations', 'type': '[MicrosoftGraphConversation]'},
        'photos': {'key': 'photos', 'type': '[MicrosoftGraphProfilePhoto]'},
        'accepted_senders': {'key': 'acceptedSenders', 'type': '[MicrosoftGraphDirectoryObject]'},
        'rejected_senders': {'key': 'rejectedSenders', 'type': '[MicrosoftGraphDirectoryObject]'},
        'threads': {'key': 'threads', 'type': '[MicrosoftGraphConversationThread]'},
        'calendar': {'key': 'calendar', 'type': 'MicrosoftGraphCalendar'},
        'calendar_view': {'key': 'calendarView', 'type': '[MicrosoftGraphEvent]'},
        'events': {'key': 'events', 'type': '[MicrosoftGraphEvent]'},
        'photo': {'key': 'photo', 'type': 'MicrosoftGraphProfilePhoto'},
        'drive': {'key': 'drive', 'type': 'MicrosoftGraphDrive'},
        'drives': {'key': 'drives', 'type': '[MicrosoftGraphDrive]'},
        'sites': {'key': 'sites', 'type': '[MicrosoftGraphSite]'},
        'extensions': {'key': 'extensions', 'type': '[MicrosoftGraphEntity]'},
        'group_lifecycle_policies': {'key': 'groupLifecyclePolicies', 'type': '[MicrosoftGraphGroupLifecyclePolicy]'},
        'planner': {'key': 'planner', 'type': 'MicrosoftGraphPlannerGroup'},
        'onenote': {'key': 'onenote', 'type': 'MicrosoftGraphOnenote'},
        'team': {'key': 'team', 'type': 'MicrosoftGraphTeam'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphGroup, self).__init__(**kwargs)
        self.assigned_labels = kwargs.get('assigned_labels', None)
        self.assigned_licenses = kwargs.get('assigned_licenses', None)
        self.classification = kwargs.get('classification', None)
        self.created_by_app_id = kwargs.get('created_by_app_id', None)
        self.created_date_time = kwargs.get('created_date_time', None)
        self.description = kwargs.get('description', None)
        self.display_name = kwargs.get('display_name', None)
        self.expiration_date_time = kwargs.get('expiration_date_time', None)
        self.group_types = kwargs.get('group_types', None)
        self.has_members_with_license_errors = kwargs.get('has_members_with_license_errors', None)
        self.is_assignable_to_role = kwargs.get('is_assignable_to_role', None)
        self.license_processing_state = kwargs.get('license_processing_state', None)
        self.mail = kwargs.get('mail', None)
        self.mail_enabled = kwargs.get('mail_enabled', None)
        self.mail_nickname = kwargs.get('mail_nickname', None)
        self.mdm_app_id = kwargs.get('mdm_app_id', None)
        self.membership_rule = kwargs.get('membership_rule', None)
        self.membership_rule_processing_state = kwargs.get('membership_rule_processing_state', None)
        self.on_premises_domain_name = kwargs.get('on_premises_domain_name', None)
        self.on_premises_last_sync_date_time = kwargs.get('on_premises_last_sync_date_time', None)
        self.on_premises_net_bios_name = kwargs.get('on_premises_net_bios_name', None)
        self.on_premises_provisioning_errors = kwargs.get('on_premises_provisioning_errors', None)
        self.on_premises_sam_account_name = kwargs.get('on_premises_sam_account_name', None)
        self.on_premises_security_identifier = kwargs.get('on_premises_security_identifier', None)
        self.on_premises_sync_enabled = kwargs.get('on_premises_sync_enabled', None)
        self.preferred_data_location = kwargs.get('preferred_data_location', None)
        self.preferred_language = kwargs.get('preferred_language', None)
        self.proxy_addresses = kwargs.get('proxy_addresses', None)
        self.renewed_date_time = kwargs.get('renewed_date_time', None)
        self.resource_behavior_options = kwargs.get('resource_behavior_options', None)
        self.resource_provisioning_options = kwargs.get('resource_provisioning_options', None)
        self.security_enabled = kwargs.get('security_enabled', None)
        self.security_identifier = kwargs.get('security_identifier', None)
        self.theme = kwargs.get('theme', None)
        self.visibility = kwargs.get('visibility', None)
        self.access_type = kwargs.get('access_type', None)
        self.allow_external_senders = kwargs.get('allow_external_senders', None)
        self.auto_subscribe_new_members = kwargs.get('auto_subscribe_new_members', None)
        self.is_favorite = kwargs.get('is_favorite', None)
        self.is_subscribed_by_mail = kwargs.get('is_subscribed_by_mail', None)
        self.unseen_count = kwargs.get('unseen_count', None)
        self.unseen_conversations_count = kwargs.get('unseen_conversations_count', None)
        self.unseen_messages_count = kwargs.get('unseen_messages_count', None)
        self.hide_from_outlook_clients = kwargs.get('hide_from_outlook_clients', None)
        self.hide_from_address_lists = kwargs.get('hide_from_address_lists', None)
        self.is_archived = kwargs.get('is_archived', None)
        self.app_role_assignments = kwargs.get('app_role_assignments', None)
        self.members = kwargs.get('members', None)
        self.members_with_license_errors = kwargs.get('members_with_license_errors', None)
        self.member_of = kwargs.get('member_of', None)
        self.transitive_members = kwargs.get('transitive_members', None)
        self.transitive_member_of = kwargs.get('transitive_member_of', None)
        self.created_on_behalf_of = kwargs.get('created_on_behalf_of', None)
        self.owners = kwargs.get('owners', None)
        self.settings = kwargs.get('settings', None)
        self.endpoints = kwargs.get('endpoints', None)
        self.permission_grants = kwargs.get('permission_grants', None)
        self.conversations = kwargs.get('conversations', None)
        self.photos = kwargs.get('photos', None)
        self.accepted_senders = kwargs.get('accepted_senders', None)
        self.rejected_senders = kwargs.get('rejected_senders', None)
        self.threads = kwargs.get('threads', None)
        self.calendar = kwargs.get('calendar', None)
        self.calendar_view = kwargs.get('calendar_view', None)
        self.events = kwargs.get('events', None)
        self.photo = kwargs.get('photo', None)
        self.drive = kwargs.get('drive', None)
        self.drives = kwargs.get('drives', None)
        self.sites = kwargs.get('sites', None)
        self.extensions = kwargs.get('extensions', None)
        self.group_lifecycle_policies = kwargs.get('group_lifecycle_policies', None)
        self.planner = kwargs.get('planner', None)
        self.onenote = kwargs.get('onenote', None)
        self.team = kwargs.get('team', None)


class MicrosoftGraphGroupLifecyclePolicy(MicrosoftGraphEntity):
    """groupLifecyclePolicy.

    :param id: Read-only.
    :type id: str
    :param group_lifetime_in_days: Number of days before a group expires and needs to be renewed.
     Once renewed, the group expiration is extended by the number of days defined.
    :type group_lifetime_in_days: int
    :param managed_group_types: The group type for which the expiration policy applies. Possible
     values are All, Selected or None.
    :type managed_group_types: str
    :param alternate_notification_emails: List of email address to send notifications for groups
     without owners. Multiple email address can be defined by separating email address with a
     semicolon.
    :type alternate_notification_emails: str
    """

    _validation = {
        'group_lifetime_in_days': {'maximum': 2147483647, 'minimum': -2147483648},
    }

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'group_lifetime_in_days': {'key': 'groupLifetimeInDays', 'type': 'int'},
        'managed_group_types': {'key': 'managedGroupTypes', 'type': 'str'},
        'alternate_notification_emails': {'key': 'alternateNotificationEmails', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphGroupLifecyclePolicy, self).__init__(**kwargs)
        self.group_lifetime_in_days = kwargs.get('group_lifetime_in_days', None)
        self.managed_group_types = kwargs.get('managed_group_types', None)
        self.alternate_notification_emails = kwargs.get('alternate_notification_emails', None)


class MicrosoftGraphHardwareInformation(msrest.serialization.Model):
    """hardwareInformation.

    :param serial_number: Serial number.
    :type serial_number: str
    :param total_storage_space: Total storage space of the device.
    :type total_storage_space: long
    :param free_storage_space: Free storage space of the device.
    :type free_storage_space: long
    :param imei: IMEI.
    :type imei: str
    :param meid: MEID.
    :type meid: str
    :param manufacturer: Manufacturer of the device.
    :type manufacturer: str
    :param model: Model of the device.
    :type model: str
    :param phone_number: Phone number of the device.
    :type phone_number: str
    :param subscriber_carrier: Subscriber carrier of the device.
    :type subscriber_carrier: str
    :param cellular_technology: Cellular technology of the device.
    :type cellular_technology: str
    :param wifi_mac: WiFi MAC address of the device.
    :type wifi_mac: str
    :param operating_system_language: Operating system language of the device.
    :type operating_system_language: str
    :param is_supervised: Supervised mode of the device.
    :type is_supervised: bool
    :param is_encrypted: Encryption status of the device.
    :type is_encrypted: bool
    :param is_shared_device: Shared iPad.
    :type is_shared_device: bool
    :param shared_device_cached_users: All users on the shared Apple device.
    :type shared_device_cached_users: list[~users_user.models.MicrosoftGraphSharedAppleDeviceUser]
    :param tpm_specification_version: String that specifies the specification version.
    :type tpm_specification_version: str
    :param operating_system_edition: String that specifies the OS edition.
    :type operating_system_edition: str
    :param device_full_qualified_domain_name: Returns the fully qualified domain name of the device
     (if any). If the device is not domain-joined, it returns an empty string.
    :type device_full_qualified_domain_name: str
    :param device_guard_virtualization_based_security_hardware_requirement_state:
     deviceGuardVirtualizationBasedSecurityHardwareRequirementState. Possible values include:
     "meetHardwareRequirements", "secureBootRequired", "dmaProtectionRequired",
     "hyperVNotSupportedForGuestVM", "hyperVNotAvailable".
    :type device_guard_virtualization_based_security_hardware_requirement_state: str or
     ~users_user.models.MicrosoftGraphDeviceGuardVirtualizationBasedSecurityHardwareRequirementState
    :param device_guard_virtualization_based_security_state:
     deviceGuardVirtualizationBasedSecurityState. Possible values include: "running",
     "rebootRequired", "require64BitArchitecture", "notLicensed", "notConfigured",
     "doesNotMeetHardwareRequirements", "other".
    :type device_guard_virtualization_based_security_state: str or
     ~users_user.models.MicrosoftGraphDeviceGuardVirtualizationBasedSecurityState
    :param device_guard_local_system_authority_credential_guard_state:
     deviceGuardLocalSystemAuthorityCredentialGuardState. Possible values include: "running",
     "rebootRequired", "notLicensed", "notConfigured", "virtualizationBasedSecurityNotRunning".
    :type device_guard_local_system_authority_credential_guard_state: str or
     ~users_user.models.MicrosoftGraphDeviceGuardLocalSystemAuthorityCredentialGuardState
    :param os_build_number: Operating System Build Number on Android device.
    :type os_build_number: str
    """

    _attribute_map = {
        'serial_number': {'key': 'serialNumber', 'type': 'str'},
        'total_storage_space': {'key': 'totalStorageSpace', 'type': 'long'},
        'free_storage_space': {'key': 'freeStorageSpace', 'type': 'long'},
        'imei': {'key': 'imei', 'type': 'str'},
        'meid': {'key': 'meid', 'type': 'str'},
        'manufacturer': {'key': 'manufacturer', 'type': 'str'},
        'model': {'key': 'model', 'type': 'str'},
        'phone_number': {'key': 'phoneNumber', 'type': 'str'},
        'subscriber_carrier': {'key': 'subscriberCarrier', 'type': 'str'},
        'cellular_technology': {'key': 'cellularTechnology', 'type': 'str'},
        'wifi_mac': {'key': 'wifiMac', 'type': 'str'},
        'operating_system_language': {'key': 'operatingSystemLanguage', 'type': 'str'},
        'is_supervised': {'key': 'isSupervised', 'type': 'bool'},
        'is_encrypted': {'key': 'isEncrypted', 'type': 'bool'},
        'is_shared_device': {'key': 'isSharedDevice', 'type': 'bool'},
        'shared_device_cached_users': {'key': 'sharedDeviceCachedUsers', 'type': '[MicrosoftGraphSharedAppleDeviceUser]'},
        'tpm_specification_version': {'key': 'tpmSpecificationVersion', 'type': 'str'},
        'operating_system_edition': {'key': 'operatingSystemEdition', 'type': 'str'},
        'device_full_qualified_domain_name': {'key': 'deviceFullQualifiedDomainName', 'type': 'str'},
        'device_guard_virtualization_based_security_hardware_requirement_state': {'key': 'deviceGuardVirtualizationBasedSecurityHardwareRequirementState', 'type': 'str'},
        'device_guard_virtualization_based_security_state': {'key': 'deviceGuardVirtualizationBasedSecurityState', 'type': 'str'},
        'device_guard_local_system_authority_credential_guard_state': {'key': 'deviceGuardLocalSystemAuthorityCredentialGuardState', 'type': 'str'},
        'os_build_number': {'key': 'osBuildNumber', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphHardwareInformation, self).__init__(**kwargs)
        self.serial_number = kwargs.get('serial_number', None)
        self.total_storage_space = kwargs.get('total_storage_space', None)
        self.free_storage_space = kwargs.get('free_storage_space', None)
        self.imei = kwargs.get('imei', None)
        self.meid = kwargs.get('meid', None)
        self.manufacturer = kwargs.get('manufacturer', None)
        self.model = kwargs.get('model', None)
        self.phone_number = kwargs.get('phone_number', None)
        self.subscriber_carrier = kwargs.get('subscriber_carrier', None)
        self.cellular_technology = kwargs.get('cellular_technology', None)
        self.wifi_mac = kwargs.get('wifi_mac', None)
        self.operating_system_language = kwargs.get('operating_system_language', None)
        self.is_supervised = kwargs.get('is_supervised', None)
        self.is_encrypted = kwargs.get('is_encrypted', None)
        self.is_shared_device = kwargs.get('is_shared_device', None)
        self.shared_device_cached_users = kwargs.get('shared_device_cached_users', None)
        self.tpm_specification_version = kwargs.get('tpm_specification_version', None)
        self.operating_system_edition = kwargs.get('operating_system_edition', None)
        self.device_full_qualified_domain_name = kwargs.get('device_full_qualified_domain_name', None)
        self.device_guard_virtualization_based_security_hardware_requirement_state = kwargs.get('device_guard_virtualization_based_security_hardware_requirement_state', None)
        self.device_guard_virtualization_based_security_state = kwargs.get('device_guard_virtualization_based_security_state', None)
        self.device_guard_local_system_authority_credential_guard_state = kwargs.get('device_guard_local_system_authority_credential_guard_state', None)
        self.os_build_number = kwargs.get('os_build_number', None)


class MicrosoftGraphHashes(msrest.serialization.Model):
    """hashes.

    :param crc32_hash: The CRC32 value of the file in little endian (if available). Read-only.
    :type crc32_hash: str
    :param quick_xor_hash: A proprietary hash of the file that can be used to determine if the
     contents of the file have changed (if available). Read-only.
    :type quick_xor_hash: str
    :param sha1_hash: SHA1 hash for the contents of the file (if available). Read-only.
    :type sha1_hash: str
    :param sha256_hash:
    :type sha256_hash: str
    """

    _attribute_map = {
        'crc32_hash': {'key': 'crc32Hash', 'type': 'str'},
        'quick_xor_hash': {'key': 'quickXorHash', 'type': 'str'},
        'sha1_hash': {'key': 'sha1Hash', 'type': 'str'},
        'sha256_hash': {'key': 'sha256Hash', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphHashes, self).__init__(**kwargs)
        self.crc32_hash = kwargs.get('crc32_hash', None)
        self.quick_xor_hash = kwargs.get('quick_xor_hash', None)
        self.sha1_hash = kwargs.get('sha1_hash', None)
        self.sha256_hash = kwargs.get('sha256_hash', None)


class MicrosoftGraphIdentity(msrest.serialization.Model):
    """identity.

    :param id: Unique identifier for the identity.
    :type id: str
    :param display_name: The identity's display name. Note that this may not always be available or
     up to date. For example, if a user changes their display name, the API may show the new value
     in a future response, but the items associated with the user won't show up as having changed
     when using delta.
    :type display_name: str
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'display_name': {'key': 'displayName', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphIdentity, self).__init__(**kwargs)
        self.id = kwargs.get('id', None)
        self.display_name = kwargs.get('display_name', None)


class MicrosoftGraphIdentitySet(msrest.serialization.Model):
    """identitySet.

    :param application: identity.
    :type application: ~users_user.models.MicrosoftGraphIdentity
    :param device: identity.
    :type device: ~users_user.models.MicrosoftGraphIdentity
    :param user: identity.
    :type user: ~users_user.models.MicrosoftGraphIdentity
    """

    _attribute_map = {
        'application': {'key': 'application', 'type': 'MicrosoftGraphIdentity'},
        'device': {'key': 'device', 'type': 'MicrosoftGraphIdentity'},
        'user': {'key': 'user', 'type': 'MicrosoftGraphIdentity'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphIdentitySet, self).__init__(**kwargs)
        self.application = kwargs.get('application', None)
        self.device = kwargs.get('device', None)
        self.user = kwargs.get('user', None)


class MicrosoftGraphIdentityUserRisk(msrest.serialization.Model):
    """identityUserRisk.

    :param level: userRiskLevel. Possible values include: "unknown", "none", "low", "medium",
     "high".
    :type level: str or ~users_user.models.MicrosoftGraphUserRiskLevel
    :param last_changed_date_time:
    :type last_changed_date_time: ~datetime.datetime
    """

    _attribute_map = {
        'level': {'key': 'level', 'type': 'str'},
        'last_changed_date_time': {'key': 'lastChangedDateTime', 'type': 'iso-8601'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphIdentityUserRisk, self).__init__(**kwargs)
        self.level = kwargs.get('level', None)
        self.last_changed_date_time = kwargs.get('last_changed_date_time', None)


class MicrosoftGraphImage(msrest.serialization.Model):
    """image.

    :param height: Optional. Height of the image, in pixels. Read-only.
    :type height: int
    :param width: Optional. Width of the image, in pixels. Read-only.
    :type width: int
    """

    _validation = {
        'height': {'maximum': 2147483647, 'minimum': -2147483648},
        'width': {'maximum': 2147483647, 'minimum': -2147483648},
    }

    _attribute_map = {
        'height': {'key': 'height', 'type': 'int'},
        'width': {'key': 'width', 'type': 'int'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphImage, self).__init__(**kwargs)
        self.height = kwargs.get('height', None)
        self.width = kwargs.get('width', None)


class MicrosoftGraphImageInfo(msrest.serialization.Model):
    """imageInfo.

    :param icon_url: Optional; URI that points to an icon which represents the application used to
     generate the activity.
    :type icon_url: str
    :param alternative_text:
    :type alternative_text: str
    :param alternate_text: Optional; alt-text accessible content for the image.
    :type alternate_text: str
    :param add_image_query: Optional; parameter used to indicate the server is able to render image
     dynamically in response to parameterization. For example – a high contrast image.
    :type add_image_query: bool
    """

    _attribute_map = {
        'icon_url': {'key': 'iconUrl', 'type': 'str'},
        'alternative_text': {'key': 'alternativeText', 'type': 'str'},
        'alternate_text': {'key': 'alternateText', 'type': 'str'},
        'add_image_query': {'key': 'addImageQuery', 'type': 'bool'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphImageInfo, self).__init__(**kwargs)
        self.icon_url = kwargs.get('icon_url', None)
        self.alternative_text = kwargs.get('alternative_text', None)
        self.alternate_text = kwargs.get('alternate_text', None)
        self.add_image_query = kwargs.get('add_image_query', None)


class MicrosoftGraphIncompleteData(msrest.serialization.Model):
    """incompleteData.

    :param missing_data_before_date_time: The service does not have source data before the
     specified time.
    :type missing_data_before_date_time: ~datetime.datetime
    :param was_throttled: Some data was not recorded due to excessive activity.
    :type was_throttled: bool
    """

    _attribute_map = {
        'missing_data_before_date_time': {'key': 'missingDataBeforeDateTime', 'type': 'iso-8601'},
        'was_throttled': {'key': 'wasThrottled', 'type': 'bool'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphIncompleteData, self).__init__(**kwargs)
        self.missing_data_before_date_time = kwargs.get('missing_data_before_date_time', None)
        self.was_throttled = kwargs.get('was_throttled', None)


class MicrosoftGraphInferenceClassification(MicrosoftGraphEntity):
    """inferenceClassification.

    :param id: Read-only.
    :type id: str
    :param overrides: A set of overrides for a user to always classify messages from specific
     senders in certain ways: focused, or other. Read-only. Nullable.
    :type overrides: list[~users_user.models.MicrosoftGraphInferenceClassificationOverride]
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'overrides': {'key': 'overrides', 'type': '[MicrosoftGraphInferenceClassificationOverride]'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphInferenceClassification, self).__init__(**kwargs)
        self.overrides = kwargs.get('overrides', None)


class MicrosoftGraphInferenceClassificationOverride(MicrosoftGraphEntity):
    """inferenceClassificationOverride.

    :param id: Read-only.
    :type id: str
    :param classify_as: inferenceClassificationType. Possible values include: "focused", "other".
    :type classify_as: str or ~users_user.models.MicrosoftGraphInferenceClassificationType
    :param sender_email_address: emailAddress.
    :type sender_email_address: ~users_user.models.MicrosoftGraphEmailAddress
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'classify_as': {'key': 'classifyAs', 'type': 'str'},
        'sender_email_address': {'key': 'senderEmailAddress', 'type': 'MicrosoftGraphEmailAddress'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphInferenceClassificationOverride, self).__init__(**kwargs)
        self.classify_as = kwargs.get('classify_as', None)
        self.sender_email_address = kwargs.get('sender_email_address', None)


class MicrosoftGraphInferenceData(msrest.serialization.Model):
    """inferenceData.

    :param confidence_score:
    :type confidence_score: float
    :param user_has_verified_accuracy:
    :type user_has_verified_accuracy: bool
    """

    _attribute_map = {
        'confidence_score': {'key': 'confidenceScore', 'type': 'float'},
        'user_has_verified_accuracy': {'key': 'userHasVerifiedAccuracy', 'type': 'bool'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphInferenceData, self).__init__(**kwargs)
        self.confidence_score = kwargs.get('confidence_score', None)
        self.user_has_verified_accuracy = kwargs.get('user_has_verified_accuracy', None)


class MicrosoftGraphInformationProtection(MicrosoftGraphEntity):
    """informationProtection.

    :param id: Read-only.
    :type id: str
    :param policy: informationProtectionPolicy.
    :type policy: ~users_user.models.MicrosoftGraphInformationProtectionPolicy
    :param sensitivity_labels:
    :type sensitivity_labels: list[~users_user.models.MicrosoftGraphSensitivityLabel]
    :param sensitivity_policy_settings: sensitivityPolicySettings.
    :type sensitivity_policy_settings: ~users_user.models.MicrosoftGraphSensitivityPolicySettings
    :param data_loss_prevention_policies:
    :type data_loss_prevention_policies:
     list[~users_user.models.MicrosoftGraphDataLossPreventionPolicy]
    :param threat_assessment_requests:
    :type threat_assessment_requests:
     list[~users_user.models.MicrosoftGraphThreatAssessmentRequest]
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'policy': {'key': 'policy', 'type': 'MicrosoftGraphInformationProtectionPolicy'},
        'sensitivity_labels': {'key': 'sensitivityLabels', 'type': '[MicrosoftGraphSensitivityLabel]'},
        'sensitivity_policy_settings': {'key': 'sensitivityPolicySettings', 'type': 'MicrosoftGraphSensitivityPolicySettings'},
        'data_loss_prevention_policies': {'key': 'dataLossPreventionPolicies', 'type': '[MicrosoftGraphDataLossPreventionPolicy]'},
        'threat_assessment_requests': {'key': 'threatAssessmentRequests', 'type': '[MicrosoftGraphThreatAssessmentRequest]'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphInformationProtection, self).__init__(**kwargs)
        self.policy = kwargs.get('policy', None)
        self.sensitivity_labels = kwargs.get('sensitivity_labels', None)
        self.sensitivity_policy_settings = kwargs.get('sensitivity_policy_settings', None)
        self.data_loss_prevention_policies = kwargs.get('data_loss_prevention_policies', None)
        self.threat_assessment_requests = kwargs.get('threat_assessment_requests', None)


class MicrosoftGraphInformationProtectionLabel(MicrosoftGraphEntity):
    """informationProtectionLabel.

    :param id: Read-only.
    :type id: str
    :param name:
    :type name: str
    :param description:
    :type description: str
    :param color:
    :type color: str
    :param sensitivity:
    :type sensitivity: int
    :param tooltip:
    :type tooltip: str
    :param is_active:
    :type is_active: bool
    """

    _validation = {
        'sensitivity': {'maximum': 2147483647, 'minimum': -2147483648},
    }

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'name': {'key': 'name', 'type': 'str'},
        'description': {'key': 'description', 'type': 'str'},
        'color': {'key': 'color', 'type': 'str'},
        'sensitivity': {'key': 'sensitivity', 'type': 'int'},
        'tooltip': {'key': 'tooltip', 'type': 'str'},
        'is_active': {'key': 'isActive', 'type': 'bool'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphInformationProtectionLabel, self).__init__(**kwargs)
        self.name = kwargs.get('name', None)
        self.description = kwargs.get('description', None)
        self.color = kwargs.get('color', None)
        self.sensitivity = kwargs.get('sensitivity', None)
        self.tooltip = kwargs.get('tooltip', None)
        self.is_active = kwargs.get('is_active', None)


class MicrosoftGraphInformationProtectionPolicy(MicrosoftGraphEntity):
    """informationProtectionPolicy.

    :param id: Read-only.
    :type id: str
    :param labels:
    :type labels: list[~users_user.models.MicrosoftGraphInformationProtectionLabel]
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'labels': {'key': 'labels', 'type': '[MicrosoftGraphInformationProtectionLabel]'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphInformationProtectionPolicy, self).__init__(**kwargs)
        self.labels = kwargs.get('labels', None)


class MicrosoftGraphInsightIdentity(msrest.serialization.Model):
    """insightIdentity.

    :param display_name: The display name of the user who shared the item.
    :type display_name: str
    :param id: The id of the user who shared the item.
    :type id: str
    :param address: The email address of the user who shared the item.
    :type address: str
    """

    _attribute_map = {
        'display_name': {'key': 'displayName', 'type': 'str'},
        'id': {'key': 'id', 'type': 'str'},
        'address': {'key': 'address', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphInsightIdentity, self).__init__(**kwargs)
        self.display_name = kwargs.get('display_name', None)
        self.id = kwargs.get('id', None)
        self.address = kwargs.get('address', None)


class MicrosoftGraphInstitutionData(msrest.serialization.Model):
    """institutionData.

    :param description:
    :type description: str
    :param display_name:
    :type display_name: str
    :param location: physicalAddress.
    :type location: ~users_user.models.MicrosoftGraphPhysicalAddress
    :param web_url:
    :type web_url: str
    """

    _attribute_map = {
        'description': {'key': 'description', 'type': 'str'},
        'display_name': {'key': 'displayName', 'type': 'str'},
        'location': {'key': 'location', 'type': 'MicrosoftGraphPhysicalAddress'},
        'web_url': {'key': 'webUrl', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphInstitutionData, self).__init__(**kwargs)
        self.description = kwargs.get('description', None)
        self.display_name = kwargs.get('display_name', None)
        self.location = kwargs.get('location', None)
        self.web_url = kwargs.get('web_url', None)


class MicrosoftGraphInternetMessageHeader(msrest.serialization.Model):
    """internetMessageHeader.

    :param name: Represents the key in a key-value pair.
    :type name: str
    :param value: The value in a key-value pair.
    :type value: str
    """

    _attribute_map = {
        'name': {'key': 'name', 'type': 'str'},
        'value': {'key': 'value', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphInternetMessageHeader, self).__init__(**kwargs)
        self.name = kwargs.get('name', None)
        self.value = kwargs.get('value', None)


class MicrosoftGraphItemActionSet(msrest.serialization.Model):
    """itemActionSet.

    :param comment: commentAction.
    :type comment: ~users_user.models.MicrosoftGraphCommentAction
    :param create: Any object.
    :type create: object
    :param delete: deleteAction.
    :type delete: ~users_user.models.MicrosoftGraphDeleteAction
    :param edit: Any object.
    :type edit: object
    :param mention: mentionAction.
    :type mention: ~users_user.models.MicrosoftGraphMentionAction
    :param move: moveAction.
    :type move: ~users_user.models.MicrosoftGraphMoveAction
    :param rename: renameAction.
    :type rename: ~users_user.models.MicrosoftGraphRenameAction
    :param restore: Any object.
    :type restore: object
    :param share: shareAction.
    :type share: ~users_user.models.MicrosoftGraphShareAction
    :param version: versionAction.
    :type version: ~users_user.models.MicrosoftGraphVersionAction
    """

    _attribute_map = {
        'comment': {'key': 'comment', 'type': 'MicrosoftGraphCommentAction'},
        'create': {'key': 'create', 'type': 'object'},
        'delete': {'key': 'delete', 'type': 'MicrosoftGraphDeleteAction'},
        'edit': {'key': 'edit', 'type': 'object'},
        'mention': {'key': 'mention', 'type': 'MicrosoftGraphMentionAction'},
        'move': {'key': 'move', 'type': 'MicrosoftGraphMoveAction'},
        'rename': {'key': 'rename', 'type': 'MicrosoftGraphRenameAction'},
        'restore': {'key': 'restore', 'type': 'object'},
        'share': {'key': 'share', 'type': 'MicrosoftGraphShareAction'},
        'version': {'key': 'version', 'type': 'MicrosoftGraphVersionAction'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphItemActionSet, self).__init__(**kwargs)
        self.comment = kwargs.get('comment', None)
        self.create = kwargs.get('create', None)
        self.delete = kwargs.get('delete', None)
        self.edit = kwargs.get('edit', None)
        self.mention = kwargs.get('mention', None)
        self.move = kwargs.get('move', None)
        self.rename = kwargs.get('rename', None)
        self.restore = kwargs.get('restore', None)
        self.share = kwargs.get('share', None)
        self.version = kwargs.get('version', None)


class MicrosoftGraphItemActionStat(msrest.serialization.Model):
    """itemActionStat.

    :param action_count: The number of times the action took place. Read-only.
    :type action_count: int
    :param actor_count: The number of distinct actors that performed the action. Read-only.
    :type actor_count: int
    """

    _validation = {
        'action_count': {'maximum': 2147483647, 'minimum': -2147483648},
        'actor_count': {'maximum': 2147483647, 'minimum': -2147483648},
    }

    _attribute_map = {
        'action_count': {'key': 'actionCount', 'type': 'int'},
        'actor_count': {'key': 'actorCount', 'type': 'int'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphItemActionStat, self).__init__(**kwargs)
        self.action_count = kwargs.get('action_count', None)
        self.actor_count = kwargs.get('actor_count', None)


class MicrosoftGraphItemActivity(MicrosoftGraphEntity):
    """itemActivity.

    :param id: Read-only.
    :type id: str
    :param access: Any object.
    :type access: object
    :param activity_date_time: Details about when the activity took place. Read-only.
    :type activity_date_time: ~datetime.datetime
    :param actor: identitySet.
    :type actor: ~users_user.models.MicrosoftGraphIdentitySet
    :param drive_item: driveItem.
    :type drive_item: ~users_user.models.MicrosoftGraphDriveItem
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'access': {'key': 'access', 'type': 'object'},
        'activity_date_time': {'key': 'activityDateTime', 'type': 'iso-8601'},
        'actor': {'key': 'actor', 'type': 'MicrosoftGraphIdentitySet'},
        'drive_item': {'key': 'driveItem', 'type': 'MicrosoftGraphDriveItem'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphItemActivity, self).__init__(**kwargs)
        self.access = kwargs.get('access', None)
        self.activity_date_time = kwargs.get('activity_date_time', None)
        self.actor = kwargs.get('actor', None)
        self.drive_item = kwargs.get('drive_item', None)


class MicrosoftGraphItemActivityOld(MicrosoftGraphEntity):
    """itemActivityOLD.

    :param id: Read-only.
    :type id: str
    :param action: itemActionSet.
    :type action: ~users_user.models.MicrosoftGraphItemActionSet
    :param actor: identitySet.
    :type actor: ~users_user.models.MicrosoftGraphIdentitySet
    :param times: itemActivityTimeSet.
    :type times: ~users_user.models.MicrosoftGraphItemActivityTimeSet
    :param drive_item: driveItem.
    :type drive_item: ~users_user.models.MicrosoftGraphDriveItem
    :param list_item: listItem.
    :type list_item: ~users_user.models.MicrosoftGraphListItem
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'action': {'key': 'action', 'type': 'MicrosoftGraphItemActionSet'},
        'actor': {'key': 'actor', 'type': 'MicrosoftGraphIdentitySet'},
        'times': {'key': 'times', 'type': 'MicrosoftGraphItemActivityTimeSet'},
        'drive_item': {'key': 'driveItem', 'type': 'MicrosoftGraphDriveItem'},
        'list_item': {'key': 'listItem', 'type': 'MicrosoftGraphListItem'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphItemActivityOld, self).__init__(**kwargs)
        self.action = kwargs.get('action', None)
        self.actor = kwargs.get('actor', None)
        self.times = kwargs.get('times', None)
        self.drive_item = kwargs.get('drive_item', None)
        self.list_item = kwargs.get('list_item', None)


class MicrosoftGraphItemActivityStat(MicrosoftGraphEntity):
    """itemActivityStat.

    :param id: Read-only.
    :type id: str
    :param start_date_time: When the interval starts. Read-only.
    :type start_date_time: ~datetime.datetime
    :param end_date_time: When the interval ends. Read-only.
    :type end_date_time: ~datetime.datetime
    :param access: itemActionStat.
    :type access: ~users_user.models.MicrosoftGraphItemActionStat
    :param create: itemActionStat.
    :type create: ~users_user.models.MicrosoftGraphItemActionStat
    :param delete: itemActionStat.
    :type delete: ~users_user.models.MicrosoftGraphItemActionStat
    :param edit: itemActionStat.
    :type edit: ~users_user.models.MicrosoftGraphItemActionStat
    :param move: itemActionStat.
    :type move: ~users_user.models.MicrosoftGraphItemActionStat
    :param is_trending: Indicates whether the item is 'trending.' Read-only.
    :type is_trending: bool
    :param incomplete_data: incompleteData.
    :type incomplete_data: ~users_user.models.MicrosoftGraphIncompleteData
    :param activities: Exposes the itemActivities represented in this itemActivityStat resource.
    :type activities: list[~users_user.models.MicrosoftGraphItemActivity]
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'start_date_time': {'key': 'startDateTime', 'type': 'iso-8601'},
        'end_date_time': {'key': 'endDateTime', 'type': 'iso-8601'},
        'access': {'key': 'access', 'type': 'MicrosoftGraphItemActionStat'},
        'create': {'key': 'create', 'type': 'MicrosoftGraphItemActionStat'},
        'delete': {'key': 'delete', 'type': 'MicrosoftGraphItemActionStat'},
        'edit': {'key': 'edit', 'type': 'MicrosoftGraphItemActionStat'},
        'move': {'key': 'move', 'type': 'MicrosoftGraphItemActionStat'},
        'is_trending': {'key': 'isTrending', 'type': 'bool'},
        'incomplete_data': {'key': 'incompleteData', 'type': 'MicrosoftGraphIncompleteData'},
        'activities': {'key': 'activities', 'type': '[MicrosoftGraphItemActivity]'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphItemActivityStat, self).__init__(**kwargs)
        self.start_date_time = kwargs.get('start_date_time', None)
        self.end_date_time = kwargs.get('end_date_time', None)
        self.access = kwargs.get('access', None)
        self.create = kwargs.get('create', None)
        self.delete = kwargs.get('delete', None)
        self.edit = kwargs.get('edit', None)
        self.move = kwargs.get('move', None)
        self.is_trending = kwargs.get('is_trending', None)
        self.incomplete_data = kwargs.get('incomplete_data', None)
        self.activities = kwargs.get('activities', None)


class MicrosoftGraphItemActivityTimeSet(msrest.serialization.Model):
    """itemActivityTimeSet.

    :param last_recorded_date_time:
    :type last_recorded_date_time: ~datetime.datetime
    :param observed_date_time: When the activity was observed to take place.
    :type observed_date_time: ~datetime.datetime
    :param recorded_date_time: When the observation was recorded on the service.
    :type recorded_date_time: ~datetime.datetime
    """

    _attribute_map = {
        'last_recorded_date_time': {'key': 'lastRecordedDateTime', 'type': 'iso-8601'},
        'observed_date_time': {'key': 'observedDateTime', 'type': 'iso-8601'},
        'recorded_date_time': {'key': 'recordedDateTime', 'type': 'iso-8601'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphItemActivityTimeSet, self).__init__(**kwargs)
        self.last_recorded_date_time = kwargs.get('last_recorded_date_time', None)
        self.observed_date_time = kwargs.get('observed_date_time', None)
        self.recorded_date_time = kwargs.get('recorded_date_time', None)


class MicrosoftGraphItemAnalytics(MicrosoftGraphEntity):
    """itemAnalytics.

    :param id: Read-only.
    :type id: str
    :param item_activity_stats:
    :type item_activity_stats: list[~users_user.models.MicrosoftGraphItemActivityStat]
    :param all_time: itemActivityStat.
    :type all_time: ~users_user.models.MicrosoftGraphItemActivityStat
    :param last_seven_days: itemActivityStat.
    :type last_seven_days: ~users_user.models.MicrosoftGraphItemActivityStat
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'item_activity_stats': {'key': 'itemActivityStats', 'type': '[MicrosoftGraphItemActivityStat]'},
        'all_time': {'key': 'allTime', 'type': 'MicrosoftGraphItemActivityStat'},
        'last_seven_days': {'key': 'lastSevenDays', 'type': 'MicrosoftGraphItemActivityStat'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphItemAnalytics, self).__init__(**kwargs)
        self.item_activity_stats = kwargs.get('item_activity_stats', None)
        self.all_time = kwargs.get('all_time', None)
        self.last_seven_days = kwargs.get('last_seven_days', None)


class MicrosoftGraphItemBody(msrest.serialization.Model):
    """itemBody.

    :param content_type: bodyType. Possible values include: "text", "html".
    :type content_type: str or ~users_user.models.MicrosoftGraphBodyType
    :param content: The content of the item.
    :type content: str
    """

    _attribute_map = {
        'content_type': {'key': 'contentType', 'type': 'str'},
        'content': {'key': 'content', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphItemBody, self).__init__(**kwargs)
        self.content_type = kwargs.get('content_type', None)
        self.content = kwargs.get('content', None)


class MicrosoftGraphItemEmail(MicrosoftGraphItemFacet):
    """itemEmail.

    :param id: Read-only.
    :type id: str
    :param allowed_audiences: allowedAudiences. Possible values include: "me", "family",
     "contacts", "groupMembers", "organization", "federatedOrganizations", "everyone",
     "unknownFutureValue".
    :type allowed_audiences: str or ~users_user.models.MicrosoftGraphAllowedAudiences
    :param inference: inferenceData.
    :type inference: ~users_user.models.MicrosoftGraphInferenceData
    :param created_date_time:
    :type created_date_time: ~datetime.datetime
    :param created_by: identitySet.
    :type created_by: ~users_user.models.MicrosoftGraphIdentitySet
    :param last_modified_date_time:
    :type last_modified_date_time: ~datetime.datetime
    :param last_modified_by: identitySet.
    :type last_modified_by: ~users_user.models.MicrosoftGraphIdentitySet
    :param address:
    :type address: str
    :param display_name:
    :type display_name: str
    :param type: emailType. Possible values include: "unknown", "work", "personal", "main",
     "other".
    :type type: str or ~users_user.models.MicrosoftGraphEmailType
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'allowed_audiences': {'key': 'allowedAudiences', 'type': 'str'},
        'inference': {'key': 'inference', 'type': 'MicrosoftGraphInferenceData'},
        'created_date_time': {'key': 'createdDateTime', 'type': 'iso-8601'},
        'created_by': {'key': 'createdBy', 'type': 'MicrosoftGraphIdentitySet'},
        'last_modified_date_time': {'key': 'lastModifiedDateTime', 'type': 'iso-8601'},
        'last_modified_by': {'key': 'lastModifiedBy', 'type': 'MicrosoftGraphIdentitySet'},
        'address': {'key': 'address', 'type': 'str'},
        'display_name': {'key': 'displayName', 'type': 'str'},
        'type': {'key': 'type', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphItemEmail, self).__init__(**kwargs)
        self.address = kwargs.get('address', None)
        self.display_name = kwargs.get('display_name', None)
        self.type = kwargs.get('type', None)


class MicrosoftGraphItemPhone(MicrosoftGraphItemFacet):
    """itemPhone.

    :param id: Read-only.
    :type id: str
    :param allowed_audiences: allowedAudiences. Possible values include: "me", "family",
     "contacts", "groupMembers", "organization", "federatedOrganizations", "everyone",
     "unknownFutureValue".
    :type allowed_audiences: str or ~users_user.models.MicrosoftGraphAllowedAudiences
    :param inference: inferenceData.
    :type inference: ~users_user.models.MicrosoftGraphInferenceData
    :param created_date_time:
    :type created_date_time: ~datetime.datetime
    :param created_by: identitySet.
    :type created_by: ~users_user.models.MicrosoftGraphIdentitySet
    :param last_modified_date_time:
    :type last_modified_date_time: ~datetime.datetime
    :param last_modified_by: identitySet.
    :type last_modified_by: ~users_user.models.MicrosoftGraphIdentitySet
    :param display_name:
    :type display_name: str
    :param type: phoneType. Possible values include: "home", "business", "mobile", "other",
     "assistant", "homeFax", "businessFax", "otherFax", "pager", "radio".
    :type type: str or ~users_user.models.MicrosoftGraphPhoneType
    :param number:
    :type number: str
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'allowed_audiences': {'key': 'allowedAudiences', 'type': 'str'},
        'inference': {'key': 'inference', 'type': 'MicrosoftGraphInferenceData'},
        'created_date_time': {'key': 'createdDateTime', 'type': 'iso-8601'},
        'created_by': {'key': 'createdBy', 'type': 'MicrosoftGraphIdentitySet'},
        'last_modified_date_time': {'key': 'lastModifiedDateTime', 'type': 'iso-8601'},
        'last_modified_by': {'key': 'lastModifiedBy', 'type': 'MicrosoftGraphIdentitySet'},
        'display_name': {'key': 'displayName', 'type': 'str'},
        'type': {'key': 'type', 'type': 'str'},
        'number': {'key': 'number', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphItemPhone, self).__init__(**kwargs)
        self.display_name = kwargs.get('display_name', None)
        self.type = kwargs.get('type', None)
        self.number = kwargs.get('number', None)


class MicrosoftGraphItemReference(msrest.serialization.Model):
    """itemReference.

    :param drive_id: Unique identifier of the drive instance that contains the item. Read-only.
    :type drive_id: str
    :param drive_type: Identifies the type of drive. See [drive][] resource for values.
    :type drive_type: str
    :param id: Unique identifier of the item in the drive. Read-only.
    :type id: str
    :param name: The name of the item being referenced. Read-only.
    :type name: str
    :param path: Path that can be used to navigate to the item. Read-only.
    :type path: str
    :param share_id: A unique identifier for a shared resource that can be accessed via the
     [Shares][] API.
    :type share_id: str
    :param sharepoint_ids: sharepointIds.
    :type sharepoint_ids: ~users_user.models.MicrosoftGraphSharepointIds
    :param site_id:
    :type site_id: str
    """

    _attribute_map = {
        'drive_id': {'key': 'driveId', 'type': 'str'},
        'drive_type': {'key': 'driveType', 'type': 'str'},
        'id': {'key': 'id', 'type': 'str'},
        'name': {'key': 'name', 'type': 'str'},
        'path': {'key': 'path', 'type': 'str'},
        'share_id': {'key': 'shareId', 'type': 'str'},
        'sharepoint_ids': {'key': 'sharepointIds', 'type': 'MicrosoftGraphSharepointIds'},
        'site_id': {'key': 'siteId', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphItemReference, self).__init__(**kwargs)
        self.drive_id = kwargs.get('drive_id', None)
        self.drive_type = kwargs.get('drive_type', None)
        self.id = kwargs.get('id', None)
        self.name = kwargs.get('name', None)
        self.path = kwargs.get('path', None)
        self.share_id = kwargs.get('share_id', None)
        self.sharepoint_ids = kwargs.get('sharepoint_ids', None)
        self.site_id = kwargs.get('site_id', None)


class MicrosoftGraphKeyValuePair(msrest.serialization.Model):
    """keyValuePair.

    :param name: Name for this key-value pair.
    :type name: str
    :param value: Value for this key-value pair.
    :type value: str
    """

    _attribute_map = {
        'name': {'key': 'name', 'type': 'str'},
        'value': {'key': 'value', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphKeyValuePair, self).__init__(**kwargs)
        self.name = kwargs.get('name', None)
        self.value = kwargs.get('value', None)


class MicrosoftGraphLabelActionBase(msrest.serialization.Model):
    """labelActionBase.

    :param name:
    :type name: str
    """

    _attribute_map = {
        'name': {'key': 'name', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphLabelActionBase, self).__init__(**kwargs)
        self.name = kwargs.get('name', None)


class MicrosoftGraphLabelPolicy(msrest.serialization.Model):
    """labelPolicy.

    :param id:
    :type id: str
    :param name:
    :type name: str
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'name': {'key': 'name', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphLabelPolicy, self).__init__(**kwargs)
        self.id = kwargs.get('id', None)
        self.name = kwargs.get('name', None)


class MicrosoftGraphLanguageProficiency(MicrosoftGraphItemFacet):
    """languageProficiency.

    :param id: Read-only.
    :type id: str
    :param allowed_audiences: allowedAudiences. Possible values include: "me", "family",
     "contacts", "groupMembers", "organization", "federatedOrganizations", "everyone",
     "unknownFutureValue".
    :type allowed_audiences: str or ~users_user.models.MicrosoftGraphAllowedAudiences
    :param inference: inferenceData.
    :type inference: ~users_user.models.MicrosoftGraphInferenceData
    :param created_date_time:
    :type created_date_time: ~datetime.datetime
    :param created_by: identitySet.
    :type created_by: ~users_user.models.MicrosoftGraphIdentitySet
    :param last_modified_date_time:
    :type last_modified_date_time: ~datetime.datetime
    :param last_modified_by: identitySet.
    :type last_modified_by: ~users_user.models.MicrosoftGraphIdentitySet
    :param display_name:
    :type display_name: str
    :param tag:
    :type tag: str
    :param proficiency: languageProficiencyLevel. Possible values include: "elementary",
     "conversational", "limitedWorking", "professionalWorking", "fullProfessional",
     "nativeOrBilingual", "unknownFutureValue".
    :type proficiency: str or ~users_user.models.MicrosoftGraphLanguageProficiencyLevel
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'allowed_audiences': {'key': 'allowedAudiences', 'type': 'str'},
        'inference': {'key': 'inference', 'type': 'MicrosoftGraphInferenceData'},
        'created_date_time': {'key': 'createdDateTime', 'type': 'iso-8601'},
        'created_by': {'key': 'createdBy', 'type': 'MicrosoftGraphIdentitySet'},
        'last_modified_date_time': {'key': 'lastModifiedDateTime', 'type': 'iso-8601'},
        'last_modified_by': {'key': 'lastModifiedBy', 'type': 'MicrosoftGraphIdentitySet'},
        'display_name': {'key': 'displayName', 'type': 'str'},
        'tag': {'key': 'tag', 'type': 'str'},
        'proficiency': {'key': 'proficiency', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphLanguageProficiency, self).__init__(**kwargs)
        self.display_name = kwargs.get('display_name', None)
        self.tag = kwargs.get('tag', None)
        self.proficiency = kwargs.get('proficiency', None)


class MicrosoftGraphLicenseAssignmentState(msrest.serialization.Model):
    """licenseAssignmentState.

    :param sku_id:
    :type sku_id: str
    :param disabled_plans:
    :type disabled_plans: list[str]
    :param assigned_by_group:
    :type assigned_by_group: str
    :param state:
    :type state: str
    :param error:
    :type error: str
    """

    _attribute_map = {
        'sku_id': {'key': 'skuId', 'type': 'str'},
        'disabled_plans': {'key': 'disabledPlans', 'type': '[str]'},
        'assigned_by_group': {'key': 'assignedByGroup', 'type': 'str'},
        'state': {'key': 'state', 'type': 'str'},
        'error': {'key': 'error', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphLicenseAssignmentState, self).__init__(**kwargs)
        self.sku_id = kwargs.get('sku_id', None)
        self.disabled_plans = kwargs.get('disabled_plans', None)
        self.assigned_by_group = kwargs.get('assigned_by_group', None)
        self.state = kwargs.get('state', None)
        self.error = kwargs.get('error', None)


class MicrosoftGraphLicenseDetails(MicrosoftGraphEntity):
    """licenseDetails.

    :param id: Read-only.
    :type id: str
    :param service_plans: Information about the service plans assigned with the license. Read-only,
     Not nullable.
    :type service_plans: list[~users_user.models.MicrosoftGraphServicePlanInfo]
    :param sku_id: Unique identifier (GUID) for the service SKU. Equal to the skuId property on the
     related SubscribedSku object. Read-only.
    :type sku_id: str
    :param sku_part_number: Unique SKU display name. Equal to the skuPartNumber on the related
     SubscribedSku object; for example: 'AAD_Premium'. Read-only.
    :type sku_part_number: str
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'service_plans': {'key': 'servicePlans', 'type': '[MicrosoftGraphServicePlanInfo]'},
        'sku_id': {'key': 'skuId', 'type': 'str'},
        'sku_part_number': {'key': 'skuPartNumber', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphLicenseDetails, self).__init__(**kwargs)
        self.service_plans = kwargs.get('service_plans', None)
        self.sku_id = kwargs.get('sku_id', None)
        self.sku_part_number = kwargs.get('sku_part_number', None)


class MicrosoftGraphLicenseProcessingState(msrest.serialization.Model):
    """licenseProcessingState.

    :param state:
    :type state: str
    """

    _attribute_map = {
        'state': {'key': 'state', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphLicenseProcessingState, self).__init__(**kwargs)
        self.state = kwargs.get('state', None)


class MicrosoftGraphList(MicrosoftGraphBaseItem):
    """list.

    :param id: Read-only.
    :type id: str
    :param created_by: identitySet.
    :type created_by: ~users_user.models.MicrosoftGraphIdentitySet
    :param created_date_time: Date and time of item creation. Read-only.
    :type created_date_time: ~datetime.datetime
    :param description: Provides a user-visible description of the item. Optional.
    :type description: str
    :param e_tag: ETag for the item. Read-only.
    :type e_tag: str
    :param last_modified_by: identitySet.
    :type last_modified_by: ~users_user.models.MicrosoftGraphIdentitySet
    :param last_modified_date_time: Date and time the item was last modified. Read-only.
    :type last_modified_date_time: ~datetime.datetime
    :param name: The name of the item. Read-write.
    :type name: str
    :param parent_reference: itemReference.
    :type parent_reference: ~users_user.models.MicrosoftGraphItemReference
    :param web_url: URL that displays the resource in the browser. Read-only.
    :type web_url: str
    :param created_by_user: Represents an Azure Active Directory user object.
    :type created_by_user: ~users_user.models.MicrosoftGraphUser
    :param last_modified_by_user: Represents an Azure Active Directory user object.
    :type last_modified_by_user: ~users_user.models.MicrosoftGraphUser
    :param display_name: The displayable title of the list.
    :type display_name: str
    :param list: listInfo.
    :type list: ~users_user.models.MicrosoftGraphListInfo
    :param sharepoint_ids: sharepointIds.
    :type sharepoint_ids: ~users_user.models.MicrosoftGraphSharepointIds
    :param system: Any object.
    :type system: object
    :param activities:
    :type activities: list[~users_user.models.MicrosoftGraphItemActivityOld]
    :param columns: The collection of field definitions for this list.
    :type columns: list[~users_user.models.MicrosoftGraphColumnDefinition]
    :param content_types: The collection of content types present in this list.
    :type content_types: list[~users_user.models.MicrosoftGraphContentType]
    :param drive: drive.
    :type drive: ~users_user.models.MicrosoftGraphDrive
    :param items: All items contained in the list.
    :type items: list[~users_user.models.MicrosoftGraphListItem]
    :param subscriptions: The set of subscriptions on the list.
    :type subscriptions: list[~users_user.models.MicrosoftGraphSubscription]
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'created_by': {'key': 'createdBy', 'type': 'MicrosoftGraphIdentitySet'},
        'created_date_time': {'key': 'createdDateTime', 'type': 'iso-8601'},
        'description': {'key': 'description', 'type': 'str'},
        'e_tag': {'key': 'eTag', 'type': 'str'},
        'last_modified_by': {'key': 'lastModifiedBy', 'type': 'MicrosoftGraphIdentitySet'},
        'last_modified_date_time': {'key': 'lastModifiedDateTime', 'type': 'iso-8601'},
        'name': {'key': 'name', 'type': 'str'},
        'parent_reference': {'key': 'parentReference', 'type': 'MicrosoftGraphItemReference'},
        'web_url': {'key': 'webUrl', 'type': 'str'},
        'created_by_user': {'key': 'createdByUser', 'type': 'MicrosoftGraphUser'},
        'last_modified_by_user': {'key': 'lastModifiedByUser', 'type': 'MicrosoftGraphUser'},
        'display_name': {'key': 'displayName', 'type': 'str'},
        'list': {'key': 'list', 'type': 'MicrosoftGraphListInfo'},
        'sharepoint_ids': {'key': 'sharepointIds', 'type': 'MicrosoftGraphSharepointIds'},
        'system': {'key': 'system', 'type': 'object'},
        'activities': {'key': 'activities', 'type': '[MicrosoftGraphItemActivityOld]'},
        'columns': {'key': 'columns', 'type': '[MicrosoftGraphColumnDefinition]'},
        'content_types': {'key': 'contentTypes', 'type': '[MicrosoftGraphContentType]'},
        'drive': {'key': 'drive', 'type': 'MicrosoftGraphDrive'},
        'items': {'key': 'items', 'type': '[MicrosoftGraphListItem]'},
        'subscriptions': {'key': 'subscriptions', 'type': '[MicrosoftGraphSubscription]'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphList, self).__init__(**kwargs)
        self.display_name = kwargs.get('display_name', None)
        self.list = kwargs.get('list', None)
        self.sharepoint_ids = kwargs.get('sharepoint_ids', None)
        self.system = kwargs.get('system', None)
        self.activities = kwargs.get('activities', None)
        self.columns = kwargs.get('columns', None)
        self.content_types = kwargs.get('content_types', None)
        self.drive = kwargs.get('drive', None)
        self.items = kwargs.get('items', None)
        self.subscriptions = kwargs.get('subscriptions', None)


class MicrosoftGraphListInfo(msrest.serialization.Model):
    """listInfo.

    :param content_types_enabled: If true, indicates that content types are enabled for this list.
    :type content_types_enabled: bool
    :param hidden: If true, indicates that the list is not normally visible in the SharePoint user
     experience.
    :type hidden: bool
    :param template: An enumerated value that represents the base list template used in creating
     the list. Possible values include documentLibrary, genericList, task, survey, announcements,
     contacts, and more.
    :type template: str
    """

    _attribute_map = {
        'content_types_enabled': {'key': 'contentTypesEnabled', 'type': 'bool'},
        'hidden': {'key': 'hidden', 'type': 'bool'},
        'template': {'key': 'template', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphListInfo, self).__init__(**kwargs)
        self.content_types_enabled = kwargs.get('content_types_enabled', None)
        self.hidden = kwargs.get('hidden', None)
        self.template = kwargs.get('template', None)


class MicrosoftGraphListItem(MicrosoftGraphBaseItem):
    """listItem.

    :param id: Read-only.
    :type id: str
    :param created_by: identitySet.
    :type created_by: ~users_user.models.MicrosoftGraphIdentitySet
    :param created_date_time: Date and time of item creation. Read-only.
    :type created_date_time: ~datetime.datetime
    :param description: Provides a user-visible description of the item. Optional.
    :type description: str
    :param e_tag: ETag for the item. Read-only.
    :type e_tag: str
    :param last_modified_by: identitySet.
    :type last_modified_by: ~users_user.models.MicrosoftGraphIdentitySet
    :param last_modified_date_time: Date and time the item was last modified. Read-only.
    :type last_modified_date_time: ~datetime.datetime
    :param name: The name of the item. Read-write.
    :type name: str
    :param parent_reference: itemReference.
    :type parent_reference: ~users_user.models.MicrosoftGraphItemReference
    :param web_url: URL that displays the resource in the browser. Read-only.
    :type web_url: str
    :param created_by_user: Represents an Azure Active Directory user object.
    :type created_by_user: ~users_user.models.MicrosoftGraphUser
    :param last_modified_by_user: Represents an Azure Active Directory user object.
    :type last_modified_by_user: ~users_user.models.MicrosoftGraphUser
    :param content_type: contentTypeInfo.
    :type content_type: ~users_user.models.MicrosoftGraphContentTypeInfo
    :param sharepoint_ids: sharepointIds.
    :type sharepoint_ids: ~users_user.models.MicrosoftGraphSharepointIds
    :param activities: The list of recent activities that took place on this item.
    :type activities: list[~users_user.models.MicrosoftGraphItemActivityOld]
    :param analytics: itemAnalytics.
    :type analytics: ~users_user.models.MicrosoftGraphItemAnalytics
    :param drive_item: driveItem.
    :type drive_item: ~users_user.models.MicrosoftGraphDriveItem
    :param fields: entity.
    :type fields: ~users_user.models.MicrosoftGraphEntity
    :param versions: The list of previous versions of the list item.
    :type versions: list[~users_user.models.MicrosoftGraphListItemVersion]
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'created_by': {'key': 'createdBy', 'type': 'MicrosoftGraphIdentitySet'},
        'created_date_time': {'key': 'createdDateTime', 'type': 'iso-8601'},
        'description': {'key': 'description', 'type': 'str'},
        'e_tag': {'key': 'eTag', 'type': 'str'},
        'last_modified_by': {'key': 'lastModifiedBy', 'type': 'MicrosoftGraphIdentitySet'},
        'last_modified_date_time': {'key': 'lastModifiedDateTime', 'type': 'iso-8601'},
        'name': {'key': 'name', 'type': 'str'},
        'parent_reference': {'key': 'parentReference', 'type': 'MicrosoftGraphItemReference'},
        'web_url': {'key': 'webUrl', 'type': 'str'},
        'created_by_user': {'key': 'createdByUser', 'type': 'MicrosoftGraphUser'},
        'last_modified_by_user': {'key': 'lastModifiedByUser', 'type': 'MicrosoftGraphUser'},
        'content_type': {'key': 'contentType', 'type': 'MicrosoftGraphContentTypeInfo'},
        'sharepoint_ids': {'key': 'sharepointIds', 'type': 'MicrosoftGraphSharepointIds'},
        'activities': {'key': 'activities', 'type': '[MicrosoftGraphItemActivityOld]'},
        'analytics': {'key': 'analytics', 'type': 'MicrosoftGraphItemAnalytics'},
        'drive_item': {'key': 'driveItem', 'type': 'MicrosoftGraphDriveItem'},
        'fields': {'key': 'fields', 'type': 'MicrosoftGraphEntity'},
        'versions': {'key': 'versions', 'type': '[MicrosoftGraphListItemVersion]'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphListItem, self).__init__(**kwargs)
        self.content_type = kwargs.get('content_type', None)
        self.sharepoint_ids = kwargs.get('sharepoint_ids', None)
        self.activities = kwargs.get('activities', None)
        self.analytics = kwargs.get('analytics', None)
        self.drive_item = kwargs.get('drive_item', None)
        self.fields = kwargs.get('fields', None)
        self.versions = kwargs.get('versions', None)


class MicrosoftGraphListItemVersion(MicrosoftGraphBaseItemVersion):
    """listItemVersion.

    :param id: Read-only.
    :type id: str
    :param last_modified_by: identitySet.
    :type last_modified_by: ~users_user.models.MicrosoftGraphIdentitySet
    :param last_modified_date_time: Date and time the version was last modified. Read-only.
    :type last_modified_date_time: ~datetime.datetime
    :param publication: publicationFacet.
    :type publication: ~users_user.models.MicrosoftGraphPublicationFacet
    :param fields: entity.
    :type fields: ~users_user.models.MicrosoftGraphEntity
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'last_modified_by': {'key': 'lastModifiedBy', 'type': 'MicrosoftGraphIdentitySet'},
        'last_modified_date_time': {'key': 'lastModifiedDateTime', 'type': 'iso-8601'},
        'publication': {'key': 'publication', 'type': 'MicrosoftGraphPublicationFacet'},
        'fields': {'key': 'fields', 'type': 'MicrosoftGraphEntity'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphListItemVersion, self).__init__(**kwargs)
        self.fields = kwargs.get('fields', None)


class MicrosoftGraphLocaleInfo(msrest.serialization.Model):
    """localeInfo.

    :param locale: A locale representation for the user, which includes the user's preferred
     language and country/region. For example, 'en-us'. The language component follows 2-letter
     codes as defined in ISO 639-1, and the country component follows 2-letter codes as defined in
     ISO 3166-1 alpha-2.
    :type locale: str
    :param display_name: A name representing the user's locale in natural language, for example,
     'English (United States)'.
    :type display_name: str
    """

    _attribute_map = {
        'locale': {'key': 'locale', 'type': 'str'},
        'display_name': {'key': 'displayName', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphLocaleInfo, self).__init__(**kwargs)
        self.locale = kwargs.get('locale', None)
        self.display_name = kwargs.get('display_name', None)


class MicrosoftGraphLocation(msrest.serialization.Model):
    """location.

    :param display_name: The name associated with the location.
    :type display_name: str
    :param location_email_address: Optional email address of the location.
    :type location_email_address: str
    :param address: physicalAddress.
    :type address: ~users_user.models.MicrosoftGraphPhysicalAddress
    :param coordinates: outlookGeoCoordinates.
    :type coordinates: ~users_user.models.MicrosoftGraphOutlookGeoCoordinates
    :param location_uri: Optional URI representing the location.
    :type location_uri: str
    :param location_type: locationType. Possible values include: "default", "conferenceRoom",
     "homeAddress", "businessAddress", "geoCoordinates", "streetAddress", "hotel", "restaurant",
     "localBusiness", "postalAddress".
    :type location_type: str or ~users_user.models.MicrosoftGraphLocationType
    :param unique_id: For internal use only.
    :type unique_id: str
    :param unique_id_type: locationUniqueIdType. Possible values include: "unknown",
     "locationStore", "directory", "private", "bing".
    :type unique_id_type: str or ~users_user.models.MicrosoftGraphLocationUniqueIdType
    """

    _attribute_map = {
        'display_name': {'key': 'displayName', 'type': 'str'},
        'location_email_address': {'key': 'locationEmailAddress', 'type': 'str'},
        'address': {'key': 'address', 'type': 'MicrosoftGraphPhysicalAddress'},
        'coordinates': {'key': 'coordinates', 'type': 'MicrosoftGraphOutlookGeoCoordinates'},
        'location_uri': {'key': 'locationUri', 'type': 'str'},
        'location_type': {'key': 'locationType', 'type': 'str'},
        'unique_id': {'key': 'uniqueId', 'type': 'str'},
        'unique_id_type': {'key': 'uniqueIdType', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphLocation, self).__init__(**kwargs)
        self.display_name = kwargs.get('display_name', None)
        self.location_email_address = kwargs.get('location_email_address', None)
        self.address = kwargs.get('address', None)
        self.coordinates = kwargs.get('coordinates', None)
        self.location_uri = kwargs.get('location_uri', None)
        self.location_type = kwargs.get('location_type', None)
        self.unique_id = kwargs.get('unique_id', None)
        self.unique_id_type = kwargs.get('unique_id_type', None)


class MicrosoftGraphLoggedOnUser(msrest.serialization.Model):
    """loggedOnUser.

    :param user_id: User id.
    :type user_id: str
    :param last_log_on_date_time: Date time when user logs on.
    :type last_log_on_date_time: ~datetime.datetime
    """

    _attribute_map = {
        'user_id': {'key': 'userId', 'type': 'str'},
        'last_log_on_date_time': {'key': 'lastLogOnDateTime', 'type': 'iso-8601'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphLoggedOnUser, self).__init__(**kwargs)
        self.user_id = kwargs.get('user_id', None)
        self.last_log_on_date_time = kwargs.get('last_log_on_date_time', None)


class MicrosoftGraphLongRunningOperation(MicrosoftGraphEntity):
    """longRunningOperation.

    :param id: Read-only.
    :type id: str
    :param created_date_time:
    :type created_date_time: ~datetime.datetime
    :param last_action_date_time:
    :type last_action_date_time: ~datetime.datetime
    :param status: longRunningOperationStatus. Possible values include: "notstarted", "running",
     "succeeded", "failed".
    :type status: str or ~users_user.models.MicrosoftGraphLongRunningOperationStatus
    :param status_detail:
    :type status_detail: str
    :param resource_location:
    :type resource_location: str
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'created_date_time': {'key': 'createdDateTime', 'type': 'iso-8601'},
        'last_action_date_time': {'key': 'lastActionDateTime', 'type': 'iso-8601'},
        'status': {'key': 'status', 'type': 'str'},
        'status_detail': {'key': 'statusDetail', 'type': 'str'},
        'resource_location': {'key': 'resourceLocation', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphLongRunningOperation, self).__init__(**kwargs)
        self.created_date_time = kwargs.get('created_date_time', None)
        self.last_action_date_time = kwargs.get('last_action_date_time', None)
        self.status = kwargs.get('status', None)
        self.status_detail = kwargs.get('status_detail', None)
        self.resource_location = kwargs.get('resource_location', None)


class MicrosoftGraphLookupColumn(msrest.serialization.Model):
    """lookupColumn.

    :param allow_multiple_values: Indicates whether multiple values can be selected from the
     source.
    :type allow_multiple_values: bool
    :param allow_unlimited_length: Indicates whether values in the column should be able to exceed
     the standard limit of 255 characters.
    :type allow_unlimited_length: bool
    :param column_name: The name of the lookup source column.
    :type column_name: str
    :param list_id: The unique identifier of the lookup source list.
    :type list_id: str
    :param primary_lookup_column_id: If specified, this column is a secondary lookup, pulling an
     additional field from the list item looked up by the primary lookup. Use the list item looked
     up by the primary as the source for the column named here.
    :type primary_lookup_column_id: str
    """

    _attribute_map = {
        'allow_multiple_values': {'key': 'allowMultipleValues', 'type': 'bool'},
        'allow_unlimited_length': {'key': 'allowUnlimitedLength', 'type': 'bool'},
        'column_name': {'key': 'columnName', 'type': 'str'},
        'list_id': {'key': 'listId', 'type': 'str'},
        'primary_lookup_column_id': {'key': 'primaryLookupColumnId', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphLookupColumn, self).__init__(**kwargs)
        self.allow_multiple_values = kwargs.get('allow_multiple_values', None)
        self.allow_unlimited_length = kwargs.get('allow_unlimited_length', None)
        self.column_name = kwargs.get('column_name', None)
        self.list_id = kwargs.get('list_id', None)
        self.primary_lookup_column_id = kwargs.get('primary_lookup_column_id', None)


class MicrosoftGraphMailboxSettings(msrest.serialization.Model):
    """mailboxSettings.

    :param automatic_replies_setting: automaticRepliesSetting.
    :type automatic_replies_setting: ~users_user.models.MicrosoftGraphAutomaticRepliesSetting
    :param archive_folder: Folder ID of an archive folder for the user.
    :type archive_folder: str
    :param time_zone: The default time zone for the user's mailbox.
    :type time_zone: str
    :param language: localeInfo.
    :type language: ~users_user.models.MicrosoftGraphLocaleInfo
    :param delegate_meeting_message_delivery_options: delegateMeetingMessageDeliveryOptions.
     Possible values include: "sendToDelegateAndInformationToPrincipal",
     "sendToDelegateAndPrincipal", "sendToDelegateOnly".
    :type delegate_meeting_message_delivery_options: str or
     ~users_user.models.MicrosoftGraphDelegateMeetingMessageDeliveryOptions
    :param working_hours: workingHours.
    :type working_hours: ~users_user.models.MicrosoftGraphWorkingHours
    :param date_format: The date format for the user's mailbox.
    :type date_format: str
    :param time_format: The time format for the user's mailbox.
    :type time_format: str
    """

    _attribute_map = {
        'automatic_replies_setting': {'key': 'automaticRepliesSetting', 'type': 'MicrosoftGraphAutomaticRepliesSetting'},
        'archive_folder': {'key': 'archiveFolder', 'type': 'str'},
        'time_zone': {'key': 'timeZone', 'type': 'str'},
        'language': {'key': 'language', 'type': 'MicrosoftGraphLocaleInfo'},
        'delegate_meeting_message_delivery_options': {'key': 'delegateMeetingMessageDeliveryOptions', 'type': 'str'},
        'working_hours': {'key': 'workingHours', 'type': 'MicrosoftGraphWorkingHours'},
        'date_format': {'key': 'dateFormat', 'type': 'str'},
        'time_format': {'key': 'timeFormat', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphMailboxSettings, self).__init__(**kwargs)
        self.automatic_replies_setting = kwargs.get('automatic_replies_setting', None)
        self.archive_folder = kwargs.get('archive_folder', None)
        self.time_zone = kwargs.get('time_zone', None)
        self.language = kwargs.get('language', None)
        self.delegate_meeting_message_delivery_options = kwargs.get('delegate_meeting_message_delivery_options', None)
        self.working_hours = kwargs.get('working_hours', None)
        self.date_format = kwargs.get('date_format', None)
        self.time_format = kwargs.get('time_format', None)


class MicrosoftGraphMailFolder(MicrosoftGraphEntity):
    """mailFolder.

    :param id: Read-only.
    :type id: str
    :param display_name: The mailFolder's display name.
    :type display_name: str
    :param parent_folder_id: The unique identifier for the mailFolder's parent mailFolder.
    :type parent_folder_id: str
    :param child_folder_count: The number of immediate child mailFolders in the current mailFolder.
    :type child_folder_count: int
    :param unread_item_count: The number of items in the mailFolder marked as unread.
    :type unread_item_count: int
    :param total_item_count: The number of items in the mailFolder.
    :type total_item_count: int
    :param well_known_name:
    :type well_known_name: str
    :param single_value_extended_properties: The collection of single-value extended properties
     defined for the mailFolder. Read-only. Nullable.
    :type single_value_extended_properties:
     list[~users_user.models.MicrosoftGraphSingleValueLegacyExtendedProperty]
    :param multi_value_extended_properties: The collection of multi-value extended properties
     defined for the mailFolder. Read-only. Nullable.
    :type multi_value_extended_properties:
     list[~users_user.models.MicrosoftGraphMultiValueLegacyExtendedProperty]
    :param messages: The collection of messages in the mailFolder.
    :type messages: list[~users_user.models.MicrosoftGraphMessage]
    :param message_rules: The collection of rules that apply to the user's Inbox folder.
    :type message_rules: list[~users_user.models.MicrosoftGraphMessageRule]
    :param child_folders: The collection of child folders in the mailFolder.
    :type child_folders: list[~users_user.models.MicrosoftGraphMailFolder]
    :param user_configurations:
    :type user_configurations: list[~users_user.models.MicrosoftGraphUserConfiguration]
    """

    _validation = {
        'child_folder_count': {'maximum': 2147483647, 'minimum': -2147483648},
        'unread_item_count': {'maximum': 2147483647, 'minimum': -2147483648},
        'total_item_count': {'maximum': 2147483647, 'minimum': -2147483648},
    }

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'display_name': {'key': 'displayName', 'type': 'str'},
        'parent_folder_id': {'key': 'parentFolderId', 'type': 'str'},
        'child_folder_count': {'key': 'childFolderCount', 'type': 'int'},
        'unread_item_count': {'key': 'unreadItemCount', 'type': 'int'},
        'total_item_count': {'key': 'totalItemCount', 'type': 'int'},
        'well_known_name': {'key': 'wellKnownName', 'type': 'str'},
        'single_value_extended_properties': {'key': 'singleValueExtendedProperties', 'type': '[MicrosoftGraphSingleValueLegacyExtendedProperty]'},
        'multi_value_extended_properties': {'key': 'multiValueExtendedProperties', 'type': '[MicrosoftGraphMultiValueLegacyExtendedProperty]'},
        'messages': {'key': 'messages', 'type': '[MicrosoftGraphMessage]'},
        'message_rules': {'key': 'messageRules', 'type': '[MicrosoftGraphMessageRule]'},
        'child_folders': {'key': 'childFolders', 'type': '[MicrosoftGraphMailFolder]'},
        'user_configurations': {'key': 'userConfigurations', 'type': '[MicrosoftGraphUserConfiguration]'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphMailFolder, self).__init__(**kwargs)
        self.display_name = kwargs.get('display_name', None)
        self.parent_folder_id = kwargs.get('parent_folder_id', None)
        self.child_folder_count = kwargs.get('child_folder_count', None)
        self.unread_item_count = kwargs.get('unread_item_count', None)
        self.total_item_count = kwargs.get('total_item_count', None)
        self.well_known_name = kwargs.get('well_known_name', None)
        self.single_value_extended_properties = kwargs.get('single_value_extended_properties', None)
        self.multi_value_extended_properties = kwargs.get('multi_value_extended_properties', None)
        self.messages = kwargs.get('messages', None)
        self.message_rules = kwargs.get('message_rules', None)
        self.child_folders = kwargs.get('child_folders', None)
        self.user_configurations = kwargs.get('user_configurations', None)


class MicrosoftGraphManagedAppOperation(MicrosoftGraphEntity):
    """Represents an operation applied against an app registration.

    :param id: Read-only.
    :type id: str
    :param display_name: The operation name.
    :type display_name: str
    :param last_modified_date_time: The last time the app operation was modified.
    :type last_modified_date_time: ~datetime.datetime
    :param state: The current state of the operation.
    :type state: str
    :param version: Version of the entity.
    :type version: str
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'display_name': {'key': 'displayName', 'type': 'str'},
        'last_modified_date_time': {'key': 'lastModifiedDateTime', 'type': 'iso-8601'},
        'state': {'key': 'state', 'type': 'str'},
        'version': {'key': 'version', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphManagedAppOperation, self).__init__(**kwargs)
        self.display_name = kwargs.get('display_name', None)
        self.last_modified_date_time = kwargs.get('last_modified_date_time', None)
        self.state = kwargs.get('state', None)
        self.version = kwargs.get('version', None)


class MicrosoftGraphManagedAppPolicy(MicrosoftGraphEntity):
    """The ManagedAppPolicy resource represents a base type for platform specific policies.

    :param id: Read-only.
    :type id: str
    :param display_name: Policy display name.
    :type display_name: str
    :param description: The policy's description.
    :type description: str
    :param created_date_time: The date and time the policy was created.
    :type created_date_time: ~datetime.datetime
    :param last_modified_date_time: Last time the policy was modified.
    :type last_modified_date_time: ~datetime.datetime
    :param role_scope_tag_ids: List of Scope Tags for this Entity instance.
    :type role_scope_tag_ids: list[str]
    :param version: Version of the entity.
    :type version: str
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'display_name': {'key': 'displayName', 'type': 'str'},
        'description': {'key': 'description', 'type': 'str'},
        'created_date_time': {'key': 'createdDateTime', 'type': 'iso-8601'},
        'last_modified_date_time': {'key': 'lastModifiedDateTime', 'type': 'iso-8601'},
        'role_scope_tag_ids': {'key': 'roleScopeTagIds', 'type': '[str]'},
        'version': {'key': 'version', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphManagedAppPolicy, self).__init__(**kwargs)
        self.display_name = kwargs.get('display_name', None)
        self.description = kwargs.get('description', None)
        self.created_date_time = kwargs.get('created_date_time', None)
        self.last_modified_date_time = kwargs.get('last_modified_date_time', None)
        self.role_scope_tag_ids = kwargs.get('role_scope_tag_ids', None)
        self.version = kwargs.get('version', None)


class MicrosoftGraphManagedAppRegistration(MicrosoftGraphEntity):
    """The ManagedAppEntity is the base entity type for all other entity types under app management workflow.

    :param id: Read-only.
    :type id: str
    :param created_date_time: Date and time of creation.
    :type created_date_time: ~datetime.datetime
    :param last_sync_date_time: Date and time of last the app synced with management service.
    :type last_sync_date_time: ~datetime.datetime
    :param application_version: App version.
    :type application_version: str
    :param management_sdk_version: App management SDK version.
    :type management_sdk_version: str
    :param platform_version: Operating System version.
    :type platform_version: str
    :param device_type: Host device type.
    :type device_type: str
    :param device_tag: App management SDK generated tag, which helps relate apps hosted on the same
     device. Not guaranteed to relate apps in all conditions.
    :type device_tag: str
    :param device_name: Host device name.
    :type device_name: str
    :param managed_device_id: The Managed Device identifier of the host device. Value could be
     empty even when the host device is managed.
    :type managed_device_id: str
    :param azure_ad_device_id: The Azure Active Directory Device identifier of the host device.
     Value could be empty even when the host device is Azure Active Directory registered.
    :type azure_ad_device_id: str
    :param device_model: The device model for the current app registration.
    :type device_model: str
    :param device_manufacturer: The device manufacturer for the current app registration.
    :type device_manufacturer: str
    :param flagged_reasons: Zero or more reasons an app registration is flagged. E.g. app running
     on rooted device.
    :type flagged_reasons: list[str or ~users_user.models.MicrosoftGraphManagedAppFlaggedReason]
    :param user_id: The user Id to who this app registration belongs.
    :type user_id: str
    :param app_identifier: Any object.
    :type app_identifier: object
    :param version: Version of the entity.
    :type version: str
    :param applied_policies: Zero or more policys already applied on the registered app when it
     last synchronized with managment service.
    :type applied_policies: list[~users_user.models.MicrosoftGraphManagedAppPolicy]
    :param intended_policies: Zero or more policies admin intended for the app as of now.
    :type intended_policies: list[~users_user.models.MicrosoftGraphManagedAppPolicy]
    :param operations: Zero or more long running operations triggered on the app registration.
    :type operations: list[~users_user.models.MicrosoftGraphManagedAppOperation]
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'created_date_time': {'key': 'createdDateTime', 'type': 'iso-8601'},
        'last_sync_date_time': {'key': 'lastSyncDateTime', 'type': 'iso-8601'},
        'application_version': {'key': 'applicationVersion', 'type': 'str'},
        'management_sdk_version': {'key': 'managementSdkVersion', 'type': 'str'},
        'platform_version': {'key': 'platformVersion', 'type': 'str'},
        'device_type': {'key': 'deviceType', 'type': 'str'},
        'device_tag': {'key': 'deviceTag', 'type': 'str'},
        'device_name': {'key': 'deviceName', 'type': 'str'},
        'managed_device_id': {'key': 'managedDeviceId', 'type': 'str'},
        'azure_ad_device_id': {'key': 'azureADDeviceId', 'type': 'str'},
        'device_model': {'key': 'deviceModel', 'type': 'str'},
        'device_manufacturer': {'key': 'deviceManufacturer', 'type': 'str'},
        'flagged_reasons': {'key': 'flaggedReasons', 'type': '[str]'},
        'user_id': {'key': 'userId', 'type': 'str'},
        'app_identifier': {'key': 'appIdentifier', 'type': 'object'},
        'version': {'key': 'version', 'type': 'str'},
        'applied_policies': {'key': 'appliedPolicies', 'type': '[MicrosoftGraphManagedAppPolicy]'},
        'intended_policies': {'key': 'intendedPolicies', 'type': '[MicrosoftGraphManagedAppPolicy]'},
        'operations': {'key': 'operations', 'type': '[MicrosoftGraphManagedAppOperation]'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphManagedAppRegistration, self).__init__(**kwargs)
        self.created_date_time = kwargs.get('created_date_time', None)
        self.last_sync_date_time = kwargs.get('last_sync_date_time', None)
        self.application_version = kwargs.get('application_version', None)
        self.management_sdk_version = kwargs.get('management_sdk_version', None)
        self.platform_version = kwargs.get('platform_version', None)
        self.device_type = kwargs.get('device_type', None)
        self.device_tag = kwargs.get('device_tag', None)
        self.device_name = kwargs.get('device_name', None)
        self.managed_device_id = kwargs.get('managed_device_id', None)
        self.azure_ad_device_id = kwargs.get('azure_ad_device_id', None)
        self.device_model = kwargs.get('device_model', None)
        self.device_manufacturer = kwargs.get('device_manufacturer', None)
        self.flagged_reasons = kwargs.get('flagged_reasons', None)
        self.user_id = kwargs.get('user_id', None)
        self.app_identifier = kwargs.get('app_identifier', None)
        self.version = kwargs.get('version', None)
        self.applied_policies = kwargs.get('applied_policies', None)
        self.intended_policies = kwargs.get('intended_policies', None)
        self.operations = kwargs.get('operations', None)


class MicrosoftGraphManagedDevice(MicrosoftGraphEntity):
    """Devices that are managed or pre-enrolled through Intune.

    :param id: Read-only.
    :type id: str
    :param user_id: Unique Identifier for the user associated with the device.
    :type user_id: str
    :param device_name: Name of the device.
    :type device_name: str
    :param hardware_information: hardwareInformation.
    :type hardware_information: ~users_user.models.MicrosoftGraphHardwareInformation
    :param owner_type: ownerType. Possible values include: "unknown", "company", "personal".
    :type owner_type: str or ~users_user.models.MicrosoftGraphOwnerType
    :param managed_device_owner_type: managedDeviceOwnerType. Possible values include: "unknown",
     "company", "personal".
    :type managed_device_owner_type: str or ~users_user.models.MicrosoftGraphManagedDeviceOwnerType
    :param device_action_results: List of ComplexType deviceActionResult objects.
    :type device_action_results: list[~users_user.models.MicrosoftGraphDeviceActionResult]
    :param management_state: managementState. Possible values include: "managed", "retirePending",
     "retireFailed", "wipePending", "wipeFailed", "unhealthy", "deletePending", "retireIssued",
     "wipeIssued", "wipeCanceled", "retireCanceled", "discovered".
    :type management_state: str or ~users_user.models.MicrosoftGraphManagementState
    :param enrolled_date_time: Enrollment time of the device.
    :type enrolled_date_time: ~datetime.datetime
    :param last_sync_date_time: The date and time that the device last completed a successful sync
     with Intune.
    :type last_sync_date_time: ~datetime.datetime
    :param chassis_type: chassisType. Possible values include: "unknown", "desktop", "laptop",
     "worksWorkstation", "enterpriseServer", "phone", "tablet", "mobileOther", "mobileUnknown".
    :type chassis_type: str or ~users_user.models.MicrosoftGraphChassisType
    :param operating_system: Operating system of the device. Windows, iOS, etc.
    :type operating_system: str
    :param device_type: deviceType. Possible values include: "desktop", "windowsRT", "winMO6",
     "nokia", "windowsPhone", "mac", "winCE", "winEmbedded", "iPhone", "iPad", "iPod", "android",
     "iSocConsumer", "unix", "macMDM", "holoLens", "surfaceHub", "androidForWork",
     "androidEnterprise", "windows10x", "blackberry", "palm", "unknown".
    :type device_type: str or ~users_user.models.MicrosoftGraphDeviceType
    :param compliance_state: complianceState. Possible values include: "unknown", "compliant",
     "noncompliant", "conflict", "error", "inGracePeriod", "configManager".
    :type compliance_state: str or ~users_user.models.MicrosoftGraphComplianceState
    :param jail_broken: whether the device is jail broken or rooted.
    :type jail_broken: str
    :param management_agent: managementAgentType. Possible values include: "eas", "mdm", "easMdm",
     "intuneClient", "easIntuneClient", "configurationManagerClient",
     "configurationManagerClientMdm", "configurationManagerClientMdmEas", "unknown", "jamf",
     "googleCloudDevicePolicyController", "microsoft365ManagedMdm", "windowsManagementCloudApi".
    :type management_agent: str or ~users_user.models.MicrosoftGraphManagementAgentType
    :param os_version: Operating system version of the device.
    :type os_version: str
    :param eas_activated: Whether the device is Exchange ActiveSync activated.
    :type eas_activated: bool
    :param eas_device_id: Exchange ActiveSync Id of the device.
    :type eas_device_id: str
    :param eas_activation_date_time: Exchange ActivationSync activation time of the device.
    :type eas_activation_date_time: ~datetime.datetime
    :param aad_registered: Whether the device is Azure Active Directory registered. This property
     is read-only.
    :type aad_registered: bool
    :param azure_ad_registered: Whether the device is Azure Active Directory registered.
    :type azure_ad_registered: bool
    :param device_enrollment_type: deviceEnrollmentType. Possible values include: "unknown",
     "userEnrollment", "deviceEnrollmentManager", "appleBulkWithUser", "appleBulkWithoutUser",
     "windowsAzureADJoin", "windowsBulkUserless", "windowsAutoEnrollment",
     "windowsBulkAzureDomainJoin", "windowsCoManagement", "appleUserEnrollment",
     "appleUserEnrollmentWithServiceAccount", "azureAdJoinUsingAzureVmExtension".
    :type device_enrollment_type: str or ~users_user.models.MicrosoftGraphDeviceEnrollmentType
    :param lost_mode_state: lostModeState. Possible values include: "disabled", "enabled".
    :type lost_mode_state: str or ~users_user.models.MicrosoftGraphLostModeState
    :param activation_lock_bypass_code: Code that allows the Activation Lock on a device to be
     bypassed.
    :type activation_lock_bypass_code: str
    :param email_address: Email(s) for the user associated with the device.
    :type email_address: str
    :param azure_active_directory_device_id: The unique identifier for the Azure Active Directory
     device. Read only. This property is read-only.
    :type azure_active_directory_device_id: str
    :param azure_ad_device_id: The unique identifier for the Azure Active Directory device. Read
     only.
    :type azure_ad_device_id: str
    :param device_registration_state: deviceRegistrationState. Possible values include:
     "notRegistered", "registered", "revoked", "keyConflict", "approvalPending", "certificateReset",
     "notRegisteredPendingEnrollment", "unknown".
    :type device_registration_state: str or
     ~users_user.models.MicrosoftGraphDeviceRegistrationState
    :param device_category_display_name: Device category display name.
    :type device_category_display_name: str
    :param is_supervised: Device supervised status.
    :type is_supervised: bool
    :param exchange_last_successful_sync_date_time: Last time the device contacted Exchange.
    :type exchange_last_successful_sync_date_time: ~datetime.datetime
    :param exchange_access_state: deviceManagementExchangeAccessState. Possible values include:
     "none", "unknown", "allowed", "blocked", "quarantined".
    :type exchange_access_state: str or
     ~users_user.models.MicrosoftGraphDeviceManagementExchangeAccessState
    :param exchange_access_state_reason: deviceManagementExchangeAccessStateReason. Possible values
     include: "none", "unknown", "exchangeGlobalRule", "exchangeIndividualRule",
     "exchangeDeviceRule", "exchangeUpgrade", "exchangeMailboxPolicy", "other", "compliant",
     "notCompliant", "notEnrolled", "unknownLocation", "mfaRequired",
     "azureADBlockDueToAccessPolicy", "compromisedPassword", "deviceNotKnownWithManagedApp".
    :type exchange_access_state_reason: str or
     ~users_user.models.MicrosoftGraphDeviceManagementExchangeAccessStateReason
    :param remote_assistance_session_url: Url that allows a Remote Assistance session to be
     established with the device.
    :type remote_assistance_session_url: str
    :param remote_assistance_session_error_details: An error string that identifies issues when
     creating Remote Assistance session objects.
    :type remote_assistance_session_error_details: str
    :param is_encrypted: Device encryption status.
    :type is_encrypted: bool
    :param user_principal_name: Device user principal name.
    :type user_principal_name: str
    :param model: Model of the device.
    :type model: str
    :param manufacturer: Manufacturer of the device.
    :type manufacturer: str
    :param imei: IMEI.
    :type imei: str
    :param compliance_grace_period_expiration_date_time: The DateTime when device compliance grace
     period expires.
    :type compliance_grace_period_expiration_date_time: ~datetime.datetime
    :param serial_number: SerialNumber.
    :type serial_number: str
    :param phone_number: Phone number of the device.
    :type phone_number: str
    :param android_security_patch_level: Android security patch level.
    :type android_security_patch_level: str
    :param user_display_name: User display name.
    :type user_display_name: str
    :param configuration_manager_client_enabled_features:
     configurationManagerClientEnabledFeatures.
    :type configuration_manager_client_enabled_features:
     ~users_user.models.MicrosoftGraphConfigurationManagerClientEnabledFeatures
    :param wi_fi_mac_address: Wi-Fi MAC.
    :type wi_fi_mac_address: str
    :param device_health_attestation_state: deviceHealthAttestationState.
    :type device_health_attestation_state:
     ~users_user.models.MicrosoftGraphDeviceHealthAttestationState
    :param subscriber_carrier: Subscriber Carrier.
    :type subscriber_carrier: str
    :param meid: MEID.
    :type meid: str
    :param total_storage_space_in_bytes: Total Storage in Bytes.
    :type total_storage_space_in_bytes: long
    :param free_storage_space_in_bytes: Free Storage in Bytes.
    :type free_storage_space_in_bytes: long
    :param managed_device_name: Automatically generated name to identify a device. Can be
     overwritten to a user friendly name.
    :type managed_device_name: str
    :param partner_reported_threat_state: managedDevicePartnerReportedHealthState. Possible values
     include: "unknown", "activated", "deactivated", "secured", "lowSeverity", "mediumSeverity",
     "highSeverity", "unresponsive", "compromised", "misconfigured".
    :type partner_reported_threat_state: str or
     ~users_user.models.MicrosoftGraphManagedDevicePartnerReportedHealthState
    :param retire_after_date_time: Indicates the time after when a device will be auto retired
     because of scheduled action. This property is read-only.
    :type retire_after_date_time: ~datetime.datetime
    :param users_logged_on: Indicates the last logged on users of a device. This property is read-
     only.
    :type users_logged_on: list[~users_user.models.MicrosoftGraphLoggedOnUser]
    :param prefer_mdm_over_group_policy_applied_date_time: Reports the DateTime the
     preferMdmOverGroupPolicy setting was set.  When set, the Intune MDM settings will override
     Group Policy settings if there is a conflict. Read Only. This property is read-only.
    :type prefer_mdm_over_group_policy_applied_date_time: ~datetime.datetime
    :param autopilot_enrolled: Reports if the managed device is enrolled via auto-pilot. This
     property is read-only.
    :type autopilot_enrolled: bool
    :param require_user_enrollment_approval: Reports if the managed iOS device is user approval
     enrollment. This property is read-only.
    :type require_user_enrollment_approval: bool
    :param management_certificate_expiration_date: Reports device management certificate expiration
     date. This property is read-only.
    :type management_certificate_expiration_date: ~datetime.datetime
    :param iccid: Integrated Circuit Card Identifier, it is A SIM card's unique identification
     number. This property is read-only.
    :type iccid: str
    :param udid: Unique Device Identifier for iOS and macOS devices. This property is read-only.
    :type udid: str
    :param role_scope_tag_ids: List of Scope Tag IDs for this Device instance.
    :type role_scope_tag_ids: list[str]
    :param windows_active_malware_count: Count of active malware for this windows device. This
     property is read-only.
    :type windows_active_malware_count: int
    :param windows_remediated_malware_count: Count of remediated malware for this windows device.
     This property is read-only.
    :type windows_remediated_malware_count: int
    :param notes: Notes on the device created by IT Admin.
    :type notes: str
    :param configuration_manager_client_health_state: configurationManagerClientHealthState.
    :type configuration_manager_client_health_state:
     ~users_user.models.MicrosoftGraphConfigurationManagerClientHealthState
    :param configuration_manager_client_information: configurationManagerClientInformation.
    :type configuration_manager_client_information:
     ~users_user.models.MicrosoftGraphConfigurationManagerClientInformation
    :param ethernet_mac_address: Ethernet MAC. This property is read-only.
    :type ethernet_mac_address: str
    :param physical_memory_in_bytes: Total Memory in Bytes. This property is read-only.
    :type physical_memory_in_bytes: long
    :param processor_architecture: managedDeviceArchitecture. Possible values include: "unknown",
     "x86", "x64", "arm", "arM64".
    :type processor_architecture: str or ~users_user.models.MicrosoftGraphManagedDeviceArchitecture
    :param specification_version: Specification version. This property is read-only.
    :type specification_version: str
    :param security_baseline_states: Security baseline states for this device.
    :type security_baseline_states: list[~users_user.models.MicrosoftGraphSecurityBaselineState]
    :param device_configuration_states: Device configuration states for this device.
    :type device_configuration_states:
     list[~users_user.models.MicrosoftGraphDeviceConfigurationState]
    :param device_compliance_policy_states: Device compliance policy states for this device.
    :type device_compliance_policy_states:
     list[~users_user.models.MicrosoftGraphDeviceCompliancePolicyState]
    :param managed_device_mobile_app_configuration_states: Managed device mobile app configuration
     states for this device.
    :type managed_device_mobile_app_configuration_states:
     list[~users_user.models.MicrosoftGraphManagedDeviceMobileAppConfigurationState]
    :param detected_apps: All applications currently installed on the device.
    :type detected_apps: list[~users_user.models.MicrosoftGraphDetectedApp]
    :param device_category: Device categories provides a way to organize your devices. Using device
     categories, company administrators can define their own categories that make sense to their
     company.??These categories can then be applied to a device in the Intune Azure console or
     selected by a user during device enrollment. You can filter reports and create dynamic Azure
     Active Directory device groups based on device categories.
    :type device_category: ~users_user.models.MicrosoftGraphDeviceCategory
    :param windows_protection_state: Device protection status entity.
    :type windows_protection_state: ~users_user.models.MicrosoftGraphWindowsProtectionState
    :param users: The primary users associated with the managed device.
    :type users: list[~users_user.models.MicrosoftGraphUser]
    """

    _validation = {
        'windows_active_malware_count': {'maximum': 2147483647, 'minimum': -2147483648},
        'windows_remediated_malware_count': {'maximum': 2147483647, 'minimum': -2147483648},
    }

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'user_id': {'key': 'userId', 'type': 'str'},
        'device_name': {'key': 'deviceName', 'type': 'str'},
        'hardware_information': {'key': 'hardwareInformation', 'type': 'MicrosoftGraphHardwareInformation'},
        'owner_type': {'key': 'ownerType', 'type': 'str'},
        'managed_device_owner_type': {'key': 'managedDeviceOwnerType', 'type': 'str'},
        'device_action_results': {'key': 'deviceActionResults', 'type': '[MicrosoftGraphDeviceActionResult]'},
        'management_state': {'key': 'managementState', 'type': 'str'},
        'enrolled_date_time': {'key': 'enrolledDateTime', 'type': 'iso-8601'},
        'last_sync_date_time': {'key': 'lastSyncDateTime', 'type': 'iso-8601'},
        'chassis_type': {'key': 'chassisType', 'type': 'str'},
        'operating_system': {'key': 'operatingSystem', 'type': 'str'},
        'device_type': {'key': 'deviceType', 'type': 'str'},
        'compliance_state': {'key': 'complianceState', 'type': 'str'},
        'jail_broken': {'key': 'jailBroken', 'type': 'str'},
        'management_agent': {'key': 'managementAgent', 'type': 'str'},
        'os_version': {'key': 'osVersion', 'type': 'str'},
        'eas_activated': {'key': 'easActivated', 'type': 'bool'},
        'eas_device_id': {'key': 'easDeviceId', 'type': 'str'},
        'eas_activation_date_time': {'key': 'easActivationDateTime', 'type': 'iso-8601'},
        'aad_registered': {'key': 'aadRegistered', 'type': 'bool'},
        'azure_ad_registered': {'key': 'azureADRegistered', 'type': 'bool'},
        'device_enrollment_type': {'key': 'deviceEnrollmentType', 'type': 'str'},
        'lost_mode_state': {'key': 'lostModeState', 'type': 'str'},
        'activation_lock_bypass_code': {'key': 'activationLockBypassCode', 'type': 'str'},
        'email_address': {'key': 'emailAddress', 'type': 'str'},
        'azure_active_directory_device_id': {'key': 'azureActiveDirectoryDeviceId', 'type': 'str'},
        'azure_ad_device_id': {'key': 'azureADDeviceId', 'type': 'str'},
        'device_registration_state': {'key': 'deviceRegistrationState', 'type': 'str'},
        'device_category_display_name': {'key': 'deviceCategoryDisplayName', 'type': 'str'},
        'is_supervised': {'key': 'isSupervised', 'type': 'bool'},
        'exchange_last_successful_sync_date_time': {'key': 'exchangeLastSuccessfulSyncDateTime', 'type': 'iso-8601'},
        'exchange_access_state': {'key': 'exchangeAccessState', 'type': 'str'},
        'exchange_access_state_reason': {'key': 'exchangeAccessStateReason', 'type': 'str'},
        'remote_assistance_session_url': {'key': 'remoteAssistanceSessionUrl', 'type': 'str'},
        'remote_assistance_session_error_details': {'key': 'remoteAssistanceSessionErrorDetails', 'type': 'str'},
        'is_encrypted': {'key': 'isEncrypted', 'type': 'bool'},
        'user_principal_name': {'key': 'userPrincipalName', 'type': 'str'},
        'model': {'key': 'model', 'type': 'str'},
        'manufacturer': {'key': 'manufacturer', 'type': 'str'},
        'imei': {'key': 'imei', 'type': 'str'},
        'compliance_grace_period_expiration_date_time': {'key': 'complianceGracePeriodExpirationDateTime', 'type': 'iso-8601'},
        'serial_number': {'key': 'serialNumber', 'type': 'str'},
        'phone_number': {'key': 'phoneNumber', 'type': 'str'},
        'android_security_patch_level': {'key': 'androidSecurityPatchLevel', 'type': 'str'},
        'user_display_name': {'key': 'userDisplayName', 'type': 'str'},
        'configuration_manager_client_enabled_features': {'key': 'configurationManagerClientEnabledFeatures', 'type': 'MicrosoftGraphConfigurationManagerClientEnabledFeatures'},
        'wi_fi_mac_address': {'key': 'wiFiMacAddress', 'type': 'str'},
        'device_health_attestation_state': {'key': 'deviceHealthAttestationState', 'type': 'MicrosoftGraphDeviceHealthAttestationState'},
        'subscriber_carrier': {'key': 'subscriberCarrier', 'type': 'str'},
        'meid': {'key': 'meid', 'type': 'str'},
        'total_storage_space_in_bytes': {'key': 'totalStorageSpaceInBytes', 'type': 'long'},
        'free_storage_space_in_bytes': {'key': 'freeStorageSpaceInBytes', 'type': 'long'},
        'managed_device_name': {'key': 'managedDeviceName', 'type': 'str'},
        'partner_reported_threat_state': {'key': 'partnerReportedThreatState', 'type': 'str'},
        'retire_after_date_time': {'key': 'retireAfterDateTime', 'type': 'iso-8601'},
        'users_logged_on': {'key': 'usersLoggedOn', 'type': '[MicrosoftGraphLoggedOnUser]'},
        'prefer_mdm_over_group_policy_applied_date_time': {'key': 'preferMdmOverGroupPolicyAppliedDateTime', 'type': 'iso-8601'},
        'autopilot_enrolled': {'key': 'autopilotEnrolled', 'type': 'bool'},
        'require_user_enrollment_approval': {'key': 'requireUserEnrollmentApproval', 'type': 'bool'},
        'management_certificate_expiration_date': {'key': 'managementCertificateExpirationDate', 'type': 'iso-8601'},
        'iccid': {'key': 'iccid', 'type': 'str'},
        'udid': {'key': 'udid', 'type': 'str'},
        'role_scope_tag_ids': {'key': 'roleScopeTagIds', 'type': '[str]'},
        'windows_active_malware_count': {'key': 'windowsActiveMalwareCount', 'type': 'int'},
        'windows_remediated_malware_count': {'key': 'windowsRemediatedMalwareCount', 'type': 'int'},
        'notes': {'key': 'notes', 'type': 'str'},
        'configuration_manager_client_health_state': {'key': 'configurationManagerClientHealthState', 'type': 'MicrosoftGraphConfigurationManagerClientHealthState'},
        'configuration_manager_client_information': {'key': 'configurationManagerClientInformation', 'type': 'MicrosoftGraphConfigurationManagerClientInformation'},
        'ethernet_mac_address': {'key': 'ethernetMacAddress', 'type': 'str'},
        'physical_memory_in_bytes': {'key': 'physicalMemoryInBytes', 'type': 'long'},
        'processor_architecture': {'key': 'processorArchitecture', 'type': 'str'},
        'specification_version': {'key': 'specificationVersion', 'type': 'str'},
        'security_baseline_states': {'key': 'securityBaselineStates', 'type': '[MicrosoftGraphSecurityBaselineState]'},
        'device_configuration_states': {'key': 'deviceConfigurationStates', 'type': '[MicrosoftGraphDeviceConfigurationState]'},
        'device_compliance_policy_states': {'key': 'deviceCompliancePolicyStates', 'type': '[MicrosoftGraphDeviceCompliancePolicyState]'},
        'managed_device_mobile_app_configuration_states': {'key': 'managedDeviceMobileAppConfigurationStates', 'type': '[MicrosoftGraphManagedDeviceMobileAppConfigurationState]'},
        'detected_apps': {'key': 'detectedApps', 'type': '[MicrosoftGraphDetectedApp]'},
        'device_category': {'key': 'deviceCategory', 'type': 'MicrosoftGraphDeviceCategory'},
        'windows_protection_state': {'key': 'windowsProtectionState', 'type': 'MicrosoftGraphWindowsProtectionState'},
        'users': {'key': 'users', 'type': '[MicrosoftGraphUser]'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphManagedDevice, self).__init__(**kwargs)
        self.user_id = kwargs.get('user_id', None)
        self.device_name = kwargs.get('device_name', None)
        self.hardware_information = kwargs.get('hardware_information', None)
        self.owner_type = kwargs.get('owner_type', None)
        self.managed_device_owner_type = kwargs.get('managed_device_owner_type', None)
        self.device_action_results = kwargs.get('device_action_results', None)
        self.management_state = kwargs.get('management_state', None)
        self.enrolled_date_time = kwargs.get('enrolled_date_time', None)
        self.last_sync_date_time = kwargs.get('last_sync_date_time', None)
        self.chassis_type = kwargs.get('chassis_type', None)
        self.operating_system = kwargs.get('operating_system', None)
        self.device_type = kwargs.get('device_type', None)
        self.compliance_state = kwargs.get('compliance_state', None)
        self.jail_broken = kwargs.get('jail_broken', None)
        self.management_agent = kwargs.get('management_agent', None)
        self.os_version = kwargs.get('os_version', None)
        self.eas_activated = kwargs.get('eas_activated', None)
        self.eas_device_id = kwargs.get('eas_device_id', None)
        self.eas_activation_date_time = kwargs.get('eas_activation_date_time', None)
        self.aad_registered = kwargs.get('aad_registered', None)
        self.azure_ad_registered = kwargs.get('azure_ad_registered', None)
        self.device_enrollment_type = kwargs.get('device_enrollment_type', None)
        self.lost_mode_state = kwargs.get('lost_mode_state', None)
        self.activation_lock_bypass_code = kwargs.get('activation_lock_bypass_code', None)
        self.email_address = kwargs.get('email_address', None)
        self.azure_active_directory_device_id = kwargs.get('azure_active_directory_device_id', None)
        self.azure_ad_device_id = kwargs.get('azure_ad_device_id', None)
        self.device_registration_state = kwargs.get('device_registration_state', None)
        self.device_category_display_name = kwargs.get('device_category_display_name', None)
        self.is_supervised = kwargs.get('is_supervised', None)
        self.exchange_last_successful_sync_date_time = kwargs.get('exchange_last_successful_sync_date_time', None)
        self.exchange_access_state = kwargs.get('exchange_access_state', None)
        self.exchange_access_state_reason = kwargs.get('exchange_access_state_reason', None)
        self.remote_assistance_session_url = kwargs.get('remote_assistance_session_url', None)
        self.remote_assistance_session_error_details = kwargs.get('remote_assistance_session_error_details', None)
        self.is_encrypted = kwargs.get('is_encrypted', None)
        self.user_principal_name = kwargs.get('user_principal_name', None)
        self.model = kwargs.get('model', None)
        self.manufacturer = kwargs.get('manufacturer', None)
        self.imei = kwargs.get('imei', None)
        self.compliance_grace_period_expiration_date_time = kwargs.get('compliance_grace_period_expiration_date_time', None)
        self.serial_number = kwargs.get('serial_number', None)
        self.phone_number = kwargs.get('phone_number', None)
        self.android_security_patch_level = kwargs.get('android_security_patch_level', None)
        self.user_display_name = kwargs.get('user_display_name', None)
        self.configuration_manager_client_enabled_features = kwargs.get('configuration_manager_client_enabled_features', None)
        self.wi_fi_mac_address = kwargs.get('wi_fi_mac_address', None)
        self.device_health_attestation_state = kwargs.get('device_health_attestation_state', None)
        self.subscriber_carrier = kwargs.get('subscriber_carrier', None)
        self.meid = kwargs.get('meid', None)
        self.total_storage_space_in_bytes = kwargs.get('total_storage_space_in_bytes', None)
        self.free_storage_space_in_bytes = kwargs.get('free_storage_space_in_bytes', None)
        self.managed_device_name = kwargs.get('managed_device_name', None)
        self.partner_reported_threat_state = kwargs.get('partner_reported_threat_state', None)
        self.retire_after_date_time = kwargs.get('retire_after_date_time', None)
        self.users_logged_on = kwargs.get('users_logged_on', None)
        self.prefer_mdm_over_group_policy_applied_date_time = kwargs.get('prefer_mdm_over_group_policy_applied_date_time', None)
        self.autopilot_enrolled = kwargs.get('autopilot_enrolled', None)
        self.require_user_enrollment_approval = kwargs.get('require_user_enrollment_approval', None)
        self.management_certificate_expiration_date = kwargs.get('management_certificate_expiration_date', None)
        self.iccid = kwargs.get('iccid', None)
        self.udid = kwargs.get('udid', None)
        self.role_scope_tag_ids = kwargs.get('role_scope_tag_ids', None)
        self.windows_active_malware_count = kwargs.get('windows_active_malware_count', None)
        self.windows_remediated_malware_count = kwargs.get('windows_remediated_malware_count', None)
        self.notes = kwargs.get('notes', None)
        self.configuration_manager_client_health_state = kwargs.get('configuration_manager_client_health_state', None)
        self.configuration_manager_client_information = kwargs.get('configuration_manager_client_information', None)
        self.ethernet_mac_address = kwargs.get('ethernet_mac_address', None)
        self.physical_memory_in_bytes = kwargs.get('physical_memory_in_bytes', None)
        self.processor_architecture = kwargs.get('processor_architecture', None)
        self.specification_version = kwargs.get('specification_version', None)
        self.security_baseline_states = kwargs.get('security_baseline_states', None)
        self.device_configuration_states = kwargs.get('device_configuration_states', None)
        self.device_compliance_policy_states = kwargs.get('device_compliance_policy_states', None)
        self.managed_device_mobile_app_configuration_states = kwargs.get('managed_device_mobile_app_configuration_states', None)
        self.detected_apps = kwargs.get('detected_apps', None)
        self.device_category = kwargs.get('device_category', None)
        self.windows_protection_state = kwargs.get('windows_protection_state', None)
        self.users = kwargs.get('users', None)


class MicrosoftGraphManagedDeviceMobileAppConfigurationSettingState(msrest.serialization.Model):
    """managedDeviceMobileAppConfigurationSettingState.

    :param setting: The setting that is being reported.
    :type setting: str
    :param setting_name: Localized/user friendly setting name that is being reported.
    :type setting_name: str
    :param instance_display_name: Name of setting instance that is being reported.
    :type instance_display_name: str
    :param state: complianceStatus. Possible values include: "unknown", "notApplicable",
     "compliant", "remediated", "nonCompliant", "error", "conflict", "notAssigned".
    :type state: str or ~users_user.models.MicrosoftGraphComplianceStatus
    :param error_code: Error code for the setting.
    :type error_code: long
    :param error_description: Error description.
    :type error_description: str
    :param user_id: UserId.
    :type user_id: str
    :param user_name: UserName.
    :type user_name: str
    :param user_email: UserEmail.
    :type user_email: str
    :param user_principal_name: UserPrincipalName.
    :type user_principal_name: str
    :param sources: Contributing policies.
    :type sources: list[~users_user.models.MicrosoftGraphSettingSource]
    :param current_value: Current value of setting on device.
    :type current_value: str
    :param setting_instance_id: SettingInstanceId.
    :type setting_instance_id: str
    """

    _attribute_map = {
        'setting': {'key': 'setting', 'type': 'str'},
        'setting_name': {'key': 'settingName', 'type': 'str'},
        'instance_display_name': {'key': 'instanceDisplayName', 'type': 'str'},
        'state': {'key': 'state', 'type': 'str'},
        'error_code': {'key': 'errorCode', 'type': 'long'},
        'error_description': {'key': 'errorDescription', 'type': 'str'},
        'user_id': {'key': 'userId', 'type': 'str'},
        'user_name': {'key': 'userName', 'type': 'str'},
        'user_email': {'key': 'userEmail', 'type': 'str'},
        'user_principal_name': {'key': 'userPrincipalName', 'type': 'str'},
        'sources': {'key': 'sources', 'type': '[MicrosoftGraphSettingSource]'},
        'current_value': {'key': 'currentValue', 'type': 'str'},
        'setting_instance_id': {'key': 'settingInstanceId', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphManagedDeviceMobileAppConfigurationSettingState, self).__init__(**kwargs)
        self.setting = kwargs.get('setting', None)
        self.setting_name = kwargs.get('setting_name', None)
        self.instance_display_name = kwargs.get('instance_display_name', None)
        self.state = kwargs.get('state', None)
        self.error_code = kwargs.get('error_code', None)
        self.error_description = kwargs.get('error_description', None)
        self.user_id = kwargs.get('user_id', None)
        self.user_name = kwargs.get('user_name', None)
        self.user_email = kwargs.get('user_email', None)
        self.user_principal_name = kwargs.get('user_principal_name', None)
        self.sources = kwargs.get('sources', None)
        self.current_value = kwargs.get('current_value', None)
        self.setting_instance_id = kwargs.get('setting_instance_id', None)


class MicrosoftGraphManagedDeviceMobileAppConfigurationState(MicrosoftGraphEntity):
    """Managed Device Mobile App Configuration State for a given device.

    :param id: Read-only.
    :type id: str
    :param setting_states:
    :type setting_states:
     list[~users_user.models.MicrosoftGraphManagedDeviceMobileAppConfigurationSettingState]
    :param display_name: The name of the policy for this policyBase.
    :type display_name: str
    :param version: The version of the policy.
    :type version: int
    :param platform_type: policyPlatformType. Possible values include: "android", "androidForWork",
     "iOS", "macOS", "windowsPhone81", "windows81AndLater", "windows10AndLater",
     "androidWorkProfile", "windows10XProfile", "all".
    :type platform_type: str or ~users_user.models.MicrosoftGraphPolicyPlatformType
    :param state: complianceStatus. Possible values include: "unknown", "notApplicable",
     "compliant", "remediated", "nonCompliant", "error", "conflict", "notAssigned".
    :type state: str or ~users_user.models.MicrosoftGraphComplianceStatus
    :param setting_count: Count of how many setting a policy holds.
    :type setting_count: int
    :param user_id: User unique identifier, must be Guid.
    :type user_id: str
    :param user_principal_name: User Principal Name.
    :type user_principal_name: str
    """

    _validation = {
        'version': {'maximum': 2147483647, 'minimum': -2147483648},
        'setting_count': {'maximum': 2147483647, 'minimum': -2147483648},
    }

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'setting_states': {'key': 'settingStates', 'type': '[MicrosoftGraphManagedDeviceMobileAppConfigurationSettingState]'},
        'display_name': {'key': 'displayName', 'type': 'str'},
        'version': {'key': 'version', 'type': 'int'},
        'platform_type': {'key': 'platformType', 'type': 'str'},
        'state': {'key': 'state', 'type': 'str'},
        'setting_count': {'key': 'settingCount', 'type': 'int'},
        'user_id': {'key': 'userId', 'type': 'str'},
        'user_principal_name': {'key': 'userPrincipalName', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphManagedDeviceMobileAppConfigurationState, self).__init__(**kwargs)
        self.setting_states = kwargs.get('setting_states', None)
        self.display_name = kwargs.get('display_name', None)
        self.version = kwargs.get('version', None)
        self.platform_type = kwargs.get('platform_type', None)
        self.state = kwargs.get('state', None)
        self.setting_count = kwargs.get('setting_count', None)
        self.user_id = kwargs.get('user_id', None)
        self.user_principal_name = kwargs.get('user_principal_name', None)


class MicrosoftGraphMeetingParticipantInfo(msrest.serialization.Model):
    """meetingParticipantInfo.

    :param identity: identitySet.
    :type identity: ~users_user.models.MicrosoftGraphIdentitySet
    :param upn: User principal name of the participant.
    :type upn: str
    """

    _attribute_map = {
        'identity': {'key': 'identity', 'type': 'MicrosoftGraphIdentitySet'},
        'upn': {'key': 'upn', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphMeetingParticipantInfo, self).__init__(**kwargs)
        self.identity = kwargs.get('identity', None)
        self.upn = kwargs.get('upn', None)


class MicrosoftGraphMeetingParticipants(msrest.serialization.Model):
    """meetingParticipants.

    :param organizer: meetingParticipantInfo.
    :type organizer: ~users_user.models.MicrosoftGraphMeetingParticipantInfo
    :param attendees:
    :type attendees: list[~users_user.models.MicrosoftGraphMeetingParticipantInfo]
    :param producers:
    :type producers: list[~users_user.models.MicrosoftGraphMeetingParticipantInfo]
    :param contributors:
    :type contributors: list[~users_user.models.MicrosoftGraphMeetingParticipantInfo]
    """

    _attribute_map = {
        'organizer': {'key': 'organizer', 'type': 'MicrosoftGraphMeetingParticipantInfo'},
        'attendees': {'key': 'attendees', 'type': '[MicrosoftGraphMeetingParticipantInfo]'},
        'producers': {'key': 'producers', 'type': '[MicrosoftGraphMeetingParticipantInfo]'},
        'contributors': {'key': 'contributors', 'type': '[MicrosoftGraphMeetingParticipantInfo]'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphMeetingParticipants, self).__init__(**kwargs)
        self.organizer = kwargs.get('organizer', None)
        self.attendees = kwargs.get('attendees', None)
        self.producers = kwargs.get('producers', None)
        self.contributors = kwargs.get('contributors', None)


class MicrosoftGraphMention(MicrosoftGraphEntity):
    """mention.

    :param id: Read-only.
    :type id: str
    :param mentioned: emailAddress.
    :type mentioned: ~users_user.models.MicrosoftGraphEmailAddress
    :param mention_text:
    :type mention_text: str
    :param client_reference:
    :type client_reference: str
    :param created_by: emailAddress.
    :type created_by: ~users_user.models.MicrosoftGraphEmailAddress
    :param created_date_time:
    :type created_date_time: ~datetime.datetime
    :param server_created_date_time:
    :type server_created_date_time: ~datetime.datetime
    :param deep_link:
    :type deep_link: str
    :param application:
    :type application: str
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'mentioned': {'key': 'mentioned', 'type': 'MicrosoftGraphEmailAddress'},
        'mention_text': {'key': 'mentionText', 'type': 'str'},
        'client_reference': {'key': 'clientReference', 'type': 'str'},
        'created_by': {'key': 'createdBy', 'type': 'MicrosoftGraphEmailAddress'},
        'created_date_time': {'key': 'createdDateTime', 'type': 'iso-8601'},
        'server_created_date_time': {'key': 'serverCreatedDateTime', 'type': 'iso-8601'},
        'deep_link': {'key': 'deepLink', 'type': 'str'},
        'application': {'key': 'application', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphMention, self).__init__(**kwargs)
        self.mentioned = kwargs.get('mentioned', None)
        self.mention_text = kwargs.get('mention_text', None)
        self.client_reference = kwargs.get('client_reference', None)
        self.created_by = kwargs.get('created_by', None)
        self.created_date_time = kwargs.get('created_date_time', None)
        self.server_created_date_time = kwargs.get('server_created_date_time', None)
        self.deep_link = kwargs.get('deep_link', None)
        self.application = kwargs.get('application', None)


class MicrosoftGraphMentionAction(msrest.serialization.Model):
    """mentionAction.

    :param mentionees: The identities of the users mentioned in this action.
    :type mentionees: list[~users_user.models.MicrosoftGraphIdentitySet]
    """

    _attribute_map = {
        'mentionees': {'key': 'mentionees', 'type': '[MicrosoftGraphIdentitySet]'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphMentionAction, self).__init__(**kwargs)
        self.mentionees = kwargs.get('mentionees', None)


class MicrosoftGraphMentionsPreview(msrest.serialization.Model):
    """mentionsPreview.

    :param is_mentioned:
    :type is_mentioned: bool
    """

    _attribute_map = {
        'is_mentioned': {'key': 'isMentioned', 'type': 'bool'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphMentionsPreview, self).__init__(**kwargs)
        self.is_mentioned = kwargs.get('is_mentioned', None)


class MicrosoftGraphMessage(MicrosoftGraphOutlookItem):
    """message.

    :param id: Read-only.
    :type id: str
    :param created_date_time: The Timestamp type represents date and time information using ISO
     8601 format and is always in UTC time. For example, midnight UTC on Jan 1, 2014 would look like
     this: '2014-01-01T00:00:00Z'.
    :type created_date_time: ~datetime.datetime
    :param last_modified_date_time: The Timestamp type represents date and time information using
     ISO 8601 format and is always in UTC time. For example, midnight UTC on Jan 1, 2014 would look
     like this: '2014-01-01T00:00:00Z'.
    :type last_modified_date_time: ~datetime.datetime
    :param change_key: Identifies the version of the item. Every time the item is changed,
     changeKey changes as well. This allows Exchange to apply changes to the correct version of the
     object. Read-only.
    :type change_key: str
    :param categories: The categories associated with the item.
    :type categories: list[str]
    :param received_date_time: The date and time the message was received.
    :type received_date_time: ~datetime.datetime
    :param sent_date_time: The date and time the message was sent.
    :type sent_date_time: ~datetime.datetime
    :param has_attachments: Indicates whether the message has attachments. This property doesn't
     include inline attachments, so if a message contains only inline attachments, this property is
     false. To verify the existence of inline attachments, parse the body property to look for a src
     attribute, such as :code:`<IMG src='cid:image001.jpg@01D26CD8.6C05F070'>`.
    :type has_attachments: bool
    :param internet_message_id: The message ID in the format specified by RFC2822.
    :type internet_message_id: str
    :param internet_message_headers: A collection of message headers defined by RFC5322. The set
     includes message headers indicating the network path taken by a message from the sender to the
     recipient. It can also contain custom message headers that hold app data for the message.
     Returned only on applying a $select query option. Read-only.
    :type internet_message_headers: list[~users_user.models.MicrosoftGraphInternetMessageHeader]
    :param subject: The subject of the message.
    :type subject: str
    :param body: itemBody.
    :type body: ~users_user.models.MicrosoftGraphItemBody
    :param body_preview: The first 255 characters of the message body. It is in text format.
    :type body_preview: str
    :param importance: importance. Possible values include: "low", "normal", "high".
    :type importance: str or ~users_user.models.MicrosoftGraphImportance
    :param parent_folder_id: The unique identifier for the message's parent mailFolder.
    :type parent_folder_id: str
    :param sender: recipient.
    :type sender: ~users_user.models.MicrosoftGraphRecipient
    :param from_property: recipient.
    :type from_property: ~users_user.models.MicrosoftGraphRecipient
    :param to_recipients: The To: recipients for the message.
    :type to_recipients: list[~users_user.models.MicrosoftGraphRecipient]
    :param cc_recipients: The Cc: recipients for the message.
    :type cc_recipients: list[~users_user.models.MicrosoftGraphRecipient]
    :param bcc_recipients: The Bcc: recipients for the message.
    :type bcc_recipients: list[~users_user.models.MicrosoftGraphRecipient]
    :param reply_to: The email addresses to use when replying.
    :type reply_to: list[~users_user.models.MicrosoftGraphRecipient]
    :param conversation_id: The ID of the conversation the email belongs to.
    :type conversation_id: str
    :param conversation_index: Indicates the position of the message within the conversation.
    :type conversation_index: bytes
    :param unique_body: itemBody.
    :type unique_body: ~users_user.models.MicrosoftGraphItemBody
    :param is_delivery_receipt_requested: Indicates whether a read receipt is requested for the
     message.
    :type is_delivery_receipt_requested: bool
    :param is_read_receipt_requested: Indicates whether a read receipt is requested for the
     message.
    :type is_read_receipt_requested: bool
    :param is_read: Indicates whether the message has been read.
    :type is_read: bool
    :param is_draft: Indicates whether the message is a draft. A message is a draft if it hasn't
     been sent yet.
    :type is_draft: bool
    :param web_link: The URL to open the message in Outlook Web App.You can append an ispopout
     argument to the end of the URL to change how the message is displayed. If ispopout is not
     present or if it is set to 1, then the message is shown in a popout window. If ispopout is set
     to 0, then the browser will show the message in the Outlook Web App review pane.The message
     will open in the browser if you are logged in to your mailbox via Outlook Web App. You will be
     prompted to login if you are not already logged in with the browser.This URL can be accessed
     from within an iFrame.
    :type web_link: str
    :param mentions_preview: mentionsPreview.
    :type mentions_preview: ~users_user.models.MicrosoftGraphMentionsPreview
    :param inference_classification: inferenceClassificationType. Possible values include:
     "focused", "other".
    :type inference_classification: str or
     ~users_user.models.MicrosoftGraphInferenceClassificationType
    :param unsubscribe_data:
    :type unsubscribe_data: list[str]
    :param unsubscribe_enabled:
    :type unsubscribe_enabled: bool
    :param flag: followupFlag.
    :type flag: ~users_user.models.MicrosoftGraphFollowupFlag
    :param single_value_extended_properties: The collection of single-value extended properties
     defined for the message. Nullable.
    :type single_value_extended_properties:
     list[~users_user.models.MicrosoftGraphSingleValueLegacyExtendedProperty]
    :param multi_value_extended_properties: The collection of multi-value extended properties
     defined for the message. Nullable.
    :type multi_value_extended_properties:
     list[~users_user.models.MicrosoftGraphMultiValueLegacyExtendedProperty]
    :param attachments: The fileAttachment and itemAttachment attachments for the message.
    :type attachments: list[~users_user.models.MicrosoftGraphAttachment]
    :param extensions: The collection of open extensions defined for the message. Nullable.
    :type extensions: list[~users_user.models.MicrosoftGraphEntity]
    :param mentions:
    :type mentions: list[~users_user.models.MicrosoftGraphMention]
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'created_date_time': {'key': 'createdDateTime', 'type': 'iso-8601'},
        'last_modified_date_time': {'key': 'lastModifiedDateTime', 'type': 'iso-8601'},
        'change_key': {'key': 'changeKey', 'type': 'str'},
        'categories': {'key': 'categories', 'type': '[str]'},
        'received_date_time': {'key': 'receivedDateTime', 'type': 'iso-8601'},
        'sent_date_time': {'key': 'sentDateTime', 'type': 'iso-8601'},
        'has_attachments': {'key': 'hasAttachments', 'type': 'bool'},
        'internet_message_id': {'key': 'internetMessageId', 'type': 'str'},
        'internet_message_headers': {'key': 'internetMessageHeaders', 'type': '[MicrosoftGraphInternetMessageHeader]'},
        'subject': {'key': 'subject', 'type': 'str'},
        'body': {'key': 'body', 'type': 'MicrosoftGraphItemBody'},
        'body_preview': {'key': 'bodyPreview', 'type': 'str'},
        'importance': {'key': 'importance', 'type': 'str'},
        'parent_folder_id': {'key': 'parentFolderId', 'type': 'str'},
        'sender': {'key': 'sender', 'type': 'MicrosoftGraphRecipient'},
        'from_property': {'key': 'from', 'type': 'MicrosoftGraphRecipient'},
        'to_recipients': {'key': 'toRecipients', 'type': '[MicrosoftGraphRecipient]'},
        'cc_recipients': {'key': 'ccRecipients', 'type': '[MicrosoftGraphRecipient]'},
        'bcc_recipients': {'key': 'bccRecipients', 'type': '[MicrosoftGraphRecipient]'},
        'reply_to': {'key': 'replyTo', 'type': '[MicrosoftGraphRecipient]'},
        'conversation_id': {'key': 'conversationId', 'type': 'str'},
        'conversation_index': {'key': 'conversationIndex', 'type': 'base64'},
        'unique_body': {'key': 'uniqueBody', 'type': 'MicrosoftGraphItemBody'},
        'is_delivery_receipt_requested': {'key': 'isDeliveryReceiptRequested', 'type': 'bool'},
        'is_read_receipt_requested': {'key': 'isReadReceiptRequested', 'type': 'bool'},
        'is_read': {'key': 'isRead', 'type': 'bool'},
        'is_draft': {'key': 'isDraft', 'type': 'bool'},
        'web_link': {'key': 'webLink', 'type': 'str'},
        'mentions_preview': {'key': 'mentionsPreview', 'type': 'MicrosoftGraphMentionsPreview'},
        'inference_classification': {'key': 'inferenceClassification', 'type': 'str'},
        'unsubscribe_data': {'key': 'unsubscribeData', 'type': '[str]'},
        'unsubscribe_enabled': {'key': 'unsubscribeEnabled', 'type': 'bool'},
        'flag': {'key': 'flag', 'type': 'MicrosoftGraphFollowupFlag'},
        'single_value_extended_properties': {'key': 'singleValueExtendedProperties', 'type': '[MicrosoftGraphSingleValueLegacyExtendedProperty]'},
        'multi_value_extended_properties': {'key': 'multiValueExtendedProperties', 'type': '[MicrosoftGraphMultiValueLegacyExtendedProperty]'},
        'attachments': {'key': 'attachments', 'type': '[MicrosoftGraphAttachment]'},
        'extensions': {'key': 'extensions', 'type': '[MicrosoftGraphEntity]'},
        'mentions': {'key': 'mentions', 'type': '[MicrosoftGraphMention]'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphMessage, self).__init__(**kwargs)
        self.received_date_time = kwargs.get('received_date_time', None)
        self.sent_date_time = kwargs.get('sent_date_time', None)
        self.has_attachments = kwargs.get('has_attachments', None)
        self.internet_message_id = kwargs.get('internet_message_id', None)
        self.internet_message_headers = kwargs.get('internet_message_headers', None)
        self.subject = kwargs.get('subject', None)
        self.body = kwargs.get('body', None)
        self.body_preview = kwargs.get('body_preview', None)
        self.importance = kwargs.get('importance', None)
        self.parent_folder_id = kwargs.get('parent_folder_id', None)
        self.sender = kwargs.get('sender', None)
        self.from_property = kwargs.get('from_property', None)
        self.to_recipients = kwargs.get('to_recipients', None)
        self.cc_recipients = kwargs.get('cc_recipients', None)
        self.bcc_recipients = kwargs.get('bcc_recipients', None)
        self.reply_to = kwargs.get('reply_to', None)
        self.conversation_id = kwargs.get('conversation_id', None)
        self.conversation_index = kwargs.get('conversation_index', None)
        self.unique_body = kwargs.get('unique_body', None)
        self.is_delivery_receipt_requested = kwargs.get('is_delivery_receipt_requested', None)
        self.is_read_receipt_requested = kwargs.get('is_read_receipt_requested', None)
        self.is_read = kwargs.get('is_read', None)
        self.is_draft = kwargs.get('is_draft', None)
        self.web_link = kwargs.get('web_link', None)
        self.mentions_preview = kwargs.get('mentions_preview', None)
        self.inference_classification = kwargs.get('inference_classification', None)
        self.unsubscribe_data = kwargs.get('unsubscribe_data', None)
        self.unsubscribe_enabled = kwargs.get('unsubscribe_enabled', None)
        self.flag = kwargs.get('flag', None)
        self.single_value_extended_properties = kwargs.get('single_value_extended_properties', None)
        self.multi_value_extended_properties = kwargs.get('multi_value_extended_properties', None)
        self.attachments = kwargs.get('attachments', None)
        self.extensions = kwargs.get('extensions', None)
        self.mentions = kwargs.get('mentions', None)


class MicrosoftGraphMessageRule(MicrosoftGraphEntity):
    """messageRule.

    :param id: Read-only.
    :type id: str
    :param display_name: The display name of the rule.
    :type display_name: str
    :param sequence: Indicates the order in which the rule is executed, among other rules.
    :type sequence: int
    :param conditions: messageRulePredicates.
    :type conditions: ~users_user.models.MicrosoftGraphMessageRulePredicates
    :param actions: messageRuleActions.
    :type actions: ~users_user.models.MicrosoftGraphMessageRuleActions
    :param exceptions: messageRulePredicates.
    :type exceptions: ~users_user.models.MicrosoftGraphMessageRulePredicates
    :param is_enabled: Indicates whether the rule is enabled to be applied to messages.
    :type is_enabled: bool
    :param has_error: Indicates whether the rule is in an error condition. Read-only.
    :type has_error: bool
    :param is_read_only: Indicates if the rule is read-only and cannot be modified or deleted by
     the rules REST API.
    :type is_read_only: bool
    """

    _validation = {
        'sequence': {'maximum': 2147483647, 'minimum': -2147483648},
    }

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'display_name': {'key': 'displayName', 'type': 'str'},
        'sequence': {'key': 'sequence', 'type': 'int'},
        'conditions': {'key': 'conditions', 'type': 'MicrosoftGraphMessageRulePredicates'},
        'actions': {'key': 'actions', 'type': 'MicrosoftGraphMessageRuleActions'},
        'exceptions': {'key': 'exceptions', 'type': 'MicrosoftGraphMessageRulePredicates'},
        'is_enabled': {'key': 'isEnabled', 'type': 'bool'},
        'has_error': {'key': 'hasError', 'type': 'bool'},
        'is_read_only': {'key': 'isReadOnly', 'type': 'bool'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphMessageRule, self).__init__(**kwargs)
        self.display_name = kwargs.get('display_name', None)
        self.sequence = kwargs.get('sequence', None)
        self.conditions = kwargs.get('conditions', None)
        self.actions = kwargs.get('actions', None)
        self.exceptions = kwargs.get('exceptions', None)
        self.is_enabled = kwargs.get('is_enabled', None)
        self.has_error = kwargs.get('has_error', None)
        self.is_read_only = kwargs.get('is_read_only', None)


class MicrosoftGraphMessageRuleActions(msrest.serialization.Model):
    """messageRuleActions.

    :param move_to_folder: The ID of the folder that a message will be moved to.
    :type move_to_folder: str
    :param copy_to_folder: The ID of a folder that a message is to be copied to.
    :type copy_to_folder: str
    :param delete: Indicates whether a message should be moved to the Deleted Items folder.
    :type delete: bool
    :param permanent_delete: Indicates whether a message should be permanently deleted and not
     saved to the Deleted Items folder.
    :type permanent_delete: bool
    :param mark_as_read: Indicates whether a message should be marked as read.
    :type mark_as_read: bool
    :param mark_importance: importance. Possible values include: "low", "normal", "high".
    :type mark_importance: str or ~users_user.models.MicrosoftGraphImportance
    :param forward_to: The email addresses of the recipients to which a message should be
     forwarded.
    :type forward_to: list[~users_user.models.MicrosoftGraphRecipient]
    :param forward_as_attachment_to: The email addresses of the recipients to which a message
     should be forwarded as an attachment.
    :type forward_as_attachment_to: list[~users_user.models.MicrosoftGraphRecipient]
    :param redirect_to: The email addresses to which a message should be redirected.
    :type redirect_to: list[~users_user.models.MicrosoftGraphRecipient]
    :param assign_categories: A list of categories to be assigned to a message.
    :type assign_categories: list[str]
    :param stop_processing_rules: Indicates whether subsequent rules should be evaluated.
    :type stop_processing_rules: bool
    """

    _attribute_map = {
        'move_to_folder': {'key': 'moveToFolder', 'type': 'str'},
        'copy_to_folder': {'key': 'copyToFolder', 'type': 'str'},
        'delete': {'key': 'delete', 'type': 'bool'},
        'permanent_delete': {'key': 'permanentDelete', 'type': 'bool'},
        'mark_as_read': {'key': 'markAsRead', 'type': 'bool'},
        'mark_importance': {'key': 'markImportance', 'type': 'str'},
        'forward_to': {'key': 'forwardTo', 'type': '[MicrosoftGraphRecipient]'},
        'forward_as_attachment_to': {'key': 'forwardAsAttachmentTo', 'type': '[MicrosoftGraphRecipient]'},
        'redirect_to': {'key': 'redirectTo', 'type': '[MicrosoftGraphRecipient]'},
        'assign_categories': {'key': 'assignCategories', 'type': '[str]'},
        'stop_processing_rules': {'key': 'stopProcessingRules', 'type': 'bool'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphMessageRuleActions, self).__init__(**kwargs)
        self.move_to_folder = kwargs.get('move_to_folder', None)
        self.copy_to_folder = kwargs.get('copy_to_folder', None)
        self.delete = kwargs.get('delete', None)
        self.permanent_delete = kwargs.get('permanent_delete', None)
        self.mark_as_read = kwargs.get('mark_as_read', None)
        self.mark_importance = kwargs.get('mark_importance', None)
        self.forward_to = kwargs.get('forward_to', None)
        self.forward_as_attachment_to = kwargs.get('forward_as_attachment_to', None)
        self.redirect_to = kwargs.get('redirect_to', None)
        self.assign_categories = kwargs.get('assign_categories', None)
        self.stop_processing_rules = kwargs.get('stop_processing_rules', None)


class MicrosoftGraphMessageRulePredicates(msrest.serialization.Model):
    """messageRulePredicates.

    :param categories: Represents the categories that an incoming message should be labeled with in
     order for the condition or exception to apply.
    :type categories: list[str]
    :param subject_contains: Represents the strings that appear in the subject of an incoming
     message in order for the condition or exception to apply.
    :type subject_contains: list[str]
    :param body_contains: Represents the strings that should appear in the body of an incoming
     message in order for the condition or exception to apply.
    :type body_contains: list[str]
    :param body_or_subject_contains: Represents the strings that should appear in the body or
     subject of an incoming message in order for the condition or exception to apply.
    :type body_or_subject_contains: list[str]
    :param sender_contains: Represents the strings that appear in the from property of an incoming
     message in order for the condition or exception to apply.
    :type sender_contains: list[str]
    :param recipient_contains: Represents the strings that appear in either the toRecipients or
     ccRecipients properties of an incoming message in order for the condition or exception to
     apply.
    :type recipient_contains: list[str]
    :param header_contains: Represents the strings that appear in the headers of an incoming
     message in order for the condition or exception to apply.
    :type header_contains: list[str]
    :param message_action_flag: messageActionFlag. Possible values include: "any", "call",
     "doNotForward", "followUp", "fyi", "forward", "noResponseNecessary", "read", "reply",
     "replyToAll", "review".
    :type message_action_flag: str or ~users_user.models.MicrosoftGraphMessageActionFlag
    :param importance: importance. Possible values include: "low", "normal", "high".
    :type importance: str or ~users_user.models.MicrosoftGraphImportance
    :param sensitivity: sensitivity. Possible values include: "normal", "personal", "private",
     "confidential".
    :type sensitivity: str or ~users_user.models.MicrosoftGraphSensitivity
    :param from_addresses: Represents the specific sender email addresses of an incoming message in
     order for the condition or exception to apply.
    :type from_addresses: list[~users_user.models.MicrosoftGraphRecipient]
    :param sent_to_addresses: Represents the email addresses that an incoming message must have
     been sent to in order for the condition or exception to apply.
    :type sent_to_addresses: list[~users_user.models.MicrosoftGraphRecipient]
    :param sent_to_me: Indicates whether the owner of the mailbox must be in the toRecipients
     property of an incoming message in order for the condition or exception to apply.
    :type sent_to_me: bool
    :param sent_only_to_me: Indicates whether the owner of the mailbox must be the only recipient
     in an incoming message in order for the condition or exception to apply.
    :type sent_only_to_me: bool
    :param sent_cc_me: Indicates whether the owner of the mailbox must be in the ccRecipients
     property of an incoming message in order for the condition or exception to apply.
    :type sent_cc_me: bool
    :param sent_to_or_cc_me: Indicates whether the owner of the mailbox must be in either a
     toRecipients or ccRecipients property of an incoming message in order for the condition or
     exception to apply.
    :type sent_to_or_cc_me: bool
    :param not_sent_to_me: Indicates whether the owner of the mailbox must not be a recipient of an
     incoming message in order for the condition or exception to apply.
    :type not_sent_to_me: bool
    :param has_attachments: Indicates whether an incoming message must have attachments in order
     for the condition or exception to apply.
    :type has_attachments: bool
    :param is_approval_request: Indicates whether an incoming message must be an approval request
     in order for the condition or exception to apply.
    :type is_approval_request: bool
    :param is_automatic_forward: Indicates whether an incoming message must be automatically
     forwarded in order for the condition or exception to apply.
    :type is_automatic_forward: bool
    :param is_automatic_reply: Indicates whether an incoming message must be an auto reply in order
     for the condition or exception to apply.
    :type is_automatic_reply: bool
    :param is_encrypted: Indicates whether an incoming message must be encrypted in order for the
     condition or exception to apply.
    :type is_encrypted: bool
    :param is_meeting_request: Indicates whether an incoming message must be a meeting request in
     order for the condition or exception to apply.
    :type is_meeting_request: bool
    :param is_meeting_response: Indicates whether an incoming message must be a meeting response in
     order for the condition or exception to apply.
    :type is_meeting_response: bool
    :param is_non_delivery_report: Indicates whether an incoming message must be a non-delivery
     report in order for the condition or exception to apply.
    :type is_non_delivery_report: bool
    :param is_permission_controlled: Indicates whether an incoming message must be permission
     controlled (RMS-protected) in order for the condition or exception to apply.
    :type is_permission_controlled: bool
    :param is_read_receipt: Indicates whether an incoming message must be a read receipt in order
     for the condition or exception to apply.
    :type is_read_receipt: bool
    :param is_signed: Indicates whether an incoming message must be S/MIME-signed in order for the
     condition or exception to apply.
    :type is_signed: bool
    :param is_voicemail: Indicates whether an incoming message must be a voice mail in order for
     the condition or exception to apply.
    :type is_voicemail: bool
    :param within_size_range: sizeRange.
    :type within_size_range: ~users_user.models.MicrosoftGraphSizeRange
    """

    _attribute_map = {
        'categories': {'key': 'categories', 'type': '[str]'},
        'subject_contains': {'key': 'subjectContains', 'type': '[str]'},
        'body_contains': {'key': 'bodyContains', 'type': '[str]'},
        'body_or_subject_contains': {'key': 'bodyOrSubjectContains', 'type': '[str]'},
        'sender_contains': {'key': 'senderContains', 'type': '[str]'},
        'recipient_contains': {'key': 'recipientContains', 'type': '[str]'},
        'header_contains': {'key': 'headerContains', 'type': '[str]'},
        'message_action_flag': {'key': 'messageActionFlag', 'type': 'str'},
        'importance': {'key': 'importance', 'type': 'str'},
        'sensitivity': {'key': 'sensitivity', 'type': 'str'},
        'from_addresses': {'key': 'fromAddresses', 'type': '[MicrosoftGraphRecipient]'},
        'sent_to_addresses': {'key': 'sentToAddresses', 'type': '[MicrosoftGraphRecipient]'},
        'sent_to_me': {'key': 'sentToMe', 'type': 'bool'},
        'sent_only_to_me': {'key': 'sentOnlyToMe', 'type': 'bool'},
        'sent_cc_me': {'key': 'sentCcMe', 'type': 'bool'},
        'sent_to_or_cc_me': {'key': 'sentToOrCcMe', 'type': 'bool'},
        'not_sent_to_me': {'key': 'notSentToMe', 'type': 'bool'},
        'has_attachments': {'key': 'hasAttachments', 'type': 'bool'},
        'is_approval_request': {'key': 'isApprovalRequest', 'type': 'bool'},
        'is_automatic_forward': {'key': 'isAutomaticForward', 'type': 'bool'},
        'is_automatic_reply': {'key': 'isAutomaticReply', 'type': 'bool'},
        'is_encrypted': {'key': 'isEncrypted', 'type': 'bool'},
        'is_meeting_request': {'key': 'isMeetingRequest', 'type': 'bool'},
        'is_meeting_response': {'key': 'isMeetingResponse', 'type': 'bool'},
        'is_non_delivery_report': {'key': 'isNonDeliveryReport', 'type': 'bool'},
        'is_permission_controlled': {'key': 'isPermissionControlled', 'type': 'bool'},
        'is_read_receipt': {'key': 'isReadReceipt', 'type': 'bool'},
        'is_signed': {'key': 'isSigned', 'type': 'bool'},
        'is_voicemail': {'key': 'isVoicemail', 'type': 'bool'},
        'within_size_range': {'key': 'withinSizeRange', 'type': 'MicrosoftGraphSizeRange'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphMessageRulePredicates, self).__init__(**kwargs)
        self.categories = kwargs.get('categories', None)
        self.subject_contains = kwargs.get('subject_contains', None)
        self.body_contains = kwargs.get('body_contains', None)
        self.body_or_subject_contains = kwargs.get('body_or_subject_contains', None)
        self.sender_contains = kwargs.get('sender_contains', None)
        self.recipient_contains = kwargs.get('recipient_contains', None)
        self.header_contains = kwargs.get('header_contains', None)
        self.message_action_flag = kwargs.get('message_action_flag', None)
        self.importance = kwargs.get('importance', None)
        self.sensitivity = kwargs.get('sensitivity', None)
        self.from_addresses = kwargs.get('from_addresses', None)
        self.sent_to_addresses = kwargs.get('sent_to_addresses', None)
        self.sent_to_me = kwargs.get('sent_to_me', None)
        self.sent_only_to_me = kwargs.get('sent_only_to_me', None)
        self.sent_cc_me = kwargs.get('sent_cc_me', None)
        self.sent_to_or_cc_me = kwargs.get('sent_to_or_cc_me', None)
        self.not_sent_to_me = kwargs.get('not_sent_to_me', None)
        self.has_attachments = kwargs.get('has_attachments', None)
        self.is_approval_request = kwargs.get('is_approval_request', None)
        self.is_automatic_forward = kwargs.get('is_automatic_forward', None)
        self.is_automatic_reply = kwargs.get('is_automatic_reply', None)
        self.is_encrypted = kwargs.get('is_encrypted', None)
        self.is_meeting_request = kwargs.get('is_meeting_request', None)
        self.is_meeting_response = kwargs.get('is_meeting_response', None)
        self.is_non_delivery_report = kwargs.get('is_non_delivery_report', None)
        self.is_permission_controlled = kwargs.get('is_permission_controlled', None)
        self.is_read_receipt = kwargs.get('is_read_receipt', None)
        self.is_signed = kwargs.get('is_signed', None)
        self.is_voicemail = kwargs.get('is_voicemail', None)
        self.within_size_range = kwargs.get('within_size_range', None)


class MicrosoftGraphMobileAppIntentAndState(MicrosoftGraphEntity):
    """MobileApp Intent and Install State for a given device.

    :param id: Read-only.
    :type id: str
    :param managed_device_identifier: Device identifier created or collected by Intune.
    :type managed_device_identifier: str
    :param user_id: Identifier for the user that tried to enroll the device.
    :type user_id: str
    :param mobile_app_list: The list of payload intents and states for the tenant.
    :type mobile_app_list: list[~users_user.models.MicrosoftGraphMobileAppIntentAndStateDetail]
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'managed_device_identifier': {'key': 'managedDeviceIdentifier', 'type': 'str'},
        'user_id': {'key': 'userId', 'type': 'str'},
        'mobile_app_list': {'key': 'mobileAppList', 'type': '[MicrosoftGraphMobileAppIntentAndStateDetail]'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphMobileAppIntentAndState, self).__init__(**kwargs)
        self.managed_device_identifier = kwargs.get('managed_device_identifier', None)
        self.user_id = kwargs.get('user_id', None)
        self.mobile_app_list = kwargs.get('mobile_app_list', None)


class MicrosoftGraphMobileAppIntentAndStateDetail(msrest.serialization.Model):
    """mobileAppIntentAndStateDetail.

    :param application_id: MobieApp identifier.
    :type application_id: str
    :param display_name: The admin provided or imported title of the app.
    :type display_name: str
    :param mobile_app_intent: mobileAppIntent. Possible values include: "available",
     "notAvailable", "requiredInstall", "requiredUninstall", "requiredAndAvailableInstall",
     "availableInstallWithoutEnrollment", "exclude".
    :type mobile_app_intent: str or ~users_user.models.MicrosoftGraphMobileAppIntent
    :param display_version: Human readable version of the application.
    :type display_version: str
    :param install_state: resultantAppState. Possible values include: "installed", "failed",
     "notInstalled", "uninstallFailed", "pendingInstall", "unknown", "notApplicable".
    :type install_state: str or ~users_user.models.MicrosoftGraphResultantAppState
    :param supported_device_types: The supported platforms for the app.
    :type supported_device_types:
     list[~users_user.models.MicrosoftGraphMobileAppSupportedDeviceType]
    """

    _attribute_map = {
        'application_id': {'key': 'applicationId', 'type': 'str'},
        'display_name': {'key': 'displayName', 'type': 'str'},
        'mobile_app_intent': {'key': 'mobileAppIntent', 'type': 'str'},
        'display_version': {'key': 'displayVersion', 'type': 'str'},
        'install_state': {'key': 'installState', 'type': 'str'},
        'supported_device_types': {'key': 'supportedDeviceTypes', 'type': '[MicrosoftGraphMobileAppSupportedDeviceType]'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphMobileAppIntentAndStateDetail, self).__init__(**kwargs)
        self.application_id = kwargs.get('application_id', None)
        self.display_name = kwargs.get('display_name', None)
        self.mobile_app_intent = kwargs.get('mobile_app_intent', None)
        self.display_version = kwargs.get('display_version', None)
        self.install_state = kwargs.get('install_state', None)
        self.supported_device_types = kwargs.get('supported_device_types', None)


class MicrosoftGraphMobileAppSupportedDeviceType(msrest.serialization.Model):
    """mobileAppSupportedDeviceType.

    :param type: deviceType. Possible values include: "desktop", "windowsRT", "winMO6", "nokia",
     "windowsPhone", "mac", "winCE", "winEmbedded", "iPhone", "iPad", "iPod", "android",
     "iSocConsumer", "unix", "macMDM", "holoLens", "surfaceHub", "androidForWork",
     "androidEnterprise", "windows10x", "blackberry", "palm", "unknown".
    :type type: str or ~users_user.models.MicrosoftGraphDeviceType
    :param minimum_operating_system_version: Minimum OS version.
    :type minimum_operating_system_version: str
    :param maximum_operating_system_version: Maximum OS version.
    :type maximum_operating_system_version: str
    """

    _attribute_map = {
        'type': {'key': 'type', 'type': 'str'},
        'minimum_operating_system_version': {'key': 'minimumOperatingSystemVersion', 'type': 'str'},
        'maximum_operating_system_version': {'key': 'maximumOperatingSystemVersion', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphMobileAppSupportedDeviceType, self).__init__(**kwargs)
        self.type = kwargs.get('type', None)
        self.minimum_operating_system_version = kwargs.get('minimum_operating_system_version', None)
        self.maximum_operating_system_version = kwargs.get('maximum_operating_system_version', None)


class MicrosoftGraphMobileAppTroubleshootingEvent(MicrosoftGraphDeviceManagementTroubleshootingEvent):
    """MobileAppTroubleshootingEvent Entity.

    :param id: Read-only.
    :type id: str
    :param event_date_time: Time when the event occurred .
    :type event_date_time: ~datetime.datetime
    :param correlation_id: Id used for tracing the failure in the service.
    :type correlation_id: str
    :param troubleshooting_error_details: deviceManagementTroubleshootingErrorDetails.
    :type troubleshooting_error_details:
     ~users_user.models.MicrosoftGraphDeviceManagementTroubleshootingErrorDetails
    :param event_name: Event Name corresponding to the Troubleshooting Event. It is an Optional
     field.
    :type event_name: str
    :param additional_information: A set of string key and string value pairs which provides
     additional information on the Troubleshooting event.
    :type additional_information: list[~users_user.models.MicrosoftGraphKeyValuePair]
    :param managed_device_identifier: Device identifier created or collected by Intune.
    :type managed_device_identifier: str
    :param user_id: Identifier for the user that tried to enroll the device.
    :type user_id: str
    :param application_id: Intune application identifier.
    :type application_id: str
    :param history: Intune Mobile Application Troubleshooting History Item.
    :type history: list[~users_user.models.MicrosoftGraphMobileAppTroubleshootingHistoryItem]
    :param app_log_collection_requests: The collection property of AppLogUploadRequest.
    :type app_log_collection_requests:
     list[~users_user.models.MicrosoftGraphAppLogCollectionRequest]
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'event_date_time': {'key': 'eventDateTime', 'type': 'iso-8601'},
        'correlation_id': {'key': 'correlationId', 'type': 'str'},
        'troubleshooting_error_details': {'key': 'troubleshootingErrorDetails', 'type': 'MicrosoftGraphDeviceManagementTroubleshootingErrorDetails'},
        'event_name': {'key': 'eventName', 'type': 'str'},
        'additional_information': {'key': 'additionalInformation', 'type': '[MicrosoftGraphKeyValuePair]'},
        'managed_device_identifier': {'key': 'managedDeviceIdentifier', 'type': 'str'},
        'user_id': {'key': 'userId', 'type': 'str'},
        'application_id': {'key': 'applicationId', 'type': 'str'},
        'history': {'key': 'history', 'type': '[MicrosoftGraphMobileAppTroubleshootingHistoryItem]'},
        'app_log_collection_requests': {'key': 'appLogCollectionRequests', 'type': '[MicrosoftGraphAppLogCollectionRequest]'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphMobileAppTroubleshootingEvent, self).__init__(**kwargs)
        self.managed_device_identifier = kwargs.get('managed_device_identifier', None)
        self.user_id = kwargs.get('user_id', None)
        self.application_id = kwargs.get('application_id', None)
        self.history = kwargs.get('history', None)
        self.app_log_collection_requests = kwargs.get('app_log_collection_requests', None)


class MicrosoftGraphMobileAppTroubleshootingHistoryItem(msrest.serialization.Model):
    """mobileAppTroubleshootingHistoryItem.

    :param occurrence_date_time: Time when the history item occurred.
    :type occurrence_date_time: ~datetime.datetime
    :param troubleshooting_error_details: deviceManagementTroubleshootingErrorDetails.
    :type troubleshooting_error_details:
     ~users_user.models.MicrosoftGraphDeviceManagementTroubleshootingErrorDetails
    """

    _attribute_map = {
        'occurrence_date_time': {'key': 'occurrenceDateTime', 'type': 'iso-8601'},
        'troubleshooting_error_details': {'key': 'troubleshootingErrorDetails', 'type': 'MicrosoftGraphDeviceManagementTroubleshootingErrorDetails'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphMobileAppTroubleshootingHistoryItem, self).__init__(**kwargs)
        self.occurrence_date_time = kwargs.get('occurrence_date_time', None)
        self.troubleshooting_error_details = kwargs.get('troubleshooting_error_details', None)


class MicrosoftGraphMoveAction(msrest.serialization.Model):
    """moveAction.

    :param from_property: The name of the location the item was moved from.
    :type from_property: str
    :param to: The name of the location the item was moved to.
    :type to: str
    """

    _attribute_map = {
        'from_property': {'key': 'from', 'type': 'str'},
        'to': {'key': 'to', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphMoveAction, self).__init__(**kwargs)
        self.from_property = kwargs.get('from_property', None)
        self.to = kwargs.get('to', None)


class MicrosoftGraphMultiValueLegacyExtendedProperty(MicrosoftGraphEntity):
    """multiValueLegacyExtendedProperty.

    :param id: Read-only.
    :type id: str
    :param value: A collection of property values.
    :type value: list[str]
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'value': {'key': 'value', 'type': '[str]'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphMultiValueLegacyExtendedProperty, self).__init__(**kwargs)
        self.value = kwargs.get('value', None)


class MicrosoftGraphOnenoteEntityBaseModel(MicrosoftGraphEntity):
    """onenoteEntityBaseModel.

    :param id: Read-only.
    :type id: str
    :param self_property: The endpoint where you can get details about the page. Read-only.
    :type self_property: str
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'self_property': {'key': 'self', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphOnenoteEntityBaseModel, self).__init__(**kwargs)
        self.self_property = kwargs.get('self_property', None)


class MicrosoftGraphOnenoteEntitySchemaObjectModel(MicrosoftGraphOnenoteEntityBaseModel):
    """onenoteEntitySchemaObjectModel.

    :param id: Read-only.
    :type id: str
    :param self_property: The endpoint where you can get details about the page. Read-only.
    :type self_property: str
    :param created_date_time: The date and time when the page was created. The timestamp represents
     date and time information using ISO 8601 format and is always in UTC time. For example,
     midnight UTC on Jan 1, 2014 would look like this: '2014-01-01T00:00:00Z'. Read-only.
    :type created_date_time: ~datetime.datetime
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'self_property': {'key': 'self', 'type': 'str'},
        'created_date_time': {'key': 'createdDateTime', 'type': 'iso-8601'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphOnenoteEntitySchemaObjectModel, self).__init__(**kwargs)
        self.created_date_time = kwargs.get('created_date_time', None)


class MicrosoftGraphOnenoteEntityHierarchyModel(MicrosoftGraphOnenoteEntitySchemaObjectModel):
    """onenoteEntityHierarchyModel.

    :param id: Read-only.
    :type id: str
    :param id: Read-only.
    :type id: str
    :param self_property: The endpoint where you can get details about the page. Read-only.
    :type self_property: str
    :param created_date_time: The date and time when the page was created. The timestamp represents
     date and time information using ISO 8601 format and is always in UTC time. For example,
     midnight UTC on Jan 1, 2014 would look like this: '2014-01-01T00:00:00Z'. Read-only.
    :type created_date_time: ~datetime.datetime
    :param display_name: The name of the notebook.
    :type display_name: str
    :param created_by: identitySet.
    :type created_by: ~users_user.models.MicrosoftGraphIdentitySet
    :param last_modified_by: identitySet.
    :type last_modified_by: ~users_user.models.MicrosoftGraphIdentitySet
    :param last_modified_date_time: The date and time when the notebook was last modified. The
     timestamp represents date and time information using ISO 8601 format and is always in UTC time.
     For example, midnight UTC on Jan 1, 2014 would look like this: '2014-01-01T00:00:00Z'. Read-
     only.
    :type last_modified_date_time: ~datetime.datetime
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'id': {'key': 'id', 'type': 'str'},
        'self_property': {'key': 'self', 'type': 'str'},
        'created_date_time': {'key': 'createdDateTime', 'type': 'iso-8601'},
        'display_name': {'key': 'displayName', 'type': 'str'},
        'created_by': {'key': 'createdBy', 'type': 'MicrosoftGraphIdentitySet'},
        'last_modified_by': {'key': 'lastModifiedBy', 'type': 'MicrosoftGraphIdentitySet'},
        'last_modified_date_time': {'key': 'lastModifiedDateTime', 'type': 'iso-8601'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphOnenoteEntityHierarchyModel, self).__init__(**kwargs)
        self.display_name = kwargs.get('display_name', None)
        self.created_by = kwargs.get('created_by', None)
        self.last_modified_by = kwargs.get('last_modified_by', None)
        self.last_modified_date_time = kwargs.get('last_modified_date_time', None)


class MicrosoftGraphNotebook(MicrosoftGraphOnenoteEntityHierarchyModel):
    """notebook.

    :param id: Read-only.
    :type id: str
    :param self_property: The endpoint where you can get details about the page. Read-only.
    :type self_property: str
    :param created_date_time: The date and time when the page was created. The timestamp represents
     date and time information using ISO 8601 format and is always in UTC time. For example,
     midnight UTC on Jan 1, 2014 would look like this: '2014-01-01T00:00:00Z'. Read-only.
    :type created_date_time: ~datetime.datetime
    :param display_name: The name of the notebook.
    :type display_name: str
    :param created_by: identitySet.
    :type created_by: ~users_user.models.MicrosoftGraphIdentitySet
    :param last_modified_by: identitySet.
    :type last_modified_by: ~users_user.models.MicrosoftGraphIdentitySet
    :param last_modified_date_time: The date and time when the notebook was last modified. The
     timestamp represents date and time information using ISO 8601 format and is always in UTC time.
     For example, midnight UTC on Jan 1, 2014 would look like this: '2014-01-01T00:00:00Z'. Read-
     only.
    :type last_modified_date_time: ~datetime.datetime
    :param is_default: Indicates whether this is the user's default notebook. Read-only.
    :type is_default: bool
    :param user_role: onenoteUserRole. Possible values include: "Owner", "Contributor", "Reader",
     "None".
    :type user_role: str or ~users_user.models.MicrosoftGraphOnenoteUserRole
    :param is_shared: Indicates whether the notebook is shared. If true, the contents of the
     notebook can be seen by people other than the owner. Read-only.
    :type is_shared: bool
    :param sections_url: The URL for the sections navigation property, which returns all the
     sections in the notebook. Read-only.
    :type sections_url: str
    :param section_groups_url: The URL for the sectionGroups navigation property, which returns all
     the section groups in the notebook. Read-only.
    :type section_groups_url: str
    :param links: notebookLinks.
    :type links: ~users_user.models.MicrosoftGraphNotebookLinks
    :param sections: The sections in the notebook. Read-only. Nullable.
    :type sections: list[~users_user.models.MicrosoftGraphOnenoteSection]
    :param section_groups: The section groups in the notebook. Read-only. Nullable.
    :type section_groups: list[~users_user.models.MicrosoftGraphSectionGroup]
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'self_property': {'key': 'self', 'type': 'str'},
        'created_date_time': {'key': 'createdDateTime', 'type': 'iso-8601'},
        'display_name': {'key': 'displayName', 'type': 'str'},
        'created_by': {'key': 'createdBy', 'type': 'MicrosoftGraphIdentitySet'},
        'last_modified_by': {'key': 'lastModifiedBy', 'type': 'MicrosoftGraphIdentitySet'},
        'last_modified_date_time': {'key': 'lastModifiedDateTime', 'type': 'iso-8601'},
        'is_default': {'key': 'isDefault', 'type': 'bool'},
        'user_role': {'key': 'userRole', 'type': 'str'},
        'is_shared': {'key': 'isShared', 'type': 'bool'},
        'sections_url': {'key': 'sectionsUrl', 'type': 'str'},
        'section_groups_url': {'key': 'sectionGroupsUrl', 'type': 'str'},
        'links': {'key': 'links', 'type': 'MicrosoftGraphNotebookLinks'},
        'sections': {'key': 'sections', 'type': '[MicrosoftGraphOnenoteSection]'},
        'section_groups': {'key': 'sectionGroups', 'type': '[MicrosoftGraphSectionGroup]'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphNotebook, self).__init__(**kwargs)
        self.is_default = kwargs.get('is_default', None)
        self.user_role = kwargs.get('user_role', None)
        self.is_shared = kwargs.get('is_shared', None)
        self.sections_url = kwargs.get('sections_url', None)
        self.section_groups_url = kwargs.get('section_groups_url', None)
        self.links = kwargs.get('links', None)
        self.sections = kwargs.get('sections', None)
        self.section_groups = kwargs.get('section_groups', None)


class MicrosoftGraphNotebookLinks(msrest.serialization.Model):
    """notebookLinks.

    :param one_note_client_url: externalLink.
    :type one_note_client_url: ~users_user.models.MicrosoftGraphExternalLink
    :param one_note_web_url: externalLink.
    :type one_note_web_url: ~users_user.models.MicrosoftGraphExternalLink
    """

    _attribute_map = {
        'one_note_client_url': {'key': 'oneNoteClientUrl', 'type': 'MicrosoftGraphExternalLink'},
        'one_note_web_url': {'key': 'oneNoteWebUrl', 'type': 'MicrosoftGraphExternalLink'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphNotebookLinks, self).__init__(**kwargs)
        self.one_note_client_url = kwargs.get('one_note_client_url', None)
        self.one_note_web_url = kwargs.get('one_note_web_url', None)


class MicrosoftGraphNotification(MicrosoftGraphEntity):
    """notification.

    :param id: Read-only.
    :type id: str
    :param target_host_name:
    :type target_host_name: str
    :param expiration_date_time:
    :type expiration_date_time: ~datetime.datetime
    :param payload: payloadTypes.
    :type payload: ~users_user.models.MicrosoftGraphPayloadTypes
    :param display_time_to_live:
    :type display_time_to_live: int
    :param priority: priority. Possible values include: "None", "High", "Low".
    :type priority: str or ~users_user.models.MicrosoftGraphPriority
    :param group_name:
    :type group_name: str
    :param target_policy: targetPolicyEndpoints.
    :type target_policy: ~users_user.models.MicrosoftGraphTargetPolicyEndpoints
    """

    _validation = {
        'display_time_to_live': {'maximum': 2147483647, 'minimum': -2147483648},
    }

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'target_host_name': {'key': 'targetHostName', 'type': 'str'},
        'expiration_date_time': {'key': 'expirationDateTime', 'type': 'iso-8601'},
        'payload': {'key': 'payload', 'type': 'MicrosoftGraphPayloadTypes'},
        'display_time_to_live': {'key': 'displayTimeToLive', 'type': 'int'},
        'priority': {'key': 'priority', 'type': 'str'},
        'group_name': {'key': 'groupName', 'type': 'str'},
        'target_policy': {'key': 'targetPolicy', 'type': 'MicrosoftGraphTargetPolicyEndpoints'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphNotification, self).__init__(**kwargs)
        self.target_host_name = kwargs.get('target_host_name', None)
        self.expiration_date_time = kwargs.get('expiration_date_time', None)
        self.payload = kwargs.get('payload', None)
        self.display_time_to_live = kwargs.get('display_time_to_live', None)
        self.priority = kwargs.get('priority', None)
        self.group_name = kwargs.get('group_name', None)
        self.target_policy = kwargs.get('target_policy', None)


class MicrosoftGraphNumberColumn(msrest.serialization.Model):
    """numberColumn.

    :param decimal_places: How many decimal places to display. See below for information about the
     possible values.
    :type decimal_places: str
    :param display_as: How the value should be presented in the UX. Must be one of number or
     percentage. If unspecified, treated as number.
    :type display_as: str
    :param maximum: The maximum permitted value.
    :type maximum: float
    :param minimum: The minimum permitted value.
    :type minimum: float
    """

    _attribute_map = {
        'decimal_places': {'key': 'decimalPlaces', 'type': 'str'},
        'display_as': {'key': 'displayAs', 'type': 'str'},
        'maximum': {'key': 'maximum', 'type': 'float'},
        'minimum': {'key': 'minimum', 'type': 'float'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphNumberColumn, self).__init__(**kwargs)
        self.decimal_places = kwargs.get('decimal_places', None)
        self.display_as = kwargs.get('display_as', None)
        self.maximum = kwargs.get('maximum', None)
        self.minimum = kwargs.get('minimum', None)


class MicrosoftGraphObjectIdentity(msrest.serialization.Model):
    """objectIdentity.

    :param sign_in_type: Specifies the user sign-in types in your directory, such as emailAddress,
     userName or federated. Here, federated represents a unique identifier for a user from an
     issuer, that can be in any format chosen by the issuer. Additional validation is enforced on
     issuerAssignedId when the sign-in type is set to emailAddress or userName. This property can
     also be set to any custom string.
    :type sign_in_type: str
    :param issuer: Specifies the issuer of the identity, for example facebook.com.For local
     accounts (where signInType is not federated), this property is the local B2C tenant default
     domain name, for example contoso.onmicrosoft.com.For external users from other Azure AD
     organization, this will be the domain of the federated organization, for example
     contoso.com.Supports $filter. 512 character limit.
    :type issuer: str
    :param issuer_assigned_id: Specifies the unique identifier assigned to the user by the issuer.
     The combination of issuer and issuerAssignedId must be unique within the organization.
     Represents the sign-in name for the user, when signInType is set to emailAddress or userName
     (also known as local accounts).When signInType is set to: emailAddress, (or starts with
     emailAddress like emailAddress1) issuerAssignedId must be a valid email addressuserName,
     issuerAssignedId must be a valid local part of an email addressSupports $filter. 512 character
     limit.
    :type issuer_assigned_id: str
    """

    _attribute_map = {
        'sign_in_type': {'key': 'signInType', 'type': 'str'},
        'issuer': {'key': 'issuer', 'type': 'str'},
        'issuer_assigned_id': {'key': 'issuerAssignedId', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphObjectIdentity, self).__init__(**kwargs)
        self.sign_in_type = kwargs.get('sign_in_type', None)
        self.issuer = kwargs.get('issuer', None)
        self.issuer_assigned_id = kwargs.get('issuer_assigned_id', None)


class MicrosoftGraphScheduleChangeRequest(MicrosoftGraphChangeTrackedEntity):
    """scheduleChangeRequest.

    :param id: Read-only.
    :type id: str
    :param created_date_time:
    :type created_date_time: ~datetime.datetime
    :param last_modified_date_time:
    :type last_modified_date_time: ~datetime.datetime
    :param last_modified_by: identitySet.
    :type last_modified_by: ~users_user.models.MicrosoftGraphIdentitySet
    :param assigned_to: scheduleChangeRequestActor. Possible values include: "sender", "recipient",
     "manager", "system", "unknownFutureValue".
    :type assigned_to: str or ~users_user.models.MicrosoftGraphScheduleChangeRequestActor
    :param state: scheduleChangeState. Possible values include: "pending", "approved", "declined",
     "unknownFutureValue".
    :type state: str or ~users_user.models.MicrosoftGraphScheduleChangeState
    :param sender_message:
    :type sender_message: str
    :param sender_date_time:
    :type sender_date_time: ~datetime.datetime
    :param manager_action_message:
    :type manager_action_message: str
    :param manager_action_date_time:
    :type manager_action_date_time: ~datetime.datetime
    :param sender_user_id:
    :type sender_user_id: str
    :param manager_user_id:
    :type manager_user_id: str
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'created_date_time': {'key': 'createdDateTime', 'type': 'iso-8601'},
        'last_modified_date_time': {'key': 'lastModifiedDateTime', 'type': 'iso-8601'},
        'last_modified_by': {'key': 'lastModifiedBy', 'type': 'MicrosoftGraphIdentitySet'},
        'assigned_to': {'key': 'assignedTo', 'type': 'str'},
        'state': {'key': 'state', 'type': 'str'},
        'sender_message': {'key': 'senderMessage', 'type': 'str'},
        'sender_date_time': {'key': 'senderDateTime', 'type': 'iso-8601'},
        'manager_action_message': {'key': 'managerActionMessage', 'type': 'str'},
        'manager_action_date_time': {'key': 'managerActionDateTime', 'type': 'iso-8601'},
        'sender_user_id': {'key': 'senderUserId', 'type': 'str'},
        'manager_user_id': {'key': 'managerUserId', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphScheduleChangeRequest, self).__init__(**kwargs)
        self.assigned_to = kwargs.get('assigned_to', None)
        self.state = kwargs.get('state', None)
        self.sender_message = kwargs.get('sender_message', None)
        self.sender_date_time = kwargs.get('sender_date_time', None)
        self.manager_action_message = kwargs.get('manager_action_message', None)
        self.manager_action_date_time = kwargs.get('manager_action_date_time', None)
        self.sender_user_id = kwargs.get('sender_user_id', None)
        self.manager_user_id = kwargs.get('manager_user_id', None)


class MicrosoftGraphOfferShiftRequest(MicrosoftGraphScheduleChangeRequest):
    """offerShiftRequest.

    :param id: Read-only.
    :type id: str
    :param created_date_time:
    :type created_date_time: ~datetime.datetime
    :param last_modified_date_time:
    :type last_modified_date_time: ~datetime.datetime
    :param last_modified_by: identitySet.
    :type last_modified_by: ~users_user.models.MicrosoftGraphIdentitySet
    :param assigned_to: scheduleChangeRequestActor. Possible values include: "sender", "recipient",
     "manager", "system", "unknownFutureValue".
    :type assigned_to: str or ~users_user.models.MicrosoftGraphScheduleChangeRequestActor
    :param state: scheduleChangeState. Possible values include: "pending", "approved", "declined",
     "unknownFutureValue".
    :type state: str or ~users_user.models.MicrosoftGraphScheduleChangeState
    :param sender_message:
    :type sender_message: str
    :param sender_date_time:
    :type sender_date_time: ~datetime.datetime
    :param manager_action_message:
    :type manager_action_message: str
    :param manager_action_date_time:
    :type manager_action_date_time: ~datetime.datetime
    :param sender_user_id:
    :type sender_user_id: str
    :param manager_user_id:
    :type manager_user_id: str
    :param recipient_action_message:
    :type recipient_action_message: str
    :param recipient_action_date_time:
    :type recipient_action_date_time: ~datetime.datetime
    :param sender_shift_id:
    :type sender_shift_id: str
    :param recipient_user_id:
    :type recipient_user_id: str
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'created_date_time': {'key': 'createdDateTime', 'type': 'iso-8601'},
        'last_modified_date_time': {'key': 'lastModifiedDateTime', 'type': 'iso-8601'},
        'last_modified_by': {'key': 'lastModifiedBy', 'type': 'MicrosoftGraphIdentitySet'},
        'assigned_to': {'key': 'assignedTo', 'type': 'str'},
        'state': {'key': 'state', 'type': 'str'},
        'sender_message': {'key': 'senderMessage', 'type': 'str'},
        'sender_date_time': {'key': 'senderDateTime', 'type': 'iso-8601'},
        'manager_action_message': {'key': 'managerActionMessage', 'type': 'str'},
        'manager_action_date_time': {'key': 'managerActionDateTime', 'type': 'iso-8601'},
        'sender_user_id': {'key': 'senderUserId', 'type': 'str'},
        'manager_user_id': {'key': 'managerUserId', 'type': 'str'},
        'recipient_action_message': {'key': 'recipientActionMessage', 'type': 'str'},
        'recipient_action_date_time': {'key': 'recipientActionDateTime', 'type': 'iso-8601'},
        'sender_shift_id': {'key': 'senderShiftId', 'type': 'str'},
        'recipient_user_id': {'key': 'recipientUserId', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphOfferShiftRequest, self).__init__(**kwargs)
        self.recipient_action_message = kwargs.get('recipient_action_message', None)
        self.recipient_action_date_time = kwargs.get('recipient_action_date_time', None)
        self.sender_shift_id = kwargs.get('sender_shift_id', None)
        self.recipient_user_id = kwargs.get('recipient_user_id', None)


class MicrosoftGraphOfficeGraphInsights(MicrosoftGraphEntity):
    """officeGraphInsights.

    :param id: Read-only.
    :type id: str
    :param trending: Calculated relationship identifying trending documents. Trending documents can
     be stored in OneDrive or in SharePoint sites.
    :type trending: list[~users_user.models.MicrosoftGraphTrending]
    :param shared: Calculated relationship identifying documents shared with a user. Documents can
     be shared as email attachments or as OneDrive for Business links sent in emails.
    :type shared: list[~users_user.models.MicrosoftGraphSharedInsight]
    :param used: Calculated relationship identifying documents viewed and modified by a user.
     Includes documents the user used in OneDrive for Business, SharePoint, opened as email
     attachments, and as link attachments from sources like Box, DropBox and Google Drive.
    :type used: list[~users_user.models.MicrosoftGraphUsedInsight]
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'trending': {'key': 'trending', 'type': '[MicrosoftGraphTrending]'},
        'shared': {'key': 'shared', 'type': '[MicrosoftGraphSharedInsight]'},
        'used': {'key': 'used', 'type': '[MicrosoftGraphUsedInsight]'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphOfficeGraphInsights, self).__init__(**kwargs)
        self.trending = kwargs.get('trending', None)
        self.shared = kwargs.get('shared', None)
        self.used = kwargs.get('used', None)


class MicrosoftGraphOnenote(MicrosoftGraphEntity):
    """onenote.

    :param id: Read-only.
    :type id: str
    :param notebooks: The collection of OneNote notebooks that are owned by the user or group.
     Read-only. Nullable.
    :type notebooks: list[~users_user.models.MicrosoftGraphNotebook]
    :param sections: The sections in all OneNote notebooks that are owned by the user or group.
     Read-only. Nullable.
    :type sections: list[~users_user.models.MicrosoftGraphOnenoteSection]
    :param section_groups: The section groups in all OneNote notebooks that are owned by the user
     or group.  Read-only. Nullable.
    :type section_groups: list[~users_user.models.MicrosoftGraphSectionGroup]
    :param pages: The pages in all OneNote notebooks that are owned by the user or group.  Read-
     only. Nullable.
    :type pages: list[~users_user.models.MicrosoftGraphOnenotePage]
    :param resources: The image and other file resources in OneNote pages. Getting a resources
     collection is not supported, but you can get the binary content of a specific resource. Read-
     only. Nullable.
    :type resources: list[~users_user.models.MicrosoftGraphOnenoteResource]
    :param operations: The status of OneNote operations. Getting an operations collection is not
     supported, but you can get the status of long-running operations if the Operation-Location
     header is returned in the response. Read-only. Nullable.
    :type operations: list[~users_user.models.MicrosoftGraphOnenoteOperation]
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'notebooks': {'key': 'notebooks', 'type': '[MicrosoftGraphNotebook]'},
        'sections': {'key': 'sections', 'type': '[MicrosoftGraphOnenoteSection]'},
        'section_groups': {'key': 'sectionGroups', 'type': '[MicrosoftGraphSectionGroup]'},
        'pages': {'key': 'pages', 'type': '[MicrosoftGraphOnenotePage]'},
        'resources': {'key': 'resources', 'type': '[MicrosoftGraphOnenoteResource]'},
        'operations': {'key': 'operations', 'type': '[MicrosoftGraphOnenoteOperation]'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphOnenote, self).__init__(**kwargs)
        self.notebooks = kwargs.get('notebooks', None)
        self.sections = kwargs.get('sections', None)
        self.section_groups = kwargs.get('section_groups', None)
        self.pages = kwargs.get('pages', None)
        self.resources = kwargs.get('resources', None)
        self.operations = kwargs.get('operations', None)


class MicrosoftGraphOperation(MicrosoftGraphEntity):
    """operation.

    :param id: Read-only.
    :type id: str
    :param status: operationStatus. Possible values include: "NotStarted", "Running", "Completed",
     "Failed".
    :type status: str or ~users_user.models.MicrosoftGraphOperationStatus
    :param created_date_time: The start time of the operation.
    :type created_date_time: ~datetime.datetime
    :param last_action_date_time: The time of the last action of the operation.
    :type last_action_date_time: ~datetime.datetime
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'status': {'key': 'status', 'type': 'str'},
        'created_date_time': {'key': 'createdDateTime', 'type': 'iso-8601'},
        'last_action_date_time': {'key': 'lastActionDateTime', 'type': 'iso-8601'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphOperation, self).__init__(**kwargs)
        self.status = kwargs.get('status', None)
        self.created_date_time = kwargs.get('created_date_time', None)
        self.last_action_date_time = kwargs.get('last_action_date_time', None)


class MicrosoftGraphOnenoteOperation(MicrosoftGraphOperation):
    """onenoteOperation.

    :param id: Read-only.
    :type id: str
    :param status: operationStatus. Possible values include: "NotStarted", "Running", "Completed",
     "Failed".
    :type status: str or ~users_user.models.MicrosoftGraphOperationStatus
    :param created_date_time: The start time of the operation.
    :type created_date_time: ~datetime.datetime
    :param last_action_date_time: The time of the last action of the operation.
    :type last_action_date_time: ~datetime.datetime
    :param resource_location: The resource URI for the object. For example, the resource URI for a
     copied page or section.
    :type resource_location: str
    :param resource_id: The resource id.
    :type resource_id: str
    :param error: onenoteOperationError.
    :type error: ~users_user.models.MicrosoftGraphOnenoteOperationError
    :param percent_complete: The operation percent complete if the operation is still in running
     status.
    :type percent_complete: str
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'status': {'key': 'status', 'type': 'str'},
        'created_date_time': {'key': 'createdDateTime', 'type': 'iso-8601'},
        'last_action_date_time': {'key': 'lastActionDateTime', 'type': 'iso-8601'},
        'resource_location': {'key': 'resourceLocation', 'type': 'str'},
        'resource_id': {'key': 'resourceId', 'type': 'str'},
        'error': {'key': 'error', 'type': 'MicrosoftGraphOnenoteOperationError'},
        'percent_complete': {'key': 'percentComplete', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphOnenoteOperation, self).__init__(**kwargs)
        self.resource_location = kwargs.get('resource_location', None)
        self.resource_id = kwargs.get('resource_id', None)
        self.error = kwargs.get('error', None)
        self.percent_complete = kwargs.get('percent_complete', None)


class MicrosoftGraphOnenoteOperationError(msrest.serialization.Model):
    """onenoteOperationError.

    :param code: The error code.
    :type code: str
    :param message: The error message.
    :type message: str
    """

    _attribute_map = {
        'code': {'key': 'code', 'type': 'str'},
        'message': {'key': 'message', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphOnenoteOperationError, self).__init__(**kwargs)
        self.code = kwargs.get('code', None)
        self.message = kwargs.get('message', None)


class MicrosoftGraphOnenotePage(MicrosoftGraphOnenoteEntitySchemaObjectModel):
    """onenotePage.

    :param id: Read-only.
    :type id: str
    :param self_property: The endpoint where you can get details about the page. Read-only.
    :type self_property: str
    :param created_date_time: The date and time when the page was created. The timestamp represents
     date and time information using ISO 8601 format and is always in UTC time. For example,
     midnight UTC on Jan 1, 2014 would look like this: '2014-01-01T00:00:00Z'. Read-only.
    :type created_date_time: ~datetime.datetime
    :param title: The title of the page.
    :type title: str
    :param created_by_app_id: The unique identifier of the application that created the page. Read-
     only.
    :type created_by_app_id: str
    :param links: pageLinks.
    :type links: ~users_user.models.MicrosoftGraphPageLinks
    :param content_url: The URL for the page's HTML content.  Read-only.
    :type content_url: str
    :param content: The page's HTML content.
    :type content: bytes
    :param last_modified_date_time: The date and time when the page was last modified. The
     timestamp represents date and time information using ISO 8601 format and is always in UTC time.
     For example, midnight UTC on Jan 1, 2014 would look like this: '2014-01-01T00:00:00Z'. Read-
     only.
    :type last_modified_date_time: ~datetime.datetime
    :param level: The indentation level of the page. Read-only.
    :type level: int
    :param order: The order of the page within its parent section. Read-only.
    :type order: int
    :param user_tags:
    :type user_tags: list[str]
    :param parent_section: onenoteSection.
    :type parent_section: ~users_user.models.MicrosoftGraphOnenoteSection
    :param parent_notebook: notebook.
    :type parent_notebook: ~users_user.models.MicrosoftGraphNotebook
    """

    _validation = {
        'level': {'maximum': 2147483647, 'minimum': -2147483648},
        'order': {'maximum': 2147483647, 'minimum': -2147483648},
    }

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'self_property': {'key': 'self', 'type': 'str'},
        'created_date_time': {'key': 'createdDateTime', 'type': 'iso-8601'},
        'title': {'key': 'title', 'type': 'str'},
        'created_by_app_id': {'key': 'createdByAppId', 'type': 'str'},
        'links': {'key': 'links', 'type': 'MicrosoftGraphPageLinks'},
        'content_url': {'key': 'contentUrl', 'type': 'str'},
        'content': {'key': 'content', 'type': 'base64'},
        'last_modified_date_time': {'key': 'lastModifiedDateTime', 'type': 'iso-8601'},
        'level': {'key': 'level', 'type': 'int'},
        'order': {'key': 'order', 'type': 'int'},
        'user_tags': {'key': 'userTags', 'type': '[str]'},
        'parent_section': {'key': 'parentSection', 'type': 'MicrosoftGraphOnenoteSection'},
        'parent_notebook': {'key': 'parentNotebook', 'type': 'MicrosoftGraphNotebook'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphOnenotePage, self).__init__(**kwargs)
        self.title = kwargs.get('title', None)
        self.created_by_app_id = kwargs.get('created_by_app_id', None)
        self.links = kwargs.get('links', None)
        self.content_url = kwargs.get('content_url', None)
        self.content = kwargs.get('content', None)
        self.last_modified_date_time = kwargs.get('last_modified_date_time', None)
        self.level = kwargs.get('level', None)
        self.order = kwargs.get('order', None)
        self.user_tags = kwargs.get('user_tags', None)
        self.parent_section = kwargs.get('parent_section', None)
        self.parent_notebook = kwargs.get('parent_notebook', None)


class MicrosoftGraphOnenoteResource(MicrosoftGraphOnenoteEntityBaseModel):
    """onenoteResource.

    :param id: Read-only.
    :type id: str
    :param self_property: The endpoint where you can get details about the page. Read-only.
    :type self_property: str
    :param content: The content stream.
    :type content: bytes
    :param content_url: The URL for downloading the content.
    :type content_url: str
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'self_property': {'key': 'self', 'type': 'str'},
        'content': {'key': 'content', 'type': 'base64'},
        'content_url': {'key': 'contentUrl', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphOnenoteResource, self).__init__(**kwargs)
        self.content = kwargs.get('content', None)
        self.content_url = kwargs.get('content_url', None)


class MicrosoftGraphOnenoteSection(MicrosoftGraphOnenoteEntityHierarchyModel):
    """onenoteSection.

    :param id: Read-only.
    :type id: str
    :param self_property: The endpoint where you can get details about the page. Read-only.
    :type self_property: str
    :param created_date_time: The date and time when the page was created. The timestamp represents
     date and time information using ISO 8601 format and is always in UTC time. For example,
     midnight UTC on Jan 1, 2014 would look like this: '2014-01-01T00:00:00Z'. Read-only.
    :type created_date_time: ~datetime.datetime
    :param display_name: The name of the notebook.
    :type display_name: str
    :param created_by: identitySet.
    :type created_by: ~users_user.models.MicrosoftGraphIdentitySet
    :param last_modified_by: identitySet.
    :type last_modified_by: ~users_user.models.MicrosoftGraphIdentitySet
    :param last_modified_date_time: The date and time when the notebook was last modified. The
     timestamp represents date and time information using ISO 8601 format and is always in UTC time.
     For example, midnight UTC on Jan 1, 2014 would look like this: '2014-01-01T00:00:00Z'. Read-
     only.
    :type last_modified_date_time: ~datetime.datetime
    :param is_default: Indicates whether this is the user's default section. Read-only.
    :type is_default: bool
    :param links: sectionLinks.
    :type links: ~users_user.models.MicrosoftGraphSectionLinks
    :param pages_url: The pages endpoint where you can get details for all the pages in the
     section. Read-only.
    :type pages_url: str
    :param parent_notebook: notebook.
    :type parent_notebook: ~users_user.models.MicrosoftGraphNotebook
    :param parent_section_group: sectionGroup.
    :type parent_section_group: ~users_user.models.MicrosoftGraphSectionGroup
    :param pages: The collection of pages in the section.  Read-only. Nullable.
    :type pages: list[~users_user.models.MicrosoftGraphOnenotePage]
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'self_property': {'key': 'self', 'type': 'str'},
        'created_date_time': {'key': 'createdDateTime', 'type': 'iso-8601'},
        'display_name': {'key': 'displayName', 'type': 'str'},
        'created_by': {'key': 'createdBy', 'type': 'MicrosoftGraphIdentitySet'},
        'last_modified_by': {'key': 'lastModifiedBy', 'type': 'MicrosoftGraphIdentitySet'},
        'last_modified_date_time': {'key': 'lastModifiedDateTime', 'type': 'iso-8601'},
        'is_default': {'key': 'isDefault', 'type': 'bool'},
        'links': {'key': 'links', 'type': 'MicrosoftGraphSectionLinks'},
        'pages_url': {'key': 'pagesUrl', 'type': 'str'},
        'parent_notebook': {'key': 'parentNotebook', 'type': 'MicrosoftGraphNotebook'},
        'parent_section_group': {'key': 'parentSectionGroup', 'type': 'MicrosoftGraphSectionGroup'},
        'pages': {'key': 'pages', 'type': '[MicrosoftGraphOnenotePage]'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphOnenoteSection, self).__init__(**kwargs)
        self.is_default = kwargs.get('is_default', None)
        self.links = kwargs.get('links', None)
        self.pages_url = kwargs.get('pages_url', None)
        self.parent_notebook = kwargs.get('parent_notebook', None)
        self.parent_section_group = kwargs.get('parent_section_group', None)
        self.pages = kwargs.get('pages', None)


class MicrosoftGraphOnlineMeeting(MicrosoftGraphEntity):
    """onlineMeeting.

    :param id: Read-only.
    :type id: str
    :param creation_date_time: The meeting creation time in UTC. Read-only.
    :type creation_date_time: ~datetime.datetime
    :param start_date_time: The meeting start time in UTC.
    :type start_date_time: ~datetime.datetime
    :param end_date_time: The meeting end time in UTC.
    :type end_date_time: ~datetime.datetime
    :param canceled_date_time:
    :type canceled_date_time: ~datetime.datetime
    :param expiration_date_time:
    :type expiration_date_time: ~datetime.datetime
    :param entry_exit_announcement:
    :type entry_exit_announcement: bool
    :param join_url:
    :type join_url: str
    :param subject: The subject of the online meeting.
    :type subject: str
    :param is_cancelled:
    :type is_cancelled: bool
    :param participants: meetingParticipants.
    :type participants: ~users_user.models.MicrosoftGraphMeetingParticipants
    :param is_broadcast:
    :type is_broadcast: bool
    :param access_level: accessLevel. Possible values include: "everyone", "invited", "locked",
     "sameEnterprise", "sameEnterpriseAndFederated".
    :type access_level: str or ~users_user.models.MicrosoftGraphAccessLevel
    :param capabilities:
    :type capabilities: list[str or ~users_user.models.MicrosoftGraphMeetingCapabilities]
    :param audio_conferencing: audioConferencing.
    :type audio_conferencing: ~users_user.models.MicrosoftGraphAudioConferencing
    :param chat_info: chatInfo.
    :type chat_info: ~users_user.models.MicrosoftGraphChatInfo
    :param video_teleconference_id: The video teleconferencing ID. Read-only.
    :type video_teleconference_id: str
    :param external_id:
    :type external_id: str
    :param join_information: itemBody.
    :type join_information: ~users_user.models.MicrosoftGraphItemBody
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'creation_date_time': {'key': 'creationDateTime', 'type': 'iso-8601'},
        'start_date_time': {'key': 'startDateTime', 'type': 'iso-8601'},
        'end_date_time': {'key': 'endDateTime', 'type': 'iso-8601'},
        'canceled_date_time': {'key': 'canceledDateTime', 'type': 'iso-8601'},
        'expiration_date_time': {'key': 'expirationDateTime', 'type': 'iso-8601'},
        'entry_exit_announcement': {'key': 'entryExitAnnouncement', 'type': 'bool'},
        'join_url': {'key': 'joinUrl', 'type': 'str'},
        'subject': {'key': 'subject', 'type': 'str'},
        'is_cancelled': {'key': 'isCancelled', 'type': 'bool'},
        'participants': {'key': 'participants', 'type': 'MicrosoftGraphMeetingParticipants'},
        'is_broadcast': {'key': 'isBroadcast', 'type': 'bool'},
        'access_level': {'key': 'accessLevel', 'type': 'str'},
        'capabilities': {'key': 'capabilities', 'type': '[str]'},
        'audio_conferencing': {'key': 'audioConferencing', 'type': 'MicrosoftGraphAudioConferencing'},
        'chat_info': {'key': 'chatInfo', 'type': 'MicrosoftGraphChatInfo'},
        'video_teleconference_id': {'key': 'videoTeleconferenceId', 'type': 'str'},
        'external_id': {'key': 'externalId', 'type': 'str'},
        'join_information': {'key': 'joinInformation', 'type': 'MicrosoftGraphItemBody'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphOnlineMeeting, self).__init__(**kwargs)
        self.creation_date_time = kwargs.get('creation_date_time', None)
        self.start_date_time = kwargs.get('start_date_time', None)
        self.end_date_time = kwargs.get('end_date_time', None)
        self.canceled_date_time = kwargs.get('canceled_date_time', None)
        self.expiration_date_time = kwargs.get('expiration_date_time', None)
        self.entry_exit_announcement = kwargs.get('entry_exit_announcement', None)
        self.join_url = kwargs.get('join_url', None)
        self.subject = kwargs.get('subject', None)
        self.is_cancelled = kwargs.get('is_cancelled', None)
        self.participants = kwargs.get('participants', None)
        self.is_broadcast = kwargs.get('is_broadcast', None)
        self.access_level = kwargs.get('access_level', None)
        self.capabilities = kwargs.get('capabilities', None)
        self.audio_conferencing = kwargs.get('audio_conferencing', None)
        self.chat_info = kwargs.get('chat_info', None)
        self.video_teleconference_id = kwargs.get('video_teleconference_id', None)
        self.external_id = kwargs.get('external_id', None)
        self.join_information = kwargs.get('join_information', None)


class MicrosoftGraphOnlineMeetingInfo(msrest.serialization.Model):
    """onlineMeetingInfo.

    :param join_url: The external link that launches the online meeting. This is a URL that clients
     will launch into a browser and will redirect the user to join the meeting.
    :type join_url: str
    :param conference_id: The ID of the conference.
    :type conference_id: str
    :param toll_number: The toll number that can be used to join the conference.
    :type toll_number: str
    :param toll_free_numbers: The toll free numbers that can be used to join the conference.
    :type toll_free_numbers: list[str]
    :param quick_dial: The pre-formatted quickdial for this call.
    :type quick_dial: str
    :param phones: All of the phone numbers associated with this conference.
    :type phones: list[~users_user.models.MicrosoftGraphPhone]
    """

    _attribute_map = {
        'join_url': {'key': 'joinUrl', 'type': 'str'},
        'conference_id': {'key': 'conferenceId', 'type': 'str'},
        'toll_number': {'key': 'tollNumber', 'type': 'str'},
        'toll_free_numbers': {'key': 'tollFreeNumbers', 'type': '[str]'},
        'quick_dial': {'key': 'quickDial', 'type': 'str'},
        'phones': {'key': 'phones', 'type': '[MicrosoftGraphPhone]'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphOnlineMeetingInfo, self).__init__(**kwargs)
        self.join_url = kwargs.get('join_url', None)
        self.conference_id = kwargs.get('conference_id', None)
        self.toll_number = kwargs.get('toll_number', None)
        self.toll_free_numbers = kwargs.get('toll_free_numbers', None)
        self.quick_dial = kwargs.get('quick_dial', None)
        self.phones = kwargs.get('phones', None)


class MicrosoftGraphOnPremisesExtensionAttributes(msrest.serialization.Model):
    """onPremisesExtensionAttributes.

    :param extension_attribute1: First customizable extension attribute.
    :type extension_attribute1: str
    :param extension_attribute2: Second customizable extension attribute.
    :type extension_attribute2: str
    :param extension_attribute3: Third customizable extension attribute.
    :type extension_attribute3: str
    :param extension_attribute4: Fourth customizable extension attribute.
    :type extension_attribute4: str
    :param extension_attribute5: Fifth customizable extension attribute.
    :type extension_attribute5: str
    :param extension_attribute6: Sixth customizable extension attribute.
    :type extension_attribute6: str
    :param extension_attribute7: Seventh customizable extension attribute.
    :type extension_attribute7: str
    :param extension_attribute8: Eighth customizable extension attribute.
    :type extension_attribute8: str
    :param extension_attribute9: Ninth customizable extension attribute.
    :type extension_attribute9: str
    :param extension_attribute10: Tenth customizable extension attribute.
    :type extension_attribute10: str
    :param extension_attribute11: Eleventh customizable extension attribute.
    :type extension_attribute11: str
    :param extension_attribute12: Twelfth customizable extension attribute.
    :type extension_attribute12: str
    :param extension_attribute13: Thirteenth customizable extension attribute.
    :type extension_attribute13: str
    :param extension_attribute14: Fourteenth customizable extension attribute.
    :type extension_attribute14: str
    :param extension_attribute15: Fifteenth customizable extension attribute.
    :type extension_attribute15: str
    """

    _attribute_map = {
        'extension_attribute1': {'key': 'extensionAttribute1', 'type': 'str'},
        'extension_attribute2': {'key': 'extensionAttribute2', 'type': 'str'},
        'extension_attribute3': {'key': 'extensionAttribute3', 'type': 'str'},
        'extension_attribute4': {'key': 'extensionAttribute4', 'type': 'str'},
        'extension_attribute5': {'key': 'extensionAttribute5', 'type': 'str'},
        'extension_attribute6': {'key': 'extensionAttribute6', 'type': 'str'},
        'extension_attribute7': {'key': 'extensionAttribute7', 'type': 'str'},
        'extension_attribute8': {'key': 'extensionAttribute8', 'type': 'str'},
        'extension_attribute9': {'key': 'extensionAttribute9', 'type': 'str'},
        'extension_attribute10': {'key': 'extensionAttribute10', 'type': 'str'},
        'extension_attribute11': {'key': 'extensionAttribute11', 'type': 'str'},
        'extension_attribute12': {'key': 'extensionAttribute12', 'type': 'str'},
        'extension_attribute13': {'key': 'extensionAttribute13', 'type': 'str'},
        'extension_attribute14': {'key': 'extensionAttribute14', 'type': 'str'},
        'extension_attribute15': {'key': 'extensionAttribute15', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphOnPremisesExtensionAttributes, self).__init__(**kwargs)
        self.extension_attribute1 = kwargs.get('extension_attribute1', None)
        self.extension_attribute2 = kwargs.get('extension_attribute2', None)
        self.extension_attribute3 = kwargs.get('extension_attribute3', None)
        self.extension_attribute4 = kwargs.get('extension_attribute4', None)
        self.extension_attribute5 = kwargs.get('extension_attribute5', None)
        self.extension_attribute6 = kwargs.get('extension_attribute6', None)
        self.extension_attribute7 = kwargs.get('extension_attribute7', None)
        self.extension_attribute8 = kwargs.get('extension_attribute8', None)
        self.extension_attribute9 = kwargs.get('extension_attribute9', None)
        self.extension_attribute10 = kwargs.get('extension_attribute10', None)
        self.extension_attribute11 = kwargs.get('extension_attribute11', None)
        self.extension_attribute12 = kwargs.get('extension_attribute12', None)
        self.extension_attribute13 = kwargs.get('extension_attribute13', None)
        self.extension_attribute14 = kwargs.get('extension_attribute14', None)
        self.extension_attribute15 = kwargs.get('extension_attribute15', None)


class MicrosoftGraphOnPremisesProvisioningError(msrest.serialization.Model):
    """onPremisesProvisioningError.

    :param value: Value of the property causing the error.
    :type value: str
    :param category: Category of the provisioning error. Note: Currently, there is only one
     possible value. Possible value: PropertyConflict - indicates a property value is not unique.
     Other objects contain the same value for the property.
    :type category: str
    :param property_causing_error: Name of the directory property causing the error. Current
     possible values: UserPrincipalName or ProxyAddress.
    :type property_causing_error: str
    :param occurred_date_time: The date and time at which the error occurred.
    :type occurred_date_time: ~datetime.datetime
    """

    _attribute_map = {
        'value': {'key': 'value', 'type': 'str'},
        'category': {'key': 'category', 'type': 'str'},
        'property_causing_error': {'key': 'propertyCausingError', 'type': 'str'},
        'occurred_date_time': {'key': 'occurredDateTime', 'type': 'iso-8601'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphOnPremisesProvisioningError, self).__init__(**kwargs)
        self.value = kwargs.get('value', None)
        self.category = kwargs.get('category', None)
        self.property_causing_error = kwargs.get('property_causing_error', None)
        self.occurred_date_time = kwargs.get('occurred_date_time', None)


class MicrosoftGraphOpenShift(MicrosoftGraphChangeTrackedEntity):
    """openShift.

    :param id: Read-only.
    :type id: str
    :param created_date_time:
    :type created_date_time: ~datetime.datetime
    :param last_modified_date_time:
    :type last_modified_date_time: ~datetime.datetime
    :param last_modified_by: identitySet.
    :type last_modified_by: ~users_user.models.MicrosoftGraphIdentitySet
    :param shared_open_shift: openShiftItem.
    :type shared_open_shift: ~users_user.models.MicrosoftGraphOpenShiftItem
    :param draft_open_shift: openShiftItem.
    :type draft_open_shift: ~users_user.models.MicrosoftGraphOpenShiftItem
    :param scheduling_group_id:
    :type scheduling_group_id: str
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'created_date_time': {'key': 'createdDateTime', 'type': 'iso-8601'},
        'last_modified_date_time': {'key': 'lastModifiedDateTime', 'type': 'iso-8601'},
        'last_modified_by': {'key': 'lastModifiedBy', 'type': 'MicrosoftGraphIdentitySet'},
        'shared_open_shift': {'key': 'sharedOpenShift', 'type': 'MicrosoftGraphOpenShiftItem'},
        'draft_open_shift': {'key': 'draftOpenShift', 'type': 'MicrosoftGraphOpenShiftItem'},
        'scheduling_group_id': {'key': 'schedulingGroupId', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphOpenShift, self).__init__(**kwargs)
        self.shared_open_shift = kwargs.get('shared_open_shift', None)
        self.draft_open_shift = kwargs.get('draft_open_shift', None)
        self.scheduling_group_id = kwargs.get('scheduling_group_id', None)


class MicrosoftGraphOpenShiftChangeRequest(MicrosoftGraphScheduleChangeRequest):
    """openShiftChangeRequest.

    :param id: Read-only.
    :type id: str
    :param created_date_time:
    :type created_date_time: ~datetime.datetime
    :param last_modified_date_time:
    :type last_modified_date_time: ~datetime.datetime
    :param last_modified_by: identitySet.
    :type last_modified_by: ~users_user.models.MicrosoftGraphIdentitySet
    :param assigned_to: scheduleChangeRequestActor. Possible values include: "sender", "recipient",
     "manager", "system", "unknownFutureValue".
    :type assigned_to: str or ~users_user.models.MicrosoftGraphScheduleChangeRequestActor
    :param state: scheduleChangeState. Possible values include: "pending", "approved", "declined",
     "unknownFutureValue".
    :type state: str or ~users_user.models.MicrosoftGraphScheduleChangeState
    :param sender_message:
    :type sender_message: str
    :param sender_date_time:
    :type sender_date_time: ~datetime.datetime
    :param manager_action_message:
    :type manager_action_message: str
    :param manager_action_date_time:
    :type manager_action_date_time: ~datetime.datetime
    :param sender_user_id:
    :type sender_user_id: str
    :param manager_user_id:
    :type manager_user_id: str
    :param open_shift_id:
    :type open_shift_id: str
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'created_date_time': {'key': 'createdDateTime', 'type': 'iso-8601'},
        'last_modified_date_time': {'key': 'lastModifiedDateTime', 'type': 'iso-8601'},
        'last_modified_by': {'key': 'lastModifiedBy', 'type': 'MicrosoftGraphIdentitySet'},
        'assigned_to': {'key': 'assignedTo', 'type': 'str'},
        'state': {'key': 'state', 'type': 'str'},
        'sender_message': {'key': 'senderMessage', 'type': 'str'},
        'sender_date_time': {'key': 'senderDateTime', 'type': 'iso-8601'},
        'manager_action_message': {'key': 'managerActionMessage', 'type': 'str'},
        'manager_action_date_time': {'key': 'managerActionDateTime', 'type': 'iso-8601'},
        'sender_user_id': {'key': 'senderUserId', 'type': 'str'},
        'manager_user_id': {'key': 'managerUserId', 'type': 'str'},
        'open_shift_id': {'key': 'openShiftId', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphOpenShiftChangeRequest, self).__init__(**kwargs)
        self.open_shift_id = kwargs.get('open_shift_id', None)


class MicrosoftGraphScheduleEntity(msrest.serialization.Model):
    """scheduleEntity.

    :param start_date_time:
    :type start_date_time: ~datetime.datetime
    :param end_date_time:
    :type end_date_time: ~datetime.datetime
    :param theme: scheduleEntityTheme. Possible values include: "white", "blue", "green", "purple",
     "pink", "yellow", "gray", "darkBlue", "darkGreen", "darkPurple", "darkPink", "darkYellow",
     "unknownFutureValue".
    :type theme: str or ~users_user.models.MicrosoftGraphScheduleEntityTheme
    """

    _attribute_map = {
        'start_date_time': {'key': 'startDateTime', 'type': 'iso-8601'},
        'end_date_time': {'key': 'endDateTime', 'type': 'iso-8601'},
        'theme': {'key': 'theme', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphScheduleEntity, self).__init__(**kwargs)
        self.start_date_time = kwargs.get('start_date_time', None)
        self.end_date_time = kwargs.get('end_date_time', None)
        self.theme = kwargs.get('theme', None)


class MicrosoftGraphShiftItem(MicrosoftGraphScheduleEntity):
    """shiftItem.

    :param start_date_time:
    :type start_date_time: ~datetime.datetime
    :param end_date_time:
    :type end_date_time: ~datetime.datetime
    :param theme: scheduleEntityTheme. Possible values include: "white", "blue", "green", "purple",
     "pink", "yellow", "gray", "darkBlue", "darkGreen", "darkPurple", "darkPink", "darkYellow",
     "unknownFutureValue".
    :type theme: str or ~users_user.models.MicrosoftGraphScheduleEntityTheme
    :param display_name:
    :type display_name: str
    :param notes:
    :type notes: str
    :param activities:
    :type activities: list[~users_user.models.MicrosoftGraphShiftActivity]
    """

    _attribute_map = {
        'start_date_time': {'key': 'startDateTime', 'type': 'iso-8601'},
        'end_date_time': {'key': 'endDateTime', 'type': 'iso-8601'},
        'theme': {'key': 'theme', 'type': 'str'},
        'display_name': {'key': 'displayName', 'type': 'str'},
        'notes': {'key': 'notes', 'type': 'str'},
        'activities': {'key': 'activities', 'type': '[MicrosoftGraphShiftActivity]'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphShiftItem, self).__init__(**kwargs)
        self.display_name = kwargs.get('display_name', None)
        self.notes = kwargs.get('notes', None)
        self.activities = kwargs.get('activities', None)


class MicrosoftGraphOpenShiftItem(MicrosoftGraphShiftItem):
    """openShiftItem.

    :param start_date_time:
    :type start_date_time: ~datetime.datetime
    :param end_date_time:
    :type end_date_time: ~datetime.datetime
    :param theme: scheduleEntityTheme. Possible values include: "white", "blue", "green", "purple",
     "pink", "yellow", "gray", "darkBlue", "darkGreen", "darkPurple", "darkPink", "darkYellow",
     "unknownFutureValue".
    :type theme: str or ~users_user.models.MicrosoftGraphScheduleEntityTheme
    :param display_name:
    :type display_name: str
    :param notes:
    :type notes: str
    :param activities:
    :type activities: list[~users_user.models.MicrosoftGraphShiftActivity]
    :param open_slot_count:
    :type open_slot_count: int
    """

    _validation = {
        'open_slot_count': {'maximum': 2147483647, 'minimum': -2147483648},
    }

    _attribute_map = {
        'start_date_time': {'key': 'startDateTime', 'type': 'iso-8601'},
        'end_date_time': {'key': 'endDateTime', 'type': 'iso-8601'},
        'theme': {'key': 'theme', 'type': 'str'},
        'display_name': {'key': 'displayName', 'type': 'str'},
        'notes': {'key': 'notes', 'type': 'str'},
        'activities': {'key': 'activities', 'type': '[MicrosoftGraphShiftActivity]'},
        'open_slot_count': {'key': 'openSlotCount', 'type': 'int'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphOpenShiftItem, self).__init__(**kwargs)
        self.open_slot_count = kwargs.get('open_slot_count', None)


class MicrosoftGraphOperationError(msrest.serialization.Model):
    """operationError.

    :param code: Operation error code.
    :type code: str
    :param message: Operation error message.
    :type message: str
    """

    _attribute_map = {
        'code': {'key': 'code', 'type': 'str'},
        'message': {'key': 'message', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphOperationError, self).__init__(**kwargs)
        self.code = kwargs.get('code', None)
        self.message = kwargs.get('message', None)


class MicrosoftGraphOutlookCategory(MicrosoftGraphEntity):
    """outlookCategory.

    :param id: Read-only.
    :type id: str
    :param display_name: A unique name that identifies a category in the user's mailbox. After a
     category is created, the name cannot be changed. Read-only.
    :type display_name: str
    :param color: categoryColor. Possible values include: "preset0", "preset1", "preset2",
     "preset3", "preset4", "preset5", "preset6", "preset7", "preset8", "preset9", "preset10",
     "preset11", "preset12", "preset13", "preset14", "preset15", "preset16", "preset17", "preset18",
     "preset19", "preset20", "preset21", "preset22", "preset23", "preset24", "none".
    :type color: str or ~users_user.models.MicrosoftGraphCategoryColor
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'display_name': {'key': 'displayName', 'type': 'str'},
        'color': {'key': 'color', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphOutlookCategory, self).__init__(**kwargs)
        self.display_name = kwargs.get('display_name', None)
        self.color = kwargs.get('color', None)


class MicrosoftGraphOutlookGeoCoordinates(msrest.serialization.Model):
    """outlookGeoCoordinates.

    :param altitude: The altitude of the location.
    :type altitude: float
    :param latitude: The latitude of the location.
    :type latitude: float
    :param longitude: The longitude of the location.
    :type longitude: float
    :param accuracy: The accuracy of the latitude and longitude. As an example, the accuracy can be
     measured in meters, such as the latitude and longitude are accurate to within 50 meters.
    :type accuracy: float
    :param altitude_accuracy: The accuracy of the altitude.
    :type altitude_accuracy: float
    """

    _attribute_map = {
        'altitude': {'key': 'altitude', 'type': 'float'},
        'latitude': {'key': 'latitude', 'type': 'float'},
        'longitude': {'key': 'longitude', 'type': 'float'},
        'accuracy': {'key': 'accuracy', 'type': 'float'},
        'altitude_accuracy': {'key': 'altitudeAccuracy', 'type': 'float'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphOutlookGeoCoordinates, self).__init__(**kwargs)
        self.altitude = kwargs.get('altitude', None)
        self.latitude = kwargs.get('latitude', None)
        self.longitude = kwargs.get('longitude', None)
        self.accuracy = kwargs.get('accuracy', None)
        self.altitude_accuracy = kwargs.get('altitude_accuracy', None)


class MicrosoftGraphOutlookTask(MicrosoftGraphOutlookItem):
    """outlookTask.

    :param id: Read-only.
    :type id: str
    :param created_date_time: The Timestamp type represents date and time information using ISO
     8601 format and is always in UTC time. For example, midnight UTC on Jan 1, 2014 would look like
     this: '2014-01-01T00:00:00Z'.
    :type created_date_time: ~datetime.datetime
    :param last_modified_date_time: The Timestamp type represents date and time information using
     ISO 8601 format and is always in UTC time. For example, midnight UTC on Jan 1, 2014 would look
     like this: '2014-01-01T00:00:00Z'.
    :type last_modified_date_time: ~datetime.datetime
    :param change_key: Identifies the version of the item. Every time the item is changed,
     changeKey changes as well. This allows Exchange to apply changes to the correct version of the
     object. Read-only.
    :type change_key: str
    :param categories: The categories associated with the item.
    :type categories: list[str]
    :param assigned_to:
    :type assigned_to: str
    :param body: itemBody.
    :type body: ~users_user.models.MicrosoftGraphItemBody
    :param completed_date_time: dateTimeTimeZone.
    :type completed_date_time: ~users_user.models.MicrosoftGraphDateTimeZone
    :param due_date_time: dateTimeTimeZone.
    :type due_date_time: ~users_user.models.MicrosoftGraphDateTimeZone
    :param has_attachments:
    :type has_attachments: bool
    :param importance: importance. Possible values include: "low", "normal", "high".
    :type importance: str or ~users_user.models.MicrosoftGraphImportance
    :param is_reminder_on:
    :type is_reminder_on: bool
    :param owner:
    :type owner: str
    :param parent_folder_id:
    :type parent_folder_id: str
    :param recurrence: patternedRecurrence.
    :type recurrence: ~users_user.models.MicrosoftGraphPatternedRecurrence
    :param reminder_date_time: dateTimeTimeZone.
    :type reminder_date_time: ~users_user.models.MicrosoftGraphDateTimeZone
    :param sensitivity: sensitivity. Possible values include: "normal", "personal", "private",
     "confidential".
    :type sensitivity: str or ~users_user.models.MicrosoftGraphSensitivity
    :param start_date_time: dateTimeTimeZone.
    :type start_date_time: ~users_user.models.MicrosoftGraphDateTimeZone
    :param status: taskStatus. Possible values include: "notStarted", "inProgress", "completed",
     "waitingOnOthers", "deferred".
    :type status: str or ~users_user.models.MicrosoftGraphTaskStatus
    :param subject:
    :type subject: str
    :param single_value_extended_properties:
    :type single_value_extended_properties:
     list[~users_user.models.MicrosoftGraphSingleValueLegacyExtendedProperty]
    :param multi_value_extended_properties:
    :type multi_value_extended_properties:
     list[~users_user.models.MicrosoftGraphMultiValueLegacyExtendedProperty]
    :param attachments:
    :type attachments: list[~users_user.models.MicrosoftGraphAttachment]
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'created_date_time': {'key': 'createdDateTime', 'type': 'iso-8601'},
        'last_modified_date_time': {'key': 'lastModifiedDateTime', 'type': 'iso-8601'},
        'change_key': {'key': 'changeKey', 'type': 'str'},
        'categories': {'key': 'categories', 'type': '[str]'},
        'assigned_to': {'key': 'assignedTo', 'type': 'str'},
        'body': {'key': 'body', 'type': 'MicrosoftGraphItemBody'},
        'completed_date_time': {'key': 'completedDateTime', 'type': 'MicrosoftGraphDateTimeZone'},
        'due_date_time': {'key': 'dueDateTime', 'type': 'MicrosoftGraphDateTimeZone'},
        'has_attachments': {'key': 'hasAttachments', 'type': 'bool'},
        'importance': {'key': 'importance', 'type': 'str'},
        'is_reminder_on': {'key': 'isReminderOn', 'type': 'bool'},
        'owner': {'key': 'owner', 'type': 'str'},
        'parent_folder_id': {'key': 'parentFolderId', 'type': 'str'},
        'recurrence': {'key': 'recurrence', 'type': 'MicrosoftGraphPatternedRecurrence'},
        'reminder_date_time': {'key': 'reminderDateTime', 'type': 'MicrosoftGraphDateTimeZone'},
        'sensitivity': {'key': 'sensitivity', 'type': 'str'},
        'start_date_time': {'key': 'startDateTime', 'type': 'MicrosoftGraphDateTimeZone'},
        'status': {'key': 'status', 'type': 'str'},
        'subject': {'key': 'subject', 'type': 'str'},
        'single_value_extended_properties': {'key': 'singleValueExtendedProperties', 'type': '[MicrosoftGraphSingleValueLegacyExtendedProperty]'},
        'multi_value_extended_properties': {'key': 'multiValueExtendedProperties', 'type': '[MicrosoftGraphMultiValueLegacyExtendedProperty]'},
        'attachments': {'key': 'attachments', 'type': '[MicrosoftGraphAttachment]'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphOutlookTask, self).__init__(**kwargs)
        self.assigned_to = kwargs.get('assigned_to', None)
        self.body = kwargs.get('body', None)
        self.completed_date_time = kwargs.get('completed_date_time', None)
        self.due_date_time = kwargs.get('due_date_time', None)
        self.has_attachments = kwargs.get('has_attachments', None)
        self.importance = kwargs.get('importance', None)
        self.is_reminder_on = kwargs.get('is_reminder_on', None)
        self.owner = kwargs.get('owner', None)
        self.parent_folder_id = kwargs.get('parent_folder_id', None)
        self.recurrence = kwargs.get('recurrence', None)
        self.reminder_date_time = kwargs.get('reminder_date_time', None)
        self.sensitivity = kwargs.get('sensitivity', None)
        self.start_date_time = kwargs.get('start_date_time', None)
        self.status = kwargs.get('status', None)
        self.subject = kwargs.get('subject', None)
        self.single_value_extended_properties = kwargs.get('single_value_extended_properties', None)
        self.multi_value_extended_properties = kwargs.get('multi_value_extended_properties', None)
        self.attachments = kwargs.get('attachments', None)


class MicrosoftGraphOutlookTaskFolder(MicrosoftGraphEntity):
    """outlookTaskFolder.

    :param id: Read-only.
    :type id: str
    :param change_key:
    :type change_key: str
    :param name:
    :type name: str
    :param is_default_folder:
    :type is_default_folder: bool
    :param parent_group_key:
    :type parent_group_key: str
    :param tasks:
    :type tasks: list[~users_user.models.MicrosoftGraphOutlookTask]
    :param single_value_extended_properties:
    :type single_value_extended_properties:
     list[~users_user.models.MicrosoftGraphSingleValueLegacyExtendedProperty]
    :param multi_value_extended_properties:
    :type multi_value_extended_properties:
     list[~users_user.models.MicrosoftGraphMultiValueLegacyExtendedProperty]
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'change_key': {'key': 'changeKey', 'type': 'str'},
        'name': {'key': 'name', 'type': 'str'},
        'is_default_folder': {'key': 'isDefaultFolder', 'type': 'bool'},
        'parent_group_key': {'key': 'parentGroupKey', 'type': 'str'},
        'tasks': {'key': 'tasks', 'type': '[MicrosoftGraphOutlookTask]'},
        'single_value_extended_properties': {'key': 'singleValueExtendedProperties', 'type': '[MicrosoftGraphSingleValueLegacyExtendedProperty]'},
        'multi_value_extended_properties': {'key': 'multiValueExtendedProperties', 'type': '[MicrosoftGraphMultiValueLegacyExtendedProperty]'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphOutlookTaskFolder, self).__init__(**kwargs)
        self.change_key = kwargs.get('change_key', None)
        self.name = kwargs.get('name', None)
        self.is_default_folder = kwargs.get('is_default_folder', None)
        self.parent_group_key = kwargs.get('parent_group_key', None)
        self.tasks = kwargs.get('tasks', None)
        self.single_value_extended_properties = kwargs.get('single_value_extended_properties', None)
        self.multi_value_extended_properties = kwargs.get('multi_value_extended_properties', None)


class MicrosoftGraphOutlookTaskGroup(MicrosoftGraphEntity):
    """outlookTaskGroup.

    :param id: Read-only.
    :type id: str
    :param change_key:
    :type change_key: str
    :param is_default_group:
    :type is_default_group: bool
    :param name:
    :type name: str
    :param group_key:
    :type group_key: str
    :param task_folders:
    :type task_folders: list[~users_user.models.MicrosoftGraphOutlookTaskFolder]
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'change_key': {'key': 'changeKey', 'type': 'str'},
        'is_default_group': {'key': 'isDefaultGroup', 'type': 'bool'},
        'name': {'key': 'name', 'type': 'str'},
        'group_key': {'key': 'groupKey', 'type': 'str'},
        'task_folders': {'key': 'taskFolders', 'type': '[MicrosoftGraphOutlookTaskFolder]'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphOutlookTaskGroup, self).__init__(**kwargs)
        self.change_key = kwargs.get('change_key', None)
        self.is_default_group = kwargs.get('is_default_group', None)
        self.name = kwargs.get('name', None)
        self.group_key = kwargs.get('group_key', None)
        self.task_folders = kwargs.get('task_folders', None)


class MicrosoftGraphOutlookUser(MicrosoftGraphEntity):
    """outlookUser.

    :param id: Read-only.
    :type id: str
    :param master_categories: A list of categories defined for the user.
    :type master_categories: list[~users_user.models.MicrosoftGraphOutlookCategory]
    :param task_groups:
    :type task_groups: list[~users_user.models.MicrosoftGraphOutlookTaskGroup]
    :param task_folders:
    :type task_folders: list[~users_user.models.MicrosoftGraphOutlookTaskFolder]
    :param tasks:
    :type tasks: list[~users_user.models.MicrosoftGraphOutlookTask]
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'master_categories': {'key': 'masterCategories', 'type': '[MicrosoftGraphOutlookCategory]'},
        'task_groups': {'key': 'taskGroups', 'type': '[MicrosoftGraphOutlookTaskGroup]'},
        'task_folders': {'key': 'taskFolders', 'type': '[MicrosoftGraphOutlookTaskFolder]'},
        'tasks': {'key': 'tasks', 'type': '[MicrosoftGraphOutlookTask]'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphOutlookUser, self).__init__(**kwargs)
        self.master_categories = kwargs.get('master_categories', None)
        self.task_groups = kwargs.get('task_groups', None)
        self.task_folders = kwargs.get('task_folders', None)
        self.tasks = kwargs.get('tasks', None)


class MicrosoftGraphPackage(msrest.serialization.Model):
    """package.

    :param type: A string indicating the type of package. While oneNote is the only currently
     defined value, you should expect other package types to be returned and handle them
     accordingly.
    :type type: str
    """

    _attribute_map = {
        'type': {'key': 'type', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphPackage, self).__init__(**kwargs)
        self.type = kwargs.get('type', None)


class MicrosoftGraphPageLinks(msrest.serialization.Model):
    """pageLinks.

    :param one_note_client_url: externalLink.
    :type one_note_client_url: ~users_user.models.MicrosoftGraphExternalLink
    :param one_note_web_url: externalLink.
    :type one_note_web_url: ~users_user.models.MicrosoftGraphExternalLink
    """

    _attribute_map = {
        'one_note_client_url': {'key': 'oneNoteClientUrl', 'type': 'MicrosoftGraphExternalLink'},
        'one_note_web_url': {'key': 'oneNoteWebUrl', 'type': 'MicrosoftGraphExternalLink'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphPageLinks, self).__init__(**kwargs)
        self.one_note_client_url = kwargs.get('one_note_client_url', None)
        self.one_note_web_url = kwargs.get('one_note_web_url', None)


class MicrosoftGraphPasswordAuthenticationMethod(MicrosoftGraphEntity):
    """passwordAuthenticationMethod.

    :param id: Read-only.
    :type id: str
    :param password:
    :type password: str
    :param creation_date_time:
    :type creation_date_time: ~datetime.datetime
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'password': {'key': 'password', 'type': 'str'},
        'creation_date_time': {'key': 'creationDateTime', 'type': 'iso-8601'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphPasswordAuthenticationMethod, self).__init__(**kwargs)
        self.password = kwargs.get('password', None)
        self.creation_date_time = kwargs.get('creation_date_time', None)


class MicrosoftGraphPasswordProfile(msrest.serialization.Model):
    """passwordProfile.

    :param password: The password for the user. This property is required when a user is created.
     It can be updated, but the user will be required to change the password on the next login. The
     password must satisfy minimum requirements as specified by the user’s passwordPolicies
     property. By default, a strong password is required.
    :type password: str
    :param force_change_password_next_sign_in: true if the user must change her password on the
     next login; otherwise false.
    :type force_change_password_next_sign_in: bool
    :param force_change_password_next_sign_in_with_mfa: If true, at next sign-in, the user must
     perform a multi-factor authentication (MFA) before being forced to change their password. The
     behavior is identical to forceChangePasswordNextSignIn except that the user is required to
     first perform a multi-factor authentication before password change. After a password change,
     this property will be automatically reset to false. If not set, default is false.
    :type force_change_password_next_sign_in_with_mfa: bool
    """

    _attribute_map = {
        'password': {'key': 'password', 'type': 'str'},
        'force_change_password_next_sign_in': {'key': 'forceChangePasswordNextSignIn', 'type': 'bool'},
        'force_change_password_next_sign_in_with_mfa': {'key': 'forceChangePasswordNextSignInWithMfa', 'type': 'bool'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphPasswordProfile, self).__init__(**kwargs)
        self.password = kwargs.get('password', None)
        self.force_change_password_next_sign_in = kwargs.get('force_change_password_next_sign_in', None)
        self.force_change_password_next_sign_in_with_mfa = kwargs.get('force_change_password_next_sign_in_with_mfa', None)


class MicrosoftGraphPatternedRecurrence(msrest.serialization.Model):
    """patternedRecurrence.

    :param pattern: recurrencePattern.
    :type pattern: ~users_user.models.MicrosoftGraphRecurrencePattern
    :param range: recurrenceRange.
    :type range: ~users_user.models.MicrosoftGraphRecurrenceRange
    """

    _attribute_map = {
        'pattern': {'key': 'pattern', 'type': 'MicrosoftGraphRecurrencePattern'},
        'range': {'key': 'range', 'type': 'MicrosoftGraphRecurrenceRange'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphPatternedRecurrence, self).__init__(**kwargs)
        self.pattern = kwargs.get('pattern', None)
        self.range = kwargs.get('range', None)


class MicrosoftGraphPayloadResponse(MicrosoftGraphEntity):
    """payloadResponse.

    :param id: Read-only.
    :type id: str
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphPayloadResponse, self).__init__(**kwargs)


class MicrosoftGraphPayloadTypes(msrest.serialization.Model):
    """payloadTypes.

    :param raw_content:
    :type raw_content: str
    :param visual_content: visualProperties.
    :type visual_content: ~users_user.models.MicrosoftGraphVisualProperties
    """

    _attribute_map = {
        'raw_content': {'key': 'rawContent', 'type': 'str'},
        'visual_content': {'key': 'visualContent', 'type': 'MicrosoftGraphVisualProperties'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphPayloadTypes, self).__init__(**kwargs)
        self.raw_content = kwargs.get('raw_content', None)
        self.visual_content = kwargs.get('visual_content', None)


class MicrosoftGraphPendingContentUpdate(msrest.serialization.Model):
    """pendingContentUpdate.

    :param queued_date_time:
    :type queued_date_time: ~datetime.datetime
    """

    _attribute_map = {
        'queued_date_time': {'key': 'queuedDateTime', 'type': 'iso-8601'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphPendingContentUpdate, self).__init__(**kwargs)
        self.queued_date_time = kwargs.get('queued_date_time', None)


class MicrosoftGraphPendingOperations(msrest.serialization.Model):
    """pendingOperations.

    :param pending_content_update: pendingContentUpdate.
    :type pending_content_update: ~users_user.models.MicrosoftGraphPendingContentUpdate
    """

    _attribute_map = {
        'pending_content_update': {'key': 'pendingContentUpdate', 'type': 'MicrosoftGraphPendingContentUpdate'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphPendingOperations, self).__init__(**kwargs)
        self.pending_content_update = kwargs.get('pending_content_update', None)


class MicrosoftGraphPermission(MicrosoftGraphEntity):
    """permission.

    :param id: Read-only.
    :type id: str
    :param expiration_date_time: A format of yyyy-MM-ddTHH:mm:ssZ of DateTimeOffset indicates the
     expiration time of the permission. DateTime.MinValue indicates there is no expiration set for
     this permission. Optional.
    :type expiration_date_time: ~datetime.datetime
    :param granted_to: identitySet.
    :type granted_to: ~users_user.models.MicrosoftGraphIdentitySet
    :param granted_to_identities: For link type permissions, the details of the users to whom
     permission was granted. Read-only.
    :type granted_to_identities: list[~users_user.models.MicrosoftGraphIdentitySet]
    :param has_password: This indicates whether password is set for this permission, it's only
     showing in response. Optional and Read-only and for OneDrive Personal only.
    :type has_password: bool
    :param inherited_from: itemReference.
    :type inherited_from: ~users_user.models.MicrosoftGraphItemReference
    :param invitation: sharingInvitation.
    :type invitation: ~users_user.models.MicrosoftGraphSharingInvitation
    :param link: sharingLink.
    :type link: ~users_user.models.MicrosoftGraphSharingLink
    :param roles: The type of permission, e.g. read. See below for the full list of roles. Read-
     only.
    :type roles: list[str]
    :param share_id: A unique token that can be used to access this shared item via the **shares**
     API. Read-only.
    :type share_id: str
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'expiration_date_time': {'key': 'expirationDateTime', 'type': 'iso-8601'},
        'granted_to': {'key': 'grantedTo', 'type': 'MicrosoftGraphIdentitySet'},
        'granted_to_identities': {'key': 'grantedToIdentities', 'type': '[MicrosoftGraphIdentitySet]'},
        'has_password': {'key': 'hasPassword', 'type': 'bool'},
        'inherited_from': {'key': 'inheritedFrom', 'type': 'MicrosoftGraphItemReference'},
        'invitation': {'key': 'invitation', 'type': 'MicrosoftGraphSharingInvitation'},
        'link': {'key': 'link', 'type': 'MicrosoftGraphSharingLink'},
        'roles': {'key': 'roles', 'type': '[str]'},
        'share_id': {'key': 'shareId', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphPermission, self).__init__(**kwargs)
        self.expiration_date_time = kwargs.get('expiration_date_time', None)
        self.granted_to = kwargs.get('granted_to', None)
        self.granted_to_identities = kwargs.get('granted_to_identities', None)
        self.has_password = kwargs.get('has_password', None)
        self.inherited_from = kwargs.get('inherited_from', None)
        self.invitation = kwargs.get('invitation', None)
        self.link = kwargs.get('link', None)
        self.roles = kwargs.get('roles', None)
        self.share_id = kwargs.get('share_id', None)


class MicrosoftGraphPerson(MicrosoftGraphEntity):
    """person.

    :param id: Read-only.
    :type id: str
    :param display_name: The person's display name.
    :type display_name: str
    :param given_name: The person's given name.
    :type given_name: str
    :param surname: The person's surname.
    :type surname: str
    :param birthday: The person's birthday.
    :type birthday: str
    :param person_notes: Free-form notes that the user has taken about this person.
    :type person_notes: str
    :param is_favorite: true if the user has flagged this person as a favorite.
    :type is_favorite: bool
    :param email_addresses:
    :type email_addresses: list[~users_user.models.MicrosoftGraphRankedEmailAddress]
    :param phones: The person's phone numbers.
    :type phones: list[~users_user.models.MicrosoftGraphPhone]
    :param postal_addresses: The person's addresses.
    :type postal_addresses: list[~users_user.models.MicrosoftGraphLocation]
    :param websites: The person's websites.
    :type websites: list[~users_user.models.MicrosoftGraphWebsite]
    :param title:
    :type title: str
    :param company_name: The name of the person's company.
    :type company_name: str
    :param yomi_company: The phonetic Japanese name of the person's company.
    :type yomi_company: str
    :param department: The person's department.
    :type department: str
    :param office_location: The location of the person's office.
    :type office_location: str
    :param profession: The person's profession.
    :type profession: str
    :param sources:
    :type sources: list[~users_user.models.MicrosoftGraphPersonDataSource]
    :param mailbox_type:
    :type mailbox_type: str
    :param person_type: The type of person.
    :type person_type: str
    :param user_principal_name: The user principal name (UPN) of the person. The UPN is an
     Internet-style login name for the person based on the Internet standard RFC 822. By convention,
     this should map to the person's email name. The general format is alias@domain.
    :type user_principal_name: str
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'display_name': {'key': 'displayName', 'type': 'str'},
        'given_name': {'key': 'givenName', 'type': 'str'},
        'surname': {'key': 'surname', 'type': 'str'},
        'birthday': {'key': 'birthday', 'type': 'str'},
        'person_notes': {'key': 'personNotes', 'type': 'str'},
        'is_favorite': {'key': 'isFavorite', 'type': 'bool'},
        'email_addresses': {'key': 'emailAddresses', 'type': '[MicrosoftGraphRankedEmailAddress]'},
        'phones': {'key': 'phones', 'type': '[MicrosoftGraphPhone]'},
        'postal_addresses': {'key': 'postalAddresses', 'type': '[MicrosoftGraphLocation]'},
        'websites': {'key': 'websites', 'type': '[MicrosoftGraphWebsite]'},
        'title': {'key': 'title', 'type': 'str'},
        'company_name': {'key': 'companyName', 'type': 'str'},
        'yomi_company': {'key': 'yomiCompany', 'type': 'str'},
        'department': {'key': 'department', 'type': 'str'},
        'office_location': {'key': 'officeLocation', 'type': 'str'},
        'profession': {'key': 'profession', 'type': 'str'},
        'sources': {'key': 'sources', 'type': '[MicrosoftGraphPersonDataSource]'},
        'mailbox_type': {'key': 'mailboxType', 'type': 'str'},
        'person_type': {'key': 'personType', 'type': 'str'},
        'user_principal_name': {'key': 'userPrincipalName', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphPerson, self).__init__(**kwargs)
        self.display_name = kwargs.get('display_name', None)
        self.given_name = kwargs.get('given_name', None)
        self.surname = kwargs.get('surname', None)
        self.birthday = kwargs.get('birthday', None)
        self.person_notes = kwargs.get('person_notes', None)
        self.is_favorite = kwargs.get('is_favorite', None)
        self.email_addresses = kwargs.get('email_addresses', None)
        self.phones = kwargs.get('phones', None)
        self.postal_addresses = kwargs.get('postal_addresses', None)
        self.websites = kwargs.get('websites', None)
        self.title = kwargs.get('title', None)
        self.company_name = kwargs.get('company_name', None)
        self.yomi_company = kwargs.get('yomi_company', None)
        self.department = kwargs.get('department', None)
        self.office_location = kwargs.get('office_location', None)
        self.profession = kwargs.get('profession', None)
        self.sources = kwargs.get('sources', None)
        self.mailbox_type = kwargs.get('mailbox_type', None)
        self.person_type = kwargs.get('person_type', None)
        self.user_principal_name = kwargs.get('user_principal_name', None)


class MicrosoftGraphPersonAnniversary(MicrosoftGraphItemFacet):
    """personAnniversary.

    :param id: Read-only.
    :type id: str
    :param allowed_audiences: allowedAudiences. Possible values include: "me", "family",
     "contacts", "groupMembers", "organization", "federatedOrganizations", "everyone",
     "unknownFutureValue".
    :type allowed_audiences: str or ~users_user.models.MicrosoftGraphAllowedAudiences
    :param inference: inferenceData.
    :type inference: ~users_user.models.MicrosoftGraphInferenceData
    :param created_date_time:
    :type created_date_time: ~datetime.datetime
    :param created_by: identitySet.
    :type created_by: ~users_user.models.MicrosoftGraphIdentitySet
    :param last_modified_date_time:
    :type last_modified_date_time: ~datetime.datetime
    :param last_modified_by: identitySet.
    :type last_modified_by: ~users_user.models.MicrosoftGraphIdentitySet
    :param type: anniversaryType. Possible values include: "birthday", "wedding",
     "unknownFutureValue".
    :type type: str or ~users_user.models.MicrosoftGraphAnniversaryType
    :param date:
    :type date: ~datetime.date
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'allowed_audiences': {'key': 'allowedAudiences', 'type': 'str'},
        'inference': {'key': 'inference', 'type': 'MicrosoftGraphInferenceData'},
        'created_date_time': {'key': 'createdDateTime', 'type': 'iso-8601'},
        'created_by': {'key': 'createdBy', 'type': 'MicrosoftGraphIdentitySet'},
        'last_modified_date_time': {'key': 'lastModifiedDateTime', 'type': 'iso-8601'},
        'last_modified_by': {'key': 'lastModifiedBy', 'type': 'MicrosoftGraphIdentitySet'},
        'type': {'key': 'type', 'type': 'str'},
        'date': {'key': 'date', 'type': 'date'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphPersonAnniversary, self).__init__(**kwargs)
        self.type = kwargs.get('type', None)
        self.date = kwargs.get('date', None)


class MicrosoftGraphPersonDataSource(msrest.serialization.Model):
    """personDataSource.

    :param type:
    :type type: str
    """

    _attribute_map = {
        'type': {'key': 'type', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphPersonDataSource, self).__init__(**kwargs)
        self.type = kwargs.get('type', None)


class MicrosoftGraphPersonInterest(MicrosoftGraphItemFacet):
    """personInterest.

    :param id: Read-only.
    :type id: str
    :param allowed_audiences: allowedAudiences. Possible values include: "me", "family",
     "contacts", "groupMembers", "organization", "federatedOrganizations", "everyone",
     "unknownFutureValue".
    :type allowed_audiences: str or ~users_user.models.MicrosoftGraphAllowedAudiences
    :param inference: inferenceData.
    :type inference: ~users_user.models.MicrosoftGraphInferenceData
    :param created_date_time:
    :type created_date_time: ~datetime.datetime
    :param created_by: identitySet.
    :type created_by: ~users_user.models.MicrosoftGraphIdentitySet
    :param last_modified_date_time:
    :type last_modified_date_time: ~datetime.datetime
    :param last_modified_by: identitySet.
    :type last_modified_by: ~users_user.models.MicrosoftGraphIdentitySet
    :param categories:
    :type categories: list[str]
    :param description:
    :type description: str
    :param display_name:
    :type display_name: str
    :param web_url:
    :type web_url: str
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'allowed_audiences': {'key': 'allowedAudiences', 'type': 'str'},
        'inference': {'key': 'inference', 'type': 'MicrosoftGraphInferenceData'},
        'created_date_time': {'key': 'createdDateTime', 'type': 'iso-8601'},
        'created_by': {'key': 'createdBy', 'type': 'MicrosoftGraphIdentitySet'},
        'last_modified_date_time': {'key': 'lastModifiedDateTime', 'type': 'iso-8601'},
        'last_modified_by': {'key': 'lastModifiedBy', 'type': 'MicrosoftGraphIdentitySet'},
        'categories': {'key': 'categories', 'type': '[str]'},
        'description': {'key': 'description', 'type': 'str'},
        'display_name': {'key': 'displayName', 'type': 'str'},
        'web_url': {'key': 'webUrl', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphPersonInterest, self).__init__(**kwargs)
        self.categories = kwargs.get('categories', None)
        self.description = kwargs.get('description', None)
        self.display_name = kwargs.get('display_name', None)
        self.web_url = kwargs.get('web_url', None)


class MicrosoftGraphPersonName(MicrosoftGraphItemFacet):
    """personName.

    :param id: Read-only.
    :type id: str
    :param allowed_audiences: allowedAudiences. Possible values include: "me", "family",
     "contacts", "groupMembers", "organization", "federatedOrganizations", "everyone",
     "unknownFutureValue".
    :type allowed_audiences: str or ~users_user.models.MicrosoftGraphAllowedAudiences
    :param inference: inferenceData.
    :type inference: ~users_user.models.MicrosoftGraphInferenceData
    :param created_date_time:
    :type created_date_time: ~datetime.datetime
    :param created_by: identitySet.
    :type created_by: ~users_user.models.MicrosoftGraphIdentitySet
    :param last_modified_date_time:
    :type last_modified_date_time: ~datetime.datetime
    :param last_modified_by: identitySet.
    :type last_modified_by: ~users_user.models.MicrosoftGraphIdentitySet
    :param display_name:
    :type display_name: str
    :param first:
    :type first: str
    :param initials:
    :type initials: str
    :param last:
    :type last: str
    :param language_tag:
    :type language_tag: str
    :param maiden:
    :type maiden: str
    :param middle:
    :type middle: str
    :param nickname:
    :type nickname: str
    :param suffix:
    :type suffix: str
    :param title:
    :type title: str
    :param pronunciation: yomiPersonName.
    :type pronunciation: ~users_user.models.MicrosoftGraphYomiPersonName
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'allowed_audiences': {'key': 'allowedAudiences', 'type': 'str'},
        'inference': {'key': 'inference', 'type': 'MicrosoftGraphInferenceData'},
        'created_date_time': {'key': 'createdDateTime', 'type': 'iso-8601'},
        'created_by': {'key': 'createdBy', 'type': 'MicrosoftGraphIdentitySet'},
        'last_modified_date_time': {'key': 'lastModifiedDateTime', 'type': 'iso-8601'},
        'last_modified_by': {'key': 'lastModifiedBy', 'type': 'MicrosoftGraphIdentitySet'},
        'display_name': {'key': 'displayName', 'type': 'str'},
        'first': {'key': 'first', 'type': 'str'},
        'initials': {'key': 'initials', 'type': 'str'},
        'last': {'key': 'last', 'type': 'str'},
        'language_tag': {'key': 'languageTag', 'type': 'str'},
        'maiden': {'key': 'maiden', 'type': 'str'},
        'middle': {'key': 'middle', 'type': 'str'},
        'nickname': {'key': 'nickname', 'type': 'str'},
        'suffix': {'key': 'suffix', 'type': 'str'},
        'title': {'key': 'title', 'type': 'str'},
        'pronunciation': {'key': 'pronunciation', 'type': 'MicrosoftGraphYomiPersonName'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphPersonName, self).__init__(**kwargs)
        self.display_name = kwargs.get('display_name', None)
        self.first = kwargs.get('first', None)
        self.initials = kwargs.get('initials', None)
        self.last = kwargs.get('last', None)
        self.language_tag = kwargs.get('language_tag', None)
        self.maiden = kwargs.get('maiden', None)
        self.middle = kwargs.get('middle', None)
        self.nickname = kwargs.get('nickname', None)
        self.suffix = kwargs.get('suffix', None)
        self.title = kwargs.get('title', None)
        self.pronunciation = kwargs.get('pronunciation', None)


class MicrosoftGraphPersonOrGroupColumn(msrest.serialization.Model):
    """personOrGroupColumn.

    :param allow_multiple_selection: Indicates whether multiple values can be selected from the
     source.
    :type allow_multiple_selection: bool
    :param choose_from_type: Whether to allow selection of people only, or people and groups. Must
     be one of peopleAndGroups or peopleOnly.
    :type choose_from_type: str
    :param display_as: How to display the information about the person or group chosen. See below.
    :type display_as: str
    """

    _attribute_map = {
        'allow_multiple_selection': {'key': 'allowMultipleSelection', 'type': 'bool'},
        'choose_from_type': {'key': 'chooseFromType', 'type': 'str'},
        'display_as': {'key': 'displayAs', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphPersonOrGroupColumn, self).__init__(**kwargs)
        self.allow_multiple_selection = kwargs.get('allow_multiple_selection', None)
        self.choose_from_type = kwargs.get('choose_from_type', None)
        self.display_as = kwargs.get('display_as', None)


class MicrosoftGraphPersonWebsite(MicrosoftGraphItemFacet):
    """personWebsite.

    :param id: Read-only.
    :type id: str
    :param allowed_audiences: allowedAudiences. Possible values include: "me", "family",
     "contacts", "groupMembers", "organization", "federatedOrganizations", "everyone",
     "unknownFutureValue".
    :type allowed_audiences: str or ~users_user.models.MicrosoftGraphAllowedAudiences
    :param inference: inferenceData.
    :type inference: ~users_user.models.MicrosoftGraphInferenceData
    :param created_date_time:
    :type created_date_time: ~datetime.datetime
    :param created_by: identitySet.
    :type created_by: ~users_user.models.MicrosoftGraphIdentitySet
    :param last_modified_date_time:
    :type last_modified_date_time: ~datetime.datetime
    :param last_modified_by: identitySet.
    :type last_modified_by: ~users_user.models.MicrosoftGraphIdentitySet
    :param categories:
    :type categories: list[str]
    :param description:
    :type description: str
    :param display_name:
    :type display_name: str
    :param web_url:
    :type web_url: str
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'allowed_audiences': {'key': 'allowedAudiences', 'type': 'str'},
        'inference': {'key': 'inference', 'type': 'MicrosoftGraphInferenceData'},
        'created_date_time': {'key': 'createdDateTime', 'type': 'iso-8601'},
        'created_by': {'key': 'createdBy', 'type': 'MicrosoftGraphIdentitySet'},
        'last_modified_date_time': {'key': 'lastModifiedDateTime', 'type': 'iso-8601'},
        'last_modified_by': {'key': 'lastModifiedBy', 'type': 'MicrosoftGraphIdentitySet'},
        'categories': {'key': 'categories', 'type': '[str]'},
        'description': {'key': 'description', 'type': 'str'},
        'display_name': {'key': 'displayName', 'type': 'str'},
        'web_url': {'key': 'webUrl', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphPersonWebsite, self).__init__(**kwargs)
        self.categories = kwargs.get('categories', None)
        self.description = kwargs.get('description', None)
        self.display_name = kwargs.get('display_name', None)
        self.web_url = kwargs.get('web_url', None)


class MicrosoftGraphPhone(msrest.serialization.Model):
    """phone.

    :param type: phoneType. Possible values include: "home", "business", "mobile", "other",
     "assistant", "homeFax", "businessFax", "otherFax", "pager", "radio".
    :type type: str or ~users_user.models.MicrosoftGraphPhoneType
    :param number: The phone number.
    :type number: str
    """

    _attribute_map = {
        'type': {'key': 'type', 'type': 'str'},
        'number': {'key': 'number', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphPhone, self).__init__(**kwargs)
        self.type = kwargs.get('type', None)
        self.number = kwargs.get('number', None)


class MicrosoftGraphPhoneAuthenticationMethod(MicrosoftGraphEntity):
    """phoneAuthenticationMethod.

    :param id: Read-only.
    :type id: str
    :param phone_number:
    :type phone_number: str
    :param phone_type: authenticationPhoneType. Possible values include: "mobile",
     "alternateMobile", "office", "unknownFutureValue".
    :type phone_type: str or ~users_user.models.MicrosoftGraphAuthenticationPhoneType
    :param sms_sign_in_state: authenticationMethodSignInState. Possible values include:
     "notSupported", "notAllowedByPolicy", "notEnabled", "phoneNumberNotUnique", "ready",
     "notConfigured", "unknownFutureValue".
    :type sms_sign_in_state: str or
     ~users_user.models.MicrosoftGraphAuthenticationMethodSignInState
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'phone_number': {'key': 'phoneNumber', 'type': 'str'},
        'phone_type': {'key': 'phoneType', 'type': 'str'},
        'sms_sign_in_state': {'key': 'smsSignInState', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphPhoneAuthenticationMethod, self).__init__(**kwargs)
        self.phone_number = kwargs.get('phone_number', None)
        self.phone_type = kwargs.get('phone_type', None)
        self.sms_sign_in_state = kwargs.get('sms_sign_in_state', None)


class MicrosoftGraphPhoto(msrest.serialization.Model):
    """photo.

    :param camera_make: Camera manufacturer. Read-only.
    :type camera_make: str
    :param camera_model: Camera model. Read-only.
    :type camera_model: str
    :param exposure_denominator: The denominator for the exposure time fraction from the camera.
     Read-only.
    :type exposure_denominator: float
    :param exposure_numerator: The numerator for the exposure time fraction from the camera. Read-
     only.
    :type exposure_numerator: float
    :param f_number: The F-stop value from the camera. Read-only.
    :type f_number: float
    :param focal_length: The focal length from the camera. Read-only.
    :type focal_length: float
    :param iso: The ISO value from the camera. Read-only.
    :type iso: int
    :param orientation:
    :type orientation: int
    :param taken_date_time: Represents the date and time the photo was taken. Read-only.
    :type taken_date_time: ~datetime.datetime
    """

    _validation = {
        'iso': {'maximum': 2147483647, 'minimum': -2147483648},
        'orientation': {'maximum': 32767, 'minimum': -32768},
    }

    _attribute_map = {
        'camera_make': {'key': 'cameraMake', 'type': 'str'},
        'camera_model': {'key': 'cameraModel', 'type': 'str'},
        'exposure_denominator': {'key': 'exposureDenominator', 'type': 'float'},
        'exposure_numerator': {'key': 'exposureNumerator', 'type': 'float'},
        'f_number': {'key': 'fNumber', 'type': 'float'},
        'focal_length': {'key': 'focalLength', 'type': 'float'},
        'iso': {'key': 'iso', 'type': 'int'},
        'orientation': {'key': 'orientation', 'type': 'int'},
        'taken_date_time': {'key': 'takenDateTime', 'type': 'iso-8601'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphPhoto, self).__init__(**kwargs)
        self.camera_make = kwargs.get('camera_make', None)
        self.camera_model = kwargs.get('camera_model', None)
        self.exposure_denominator = kwargs.get('exposure_denominator', None)
        self.exposure_numerator = kwargs.get('exposure_numerator', None)
        self.f_number = kwargs.get('f_number', None)
        self.focal_length = kwargs.get('focal_length', None)
        self.iso = kwargs.get('iso', None)
        self.orientation = kwargs.get('orientation', None)
        self.taken_date_time = kwargs.get('taken_date_time', None)


class MicrosoftGraphPhysicalAddress(msrest.serialization.Model):
    """physicalAddress.

    :param type: physicalAddressType. Possible values include: "unknown", "home", "business",
     "other".
    :type type: str or ~users_user.models.MicrosoftGraphPhysicalAddressType
    :param post_office_box:
    :type post_office_box: str
    :param street: The street.
    :type street: str
    :param city: The city.
    :type city: str
    :param state: The state.
    :type state: str
    :param country_or_region: The country or region. It's a free-format string value, for example,
     'United States'.
    :type country_or_region: str
    :param postal_code: The postal code.
    :type postal_code: str
    """

    _attribute_map = {
        'type': {'key': 'type', 'type': 'str'},
        'post_office_box': {'key': 'postOfficeBox', 'type': 'str'},
        'street': {'key': 'street', 'type': 'str'},
        'city': {'key': 'city', 'type': 'str'},
        'state': {'key': 'state', 'type': 'str'},
        'country_or_region': {'key': 'countryOrRegion', 'type': 'str'},
        'postal_code': {'key': 'postalCode', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphPhysicalAddress, self).__init__(**kwargs)
        self.type = kwargs.get('type', None)
        self.post_office_box = kwargs.get('post_office_box', None)
        self.street = kwargs.get('street', None)
        self.city = kwargs.get('city', None)
        self.state = kwargs.get('state', None)
        self.country_or_region = kwargs.get('country_or_region', None)
        self.postal_code = kwargs.get('postal_code', None)


class MicrosoftGraphPlannerAssignedToTaskBoardTaskFormat(MicrosoftGraphEntity):
    """plannerAssignedToTaskBoardTaskFormat.

    :param id: Read-only.
    :type id: str
    :param unassigned_order_hint: Hint value used to order the task on the AssignedTo view of the
     Task Board when the task is not assigned to anyone, or if the orderHintsByAssignee dictionary
     does not provide an order hint for the user the task is assigned to. The format is defined as
     outlined here.
    :type unassigned_order_hint: str
    :param order_hints_by_assignee: Any object.
    :type order_hints_by_assignee: object
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'unassigned_order_hint': {'key': 'unassignedOrderHint', 'type': 'str'},
        'order_hints_by_assignee': {'key': 'orderHintsByAssignee', 'type': 'object'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphPlannerAssignedToTaskBoardTaskFormat, self).__init__(**kwargs)
        self.unassigned_order_hint = kwargs.get('unassigned_order_hint', None)
        self.order_hints_by_assignee = kwargs.get('order_hints_by_assignee', None)


class MicrosoftGraphPlannerBucket(MicrosoftGraphEntity):
    """plannerBucket.

    :param id: Read-only.
    :type id: str
    :param name: Name of the bucket.
    :type name: str
    :param plan_id: Plan ID to which the bucket belongs.
    :type plan_id: str
    :param order_hint: Hint used to order items of this type in a list view. The format is defined
     as outlined here.
    :type order_hint: str
    :param tasks: Read-only. Nullable. The collection of tasks in the bucket.
    :type tasks: list[~users_user.models.MicrosoftGraphPlannerTask]
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'name': {'key': 'name', 'type': 'str'},
        'plan_id': {'key': 'planId', 'type': 'str'},
        'order_hint': {'key': 'orderHint', 'type': 'str'},
        'tasks': {'key': 'tasks', 'type': '[MicrosoftGraphPlannerTask]'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphPlannerBucket, self).__init__(**kwargs)
        self.name = kwargs.get('name', None)
        self.plan_id = kwargs.get('plan_id', None)
        self.order_hint = kwargs.get('order_hint', None)
        self.tasks = kwargs.get('tasks', None)


class MicrosoftGraphPlannerBucketTaskBoardTaskFormat(MicrosoftGraphEntity):
    """plannerBucketTaskBoardTaskFormat.

    :param id: Read-only.
    :type id: str
    :param order_hint: Hint used to order tasks in the Bucket view of the Task Board. The format is
     defined as outlined here.
    :type order_hint: str
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'order_hint': {'key': 'orderHint', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphPlannerBucketTaskBoardTaskFormat, self).__init__(**kwargs)
        self.order_hint = kwargs.get('order_hint', None)


class MicrosoftGraphPlannerCategoryDescriptions(msrest.serialization.Model):
    """plannerCategoryDescriptions.

    :param category1: The label associated with Category 1.
    :type category1: str
    :param category2: The label associated with Category 2.
    :type category2: str
    :param category3: The label associated with Category 3.
    :type category3: str
    :param category4: The label associated with Category 4.
    :type category4: str
    :param category5: The label associated with Category 5.
    :type category5: str
    :param category6: The label associated with Category 6.
    :type category6: str
    """

    _attribute_map = {
        'category1': {'key': 'category1', 'type': 'str'},
        'category2': {'key': 'category2', 'type': 'str'},
        'category3': {'key': 'category3', 'type': 'str'},
        'category4': {'key': 'category4', 'type': 'str'},
        'category5': {'key': 'category5', 'type': 'str'},
        'category6': {'key': 'category6', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphPlannerCategoryDescriptions, self).__init__(**kwargs)
        self.category1 = kwargs.get('category1', None)
        self.category2 = kwargs.get('category2', None)
        self.category3 = kwargs.get('category3', None)
        self.category4 = kwargs.get('category4', None)
        self.category5 = kwargs.get('category5', None)
        self.category6 = kwargs.get('category6', None)


class MicrosoftGraphPlannerDelta(MicrosoftGraphEntity):
    """plannerDelta.

    :param id: Read-only.
    :type id: str
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphPlannerDelta, self).__init__(**kwargs)


class MicrosoftGraphPlannerGroup(MicrosoftGraphEntity):
    """plannerGroup.

    :param id: Read-only.
    :type id: str
    :param plans: Read-only. Nullable. Returns the plannerPlans owned by the group.
    :type plans: list[~users_user.models.MicrosoftGraphPlannerPlan]
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'plans': {'key': 'plans', 'type': '[MicrosoftGraphPlannerPlan]'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphPlannerGroup, self).__init__(**kwargs)
        self.plans = kwargs.get('plans', None)


class MicrosoftGraphPlannerPlan(MicrosoftGraphEntity):
    """plannerPlan.

    :param id: Read-only.
    :type id: str
    :param created_by: identitySet.
    :type created_by: ~users_user.models.MicrosoftGraphIdentitySet
    :param created_date_time: Read-only. Date and time at which the plan is created. The Timestamp
     type represents date and time information using ISO 8601 format and is always in UTC time. For
     example, midnight UTC on Jan 1, 2014 would look like this: '2014-01-01T00:00:00Z'.
    :type created_date_time: ~datetime.datetime
    :param owner: ID of the Group that owns the plan. A valid group must exist before this field
     can be set. After it is set, this property can’t be updated.
    :type owner: str
    :param title: Required. Title of the plan.
    :type title: str
    :param contexts: Any object.
    :type contexts: object
    :param tasks: Read-only. Nullable. Collection of tasks in the plan.
    :type tasks: list[~users_user.models.MicrosoftGraphPlannerTask]
    :param buckets: Read-only. Nullable. Collection of buckets in the plan.
    :type buckets: list[~users_user.models.MicrosoftGraphPlannerBucket]
    :param details: plannerPlanDetails.
    :type details: ~users_user.models.MicrosoftGraphPlannerPlanDetails
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'created_by': {'key': 'createdBy', 'type': 'MicrosoftGraphIdentitySet'},
        'created_date_time': {'key': 'createdDateTime', 'type': 'iso-8601'},
        'owner': {'key': 'owner', 'type': 'str'},
        'title': {'key': 'title', 'type': 'str'},
        'contexts': {'key': 'contexts', 'type': 'object'},
        'tasks': {'key': 'tasks', 'type': '[MicrosoftGraphPlannerTask]'},
        'buckets': {'key': 'buckets', 'type': '[MicrosoftGraphPlannerBucket]'},
        'details': {'key': 'details', 'type': 'MicrosoftGraphPlannerPlanDetails'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphPlannerPlan, self).__init__(**kwargs)
        self.created_by = kwargs.get('created_by', None)
        self.created_date_time = kwargs.get('created_date_time', None)
        self.owner = kwargs.get('owner', None)
        self.title = kwargs.get('title', None)
        self.contexts = kwargs.get('contexts', None)
        self.tasks = kwargs.get('tasks', None)
        self.buckets = kwargs.get('buckets', None)
        self.details = kwargs.get('details', None)


class MicrosoftGraphPlannerPlanDetails(MicrosoftGraphEntity):
    """plannerPlanDetails.

    :param id: Read-only.
    :type id: str
    :param shared_with: Any object.
    :type shared_with: object
    :param category_descriptions: plannerCategoryDescriptions.
    :type category_descriptions: ~users_user.models.MicrosoftGraphPlannerCategoryDescriptions
    :param context_details: Any object.
    :type context_details: object
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'shared_with': {'key': 'sharedWith', 'type': 'object'},
        'category_descriptions': {'key': 'categoryDescriptions', 'type': 'MicrosoftGraphPlannerCategoryDescriptions'},
        'context_details': {'key': 'contextDetails', 'type': 'object'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphPlannerPlanDetails, self).__init__(**kwargs)
        self.shared_with = kwargs.get('shared_with', None)
        self.category_descriptions = kwargs.get('category_descriptions', None)
        self.context_details = kwargs.get('context_details', None)


class MicrosoftGraphPlannerProgressTaskBoardTaskFormat(MicrosoftGraphEntity):
    """plannerProgressTaskBoardTaskFormat.

    :param id: Read-only.
    :type id: str
    :param order_hint: Hint value used to order the task on the Progress view of the Task Board.
     The format is defined as outlined here.
    :type order_hint: str
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'order_hint': {'key': 'orderHint', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphPlannerProgressTaskBoardTaskFormat, self).__init__(**kwargs)
        self.order_hint = kwargs.get('order_hint', None)


class MicrosoftGraphPlannerTask(MicrosoftGraphEntity):
    """plannerTask.

    :param id: Read-only.
    :type id: str
    :param created_by: identitySet.
    :type created_by: ~users_user.models.MicrosoftGraphIdentitySet
    :param plan_id: Plan ID to which the task belongs.
    :type plan_id: str
    :param bucket_id: Bucket ID to which the task belongs. The bucket needs to be in the plan that
     the task is in. It is 28 characters long and case-sensitive. Format validation is done on the
     service.
    :type bucket_id: str
    :param title: Title of the task.
    :type title: str
    :param order_hint: Hint used to order items of this type in a list view. The format is defined
     as outlined here.
    :type order_hint: str
    :param assignee_priority: Hint used to order items of this type in a list view. The format is
     defined as outlined here.
    :type assignee_priority: str
    :param percent_complete: Percentage of task completion. When set to 100, the task is considered
     completed.
    :type percent_complete: int
    :param priority:
    :type priority: int
    :param start_date_time: Date and time at which the task starts. The Timestamp type represents
     date and time information using ISO 8601 format and is always in UTC time. For example,
     midnight UTC on Jan 1, 2014 would look like this: '2014-01-01T00:00:00Z'.
    :type start_date_time: ~datetime.datetime
    :param created_date_time: Read-only. Date and time at which the task is created. The Timestamp
     type represents date and time information using ISO 8601 format and is always in UTC time. For
     example, midnight UTC on Jan 1, 2014 would look like this: '2014-01-01T00:00:00Z'.
    :type created_date_time: ~datetime.datetime
    :param due_date_time: Date and time at which the task is due. The Timestamp type represents
     date and time information using ISO 8601 format and is always in UTC time. For example,
     midnight UTC on Jan 1, 2014 would look like this: '2014-01-01T00:00:00Z'.
    :type due_date_time: ~datetime.datetime
    :param has_description: Read-only. Value is true if the details object of the task has a non-
     empty description and false otherwise.
    :type has_description: bool
    :param preview_type: plannerPreviewType. Possible values include: "automatic", "noPreview",
     "checklist", "description", "reference".
    :type preview_type: str or ~users_user.models.MicrosoftGraphPlannerPreviewType
    :param completed_date_time: Read-only. Date and time at which the 'percentComplete' of the task
     is set to '100'. The Timestamp type represents date and time information using ISO 8601 format
     and is always in UTC time. For example, midnight UTC on Jan 1, 2014 would look like this:
     '2014-01-01T00:00:00Z'.
    :type completed_date_time: ~datetime.datetime
    :param completed_by: identitySet.
    :type completed_by: ~users_user.models.MicrosoftGraphIdentitySet
    :param reference_count: Number of external references that exist on the task.
    :type reference_count: int
    :param checklist_item_count: Number of checklist items that are present on the task.
    :type checklist_item_count: int
    :param active_checklist_item_count: Number of checklist items with value set to false,
     representing incomplete items.
    :type active_checklist_item_count: int
    :param applied_categories: Any object.
    :type applied_categories: object
    :param assignments: Any object.
    :type assignments: object
    :param conversation_thread_id: Thread ID of the conversation on the task. This is the ID of the
     conversation thread object created in the group.
    :type conversation_thread_id: str
    :param details: plannerTaskDetails.
    :type details: ~users_user.models.MicrosoftGraphPlannerTaskDetails
    :param assigned_to_task_board_format: plannerAssignedToTaskBoardTaskFormat.
    :type assigned_to_task_board_format:
     ~users_user.models.MicrosoftGraphPlannerAssignedToTaskBoardTaskFormat
    :param progress_task_board_format: plannerProgressTaskBoardTaskFormat.
    :type progress_task_board_format:
     ~users_user.models.MicrosoftGraphPlannerProgressTaskBoardTaskFormat
    :param bucket_task_board_format: plannerBucketTaskBoardTaskFormat.
    :type bucket_task_board_format:
     ~users_user.models.MicrosoftGraphPlannerBucketTaskBoardTaskFormat
    """

    _validation = {
        'percent_complete': {'maximum': 2147483647, 'minimum': -2147483648},
        'priority': {'maximum': 2147483647, 'minimum': -2147483648},
        'reference_count': {'maximum': 2147483647, 'minimum': -2147483648},
        'checklist_item_count': {'maximum': 2147483647, 'minimum': -2147483648},
        'active_checklist_item_count': {'maximum': 2147483647, 'minimum': -2147483648},
    }

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'created_by': {'key': 'createdBy', 'type': 'MicrosoftGraphIdentitySet'},
        'plan_id': {'key': 'planId', 'type': 'str'},
        'bucket_id': {'key': 'bucketId', 'type': 'str'},
        'title': {'key': 'title', 'type': 'str'},
        'order_hint': {'key': 'orderHint', 'type': 'str'},
        'assignee_priority': {'key': 'assigneePriority', 'type': 'str'},
        'percent_complete': {'key': 'percentComplete', 'type': 'int'},
        'priority': {'key': 'priority', 'type': 'int'},
        'start_date_time': {'key': 'startDateTime', 'type': 'iso-8601'},
        'created_date_time': {'key': 'createdDateTime', 'type': 'iso-8601'},
        'due_date_time': {'key': 'dueDateTime', 'type': 'iso-8601'},
        'has_description': {'key': 'hasDescription', 'type': 'bool'},
        'preview_type': {'key': 'previewType', 'type': 'str'},
        'completed_date_time': {'key': 'completedDateTime', 'type': 'iso-8601'},
        'completed_by': {'key': 'completedBy', 'type': 'MicrosoftGraphIdentitySet'},
        'reference_count': {'key': 'referenceCount', 'type': 'int'},
        'checklist_item_count': {'key': 'checklistItemCount', 'type': 'int'},
        'active_checklist_item_count': {'key': 'activeChecklistItemCount', 'type': 'int'},
        'applied_categories': {'key': 'appliedCategories', 'type': 'object'},
        'assignments': {'key': 'assignments', 'type': 'object'},
        'conversation_thread_id': {'key': 'conversationThreadId', 'type': 'str'},
        'details': {'key': 'details', 'type': 'MicrosoftGraphPlannerTaskDetails'},
        'assigned_to_task_board_format': {'key': 'assignedToTaskBoardFormat', 'type': 'MicrosoftGraphPlannerAssignedToTaskBoardTaskFormat'},
        'progress_task_board_format': {'key': 'progressTaskBoardFormat', 'type': 'MicrosoftGraphPlannerProgressTaskBoardTaskFormat'},
        'bucket_task_board_format': {'key': 'bucketTaskBoardFormat', 'type': 'MicrosoftGraphPlannerBucketTaskBoardTaskFormat'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphPlannerTask, self).__init__(**kwargs)
        self.created_by = kwargs.get('created_by', None)
        self.plan_id = kwargs.get('plan_id', None)
        self.bucket_id = kwargs.get('bucket_id', None)
        self.title = kwargs.get('title', None)
        self.order_hint = kwargs.get('order_hint', None)
        self.assignee_priority = kwargs.get('assignee_priority', None)
        self.percent_complete = kwargs.get('percent_complete', None)
        self.priority = kwargs.get('priority', None)
        self.start_date_time = kwargs.get('start_date_time', None)
        self.created_date_time = kwargs.get('created_date_time', None)
        self.due_date_time = kwargs.get('due_date_time', None)
        self.has_description = kwargs.get('has_description', None)
        self.preview_type = kwargs.get('preview_type', None)
        self.completed_date_time = kwargs.get('completed_date_time', None)
        self.completed_by = kwargs.get('completed_by', None)
        self.reference_count = kwargs.get('reference_count', None)
        self.checklist_item_count = kwargs.get('checklist_item_count', None)
        self.active_checklist_item_count = kwargs.get('active_checklist_item_count', None)
        self.applied_categories = kwargs.get('applied_categories', None)
        self.assignments = kwargs.get('assignments', None)
        self.conversation_thread_id = kwargs.get('conversation_thread_id', None)
        self.details = kwargs.get('details', None)
        self.assigned_to_task_board_format = kwargs.get('assigned_to_task_board_format', None)
        self.progress_task_board_format = kwargs.get('progress_task_board_format', None)
        self.bucket_task_board_format = kwargs.get('bucket_task_board_format', None)


class MicrosoftGraphPlannerTaskDetails(MicrosoftGraphEntity):
    """plannerTaskDetails.

    :param id: Read-only.
    :type id: str
    :param description: Description of the task.
    :type description: str
    :param preview_type: plannerPreviewType. Possible values include: "automatic", "noPreview",
     "checklist", "description", "reference".
    :type preview_type: str or ~users_user.models.MicrosoftGraphPlannerPreviewType
    :param references: Any object.
    :type references: object
    :param checklist: Any object.
    :type checklist: object
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'description': {'key': 'description', 'type': 'str'},
        'preview_type': {'key': 'previewType', 'type': 'str'},
        'references': {'key': 'references', 'type': 'object'},
        'checklist': {'key': 'checklist', 'type': 'object'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphPlannerTaskDetails, self).__init__(**kwargs)
        self.description = kwargs.get('description', None)
        self.preview_type = kwargs.get('preview_type', None)
        self.references = kwargs.get('references', None)
        self.checklist = kwargs.get('checklist', None)


class MicrosoftGraphPlannerUser(MicrosoftGraphEntity):
    """plannerUser.

    :param id: Read-only.
    :type id: str
    :param favorite_plan_references: Any object.
    :type favorite_plan_references: object
    :param recent_plan_references: Any object.
    :type recent_plan_references: object
    :param tasks: Read-only. Nullable. Returns the plannerPlans shared with the user.
    :type tasks: list[~users_user.models.MicrosoftGraphPlannerTask]
    :param plans: Read-only. Nullable. Returns the plannerTasks assigned to the user.
    :type plans: list[~users_user.models.MicrosoftGraphPlannerPlan]
    :param favorite_plans:
    :type favorite_plans: list[~users_user.models.MicrosoftGraphPlannerPlan]
    :param recent_plans:
    :type recent_plans: list[~users_user.models.MicrosoftGraphPlannerPlan]
    :param all:
    :type all: list[~users_user.models.MicrosoftGraphEntity]
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'favorite_plan_references': {'key': 'favoritePlanReferences', 'type': 'object'},
        'recent_plan_references': {'key': 'recentPlanReferences', 'type': 'object'},
        'tasks': {'key': 'tasks', 'type': '[MicrosoftGraphPlannerTask]'},
        'plans': {'key': 'plans', 'type': '[MicrosoftGraphPlannerPlan]'},
        'favorite_plans': {'key': 'favoritePlans', 'type': '[MicrosoftGraphPlannerPlan]'},
        'recent_plans': {'key': 'recentPlans', 'type': '[MicrosoftGraphPlannerPlan]'},
        'all': {'key': 'all', 'type': '[MicrosoftGraphEntity]'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphPlannerUser, self).__init__(**kwargs)
        self.favorite_plan_references = kwargs.get('favorite_plan_references', None)
        self.recent_plan_references = kwargs.get('recent_plan_references', None)
        self.tasks = kwargs.get('tasks', None)
        self.plans = kwargs.get('plans', None)
        self.favorite_plans = kwargs.get('favorite_plans', None)
        self.recent_plans = kwargs.get('recent_plans', None)
        self.all = kwargs.get('all', None)


class MicrosoftGraphPositionDetail(msrest.serialization.Model):
    """positionDetail.

    :param company: companyDetail.
    :type company: ~users_user.models.MicrosoftGraphCompanyDetail
    :param description:
    :type description: str
    :param end_month_year:
    :type end_month_year: ~datetime.date
    :param job_title:
    :type job_title: str
    :param role:
    :type role: str
    :param start_month_year:
    :type start_month_year: ~datetime.date
    :param summary:
    :type summary: str
    """

    _attribute_map = {
        'company': {'key': 'company', 'type': 'MicrosoftGraphCompanyDetail'},
        'description': {'key': 'description', 'type': 'str'},
        'end_month_year': {'key': 'endMonthYear', 'type': 'date'},
        'job_title': {'key': 'jobTitle', 'type': 'str'},
        'role': {'key': 'role', 'type': 'str'},
        'start_month_year': {'key': 'startMonthYear', 'type': 'date'},
        'summary': {'key': 'summary', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphPositionDetail, self).__init__(**kwargs)
        self.company = kwargs.get('company', None)
        self.description = kwargs.get('description', None)
        self.end_month_year = kwargs.get('end_month_year', None)
        self.job_title = kwargs.get('job_title', None)
        self.role = kwargs.get('role', None)
        self.start_month_year = kwargs.get('start_month_year', None)
        self.summary = kwargs.get('summary', None)


class MicrosoftGraphPost(MicrosoftGraphOutlookItem):
    """post.

    :param id: Read-only.
    :type id: str
    :param created_date_time: The Timestamp type represents date and time information using ISO
     8601 format and is always in UTC time. For example, midnight UTC on Jan 1, 2014 would look like
     this: '2014-01-01T00:00:00Z'.
    :type created_date_time: ~datetime.datetime
    :param last_modified_date_time: The Timestamp type represents date and time information using
     ISO 8601 format and is always in UTC time. For example, midnight UTC on Jan 1, 2014 would look
     like this: '2014-01-01T00:00:00Z'.
    :type last_modified_date_time: ~datetime.datetime
    :param change_key: Identifies the version of the item. Every time the item is changed,
     changeKey changes as well. This allows Exchange to apply changes to the correct version of the
     object. Read-only.
    :type change_key: str
    :param categories: The categories associated with the item.
    :type categories: list[str]
    :param body: itemBody.
    :type body: ~users_user.models.MicrosoftGraphItemBody
    :param received_date_time: Specifies when the post was received. The DateTimeOffset type
     represents date and time information using ISO 8601 format and is always in UTC time. For
     example, midnight UTC on Jan 1, 2014 would look like this: '2014-01-01T00:00:00Z'.
    :type received_date_time: ~datetime.datetime
    :param has_attachments: Indicates whether the post has at least one attachment. This is a
     default property.
    :type has_attachments: bool
    :param from_property: recipient.
    :type from_property: ~users_user.models.MicrosoftGraphRecipient
    :param sender: recipient.
    :type sender: ~users_user.models.MicrosoftGraphRecipient
    :param conversation_thread_id: Unique ID of the conversation thread. Read-only.
    :type conversation_thread_id: str
    :param new_participants: Conversation participants that were added to the thread as part of
     this post.
    :type new_participants: list[~users_user.models.MicrosoftGraphRecipient]
    :param conversation_id: Unique ID of the conversation. Read-only.
    :type conversation_id: str
    :param importance: importance. Possible values include: "low", "normal", "high".
    :type importance: str or ~users_user.models.MicrosoftGraphImportance
    :param in_reply_to: post.
    :type in_reply_to: ~users_user.models.MicrosoftGraphPost
    :param single_value_extended_properties: The collection of single-value extended properties
     defined for the post. Read-only. Nullable.
    :type single_value_extended_properties:
     list[~users_user.models.MicrosoftGraphSingleValueLegacyExtendedProperty]
    :param multi_value_extended_properties: The collection of multi-value extended properties
     defined for the post. Read-only. Nullable.
    :type multi_value_extended_properties:
     list[~users_user.models.MicrosoftGraphMultiValueLegacyExtendedProperty]
    :param extensions: The collection of open extensions defined for the post. Read-only. Nullable.
    :type extensions: list[~users_user.models.MicrosoftGraphEntity]
    :param attachments: Read-only. Nullable.
    :type attachments: list[~users_user.models.MicrosoftGraphAttachment]
    :param mentions:
    :type mentions: list[~users_user.models.MicrosoftGraphMention]
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'created_date_time': {'key': 'createdDateTime', 'type': 'iso-8601'},
        'last_modified_date_time': {'key': 'lastModifiedDateTime', 'type': 'iso-8601'},
        'change_key': {'key': 'changeKey', 'type': 'str'},
        'categories': {'key': 'categories', 'type': '[str]'},
        'body': {'key': 'body', 'type': 'MicrosoftGraphItemBody'},
        'received_date_time': {'key': 'receivedDateTime', 'type': 'iso-8601'},
        'has_attachments': {'key': 'hasAttachments', 'type': 'bool'},
        'from_property': {'key': 'from', 'type': 'MicrosoftGraphRecipient'},
        'sender': {'key': 'sender', 'type': 'MicrosoftGraphRecipient'},
        'conversation_thread_id': {'key': 'conversationThreadId', 'type': 'str'},
        'new_participants': {'key': 'newParticipants', 'type': '[MicrosoftGraphRecipient]'},
        'conversation_id': {'key': 'conversationId', 'type': 'str'},
        'importance': {'key': 'importance', 'type': 'str'},
        'in_reply_to': {'key': 'inReplyTo', 'type': 'MicrosoftGraphPost'},
        'single_value_extended_properties': {'key': 'singleValueExtendedProperties', 'type': '[MicrosoftGraphSingleValueLegacyExtendedProperty]'},
        'multi_value_extended_properties': {'key': 'multiValueExtendedProperties', 'type': '[MicrosoftGraphMultiValueLegacyExtendedProperty]'},
        'extensions': {'key': 'extensions', 'type': '[MicrosoftGraphEntity]'},
        'attachments': {'key': 'attachments', 'type': '[MicrosoftGraphAttachment]'},
        'mentions': {'key': 'mentions', 'type': '[MicrosoftGraphMention]'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphPost, self).__init__(**kwargs)
        self.body = kwargs.get('body', None)
        self.received_date_time = kwargs.get('received_date_time', None)
        self.has_attachments = kwargs.get('has_attachments', None)
        self.from_property = kwargs.get('from_property', None)
        self.sender = kwargs.get('sender', None)
        self.conversation_thread_id = kwargs.get('conversation_thread_id', None)
        self.new_participants = kwargs.get('new_participants', None)
        self.conversation_id = kwargs.get('conversation_id', None)
        self.importance = kwargs.get('importance', None)
        self.in_reply_to = kwargs.get('in_reply_to', None)
        self.single_value_extended_properties = kwargs.get('single_value_extended_properties', None)
        self.multi_value_extended_properties = kwargs.get('multi_value_extended_properties', None)
        self.extensions = kwargs.get('extensions', None)
        self.attachments = kwargs.get('attachments', None)
        self.mentions = kwargs.get('mentions', None)


class MicrosoftGraphPresence(MicrosoftGraphEntity):
    """presence.

    :param id: Read-only.
    :type id: str
    :param availability:
    :type availability: str
    :param activity:
    :type activity: str
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'availability': {'key': 'availability', 'type': 'str'},
        'activity': {'key': 'activity', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphPresence, self).__init__(**kwargs)
        self.availability = kwargs.get('availability', None)
        self.activity = kwargs.get('activity', None)


class MicrosoftGraphProfile(MicrosoftGraphEntity):
    """profile.

    :param id: Read-only.
    :type id: str
    :param account:
    :type account: list[~users_user.models.MicrosoftGraphUserAccountInformation]
    :param anniversaries:
    :type anniversaries: list[~users_user.models.MicrosoftGraphPersonAnniversary]
    :param educational_activities:
    :type educational_activities: list[~users_user.models.MicrosoftGraphEducationalActivity]
    :param emails:
    :type emails: list[~users_user.models.MicrosoftGraphItemEmail]
    :param interests:
    :type interests: list[~users_user.models.MicrosoftGraphPersonInterest]
    :param languages:
    :type languages: list[~users_user.models.MicrosoftGraphLanguageProficiency]
    :param names:
    :type names: list[~users_user.models.MicrosoftGraphPersonName]
    :param phones:
    :type phones: list[~users_user.models.MicrosoftGraphItemPhone]
    :param positions:
    :type positions: list[~users_user.models.MicrosoftGraphWorkPosition]
    :param projects:
    :type projects: list[~users_user.models.MicrosoftGraphProjectParticipation]
    :param skills:
    :type skills: list[~users_user.models.MicrosoftGraphSkillProficiency]
    :param web_accounts:
    :type web_accounts: list[~users_user.models.MicrosoftGraphWebAccount]
    :param websites:
    :type websites: list[~users_user.models.MicrosoftGraphPersonWebsite]
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'account': {'key': 'account', 'type': '[MicrosoftGraphUserAccountInformation]'},
        'anniversaries': {'key': 'anniversaries', 'type': '[MicrosoftGraphPersonAnniversary]'},
        'educational_activities': {'key': 'educationalActivities', 'type': '[MicrosoftGraphEducationalActivity]'},
        'emails': {'key': 'emails', 'type': '[MicrosoftGraphItemEmail]'},
        'interests': {'key': 'interests', 'type': '[MicrosoftGraphPersonInterest]'},
        'languages': {'key': 'languages', 'type': '[MicrosoftGraphLanguageProficiency]'},
        'names': {'key': 'names', 'type': '[MicrosoftGraphPersonName]'},
        'phones': {'key': 'phones', 'type': '[MicrosoftGraphItemPhone]'},
        'positions': {'key': 'positions', 'type': '[MicrosoftGraphWorkPosition]'},
        'projects': {'key': 'projects', 'type': '[MicrosoftGraphProjectParticipation]'},
        'skills': {'key': 'skills', 'type': '[MicrosoftGraphSkillProficiency]'},
        'web_accounts': {'key': 'webAccounts', 'type': '[MicrosoftGraphWebAccount]'},
        'websites': {'key': 'websites', 'type': '[MicrosoftGraphPersonWebsite]'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphProfile, self).__init__(**kwargs)
        self.account = kwargs.get('account', None)
        self.anniversaries = kwargs.get('anniversaries', None)
        self.educational_activities = kwargs.get('educational_activities', None)
        self.emails = kwargs.get('emails', None)
        self.interests = kwargs.get('interests', None)
        self.languages = kwargs.get('languages', None)
        self.names = kwargs.get('names', None)
        self.phones = kwargs.get('phones', None)
        self.positions = kwargs.get('positions', None)
        self.projects = kwargs.get('projects', None)
        self.skills = kwargs.get('skills', None)
        self.web_accounts = kwargs.get('web_accounts', None)
        self.websites = kwargs.get('websites', None)


class MicrosoftGraphProfilePhoto(MicrosoftGraphEntity):
    """profilePhoto.

    :param id: Read-only.
    :type id: str
    :param height: The height of the photo. Read-only.
    :type height: int
    :param width: The width of the photo. Read-only.
    :type width: int
    """

    _validation = {
        'height': {'maximum': 2147483647, 'minimum': -2147483648},
        'width': {'maximum': 2147483647, 'minimum': -2147483648},
    }

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'height': {'key': 'height', 'type': 'int'},
        'width': {'key': 'width', 'type': 'int'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphProfilePhoto, self).__init__(**kwargs)
        self.height = kwargs.get('height', None)
        self.width = kwargs.get('width', None)


class MicrosoftGraphProjectParticipation(MicrosoftGraphItemFacet):
    """projectParticipation.

    :param id: Read-only.
    :type id: str
    :param allowed_audiences: allowedAudiences. Possible values include: "me", "family",
     "contacts", "groupMembers", "organization", "federatedOrganizations", "everyone",
     "unknownFutureValue".
    :type allowed_audiences: str or ~users_user.models.MicrosoftGraphAllowedAudiences
    :param inference: inferenceData.
    :type inference: ~users_user.models.MicrosoftGraphInferenceData
    :param created_date_time:
    :type created_date_time: ~datetime.datetime
    :param created_by: identitySet.
    :type created_by: ~users_user.models.MicrosoftGraphIdentitySet
    :param last_modified_date_time:
    :type last_modified_date_time: ~datetime.datetime
    :param last_modified_by: identitySet.
    :type last_modified_by: ~users_user.models.MicrosoftGraphIdentitySet
    :param categories:
    :type categories: list[str]
    :param client: companyDetail.
    :type client: ~users_user.models.MicrosoftGraphCompanyDetail
    :param display_name:
    :type display_name: str
    :param detail: positionDetail.
    :type detail: ~users_user.models.MicrosoftGraphPositionDetail
    :param colleagues:
    :type colleagues: list[~users_user.models.MicrosoftGraphRelatedPerson]
    :param sponsors:
    :type sponsors: list[~users_user.models.MicrosoftGraphRelatedPerson]
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'allowed_audiences': {'key': 'allowedAudiences', 'type': 'str'},
        'inference': {'key': 'inference', 'type': 'MicrosoftGraphInferenceData'},
        'created_date_time': {'key': 'createdDateTime', 'type': 'iso-8601'},
        'created_by': {'key': 'createdBy', 'type': 'MicrosoftGraphIdentitySet'},
        'last_modified_date_time': {'key': 'lastModifiedDateTime', 'type': 'iso-8601'},
        'last_modified_by': {'key': 'lastModifiedBy', 'type': 'MicrosoftGraphIdentitySet'},
        'categories': {'key': 'categories', 'type': '[str]'},
        'client': {'key': 'client', 'type': 'MicrosoftGraphCompanyDetail'},
        'display_name': {'key': 'displayName', 'type': 'str'},
        'detail': {'key': 'detail', 'type': 'MicrosoftGraphPositionDetail'},
        'colleagues': {'key': 'colleagues', 'type': '[MicrosoftGraphRelatedPerson]'},
        'sponsors': {'key': 'sponsors', 'type': '[MicrosoftGraphRelatedPerson]'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphProjectParticipation, self).__init__(**kwargs)
        self.categories = kwargs.get('categories', None)
        self.client = kwargs.get('client', None)
        self.display_name = kwargs.get('display_name', None)
        self.detail = kwargs.get('detail', None)
        self.colleagues = kwargs.get('colleagues', None)
        self.sponsors = kwargs.get('sponsors', None)


class MicrosoftGraphProvisionedPlan(msrest.serialization.Model):
    """provisionedPlan.

    :param capability_status: For example, 'Enabled'.
    :type capability_status: str
    :param provisioning_status: For example, 'Success'.
    :type provisioning_status: str
    :param service: The name of the service; for example, 'AccessControlS2S'.
    :type service: str
    """

    _attribute_map = {
        'capability_status': {'key': 'capabilityStatus', 'type': 'str'},
        'provisioning_status': {'key': 'provisioningStatus', 'type': 'str'},
        'service': {'key': 'service', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphProvisionedPlan, self).__init__(**kwargs)
        self.capability_status = kwargs.get('capability_status', None)
        self.provisioning_status = kwargs.get('provisioning_status', None)
        self.service = kwargs.get('service', None)


class MicrosoftGraphPublicationFacet(msrest.serialization.Model):
    """publicationFacet.

    :param level: The state of publication for this document. Either published or checkout. Read-
     only.
    :type level: str
    :param version_id: The unique identifier for the version that is visible to the current caller.
     Read-only.
    :type version_id: str
    """

    _attribute_map = {
        'level': {'key': 'level', 'type': 'str'},
        'version_id': {'key': 'versionId', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphPublicationFacet, self).__init__(**kwargs)
        self.level = kwargs.get('level', None)
        self.version_id = kwargs.get('version_id', None)


class MicrosoftGraphQuota(msrest.serialization.Model):
    """quota.

    :param deleted: Total space consumed by files in the recycle bin, in bytes. Read-only.
    :type deleted: long
    :param remaining: Total space remaining before reaching the quota limit, in bytes. Read-only.
    :type remaining: long
    :param state: Enumeration value that indicates the state of the storage space. Read-only.
    :type state: str
    :param total: Total allowed storage space, in bytes. Read-only.
    :type total: long
    :param used: Total space used, in bytes. Read-only.
    :type used: long
    :param storage_plan_information: storagePlanInformation.
    :type storage_plan_information: ~users_user.models.MicrosoftGraphStoragePlanInformation
    """

    _attribute_map = {
        'deleted': {'key': 'deleted', 'type': 'long'},
        'remaining': {'key': 'remaining', 'type': 'long'},
        'state': {'key': 'state', 'type': 'str'},
        'total': {'key': 'total', 'type': 'long'},
        'used': {'key': 'used', 'type': 'long'},
        'storage_plan_information': {'key': 'storagePlanInformation', 'type': 'MicrosoftGraphStoragePlanInformation'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphQuota, self).__init__(**kwargs)
        self.deleted = kwargs.get('deleted', None)
        self.remaining = kwargs.get('remaining', None)
        self.state = kwargs.get('state', None)
        self.total = kwargs.get('total', None)
        self.used = kwargs.get('used', None)
        self.storage_plan_information = kwargs.get('storage_plan_information', None)


class MicrosoftGraphRankedEmailAddress(msrest.serialization.Model):
    """rankedEmailAddress.

    :param address:
    :type address: str
    :param rank:
    :type rank: float
    """

    _attribute_map = {
        'address': {'key': 'address', 'type': 'str'},
        'rank': {'key': 'rank', 'type': 'float'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphRankedEmailAddress, self).__init__(**kwargs)
        self.address = kwargs.get('address', None)
        self.rank = kwargs.get('rank', None)


class MicrosoftGraphRecurrencePattern(msrest.serialization.Model):
    """recurrencePattern.

    :param type: recurrencePatternType. Possible values include: "daily", "weekly",
     "absoluteMonthly", "relativeMonthly", "absoluteYearly", "relativeYearly".
    :type type: str or ~users_user.models.MicrosoftGraphRecurrencePatternType
    :param interval: The number of units between occurrences, where units can be in days, weeks,
     months, or years, depending on the type. Required.
    :type interval: int
    :param month: The month in which the event occurs.  This is a number from 1 to 12.
    :type month: int
    :param day_of_month: The day of the month on which the event occurs. Required if type is
     absoluteMonthly or absoluteYearly.
    :type day_of_month: int
    :param days_of_week: A collection of the days of the week on which the event occurs. The
     possible values are: sunday, monday, tuesday, wednesday, thursday, friday, saturday. If type is
     relativeMonthly or relativeYearly, and daysOfWeek specifies more than one day, the event falls
     on the first day that satisfies the pattern.  Required if type is weekly, relativeMonthly, or
     relativeYearly.
    :type days_of_week: list[str or ~users_user.models.MicrosoftGraphDayOfWeek]
    :param first_day_of_week: dayOfWeek. Possible values include: "sunday", "monday", "tuesday",
     "wednesday", "thursday", "friday", "saturday".
    :type first_day_of_week: str or ~users_user.models.MicrosoftGraphDayOfWeek
    :param index: weekIndex. Possible values include: "first", "second", "third", "fourth", "last".
    :type index: str or ~users_user.models.MicrosoftGraphWeekIndex
    """

    _validation = {
        'interval': {'maximum': 2147483647, 'minimum': -2147483648},
        'month': {'maximum': 2147483647, 'minimum': -2147483648},
        'day_of_month': {'maximum': 2147483647, 'minimum': -2147483648},
    }

    _attribute_map = {
        'type': {'key': 'type', 'type': 'str'},
        'interval': {'key': 'interval', 'type': 'int'},
        'month': {'key': 'month', 'type': 'int'},
        'day_of_month': {'key': 'dayOfMonth', 'type': 'int'},
        'days_of_week': {'key': 'daysOfWeek', 'type': '[str]'},
        'first_day_of_week': {'key': 'firstDayOfWeek', 'type': 'str'},
        'index': {'key': 'index', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphRecurrencePattern, self).__init__(**kwargs)
        self.type = kwargs.get('type', None)
        self.interval = kwargs.get('interval', None)
        self.month = kwargs.get('month', None)
        self.day_of_month = kwargs.get('day_of_month', None)
        self.days_of_week = kwargs.get('days_of_week', None)
        self.first_day_of_week = kwargs.get('first_day_of_week', None)
        self.index = kwargs.get('index', None)


class MicrosoftGraphRecurrenceRange(msrest.serialization.Model):
    """recurrenceRange.

    :param type: recurrenceRangeType. Possible values include: "endDate", "noEnd", "numbered".
    :type type: str or ~users_user.models.MicrosoftGraphRecurrenceRangeType
    :param start_date: The date to start applying the recurrence pattern. The first occurrence of
     the meeting may be this date or later, depending on the recurrence pattern of the event. Must
     be the same value as the start property of the recurring event. Required.
    :type start_date: ~datetime.date
    :param end_date: The date to stop applying the recurrence pattern. Depending on the recurrence
     pattern of the event, the last occurrence of the meeting may not be this date. Required if type
     is endDate.
    :type end_date: ~datetime.date
    :param recurrence_time_zone: Time zone for the startDate and endDate properties. Optional. If
     not specified, the time zone of the event is used.
    :type recurrence_time_zone: str
    :param number_of_occurrences: The number of times to repeat the event. Required and must be
     positive if type is numbered.
    :type number_of_occurrences: int
    """

    _validation = {
        'number_of_occurrences': {'maximum': 2147483647, 'minimum': -2147483648},
    }

    _attribute_map = {
        'type': {'key': 'type', 'type': 'str'},
        'start_date': {'key': 'startDate', 'type': 'date'},
        'end_date': {'key': 'endDate', 'type': 'date'},
        'recurrence_time_zone': {'key': 'recurrenceTimeZone', 'type': 'str'},
        'number_of_occurrences': {'key': 'numberOfOccurrences', 'type': 'int'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphRecurrenceRange, self).__init__(**kwargs)
        self.type = kwargs.get('type', None)
        self.start_date = kwargs.get('start_date', None)
        self.end_date = kwargs.get('end_date', None)
        self.recurrence_time_zone = kwargs.get('recurrence_time_zone', None)
        self.number_of_occurrences = kwargs.get('number_of_occurrences', None)


class MicrosoftGraphRelatedPerson(msrest.serialization.Model):
    """relatedPerson.

    :param display_name:
    :type display_name: str
    :param relationship: personRelationship. Possible values include: "manager", "colleague",
     "directReport", "dotLineReport", "assistant", "dotLineManager", "alternateContact", "friend",
     "spouse", "sibling", "child", "parent", "sponsor", "emergencyContact", "other",
     "unknownFutureValue".
    :type relationship: str or ~users_user.models.MicrosoftGraphPersonRelationship
    :param user_principal_name:
    :type user_principal_name: str
    """

    _attribute_map = {
        'display_name': {'key': 'displayName', 'type': 'str'},
        'relationship': {'key': 'relationship', 'type': 'str'},
        'user_principal_name': {'key': 'userPrincipalName', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphRelatedPerson, self).__init__(**kwargs)
        self.display_name = kwargs.get('display_name', None)
        self.relationship = kwargs.get('relationship', None)
        self.user_principal_name = kwargs.get('user_principal_name', None)


class MicrosoftGraphRemoteItem(msrest.serialization.Model):
    """remoteItem.

    :param created_by: identitySet.
    :type created_by: ~users_user.models.MicrosoftGraphIdentitySet
    :param created_date_time: Date and time of item creation. Read-only.
    :type created_date_time: ~datetime.datetime
    :param file: file.
    :type file: ~users_user.models.MicrosoftGraphFile
    :param file_system_info: fileSystemInfo.
    :type file_system_info: ~users_user.models.MicrosoftGraphFileSystemInfo
    :param folder: folder.
    :type folder: ~users_user.models.MicrosoftGraphFolder
    :param id: Unique identifier for the remote item in its drive. Read-only.
    :type id: str
    :param image: image.
    :type image: ~users_user.models.MicrosoftGraphImage
    :param last_modified_by: identitySet.
    :type last_modified_by: ~users_user.models.MicrosoftGraphIdentitySet
    :param last_modified_date_time: Date and time the item was last modified. Read-only.
    :type last_modified_date_time: ~datetime.datetime
    :param name: Optional. Filename of the remote item. Read-only.
    :type name: str
    :param package: package.
    :type package: ~users_user.models.MicrosoftGraphPackage
    :param parent_reference: itemReference.
    :type parent_reference: ~users_user.models.MicrosoftGraphItemReference
    :param shared: shared.
    :type shared: ~users_user.models.MicrosoftGraphShared
    :param sharepoint_ids: sharepointIds.
    :type sharepoint_ids: ~users_user.models.MicrosoftGraphSharepointIds
    :param size: Size of the remote item. Read-only.
    :type size: long
    :param special_folder: specialFolder.
    :type special_folder: ~users_user.models.MicrosoftGraphSpecialFolder
    :param video: video.
    :type video: ~users_user.models.MicrosoftGraphVideo
    :param web_dav_url: DAV compatible URL for the item.
    :type web_dav_url: str
    :param web_url: URL that displays the resource in the browser. Read-only.
    :type web_url: str
    """

    _attribute_map = {
        'created_by': {'key': 'createdBy', 'type': 'MicrosoftGraphIdentitySet'},
        'created_date_time': {'key': 'createdDateTime', 'type': 'iso-8601'},
        'file': {'key': 'file', 'type': 'MicrosoftGraphFile'},
        'file_system_info': {'key': 'fileSystemInfo', 'type': 'MicrosoftGraphFileSystemInfo'},
        'folder': {'key': 'folder', 'type': 'MicrosoftGraphFolder'},
        'id': {'key': 'id', 'type': 'str'},
        'image': {'key': 'image', 'type': 'MicrosoftGraphImage'},
        'last_modified_by': {'key': 'lastModifiedBy', 'type': 'MicrosoftGraphIdentitySet'},
        'last_modified_date_time': {'key': 'lastModifiedDateTime', 'type': 'iso-8601'},
        'name': {'key': 'name', 'type': 'str'},
        'package': {'key': 'package', 'type': 'MicrosoftGraphPackage'},
        'parent_reference': {'key': 'parentReference', 'type': 'MicrosoftGraphItemReference'},
        'shared': {'key': 'shared', 'type': 'MicrosoftGraphShared'},
        'sharepoint_ids': {'key': 'sharepointIds', 'type': 'MicrosoftGraphSharepointIds'},
        'size': {'key': 'size', 'type': 'long'},
        'special_folder': {'key': 'specialFolder', 'type': 'MicrosoftGraphSpecialFolder'},
        'video': {'key': 'video', 'type': 'MicrosoftGraphVideo'},
        'web_dav_url': {'key': 'webDavUrl', 'type': 'str'},
        'web_url': {'key': 'webUrl', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphRemoteItem, self).__init__(**kwargs)
        self.created_by = kwargs.get('created_by', None)
        self.created_date_time = kwargs.get('created_date_time', None)
        self.file = kwargs.get('file', None)
        self.file_system_info = kwargs.get('file_system_info', None)
        self.folder = kwargs.get('folder', None)
        self.id = kwargs.get('id', None)
        self.image = kwargs.get('image', None)
        self.last_modified_by = kwargs.get('last_modified_by', None)
        self.last_modified_date_time = kwargs.get('last_modified_date_time', None)
        self.name = kwargs.get('name', None)
        self.package = kwargs.get('package', None)
        self.parent_reference = kwargs.get('parent_reference', None)
        self.shared = kwargs.get('shared', None)
        self.sharepoint_ids = kwargs.get('sharepoint_ids', None)
        self.size = kwargs.get('size', None)
        self.special_folder = kwargs.get('special_folder', None)
        self.video = kwargs.get('video', None)
        self.web_dav_url = kwargs.get('web_dav_url', None)
        self.web_url = kwargs.get('web_url', None)


class MicrosoftGraphRenameAction(msrest.serialization.Model):
    """renameAction.

    :param new_name: The new name of the item.
    :type new_name: str
    :param old_name: The previous name of the item.
    :type old_name: str
    """

    _attribute_map = {
        'new_name': {'key': 'newName', 'type': 'str'},
        'old_name': {'key': 'oldName', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphRenameAction, self).__init__(**kwargs)
        self.new_name = kwargs.get('new_name', None)
        self.old_name = kwargs.get('old_name', None)


class MicrosoftGraphRequest(MicrosoftGraphEntity):
    """request.

    :param id: Read-only.
    :type id: str
    :param approval: approval.
    :type approval: ~users_user.models.MicrosoftGraphApproval
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'approval': {'key': 'approval', 'type': 'MicrosoftGraphApproval'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphRequest, self).__init__(**kwargs)
        self.approval = kwargs.get('approval', None)


class MicrosoftGraphResourceReference(msrest.serialization.Model):
    """resourceReference.

    :param web_url: A URL leading to the referenced item.
    :type web_url: str
    :param id: The item's unique identifier.
    :type id: str
    :param type: A string value that can be used to classify the item, such as
     'microsoft.graph.driveItem'.
    :type type: str
    """

    _attribute_map = {
        'web_url': {'key': 'webUrl', 'type': 'str'},
        'id': {'key': 'id', 'type': 'str'},
        'type': {'key': 'type', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphResourceReference, self).__init__(**kwargs)
        self.web_url = kwargs.get('web_url', None)
        self.id = kwargs.get('id', None)
        self.type = kwargs.get('type', None)


class MicrosoftGraphResourceSpecificPermissionGrant(MicrosoftGraphDirectoryObject):
    """Represents an Azure Active Directory object. The directoryObject type is the base type for many other directory entity types.

    :param id: Read-only.
    :type id: str
    :param deleted_date_time:
    :type deleted_date_time: ~datetime.datetime
    :param client_id:
    :type client_id: str
    :param client_app_id:
    :type client_app_id: str
    :param resource_app_id:
    :type resource_app_id: str
    :param permission_type:
    :type permission_type: str
    :param permission:
    :type permission: str
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'deleted_date_time': {'key': 'deletedDateTime', 'type': 'iso-8601'},
        'client_id': {'key': 'clientId', 'type': 'str'},
        'client_app_id': {'key': 'clientAppId', 'type': 'str'},
        'resource_app_id': {'key': 'resourceAppId', 'type': 'str'},
        'permission_type': {'key': 'permissionType', 'type': 'str'},
        'permission': {'key': 'permission', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphResourceSpecificPermissionGrant, self).__init__(**kwargs)
        self.client_id = kwargs.get('client_id', None)
        self.client_app_id = kwargs.get('client_app_id', None)
        self.resource_app_id = kwargs.get('resource_app_id', None)
        self.permission_type = kwargs.get('permission_type', None)
        self.permission = kwargs.get('permission', None)


class MicrosoftGraphResourceVisualization(msrest.serialization.Model):
    """resourceVisualization.

    :param title: The item's title text.
    :type title: str
    :param type: The item's media type. Can be used for filtering for a specific file based on a
     specific type. See below for supported types.
    :type type: str
    :param media_type: The item's media type. Can be used for filtering for a specific type of file
     based on supported IANA Media Mime Types. Note that not all Media Mime Types are supported.
    :type media_type: str
    :param preview_image_url: A URL leading to the preview image for the item.
    :type preview_image_url: str
    :param preview_text: A preview text for the item.
    :type preview_text: str
    :param container_web_url: A path leading to the folder in which the item is stored.
    :type container_web_url: str
    :param container_display_name: A string describing where the item is stored. For example, the
     name of a SharePoint site or the user name identifying the owner of the OneDrive storing the
     item.
    :type container_display_name: str
    :param container_type: Can be used for filtering by the type of container in which the file is
     stored. Such as Site or OneDriveBusiness.
    :type container_type: str
    """

    _attribute_map = {
        'title': {'key': 'title', 'type': 'str'},
        'type': {'key': 'type', 'type': 'str'},
        'media_type': {'key': 'mediaType', 'type': 'str'},
        'preview_image_url': {'key': 'previewImageUrl', 'type': 'str'},
        'preview_text': {'key': 'previewText', 'type': 'str'},
        'container_web_url': {'key': 'containerWebUrl', 'type': 'str'},
        'container_display_name': {'key': 'containerDisplayName', 'type': 'str'},
        'container_type': {'key': 'containerType', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphResourceVisualization, self).__init__(**kwargs)
        self.title = kwargs.get('title', None)
        self.type = kwargs.get('type', None)
        self.media_type = kwargs.get('media_type', None)
        self.preview_image_url = kwargs.get('preview_image_url', None)
        self.preview_text = kwargs.get('preview_text', None)
        self.container_web_url = kwargs.get('container_web_url', None)
        self.container_display_name = kwargs.get('container_display_name', None)
        self.container_type = kwargs.get('container_type', None)


class MicrosoftGraphResponseStatus(msrest.serialization.Model):
    """responseStatus.

    :param response: responseType. Possible values include: "none", "organizer",
     "tentativelyAccepted", "accepted", "declined", "notResponded".
    :type response: str or ~users_user.models.MicrosoftGraphResponseType
    :param time: The date and time that the response was returned. It uses ISO 8601 format and is
     always in UTC time. For example, midnight UTC on Jan 1, 2014 would look like this:
     '2014-01-01T00:00:00Z'.
    :type time: ~datetime.datetime
    """

    _attribute_map = {
        'response': {'key': 'response', 'type': 'str'},
        'time': {'key': 'time', 'type': 'iso-8601'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphResponseStatus, self).__init__(**kwargs)
        self.response = kwargs.get('response', None)
        self.time = kwargs.get('time', None)


class MicrosoftGraphSchedule(MicrosoftGraphEntity):
    """schedule.

    :param id: Read-only.
    :type id: str
    :param enabled:
    :type enabled: bool
    :param time_zone:
    :type time_zone: str
    :param provision_status: operationStatus. Possible values include: "NotStarted", "Running",
     "Completed", "Failed".
    :type provision_status: str or ~users_user.models.MicrosoftGraphOperationStatus
    :param provision_status_code:
    :type provision_status_code: str
    :param workforce_integration_ids:
    :type workforce_integration_ids: list[str]
    :param time_clock_enabled:
    :type time_clock_enabled: bool
    :param open_shifts_enabled:
    :type open_shifts_enabled: bool
    :param swap_shifts_requests_enabled:
    :type swap_shifts_requests_enabled: bool
    :param offer_shift_requests_enabled:
    :type offer_shift_requests_enabled: bool
    :param time_off_requests_enabled:
    :type time_off_requests_enabled: bool
    :param shifts:
    :type shifts: list[~users_user.models.MicrosoftGraphShift]
    :param open_shifts:
    :type open_shifts: list[~users_user.models.MicrosoftGraphOpenShift]
    :param times_off:
    :type times_off: list[~users_user.models.MicrosoftGraphTimeOff]
    :param time_off_reasons:
    :type time_off_reasons: list[~users_user.models.MicrosoftGraphTimeOffReason]
    :param scheduling_groups:
    :type scheduling_groups: list[~users_user.models.MicrosoftGraphSchedulingGroup]
    :param swap_shifts_change_requests:
    :type swap_shifts_change_requests:
     list[~users_user.models.MicrosoftGraphSwapShiftsChangeRequest]
    :param open_shift_change_requests:
    :type open_shift_change_requests: list[~users_user.models.MicrosoftGraphOpenShiftChangeRequest]
    :param offer_shift_requests:
    :type offer_shift_requests: list[~users_user.models.MicrosoftGraphOfferShiftRequest]
    :param time_off_requests:
    :type time_off_requests: list[~users_user.models.MicrosoftGraphTimeOffRequest]
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'enabled': {'key': 'enabled', 'type': 'bool'},
        'time_zone': {'key': 'timeZone', 'type': 'str'},
        'provision_status': {'key': 'provisionStatus', 'type': 'str'},
        'provision_status_code': {'key': 'provisionStatusCode', 'type': 'str'},
        'workforce_integration_ids': {'key': 'workforceIntegrationIds', 'type': '[str]'},
        'time_clock_enabled': {'key': 'timeClockEnabled', 'type': 'bool'},
        'open_shifts_enabled': {'key': 'openShiftsEnabled', 'type': 'bool'},
        'swap_shifts_requests_enabled': {'key': 'swapShiftsRequestsEnabled', 'type': 'bool'},
        'offer_shift_requests_enabled': {'key': 'offerShiftRequestsEnabled', 'type': 'bool'},
        'time_off_requests_enabled': {'key': 'timeOffRequestsEnabled', 'type': 'bool'},
        'shifts': {'key': 'shifts', 'type': '[MicrosoftGraphShift]'},
        'open_shifts': {'key': 'openShifts', 'type': '[MicrosoftGraphOpenShift]'},
        'times_off': {'key': 'timesOff', 'type': '[MicrosoftGraphTimeOff]'},
        'time_off_reasons': {'key': 'timeOffReasons', 'type': '[MicrosoftGraphTimeOffReason]'},
        'scheduling_groups': {'key': 'schedulingGroups', 'type': '[MicrosoftGraphSchedulingGroup]'},
        'swap_shifts_change_requests': {'key': 'swapShiftsChangeRequests', 'type': '[MicrosoftGraphSwapShiftsChangeRequest]'},
        'open_shift_change_requests': {'key': 'openShiftChangeRequests', 'type': '[MicrosoftGraphOpenShiftChangeRequest]'},
        'offer_shift_requests': {'key': 'offerShiftRequests', 'type': '[MicrosoftGraphOfferShiftRequest]'},
        'time_off_requests': {'key': 'timeOffRequests', 'type': '[MicrosoftGraphTimeOffRequest]'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphSchedule, self).__init__(**kwargs)
        self.enabled = kwargs.get('enabled', None)
        self.time_zone = kwargs.get('time_zone', None)
        self.provision_status = kwargs.get('provision_status', None)
        self.provision_status_code = kwargs.get('provision_status_code', None)
        self.workforce_integration_ids = kwargs.get('workforce_integration_ids', None)
        self.time_clock_enabled = kwargs.get('time_clock_enabled', None)
        self.open_shifts_enabled = kwargs.get('open_shifts_enabled', None)
        self.swap_shifts_requests_enabled = kwargs.get('swap_shifts_requests_enabled', None)
        self.offer_shift_requests_enabled = kwargs.get('offer_shift_requests_enabled', None)
        self.time_off_requests_enabled = kwargs.get('time_off_requests_enabled', None)
        self.shifts = kwargs.get('shifts', None)
        self.open_shifts = kwargs.get('open_shifts', None)
        self.times_off = kwargs.get('times_off', None)
        self.time_off_reasons = kwargs.get('time_off_reasons', None)
        self.scheduling_groups = kwargs.get('scheduling_groups', None)
        self.swap_shifts_change_requests = kwargs.get('swap_shifts_change_requests', None)
        self.open_shift_change_requests = kwargs.get('open_shift_change_requests', None)
        self.offer_shift_requests = kwargs.get('offer_shift_requests', None)
        self.time_off_requests = kwargs.get('time_off_requests', None)


class MicrosoftGraphSchedulingGroup(MicrosoftGraphChangeTrackedEntity):
    """schedulingGroup.

    :param id: Read-only.
    :type id: str
    :param created_date_time:
    :type created_date_time: ~datetime.datetime
    :param last_modified_date_time:
    :type last_modified_date_time: ~datetime.datetime
    :param last_modified_by: identitySet.
    :type last_modified_by: ~users_user.models.MicrosoftGraphIdentitySet
    :param display_name:
    :type display_name: str
    :param is_active:
    :type is_active: bool
    :param user_ids:
    :type user_ids: list[str]
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'created_date_time': {'key': 'createdDateTime', 'type': 'iso-8601'},
        'last_modified_date_time': {'key': 'lastModifiedDateTime', 'type': 'iso-8601'},
        'last_modified_by': {'key': 'lastModifiedBy', 'type': 'MicrosoftGraphIdentitySet'},
        'display_name': {'key': 'displayName', 'type': 'str'},
        'is_active': {'key': 'isActive', 'type': 'bool'},
        'user_ids': {'key': 'userIds', 'type': '[str]'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphSchedulingGroup, self).__init__(**kwargs)
        self.display_name = kwargs.get('display_name', None)
        self.is_active = kwargs.get('is_active', None)
        self.user_ids = kwargs.get('user_ids', None)


class MicrosoftGraphScopedRoleMembership(MicrosoftGraphEntity):
    """scopedRoleMembership.

    :param id: Read-only.
    :type id: str
    :param role_id:
    :type role_id: str
    :param administrative_unit_id:
    :type administrative_unit_id: str
    :param role_member_info: identity.
    :type role_member_info: ~users_user.models.MicrosoftGraphIdentity
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'role_id': {'key': 'roleId', 'type': 'str'},
        'administrative_unit_id': {'key': 'administrativeUnitId', 'type': 'str'},
        'role_member_info': {'key': 'roleMemberInfo', 'type': 'MicrosoftGraphIdentity'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphScopedRoleMembership, self).__init__(**kwargs)
        self.role_id = kwargs.get('role_id', None)
        self.administrative_unit_id = kwargs.get('administrative_unit_id', None)
        self.role_member_info = kwargs.get('role_member_info', None)


class MicrosoftGraphSearchResult(msrest.serialization.Model):
    """searchResult.

    :param on_click_telemetry_url: A callback URL that can be used to record telemetry information.
     The application should issue a GET on this URL if the user interacts with this item to improve
     the quality of results.
    :type on_click_telemetry_url: str
    """

    _attribute_map = {
        'on_click_telemetry_url': {'key': 'onClickTelemetryUrl', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphSearchResult, self).__init__(**kwargs)
        self.on_click_telemetry_url = kwargs.get('on_click_telemetry_url', None)


class MicrosoftGraphSectionGroup(MicrosoftGraphOnenoteEntityHierarchyModel):
    """sectionGroup.

    :param id: Read-only.
    :type id: str
    :param self_property: The endpoint where you can get details about the page. Read-only.
    :type self_property: str
    :param created_date_time: The date and time when the page was created. The timestamp represents
     date and time information using ISO 8601 format and is always in UTC time. For example,
     midnight UTC on Jan 1, 2014 would look like this: '2014-01-01T00:00:00Z'. Read-only.
    :type created_date_time: ~datetime.datetime
    :param display_name: The name of the notebook.
    :type display_name: str
    :param created_by: identitySet.
    :type created_by: ~users_user.models.MicrosoftGraphIdentitySet
    :param last_modified_by: identitySet.
    :type last_modified_by: ~users_user.models.MicrosoftGraphIdentitySet
    :param last_modified_date_time: The date and time when the notebook was last modified. The
     timestamp represents date and time information using ISO 8601 format and is always in UTC time.
     For example, midnight UTC on Jan 1, 2014 would look like this: '2014-01-01T00:00:00Z'. Read-
     only.
    :type last_modified_date_time: ~datetime.datetime
    :param sections_url: The URL for the sections navigation property, which returns all the
     sections in the section group. Read-only.
    :type sections_url: str
    :param section_groups_url: The URL for the sectionGroups navigation property, which returns all
     the section groups in the section group. Read-only.
    :type section_groups_url: str
    :param parent_notebook: notebook.
    :type parent_notebook: ~users_user.models.MicrosoftGraphNotebook
    :param parent_section_group: sectionGroup.
    :type parent_section_group: ~users_user.models.MicrosoftGraphSectionGroup
    :param sections: The sections in the section group. Read-only. Nullable.
    :type sections: list[~users_user.models.MicrosoftGraphOnenoteSection]
    :param section_groups: The section groups in the section. Read-only. Nullable.
    :type section_groups: list[~users_user.models.MicrosoftGraphSectionGroup]
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'self_property': {'key': 'self', 'type': 'str'},
        'created_date_time': {'key': 'createdDateTime', 'type': 'iso-8601'},
        'display_name': {'key': 'displayName', 'type': 'str'},
        'created_by': {'key': 'createdBy', 'type': 'MicrosoftGraphIdentitySet'},
        'last_modified_by': {'key': 'lastModifiedBy', 'type': 'MicrosoftGraphIdentitySet'},
        'last_modified_date_time': {'key': 'lastModifiedDateTime', 'type': 'iso-8601'},
        'sections_url': {'key': 'sectionsUrl', 'type': 'str'},
        'section_groups_url': {'key': 'sectionGroupsUrl', 'type': 'str'},
        'parent_notebook': {'key': 'parentNotebook', 'type': 'MicrosoftGraphNotebook'},
        'parent_section_group': {'key': 'parentSectionGroup', 'type': 'MicrosoftGraphSectionGroup'},
        'sections': {'key': 'sections', 'type': '[MicrosoftGraphOnenoteSection]'},
        'section_groups': {'key': 'sectionGroups', 'type': '[MicrosoftGraphSectionGroup]'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphSectionGroup, self).__init__(**kwargs)
        self.sections_url = kwargs.get('sections_url', None)
        self.section_groups_url = kwargs.get('section_groups_url', None)
        self.parent_notebook = kwargs.get('parent_notebook', None)
        self.parent_section_group = kwargs.get('parent_section_group', None)
        self.sections = kwargs.get('sections', None)
        self.section_groups = kwargs.get('section_groups', None)


class MicrosoftGraphSectionLinks(msrest.serialization.Model):
    """sectionLinks.

    :param one_note_client_url: externalLink.
    :type one_note_client_url: ~users_user.models.MicrosoftGraphExternalLink
    :param one_note_web_url: externalLink.
    :type one_note_web_url: ~users_user.models.MicrosoftGraphExternalLink
    """

    _attribute_map = {
        'one_note_client_url': {'key': 'oneNoteClientUrl', 'type': 'MicrosoftGraphExternalLink'},
        'one_note_web_url': {'key': 'oneNoteWebUrl', 'type': 'MicrosoftGraphExternalLink'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphSectionLinks, self).__init__(**kwargs)
        self.one_note_client_url = kwargs.get('one_note_client_url', None)
        self.one_note_web_url = kwargs.get('one_note_web_url', None)


class MicrosoftGraphSecurityBaselineSettingState(MicrosoftGraphEntity):
    """The security baseline compliance state of a setting for a device.

    :param id: Read-only.
    :type id: str
    :param setting_name: The setting name that is being reported.
    :type setting_name: str
    :param state: securityBaselineComplianceState. Possible values include: "unknown", "secure",
     "notApplicable", "notSecure", "error", "conflict".
    :type state: str or ~users_user.models.MicrosoftGraphSecurityBaselineComplianceState
    :param setting_category_id: The setting category id which this setting belongs to.
    :type setting_category_id: str
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'setting_name': {'key': 'settingName', 'type': 'str'},
        'state': {'key': 'state', 'type': 'str'},
        'setting_category_id': {'key': 'settingCategoryId', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphSecurityBaselineSettingState, self).__init__(**kwargs)
        self.setting_name = kwargs.get('setting_name', None)
        self.state = kwargs.get('state', None)
        self.setting_category_id = kwargs.get('setting_category_id', None)


class MicrosoftGraphSecurityBaselineState(MicrosoftGraphEntity):
    """Security baseline state for a device.

    :param id: Read-only.
    :type id: str
    :param security_baseline_template_id: The security baseline template id.
    :type security_baseline_template_id: str
    :param display_name: The display name of the security baseline.
    :type display_name: str
    :param setting_states: The security baseline state for different settings for a device.
    :type setting_states: list[~users_user.models.MicrosoftGraphSecurityBaselineSettingState]
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'security_baseline_template_id': {'key': 'securityBaselineTemplateId', 'type': 'str'},
        'display_name': {'key': 'displayName', 'type': 'str'},
        'setting_states': {'key': 'settingStates', 'type': '[MicrosoftGraphSecurityBaselineSettingState]'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphSecurityBaselineState, self).__init__(**kwargs)
        self.security_baseline_template_id = kwargs.get('security_baseline_template_id', None)
        self.display_name = kwargs.get('display_name', None)
        self.setting_states = kwargs.get('setting_states', None)


class MicrosoftGraphSensitivityLabel(MicrosoftGraphEntity):
    """sensitivityLabel.

    :param id: Read-only.
    :type id: str
    :param name:
    :type name: str
    :param display_name:
    :type display_name: str
    :param description:
    :type description: str
    :param tool_tip:
    :type tool_tip: str
    :param is_endpoint_protection_enabled:
    :type is_endpoint_protection_enabled: bool
    :param is_default:
    :type is_default: bool
    :param application_mode: applicationMode. Possible values include: "manual", "automatic",
     "recommended".
    :type application_mode: str or ~users_user.models.MicrosoftGraphApplicationMode
    :param label_actions:
    :type label_actions: list[~users_user.models.MicrosoftGraphLabelActionBase]
    :param assigned_policies:
    :type assigned_policies: list[~users_user.models.MicrosoftGraphLabelPolicy]
    :param priority:
    :type priority: int
    :param auto_labeling: autoLabeling.
    :type auto_labeling: ~users_user.models.MicrosoftGraphAutoLabeling
    :param applicable_to: sensitivityLabelTarget. Possible values include: "email", "site",
     "unifiedGroup", "unknownFutureValue".
    :type applicable_to: str or ~users_user.models.MicrosoftGraphSensitivityLabelTarget
    :param sublabels:
    :type sublabels: list[~users_user.models.MicrosoftGraphSensitivityLabel]
    """

    _validation = {
        'priority': {'maximum': 2147483647, 'minimum': -2147483648},
    }

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'name': {'key': 'name', 'type': 'str'},
        'display_name': {'key': 'displayName', 'type': 'str'},
        'description': {'key': 'description', 'type': 'str'},
        'tool_tip': {'key': 'toolTip', 'type': 'str'},
        'is_endpoint_protection_enabled': {'key': 'isEndpointProtectionEnabled', 'type': 'bool'},
        'is_default': {'key': 'isDefault', 'type': 'bool'},
        'application_mode': {'key': 'applicationMode', 'type': 'str'},
        'label_actions': {'key': 'labelActions', 'type': '[MicrosoftGraphLabelActionBase]'},
        'assigned_policies': {'key': 'assignedPolicies', 'type': '[MicrosoftGraphLabelPolicy]'},
        'priority': {'key': 'priority', 'type': 'int'},
        'auto_labeling': {'key': 'autoLabeling', 'type': 'MicrosoftGraphAutoLabeling'},
        'applicable_to': {'key': 'applicableTo', 'type': 'str'},
        'sublabels': {'key': 'sublabels', 'type': '[MicrosoftGraphSensitivityLabel]'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphSensitivityLabel, self).__init__(**kwargs)
        self.name = kwargs.get('name', None)
        self.display_name = kwargs.get('display_name', None)
        self.description = kwargs.get('description', None)
        self.tool_tip = kwargs.get('tool_tip', None)
        self.is_endpoint_protection_enabled = kwargs.get('is_endpoint_protection_enabled', None)
        self.is_default = kwargs.get('is_default', None)
        self.application_mode = kwargs.get('application_mode', None)
        self.label_actions = kwargs.get('label_actions', None)
        self.assigned_policies = kwargs.get('assigned_policies', None)
        self.priority = kwargs.get('priority', None)
        self.auto_labeling = kwargs.get('auto_labeling', None)
        self.applicable_to = kwargs.get('applicable_to', None)
        self.sublabels = kwargs.get('sublabels', None)


class MicrosoftGraphSensitivityPolicySettings(MicrosoftGraphEntity):
    """sensitivityPolicySettings.

    :param id: Read-only.
    :type id: str
    :param is_mandatory:
    :type is_mandatory: bool
    :param help_web_url:
    :type help_web_url: str
    :param downgrade_sensitivity_requires_justification:
    :type downgrade_sensitivity_requires_justification: bool
    :param applicable_to: sensitivityLabelTarget. Possible values include: "email", "site",
     "unifiedGroup", "unknownFutureValue".
    :type applicable_to: str or ~users_user.models.MicrosoftGraphSensitivityLabelTarget
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'is_mandatory': {'key': 'isMandatory', 'type': 'bool'},
        'help_web_url': {'key': 'helpWebUrl', 'type': 'str'},
        'downgrade_sensitivity_requires_justification': {'key': 'downgradeSensitivityRequiresJustification', 'type': 'bool'},
        'applicable_to': {'key': 'applicableTo', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphSensitivityPolicySettings, self).__init__(**kwargs)
        self.is_mandatory = kwargs.get('is_mandatory', None)
        self.help_web_url = kwargs.get('help_web_url', None)
        self.downgrade_sensitivity_requires_justification = kwargs.get('downgrade_sensitivity_requires_justification', None)
        self.applicable_to = kwargs.get('applicable_to', None)


class MicrosoftGraphServiceInformation(msrest.serialization.Model):
    """serviceInformation.

    :param name:
    :type name: str
    :param web_url:
    :type web_url: str
    """

    _attribute_map = {
        'name': {'key': 'name', 'type': 'str'},
        'web_url': {'key': 'webUrl', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphServiceInformation, self).__init__(**kwargs)
        self.name = kwargs.get('name', None)
        self.web_url = kwargs.get('web_url', None)


class MicrosoftGraphServicePlanInfo(msrest.serialization.Model):
    """servicePlanInfo.

    :param service_plan_id: The unique identifier of the service plan.
    :type service_plan_id: str
    :param service_plan_name: The name of the service plan.
    :type service_plan_name: str
    :param provisioning_status: The provisioning status of the service plan. Possible
     values:'Success' - Service is fully provisioned.'Disabled' - Service has been
     disabled.'PendingInput' - Service is not yet provisioned; awaiting service
     confirmation.'PendingActivation' - Service is provisioned but requires explicit activation by
     administrator (for example, Intune_O365 service plan)'PendingProvisioning' - Microsoft has
     added a new service to the product SKU and it has not been activated in the tenant, yet.
    :type provisioning_status: str
    :param applies_to: The object the service plan can be assigned to. Possible values:'User' -
     service plan can be assigned to individual users.'Company' - service plan can be assigned to
     the entire tenant.
    :type applies_to: str
    """

    _attribute_map = {
        'service_plan_id': {'key': 'servicePlanId', 'type': 'str'},
        'service_plan_name': {'key': 'servicePlanName', 'type': 'str'},
        'provisioning_status': {'key': 'provisioningStatus', 'type': 'str'},
        'applies_to': {'key': 'appliesTo', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphServicePlanInfo, self).__init__(**kwargs)
        self.service_plan_id = kwargs.get('service_plan_id', None)
        self.service_plan_name = kwargs.get('service_plan_name', None)
        self.provisioning_status = kwargs.get('provisioning_status', None)
        self.applies_to = kwargs.get('applies_to', None)


class MicrosoftGraphSettings(msrest.serialization.Model):
    """settings.

    :param has_license:
    :type has_license: bool
    :param has_opted_out:
    :type has_opted_out: bool
    :param has_graph_mailbox:
    :type has_graph_mailbox: bool
    """

    _attribute_map = {
        'has_license': {'key': 'hasLicense', 'type': 'bool'},
        'has_opted_out': {'key': 'hasOptedOut', 'type': 'bool'},
        'has_graph_mailbox': {'key': 'hasGraphMailbox', 'type': 'bool'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphSettings, self).__init__(**kwargs)
        self.has_license = kwargs.get('has_license', None)
        self.has_opted_out = kwargs.get('has_opted_out', None)
        self.has_graph_mailbox = kwargs.get('has_graph_mailbox', None)


class MicrosoftGraphSettingSource(msrest.serialization.Model):
    """settingSource.

    :param id: Not yet documented.
    :type id: str
    :param display_name: Not yet documented.
    :type display_name: str
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'display_name': {'key': 'displayName', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphSettingSource, self).__init__(**kwargs)
        self.id = kwargs.get('id', None)
        self.display_name = kwargs.get('display_name', None)


class MicrosoftGraphSettingValue(msrest.serialization.Model):
    """settingValue.

    :param name: Name of the setting (as defined by the groupSettingTemplate).
    :type name: str
    :param value: Value of the setting.
    :type value: str
    """

    _attribute_map = {
        'name': {'key': 'name', 'type': 'str'},
        'value': {'key': 'value', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphSettingValue, self).__init__(**kwargs)
        self.name = kwargs.get('name', None)
        self.value = kwargs.get('value', None)


class MicrosoftGraphShareAction(msrest.serialization.Model):
    """shareAction.

    :param recipients: The identities the item was shared with in this action.
    :type recipients: list[~users_user.models.MicrosoftGraphIdentitySet]
    """

    _attribute_map = {
        'recipients': {'key': 'recipients', 'type': '[MicrosoftGraphIdentitySet]'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphShareAction, self).__init__(**kwargs)
        self.recipients = kwargs.get('recipients', None)


class MicrosoftGraphShared(msrest.serialization.Model):
    """shared.

    :param owner: identitySet.
    :type owner: ~users_user.models.MicrosoftGraphIdentitySet
    :param scope: Indicates the scope of how the item is shared: anonymous, organization, or users.
     Read-only.
    :type scope: str
    :param shared_by: identitySet.
    :type shared_by: ~users_user.models.MicrosoftGraphIdentitySet
    :param shared_date_time: The UTC date and time when the item was shared. Read-only.
    :type shared_date_time: ~datetime.datetime
    """

    _attribute_map = {
        'owner': {'key': 'owner', 'type': 'MicrosoftGraphIdentitySet'},
        'scope': {'key': 'scope', 'type': 'str'},
        'shared_by': {'key': 'sharedBy', 'type': 'MicrosoftGraphIdentitySet'},
        'shared_date_time': {'key': 'sharedDateTime', 'type': 'iso-8601'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphShared, self).__init__(**kwargs)
        self.owner = kwargs.get('owner', None)
        self.scope = kwargs.get('scope', None)
        self.shared_by = kwargs.get('shared_by', None)
        self.shared_date_time = kwargs.get('shared_date_time', None)


class MicrosoftGraphSharedAppleDeviceUser(msrest.serialization.Model):
    """sharedAppleDeviceUser.

    :param user_principal_name: User name.
    :type user_principal_name: str
    :param data_to_sync: Data to sync.
    :type data_to_sync: bool
    :param data_quota: Data quota.
    :type data_quota: long
    :param data_used: Data quota.
    :type data_used: long
    """

    _attribute_map = {
        'user_principal_name': {'key': 'userPrincipalName', 'type': 'str'},
        'data_to_sync': {'key': 'dataToSync', 'type': 'bool'},
        'data_quota': {'key': 'dataQuota', 'type': 'long'},
        'data_used': {'key': 'dataUsed', 'type': 'long'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphSharedAppleDeviceUser, self).__init__(**kwargs)
        self.user_principal_name = kwargs.get('user_principal_name', None)
        self.data_to_sync = kwargs.get('data_to_sync', None)
        self.data_quota = kwargs.get('data_quota', None)
        self.data_used = kwargs.get('data_used', None)


class MicrosoftGraphSharedInsight(MicrosoftGraphEntity):
    """sharedInsight.

    :param id: Read-only.
    :type id: str
    :param last_shared: sharingDetail.
    :type last_shared: ~users_user.models.MicrosoftGraphSharingDetail
    :param sharing_history:
    :type sharing_history: list[~users_user.models.MicrosoftGraphSharingDetail]
    :param resource_visualization: resourceVisualization.
    :type resource_visualization: ~users_user.models.MicrosoftGraphResourceVisualization
    :param resource_reference: resourceReference.
    :type resource_reference: ~users_user.models.MicrosoftGraphResourceReference
    :param last_shared_method: entity.
    :type last_shared_method: ~users_user.models.MicrosoftGraphEntity
    :param resource: entity.
    :type resource: ~users_user.models.MicrosoftGraphEntity
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'last_shared': {'key': 'lastShared', 'type': 'MicrosoftGraphSharingDetail'},
        'sharing_history': {'key': 'sharingHistory', 'type': '[MicrosoftGraphSharingDetail]'},
        'resource_visualization': {'key': 'resourceVisualization', 'type': 'MicrosoftGraphResourceVisualization'},
        'resource_reference': {'key': 'resourceReference', 'type': 'MicrosoftGraphResourceReference'},
        'last_shared_method': {'key': 'lastSharedMethod', 'type': 'MicrosoftGraphEntity'},
        'resource': {'key': 'resource', 'type': 'MicrosoftGraphEntity'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphSharedInsight, self).__init__(**kwargs)
        self.last_shared = kwargs.get('last_shared', None)
        self.sharing_history = kwargs.get('sharing_history', None)
        self.resource_visualization = kwargs.get('resource_visualization', None)
        self.resource_reference = kwargs.get('resource_reference', None)
        self.last_shared_method = kwargs.get('last_shared_method', None)
        self.resource = kwargs.get('resource', None)


class MicrosoftGraphSharepointIds(msrest.serialization.Model):
    """sharepointIds.

    :param list_id: The unique identifier (guid) for the item's list in SharePoint.
    :type list_id: str
    :param list_item_id: An integer identifier for the item within the containing list.
    :type list_item_id: str
    :param list_item_unique_id: The unique identifier (guid) for the item within OneDrive for
     Business or a SharePoint site.
    :type list_item_unique_id: str
    :param site_id: The unique identifier (guid) for the item's site collection (SPSite).
    :type site_id: str
    :param site_url: The SharePoint URL for the site that contains the item.
    :type site_url: str
    :param tenant_id: The unique identifier (guid) for the tenancy.
    :type tenant_id: str
    :param web_id: The unique identifier (guid) for the item's site (SPWeb).
    :type web_id: str
    """

    _attribute_map = {
        'list_id': {'key': 'listId', 'type': 'str'},
        'list_item_id': {'key': 'listItemId', 'type': 'str'},
        'list_item_unique_id': {'key': 'listItemUniqueId', 'type': 'str'},
        'site_id': {'key': 'siteId', 'type': 'str'},
        'site_url': {'key': 'siteUrl', 'type': 'str'},
        'tenant_id': {'key': 'tenantId', 'type': 'str'},
        'web_id': {'key': 'webId', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphSharepointIds, self).__init__(**kwargs)
        self.list_id = kwargs.get('list_id', None)
        self.list_item_id = kwargs.get('list_item_id', None)
        self.list_item_unique_id = kwargs.get('list_item_unique_id', None)
        self.site_id = kwargs.get('site_id', None)
        self.site_url = kwargs.get('site_url', None)
        self.tenant_id = kwargs.get('tenant_id', None)
        self.web_id = kwargs.get('web_id', None)


class MicrosoftGraphSharingDetail(msrest.serialization.Model):
    """sharingDetail.

    :param shared_by: insightIdentity.
    :type shared_by: ~users_user.models.MicrosoftGraphInsightIdentity
    :param shared_date_time: The date and time the file was last shared. The timestamp represents
     date and time information using ISO 8601 format and is always in UTC time. For example,
     midnight UTC on Jan 1, 2014 would look like this: 2014-01-01T00:00:00Z. Read-only.
    :type shared_date_time: ~datetime.datetime
    :param sharing_subject: The subject with which the document was shared.
    :type sharing_subject: str
    :param sharing_type: Determines the way the document was shared, can be by a 'Link',
     'Attachment', 'Group', 'Site'.
    :type sharing_type: str
    :param sharing_reference: resourceReference.
    :type sharing_reference: ~users_user.models.MicrosoftGraphResourceReference
    """

    _attribute_map = {
        'shared_by': {'key': 'sharedBy', 'type': 'MicrosoftGraphInsightIdentity'},
        'shared_date_time': {'key': 'sharedDateTime', 'type': 'iso-8601'},
        'sharing_subject': {'key': 'sharingSubject', 'type': 'str'},
        'sharing_type': {'key': 'sharingType', 'type': 'str'},
        'sharing_reference': {'key': 'sharingReference', 'type': 'MicrosoftGraphResourceReference'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphSharingDetail, self).__init__(**kwargs)
        self.shared_by = kwargs.get('shared_by', None)
        self.shared_date_time = kwargs.get('shared_date_time', None)
        self.sharing_subject = kwargs.get('sharing_subject', None)
        self.sharing_type = kwargs.get('sharing_type', None)
        self.sharing_reference = kwargs.get('sharing_reference', None)


class MicrosoftGraphSharingInvitation(msrest.serialization.Model):
    """sharingInvitation.

    :param email: The email address provided for the recipient of the sharing invitation. Read-
     only.
    :type email: str
    :param invited_by: identitySet.
    :type invited_by: ~users_user.models.MicrosoftGraphIdentitySet
    :param redeemed_by:
    :type redeemed_by: str
    :param sign_in_required: If true the recipient of the invitation needs to sign in in order to
     access the shared item. Read-only.
    :type sign_in_required: bool
    """

    _attribute_map = {
        'email': {'key': 'email', 'type': 'str'},
        'invited_by': {'key': 'invitedBy', 'type': 'MicrosoftGraphIdentitySet'},
        'redeemed_by': {'key': 'redeemedBy', 'type': 'str'},
        'sign_in_required': {'key': 'signInRequired', 'type': 'bool'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphSharingInvitation, self).__init__(**kwargs)
        self.email = kwargs.get('email', None)
        self.invited_by = kwargs.get('invited_by', None)
        self.redeemed_by = kwargs.get('redeemed_by', None)
        self.sign_in_required = kwargs.get('sign_in_required', None)


class MicrosoftGraphSharingLink(msrest.serialization.Model):
    """sharingLink.

    :param application: identity.
    :type application: ~users_user.models.MicrosoftGraphIdentity
    :param prevents_download: If true then the user can only use this link to view the item on the
     web, and cannot use it to download the contents of the item. Only for OneDrive for Business and
     SharePoint.
    :type prevents_download: bool
    :param configurator_url:
    :type configurator_url: str
    :param scope: The scope of the link represented by this permission. Value anonymous indicates
     the link is usable by anyone, organization indicates the link is only usable for users signed
     into the same tenant.
    :type scope: str
    :param type: The type of the link created.
    :type type: str
    :param web_html: For embed links, this property contains the HTML code for an :code:`<iframe>`
     element that will embed the item in a webpage.
    :type web_html: str
    :param web_url: A URL that opens the item in the browser on the OneDrive website.
    :type web_url: str
    """

    _attribute_map = {
        'application': {'key': 'application', 'type': 'MicrosoftGraphIdentity'},
        'prevents_download': {'key': 'preventsDownload', 'type': 'bool'},
        'configurator_url': {'key': 'configuratorUrl', 'type': 'str'},
        'scope': {'key': 'scope', 'type': 'str'},
        'type': {'key': 'type', 'type': 'str'},
        'web_html': {'key': 'webHtml', 'type': 'str'},
        'web_url': {'key': 'webUrl', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphSharingLink, self).__init__(**kwargs)
        self.application = kwargs.get('application', None)
        self.prevents_download = kwargs.get('prevents_download', None)
        self.configurator_url = kwargs.get('configurator_url', None)
        self.scope = kwargs.get('scope', None)
        self.type = kwargs.get('type', None)
        self.web_html = kwargs.get('web_html', None)
        self.web_url = kwargs.get('web_url', None)


class MicrosoftGraphShift(MicrosoftGraphChangeTrackedEntity):
    """shift.

    :param id: Read-only.
    :type id: str
    :param created_date_time:
    :type created_date_time: ~datetime.datetime
    :param last_modified_date_time:
    :type last_modified_date_time: ~datetime.datetime
    :param last_modified_by: identitySet.
    :type last_modified_by: ~users_user.models.MicrosoftGraphIdentitySet
    :param shared_shift: shiftItem.
    :type shared_shift: ~users_user.models.MicrosoftGraphShiftItem
    :param draft_shift: shiftItem.
    :type draft_shift: ~users_user.models.MicrosoftGraphShiftItem
    :param user_id:
    :type user_id: str
    :param scheduling_group_id:
    :type scheduling_group_id: str
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'created_date_time': {'key': 'createdDateTime', 'type': 'iso-8601'},
        'last_modified_date_time': {'key': 'lastModifiedDateTime', 'type': 'iso-8601'},
        'last_modified_by': {'key': 'lastModifiedBy', 'type': 'MicrosoftGraphIdentitySet'},
        'shared_shift': {'key': 'sharedShift', 'type': 'MicrosoftGraphShiftItem'},
        'draft_shift': {'key': 'draftShift', 'type': 'MicrosoftGraphShiftItem'},
        'user_id': {'key': 'userId', 'type': 'str'},
        'scheduling_group_id': {'key': 'schedulingGroupId', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphShift, self).__init__(**kwargs)
        self.shared_shift = kwargs.get('shared_shift', None)
        self.draft_shift = kwargs.get('draft_shift', None)
        self.user_id = kwargs.get('user_id', None)
        self.scheduling_group_id = kwargs.get('scheduling_group_id', None)


class MicrosoftGraphShiftActivity(msrest.serialization.Model):
    """shiftActivity.

    :param is_paid:
    :type is_paid: bool
    :param start_date_time:
    :type start_date_time: ~datetime.datetime
    :param end_date_time:
    :type end_date_time: ~datetime.datetime
    :param code:
    :type code: str
    :param display_name:
    :type display_name: str
    :param theme: scheduleEntityTheme. Possible values include: "white", "blue", "green", "purple",
     "pink", "yellow", "gray", "darkBlue", "darkGreen", "darkPurple", "darkPink", "darkYellow",
     "unknownFutureValue".
    :type theme: str or ~users_user.models.MicrosoftGraphScheduleEntityTheme
    """

    _attribute_map = {
        'is_paid': {'key': 'isPaid', 'type': 'bool'},
        'start_date_time': {'key': 'startDateTime', 'type': 'iso-8601'},
        'end_date_time': {'key': 'endDateTime', 'type': 'iso-8601'},
        'code': {'key': 'code', 'type': 'str'},
        'display_name': {'key': 'displayName', 'type': 'str'},
        'theme': {'key': 'theme', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphShiftActivity, self).__init__(**kwargs)
        self.is_paid = kwargs.get('is_paid', None)
        self.start_date_time = kwargs.get('start_date_time', None)
        self.end_date_time = kwargs.get('end_date_time', None)
        self.code = kwargs.get('code', None)
        self.display_name = kwargs.get('display_name', None)
        self.theme = kwargs.get('theme', None)


class MicrosoftGraphShiftAvailability(msrest.serialization.Model):
    """shiftAvailability.

    :param recurrence: patternedRecurrence.
    :type recurrence: ~users_user.models.MicrosoftGraphPatternedRecurrence
    :param time_zone:
    :type time_zone: str
    :param time_slots:
    :type time_slots: list[~users_user.models.MicrosoftGraphTimeRange]
    """

    _attribute_map = {
        'recurrence': {'key': 'recurrence', 'type': 'MicrosoftGraphPatternedRecurrence'},
        'time_zone': {'key': 'timeZone', 'type': 'str'},
        'time_slots': {'key': 'timeSlots', 'type': '[MicrosoftGraphTimeRange]'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphShiftAvailability, self).__init__(**kwargs)
        self.recurrence = kwargs.get('recurrence', None)
        self.time_zone = kwargs.get('time_zone', None)
        self.time_slots = kwargs.get('time_slots', None)


class MicrosoftGraphShiftPreferences(MicrosoftGraphChangeTrackedEntity):
    """shiftPreferences.

    :param id: Read-only.
    :type id: str
    :param created_date_time:
    :type created_date_time: ~datetime.datetime
    :param last_modified_date_time:
    :type last_modified_date_time: ~datetime.datetime
    :param last_modified_by: identitySet.
    :type last_modified_by: ~users_user.models.MicrosoftGraphIdentitySet
    :param availability:
    :type availability: list[~users_user.models.MicrosoftGraphShiftAvailability]
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'created_date_time': {'key': 'createdDateTime', 'type': 'iso-8601'},
        'last_modified_date_time': {'key': 'lastModifiedDateTime', 'type': 'iso-8601'},
        'last_modified_by': {'key': 'lastModifiedBy', 'type': 'MicrosoftGraphIdentitySet'},
        'availability': {'key': 'availability', 'type': '[MicrosoftGraphShiftAvailability]'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphShiftPreferences, self).__init__(**kwargs)
        self.availability = kwargs.get('availability', None)


class MicrosoftGraphSignInActivity(msrest.serialization.Model):
    """signInActivity.

    :param last_sign_in_date_time:
    :type last_sign_in_date_time: ~datetime.datetime
    :param last_sign_in_request_id:
    :type last_sign_in_request_id: str
    """

    _attribute_map = {
        'last_sign_in_date_time': {'key': 'lastSignInDateTime', 'type': 'iso-8601'},
        'last_sign_in_request_id': {'key': 'lastSignInRequestId', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphSignInActivity, self).__init__(**kwargs)
        self.last_sign_in_date_time = kwargs.get('last_sign_in_date_time', None)
        self.last_sign_in_request_id = kwargs.get('last_sign_in_request_id', None)


class MicrosoftGraphSingleValueLegacyExtendedProperty(MicrosoftGraphEntity):
    """singleValueLegacyExtendedProperty.

    :param id: Read-only.
    :type id: str
    :param value: A property value.
    :type value: str
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'value': {'key': 'value', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphSingleValueLegacyExtendedProperty, self).__init__(**kwargs)
        self.value = kwargs.get('value', None)


class MicrosoftGraphSite(MicrosoftGraphBaseItem):
    """site.

    :param id: Read-only.
    :type id: str
    :param created_by: identitySet.
    :type created_by: ~users_user.models.MicrosoftGraphIdentitySet
    :param created_date_time: Date and time of item creation. Read-only.
    :type created_date_time: ~datetime.datetime
    :param description: Provides a user-visible description of the item. Optional.
    :type description: str
    :param e_tag: ETag for the item. Read-only.
    :type e_tag: str
    :param last_modified_by: identitySet.
    :type last_modified_by: ~users_user.models.MicrosoftGraphIdentitySet
    :param last_modified_date_time: Date and time the item was last modified. Read-only.
    :type last_modified_date_time: ~datetime.datetime
    :param name: The name of the item. Read-write.
    :type name: str
    :param parent_reference: itemReference.
    :type parent_reference: ~users_user.models.MicrosoftGraphItemReference
    :param web_url: URL that displays the resource in the browser. Read-only.
    :type web_url: str
    :param created_by_user: Represents an Azure Active Directory user object.
    :type created_by_user: ~users_user.models.MicrosoftGraphUser
    :param last_modified_by_user: Represents an Azure Active Directory user object.
    :type last_modified_by_user: ~users_user.models.MicrosoftGraphUser
    :param display_name: The full title for the site. Read-only.
    :type display_name: str
    :param root: Any object.
    :type root: object
    :param sharepoint_ids: sharepointIds.
    :type sharepoint_ids: ~users_user.models.MicrosoftGraphSharepointIds
    :param site_collection: siteCollection.
    :type site_collection: ~users_user.models.MicrosoftGraphSiteCollection
    :param analytics: itemAnalytics.
    :type analytics: ~users_user.models.MicrosoftGraphItemAnalytics
    :param columns: The collection of column definitions reusable across lists under this site.
    :type columns: list[~users_user.models.MicrosoftGraphColumnDefinition]
    :param content_types: The collection of content types defined for this site.
    :type content_types: list[~users_user.models.MicrosoftGraphContentType]
    :param drive: drive.
    :type drive: ~users_user.models.MicrosoftGraphDrive
    :param drives: The collection of drives (document libraries) under this site.
    :type drives: list[~users_user.models.MicrosoftGraphDrive]
    :param items: Used to address any item contained in this site. This collection cannot be
     enumerated.
    :type items: list[~users_user.models.MicrosoftGraphBaseItem]
    :param lists: The collection of lists under this site.
    :type lists: list[~users_user.models.MicrosoftGraphList]
    :param pages:
    :type pages: list[~users_user.models.MicrosoftGraphSitePage]
    :param sites: The collection of the sub-sites under this site.
    :type sites: list[~users_user.models.MicrosoftGraphSite]
    :param onenote: onenote.
    :type onenote: ~users_user.models.MicrosoftGraphOnenote
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'created_by': {'key': 'createdBy', 'type': 'MicrosoftGraphIdentitySet'},
        'created_date_time': {'key': 'createdDateTime', 'type': 'iso-8601'},
        'description': {'key': 'description', 'type': 'str'},
        'e_tag': {'key': 'eTag', 'type': 'str'},
        'last_modified_by': {'key': 'lastModifiedBy', 'type': 'MicrosoftGraphIdentitySet'},
        'last_modified_date_time': {'key': 'lastModifiedDateTime', 'type': 'iso-8601'},
        'name': {'key': 'name', 'type': 'str'},
        'parent_reference': {'key': 'parentReference', 'type': 'MicrosoftGraphItemReference'},
        'web_url': {'key': 'webUrl', 'type': 'str'},
        'created_by_user': {'key': 'createdByUser', 'type': 'MicrosoftGraphUser'},
        'last_modified_by_user': {'key': 'lastModifiedByUser', 'type': 'MicrosoftGraphUser'},
        'display_name': {'key': 'displayName', 'type': 'str'},
        'root': {'key': 'root', 'type': 'object'},
        'sharepoint_ids': {'key': 'sharepointIds', 'type': 'MicrosoftGraphSharepointIds'},
        'site_collection': {'key': 'siteCollection', 'type': 'MicrosoftGraphSiteCollection'},
        'analytics': {'key': 'analytics', 'type': 'MicrosoftGraphItemAnalytics'},
        'columns': {'key': 'columns', 'type': '[MicrosoftGraphColumnDefinition]'},
        'content_types': {'key': 'contentTypes', 'type': '[MicrosoftGraphContentType]'},
        'drive': {'key': 'drive', 'type': 'MicrosoftGraphDrive'},
        'drives': {'key': 'drives', 'type': '[MicrosoftGraphDrive]'},
        'items': {'key': 'items', 'type': '[MicrosoftGraphBaseItem]'},
        'lists': {'key': 'lists', 'type': '[MicrosoftGraphList]'},
        'pages': {'key': 'pages', 'type': '[MicrosoftGraphSitePage]'},
        'sites': {'key': 'sites', 'type': '[MicrosoftGraphSite]'},
        'onenote': {'key': 'onenote', 'type': 'MicrosoftGraphOnenote'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphSite, self).__init__(**kwargs)
        self.display_name = kwargs.get('display_name', None)
        self.root = kwargs.get('root', None)
        self.sharepoint_ids = kwargs.get('sharepoint_ids', None)
        self.site_collection = kwargs.get('site_collection', None)
        self.analytics = kwargs.get('analytics', None)
        self.columns = kwargs.get('columns', None)
        self.content_types = kwargs.get('content_types', None)
        self.drive = kwargs.get('drive', None)
        self.drives = kwargs.get('drives', None)
        self.items = kwargs.get('items', None)
        self.lists = kwargs.get('lists', None)
        self.pages = kwargs.get('pages', None)
        self.sites = kwargs.get('sites', None)
        self.onenote = kwargs.get('onenote', None)


class MicrosoftGraphSiteCollection(msrest.serialization.Model):
    """siteCollection.

    :param data_location_code: The geographic region code for where this site collection resides.
     Read-only.
    :type data_location_code: str
    :param hostname: The hostname for the site collection. Read-only.
    :type hostname: str
    :param root: Any object.
    :type root: object
    """

    _attribute_map = {
        'data_location_code': {'key': 'dataLocationCode', 'type': 'str'},
        'hostname': {'key': 'hostname', 'type': 'str'},
        'root': {'key': 'root', 'type': 'object'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphSiteCollection, self).__init__(**kwargs)
        self.data_location_code = kwargs.get('data_location_code', None)
        self.hostname = kwargs.get('hostname', None)
        self.root = kwargs.get('root', None)


class MicrosoftGraphSitePage(MicrosoftGraphBaseItem):
    """sitePage.

    :param id: Read-only.
    :type id: str
    :param created_by: identitySet.
    :type created_by: ~users_user.models.MicrosoftGraphIdentitySet
    :param created_date_time: Date and time of item creation. Read-only.
    :type created_date_time: ~datetime.datetime
    :param description: Provides a user-visible description of the item. Optional.
    :type description: str
    :param e_tag: ETag for the item. Read-only.
    :type e_tag: str
    :param last_modified_by: identitySet.
    :type last_modified_by: ~users_user.models.MicrosoftGraphIdentitySet
    :param last_modified_date_time: Date and time the item was last modified. Read-only.
    :type last_modified_date_time: ~datetime.datetime
    :param name: The name of the item. Read-write.
    :type name: str
    :param parent_reference: itemReference.
    :type parent_reference: ~users_user.models.MicrosoftGraphItemReference
    :param web_url: URL that displays the resource in the browser. Read-only.
    :type web_url: str
    :param created_by_user: Represents an Azure Active Directory user object.
    :type created_by_user: ~users_user.models.MicrosoftGraphUser
    :param last_modified_by_user: Represents an Azure Active Directory user object.
    :type last_modified_by_user: ~users_user.models.MicrosoftGraphUser
    :param title:
    :type title: str
    :param content_type: contentTypeInfo.
    :type content_type: ~users_user.models.MicrosoftGraphContentTypeInfo
    :param page_layout_type:
    :type page_layout_type: str
    :param web_parts:
    :type web_parts: list[~users_user.models.MicrosoftGraphWebPart]
    :param publishing_state: publicationFacet.
    :type publishing_state: ~users_user.models.MicrosoftGraphPublicationFacet
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'created_by': {'key': 'createdBy', 'type': 'MicrosoftGraphIdentitySet'},
        'created_date_time': {'key': 'createdDateTime', 'type': 'iso-8601'},
        'description': {'key': 'description', 'type': 'str'},
        'e_tag': {'key': 'eTag', 'type': 'str'},
        'last_modified_by': {'key': 'lastModifiedBy', 'type': 'MicrosoftGraphIdentitySet'},
        'last_modified_date_time': {'key': 'lastModifiedDateTime', 'type': 'iso-8601'},
        'name': {'key': 'name', 'type': 'str'},
        'parent_reference': {'key': 'parentReference', 'type': 'MicrosoftGraphItemReference'},
        'web_url': {'key': 'webUrl', 'type': 'str'},
        'created_by_user': {'key': 'createdByUser', 'type': 'MicrosoftGraphUser'},
        'last_modified_by_user': {'key': 'lastModifiedByUser', 'type': 'MicrosoftGraphUser'},
        'title': {'key': 'title', 'type': 'str'},
        'content_type': {'key': 'contentType', 'type': 'MicrosoftGraphContentTypeInfo'},
        'page_layout_type': {'key': 'pageLayoutType', 'type': 'str'},
        'web_parts': {'key': 'webParts', 'type': '[MicrosoftGraphWebPart]'},
        'publishing_state': {'key': 'publishingState', 'type': 'MicrosoftGraphPublicationFacet'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphSitePage, self).__init__(**kwargs)
        self.title = kwargs.get('title', None)
        self.content_type = kwargs.get('content_type', None)
        self.page_layout_type = kwargs.get('page_layout_type', None)
        self.web_parts = kwargs.get('web_parts', None)
        self.publishing_state = kwargs.get('publishing_state', None)


class MicrosoftGraphSizeRange(msrest.serialization.Model):
    """sizeRange.

    :param minimum_size: The minimum size (in kilobytes) that an incoming message must have in
     order for a condition or exception to apply.
    :type minimum_size: int
    :param maximum_size: The maximum size (in kilobytes) that an incoming message must have in
     order for a condition or exception to apply.
    :type maximum_size: int
    """

    _validation = {
        'minimum_size': {'maximum': 2147483647, 'minimum': -2147483648},
        'maximum_size': {'maximum': 2147483647, 'minimum': -2147483648},
    }

    _attribute_map = {
        'minimum_size': {'key': 'minimumSize', 'type': 'int'},
        'maximum_size': {'key': 'maximumSize', 'type': 'int'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphSizeRange, self).__init__(**kwargs)
        self.minimum_size = kwargs.get('minimum_size', None)
        self.maximum_size = kwargs.get('maximum_size', None)


class MicrosoftGraphSkillProficiency(MicrosoftGraphItemFacet):
    """skillProficiency.

    :param id: Read-only.
    :type id: str
    :param allowed_audiences: allowedAudiences. Possible values include: "me", "family",
     "contacts", "groupMembers", "organization", "federatedOrganizations", "everyone",
     "unknownFutureValue".
    :type allowed_audiences: str or ~users_user.models.MicrosoftGraphAllowedAudiences
    :param inference: inferenceData.
    :type inference: ~users_user.models.MicrosoftGraphInferenceData
    :param created_date_time:
    :type created_date_time: ~datetime.datetime
    :param created_by: identitySet.
    :type created_by: ~users_user.models.MicrosoftGraphIdentitySet
    :param last_modified_date_time:
    :type last_modified_date_time: ~datetime.datetime
    :param last_modified_by: identitySet.
    :type last_modified_by: ~users_user.models.MicrosoftGraphIdentitySet
    :param categories:
    :type categories: list[str]
    :param display_name:
    :type display_name: str
    :param proficiency: skillProficiencyLevel. Possible values include: "elementary",
     "limitedWorking", "generalProfessional", "advancedProfessional", "expert",
     "unknownFutureValue".
    :type proficiency: str or ~users_user.models.MicrosoftGraphSkillProficiencyLevel
    :param web_url:
    :type web_url: str
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'allowed_audiences': {'key': 'allowedAudiences', 'type': 'str'},
        'inference': {'key': 'inference', 'type': 'MicrosoftGraphInferenceData'},
        'created_date_time': {'key': 'createdDateTime', 'type': 'iso-8601'},
        'created_by': {'key': 'createdBy', 'type': 'MicrosoftGraphIdentitySet'},
        'last_modified_date_time': {'key': 'lastModifiedDateTime', 'type': 'iso-8601'},
        'last_modified_by': {'key': 'lastModifiedBy', 'type': 'MicrosoftGraphIdentitySet'},
        'categories': {'key': 'categories', 'type': '[str]'},
        'display_name': {'key': 'displayName', 'type': 'str'},
        'proficiency': {'key': 'proficiency', 'type': 'str'},
        'web_url': {'key': 'webUrl', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphSkillProficiency, self).__init__(**kwargs)
        self.categories = kwargs.get('categories', None)
        self.display_name = kwargs.get('display_name', None)
        self.proficiency = kwargs.get('proficiency', None)
        self.web_url = kwargs.get('web_url', None)


class MicrosoftGraphSpecialFolder(msrest.serialization.Model):
    """specialFolder.

    :param name: The unique identifier for this item in the /drive/special collection.
    :type name: str
    """

    _attribute_map = {
        'name': {'key': 'name', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphSpecialFolder, self).__init__(**kwargs)
        self.name = kwargs.get('name', None)


class MicrosoftGraphStoragePlanInformation(msrest.serialization.Model):
    """storagePlanInformation.

    :param upgrade_available:
    :type upgrade_available: bool
    """

    _attribute_map = {
        'upgrade_available': {'key': 'upgradeAvailable', 'type': 'bool'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphStoragePlanInformation, self).__init__(**kwargs)
        self.upgrade_available = kwargs.get('upgrade_available', None)


class MicrosoftGraphSubscription(MicrosoftGraphEntity):
    """subscription.

    :param id: Read-only.
    :type id: str
    :param resource: Required. Specifies the resource that will be monitored for changes. Do not
     include the base URL (https://graph.microsoft.com/v1.0/). See the possible resource path values
     for each supported resource.
    :type resource: str
    :param change_type: Required. Indicates the type of change in the subscribed resource that will
     raise a notification. The supported values are: created, updated, deleted. Multiple values can
     be combined using a comma-separated list.Note: Drive root item and list notifications support
     only the updated changeType. User and group notifications support updated and deleted
     changeType.
    :type change_type: str
    :param client_state: Optional. Specifies the value of the clientState property sent by the
     service in each notification. The maximum length is 128 characters. The client can check that
     the notification came from the service by comparing the value of the clientState property sent
     with the subscription with the value of the clientState property received with each
     notification.
    :type client_state: str
    :param notification_url: Required. The URL of the endpoint that will receive the notifications.
     This URL must make use of the HTTPS protocol.
    :type notification_url: str
    :param expiration_date_time: Required. Specifies the date and time when the webhook
     subscription expires. The time is in UTC, and can be an amount of time from subscription
     creation that varies for the resource subscribed to.  See the table below for maximum supported
     subscription length of time.
    :type expiration_date_time: ~datetime.datetime
    :param application_id: Identifier of the application used to create the subscription. Read-
     only.
    :type application_id: str
    :param creator_id: Identifier of the user or service principal that created the subscription.
     If the app used delegated permissions to create the subscription, this field contains the id of
     the signed-in user the app called on behalf of. If the app used application permissions, this
     field contains the id of the service principal corresponding to the app. Read-only.
    :type creator_id: str
    :param include_properties:
    :type include_properties: bool
    :param include_resource_data:
    :type include_resource_data: bool
    :param lifecycle_notification_url:
    :type lifecycle_notification_url: str
    :param encryption_certificate:
    :type encryption_certificate: str
    :param encryption_certificate_id:
    :type encryption_certificate_id: str
    :param latest_supported_tls_version: Specifies the latest version of Transport Layer Security
     (TLS) that the notification endpoint, specified by notificationUrl, supports. The possible
     values are: v1_0, v1_1, v1_2, v1_3. For subscribers whose notification endpoint supports a
     version lower than the currently recommended version (TLS 1.2), specifying this property by a
     set timeline allows them to temporarily use their deprecated version of TLS before completing
     their upgrade to TLS 1.2. For these subscribers, not setting this property per the timeline
     would result in subscription operations failing. For subscribers whose notification endpoint
     already supports TLS 1.2, setting this property is optional. In such cases, Microsoft Graph
     defaults the property to v1_2.
    :type latest_supported_tls_version: str
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'resource': {'key': 'resource', 'type': 'str'},
        'change_type': {'key': 'changeType', 'type': 'str'},
        'client_state': {'key': 'clientState', 'type': 'str'},
        'notification_url': {'key': 'notificationUrl', 'type': 'str'},
        'expiration_date_time': {'key': 'expirationDateTime', 'type': 'iso-8601'},
        'application_id': {'key': 'applicationId', 'type': 'str'},
        'creator_id': {'key': 'creatorId', 'type': 'str'},
        'include_properties': {'key': 'includeProperties', 'type': 'bool'},
        'include_resource_data': {'key': 'includeResourceData', 'type': 'bool'},
        'lifecycle_notification_url': {'key': 'lifecycleNotificationUrl', 'type': 'str'},
        'encryption_certificate': {'key': 'encryptionCertificate', 'type': 'str'},
        'encryption_certificate_id': {'key': 'encryptionCertificateId', 'type': 'str'},
        'latest_supported_tls_version': {'key': 'latestSupportedTlsVersion', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphSubscription, self).__init__(**kwargs)
        self.resource = kwargs.get('resource', None)
        self.change_type = kwargs.get('change_type', None)
        self.client_state = kwargs.get('client_state', None)
        self.notification_url = kwargs.get('notification_url', None)
        self.expiration_date_time = kwargs.get('expiration_date_time', None)
        self.application_id = kwargs.get('application_id', None)
        self.creator_id = kwargs.get('creator_id', None)
        self.include_properties = kwargs.get('include_properties', None)
        self.include_resource_data = kwargs.get('include_resource_data', None)
        self.lifecycle_notification_url = kwargs.get('lifecycle_notification_url', None)
        self.encryption_certificate = kwargs.get('encryption_certificate', None)
        self.encryption_certificate_id = kwargs.get('encryption_certificate_id', None)
        self.latest_supported_tls_version = kwargs.get('latest_supported_tls_version', None)


class MicrosoftGraphSwapShiftsChangeRequest(MicrosoftGraphOfferShiftRequest):
    """swapShiftsChangeRequest.

    :param id: Read-only.
    :type id: str
    :param created_date_time:
    :type created_date_time: ~datetime.datetime
    :param last_modified_date_time:
    :type last_modified_date_time: ~datetime.datetime
    :param last_modified_by: identitySet.
    :type last_modified_by: ~users_user.models.MicrosoftGraphIdentitySet
    :param assigned_to: scheduleChangeRequestActor. Possible values include: "sender", "recipient",
     "manager", "system", "unknownFutureValue".
    :type assigned_to: str or ~users_user.models.MicrosoftGraphScheduleChangeRequestActor
    :param state: scheduleChangeState. Possible values include: "pending", "approved", "declined",
     "unknownFutureValue".
    :type state: str or ~users_user.models.MicrosoftGraphScheduleChangeState
    :param sender_message:
    :type sender_message: str
    :param sender_date_time:
    :type sender_date_time: ~datetime.datetime
    :param manager_action_message:
    :type manager_action_message: str
    :param manager_action_date_time:
    :type manager_action_date_time: ~datetime.datetime
    :param sender_user_id:
    :type sender_user_id: str
    :param manager_user_id:
    :type manager_user_id: str
    :param recipient_action_message:
    :type recipient_action_message: str
    :param recipient_action_date_time:
    :type recipient_action_date_time: ~datetime.datetime
    :param sender_shift_id:
    :type sender_shift_id: str
    :param recipient_user_id:
    :type recipient_user_id: str
    :param recipient_shift_id:
    :type recipient_shift_id: str
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'created_date_time': {'key': 'createdDateTime', 'type': 'iso-8601'},
        'last_modified_date_time': {'key': 'lastModifiedDateTime', 'type': 'iso-8601'},
        'last_modified_by': {'key': 'lastModifiedBy', 'type': 'MicrosoftGraphIdentitySet'},
        'assigned_to': {'key': 'assignedTo', 'type': 'str'},
        'state': {'key': 'state', 'type': 'str'},
        'sender_message': {'key': 'senderMessage', 'type': 'str'},
        'sender_date_time': {'key': 'senderDateTime', 'type': 'iso-8601'},
        'manager_action_message': {'key': 'managerActionMessage', 'type': 'str'},
        'manager_action_date_time': {'key': 'managerActionDateTime', 'type': 'iso-8601'},
        'sender_user_id': {'key': 'senderUserId', 'type': 'str'},
        'manager_user_id': {'key': 'managerUserId', 'type': 'str'},
        'recipient_action_message': {'key': 'recipientActionMessage', 'type': 'str'},
        'recipient_action_date_time': {'key': 'recipientActionDateTime', 'type': 'iso-8601'},
        'sender_shift_id': {'key': 'senderShiftId', 'type': 'str'},
        'recipient_user_id': {'key': 'recipientUserId', 'type': 'str'},
        'recipient_shift_id': {'key': 'recipientShiftId', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphSwapShiftsChangeRequest, self).__init__(**kwargs)
        self.recipient_shift_id = kwargs.get('recipient_shift_id', None)


class MicrosoftGraphTargetPolicyEndpoints(msrest.serialization.Model):
    """targetPolicyEndpoints.

    :param platform_types:
    :type platform_types: list[str]
    """

    _attribute_map = {
        'platform_types': {'key': 'platformTypes', 'type': '[str]'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphTargetPolicyEndpoints, self).__init__(**kwargs)
        self.platform_types = kwargs.get('platform_types', None)


class MicrosoftGraphTeam(MicrosoftGraphEntity):
    """team.

    :param id: Read-only.
    :type id: str
    :param display_name:
    :type display_name: str
    :param description:
    :type description: str
    :param internal_id: A unique ID for the team that has been used in a few places such as the
     audit log/Office 365 Management Activity API.
    :type internal_id: str
    :param classification:
    :type classification: str
    :param specialization: teamSpecialization. Possible values include: "none",
     "educationStandard", "educationClass", "educationProfessionalLearningCommunity",
     "educationStaff", "healthcareStandard", "healthcareCareCoordination", "unknownFutureValue".
    :type specialization: str or ~users_user.models.MicrosoftGraphTeamSpecialization
    :param visibility: teamVisibilityType. Possible values include: "private", "public",
     "hiddenMembership", "unknownFutureValue".
    :type visibility: str or ~users_user.models.MicrosoftGraphTeamVisibilityType
    :param web_url: A hyperlink that will go to the team in the Microsoft Teams client. This is the
     URL that you get when you right-click a team in the Microsoft Teams client and select Get link
     to team. This URL should be treated as an opaque blob, and not parsed.
    :type web_url: str
    :param member_settings: teamMemberSettings.
    :type member_settings: ~users_user.models.MicrosoftGraphTeamMemberSettings
    :param guest_settings: teamGuestSettings.
    :type guest_settings: ~users_user.models.MicrosoftGraphTeamGuestSettings
    :param messaging_settings: teamMessagingSettings.
    :type messaging_settings: ~users_user.models.MicrosoftGraphTeamMessagingSettings
    :param fun_settings: teamFunSettings.
    :type fun_settings: ~users_user.models.MicrosoftGraphTeamFunSettings
    :param discovery_settings: teamDiscoverySettings.
    :type discovery_settings: ~users_user.models.MicrosoftGraphTeamDiscoverySettings
    :param is_archived: Whether this team is in read-only mode.
    :type is_archived: bool
    :param schedule: schedule.
    :type schedule: ~users_user.models.MicrosoftGraphSchedule
    :param group: Represents an Azure Active Directory object. The directoryObject type is the base
     type for many other directory entity types.
    :type group: ~users_user.models.MicrosoftGraphGroup
    :param template: entity.
    :type template: ~users_user.models.MicrosoftGraphEntity
    :param photo: profilePhoto.
    :type photo: ~users_user.models.MicrosoftGraphProfilePhoto
    :param owners:
    :type owners: list[~users_user.models.MicrosoftGraphUser]
    :param channels: The collection of channels & messages associated with the team.
    :type channels: list[~users_user.models.MicrosoftGraphChannel]
    :param primary_channel: channel.
    :type primary_channel: ~users_user.models.MicrosoftGraphChannel
    :param apps:
    :type apps: list[~users_user.models.MicrosoftGraphTeamsCatalogApp]
    :param installed_apps: The apps installed in this team.
    :type installed_apps: list[~users_user.models.MicrosoftGraphTeamsAppInstallation]
    :param operations:
    :type operations: list[~users_user.models.MicrosoftGraphTeamsAsyncOperation]
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'display_name': {'key': 'displayName', 'type': 'str'},
        'description': {'key': 'description', 'type': 'str'},
        'internal_id': {'key': 'internalId', 'type': 'str'},
        'classification': {'key': 'classification', 'type': 'str'},
        'specialization': {'key': 'specialization', 'type': 'str'},
        'visibility': {'key': 'visibility', 'type': 'str'},
        'web_url': {'key': 'webUrl', 'type': 'str'},
        'member_settings': {'key': 'memberSettings', 'type': 'MicrosoftGraphTeamMemberSettings'},
        'guest_settings': {'key': 'guestSettings', 'type': 'MicrosoftGraphTeamGuestSettings'},
        'messaging_settings': {'key': 'messagingSettings', 'type': 'MicrosoftGraphTeamMessagingSettings'},
        'fun_settings': {'key': 'funSettings', 'type': 'MicrosoftGraphTeamFunSettings'},
        'discovery_settings': {'key': 'discoverySettings', 'type': 'MicrosoftGraphTeamDiscoverySettings'},
        'is_archived': {'key': 'isArchived', 'type': 'bool'},
        'schedule': {'key': 'schedule', 'type': 'MicrosoftGraphSchedule'},
        'group': {'key': 'group', 'type': 'MicrosoftGraphGroup'},
        'template': {'key': 'template', 'type': 'MicrosoftGraphEntity'},
        'photo': {'key': 'photo', 'type': 'MicrosoftGraphProfilePhoto'},
        'owners': {'key': 'owners', 'type': '[MicrosoftGraphUser]'},
        'channels': {'key': 'channels', 'type': '[MicrosoftGraphChannel]'},
        'primary_channel': {'key': 'primaryChannel', 'type': 'MicrosoftGraphChannel'},
        'apps': {'key': 'apps', 'type': '[MicrosoftGraphTeamsCatalogApp]'},
        'installed_apps': {'key': 'installedApps', 'type': '[MicrosoftGraphTeamsAppInstallation]'},
        'operations': {'key': 'operations', 'type': '[MicrosoftGraphTeamsAsyncOperation]'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphTeam, self).__init__(**kwargs)
        self.display_name = kwargs.get('display_name', None)
        self.description = kwargs.get('description', None)
        self.internal_id = kwargs.get('internal_id', None)
        self.classification = kwargs.get('classification', None)
        self.specialization = kwargs.get('specialization', None)
        self.visibility = kwargs.get('visibility', None)
        self.web_url = kwargs.get('web_url', None)
        self.member_settings = kwargs.get('member_settings', None)
        self.guest_settings = kwargs.get('guest_settings', None)
        self.messaging_settings = kwargs.get('messaging_settings', None)
        self.fun_settings = kwargs.get('fun_settings', None)
        self.discovery_settings = kwargs.get('discovery_settings', None)
        self.is_archived = kwargs.get('is_archived', None)
        self.schedule = kwargs.get('schedule', None)
        self.group = kwargs.get('group', None)
        self.template = kwargs.get('template', None)
        self.photo = kwargs.get('photo', None)
        self.owners = kwargs.get('owners', None)
        self.channels = kwargs.get('channels', None)
        self.primary_channel = kwargs.get('primary_channel', None)
        self.apps = kwargs.get('apps', None)
        self.installed_apps = kwargs.get('installed_apps', None)
        self.operations = kwargs.get('operations', None)


class MicrosoftGraphTeamDiscoverySettings(msrest.serialization.Model):
    """teamDiscoverySettings.

    :param show_in_teams_search_and_suggestions:
    :type show_in_teams_search_and_suggestions: bool
    """

    _attribute_map = {
        'show_in_teams_search_and_suggestions': {'key': 'showInTeamsSearchAndSuggestions', 'type': 'bool'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphTeamDiscoverySettings, self).__init__(**kwargs)
        self.show_in_teams_search_and_suggestions = kwargs.get('show_in_teams_search_and_suggestions', None)


class MicrosoftGraphTeamFunSettings(msrest.serialization.Model):
    """teamFunSettings.

    :param allow_giphy: If set to true, enables Giphy use.
    :type allow_giphy: bool
    :param giphy_content_rating: giphyRatingType. Possible values include: "strict", "moderate",
     "unknownFutureValue".
    :type giphy_content_rating: str or ~users_user.models.MicrosoftGraphGiphyRatingType
    :param allow_stickers_and_memes: If set to true, enables users to include stickers and memes.
    :type allow_stickers_and_memes: bool
    :param allow_custom_memes: If set to true, enables users to include custom memes.
    :type allow_custom_memes: bool
    """

    _attribute_map = {
        'allow_giphy': {'key': 'allowGiphy', 'type': 'bool'},
        'giphy_content_rating': {'key': 'giphyContentRating', 'type': 'str'},
        'allow_stickers_and_memes': {'key': 'allowStickersAndMemes', 'type': 'bool'},
        'allow_custom_memes': {'key': 'allowCustomMemes', 'type': 'bool'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphTeamFunSettings, self).__init__(**kwargs)
        self.allow_giphy = kwargs.get('allow_giphy', None)
        self.giphy_content_rating = kwargs.get('giphy_content_rating', None)
        self.allow_stickers_and_memes = kwargs.get('allow_stickers_and_memes', None)
        self.allow_custom_memes = kwargs.get('allow_custom_memes', None)


class MicrosoftGraphTeamGuestSettings(msrest.serialization.Model):
    """teamGuestSettings.

    :param allow_create_update_channels: If set to true, guests can add and update channels.
    :type allow_create_update_channels: bool
    :param allow_delete_channels: If set to true, guests can delete channels.
    :type allow_delete_channels: bool
    """

    _attribute_map = {
        'allow_create_update_channels': {'key': 'allowCreateUpdateChannels', 'type': 'bool'},
        'allow_delete_channels': {'key': 'allowDeleteChannels', 'type': 'bool'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphTeamGuestSettings, self).__init__(**kwargs)
        self.allow_create_update_channels = kwargs.get('allow_create_update_channels', None)
        self.allow_delete_channels = kwargs.get('allow_delete_channels', None)


class MicrosoftGraphTeamMemberSettings(msrest.serialization.Model):
    """teamMemberSettings.

    :param allow_create_update_channels: If set to true, members can add and update channels.
    :type allow_create_update_channels: bool
    :param allow_create_private_channels: If set to true, members can add and update private
     channels.
    :type allow_create_private_channels: bool
    :param allow_delete_channels: If set to true, members can delete channels.
    :type allow_delete_channels: bool
    :param allow_add_remove_apps: If set to true, members can add and remove apps.
    :type allow_add_remove_apps: bool
    :param allow_create_update_remove_tabs: If set to true, members can add, update, and remove
     tabs.
    :type allow_create_update_remove_tabs: bool
    :param allow_create_update_remove_connectors: If set to true, members can add, update, and
     remove connectors.
    :type allow_create_update_remove_connectors: bool
    """

    _attribute_map = {
        'allow_create_update_channels': {'key': 'allowCreateUpdateChannels', 'type': 'bool'},
        'allow_create_private_channels': {'key': 'allowCreatePrivateChannels', 'type': 'bool'},
        'allow_delete_channels': {'key': 'allowDeleteChannels', 'type': 'bool'},
        'allow_add_remove_apps': {'key': 'allowAddRemoveApps', 'type': 'bool'},
        'allow_create_update_remove_tabs': {'key': 'allowCreateUpdateRemoveTabs', 'type': 'bool'},
        'allow_create_update_remove_connectors': {'key': 'allowCreateUpdateRemoveConnectors', 'type': 'bool'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphTeamMemberSettings, self).__init__(**kwargs)
        self.allow_create_update_channels = kwargs.get('allow_create_update_channels', None)
        self.allow_create_private_channels = kwargs.get('allow_create_private_channels', None)
        self.allow_delete_channels = kwargs.get('allow_delete_channels', None)
        self.allow_add_remove_apps = kwargs.get('allow_add_remove_apps', None)
        self.allow_create_update_remove_tabs = kwargs.get('allow_create_update_remove_tabs', None)
        self.allow_create_update_remove_connectors = kwargs.get('allow_create_update_remove_connectors', None)


class MicrosoftGraphTeamMessagingSettings(msrest.serialization.Model):
    """teamMessagingSettings.

    :param allow_user_edit_messages: If set to true, users can edit their messages.
    :type allow_user_edit_messages: bool
    :param allow_user_delete_messages: If set to true, users can delete their messages.
    :type allow_user_delete_messages: bool
    :param allow_owner_delete_messages: If set to true, owners can delete any message.
    :type allow_owner_delete_messages: bool
    :param allow_team_mentions: If set to true, @team mentions are allowed.
    :type allow_team_mentions: bool
    :param allow_channel_mentions: If set to true, @channel mentions are allowed.
    :type allow_channel_mentions: bool
    """

    _attribute_map = {
        'allow_user_edit_messages': {'key': 'allowUserEditMessages', 'type': 'bool'},
        'allow_user_delete_messages': {'key': 'allowUserDeleteMessages', 'type': 'bool'},
        'allow_owner_delete_messages': {'key': 'allowOwnerDeleteMessages', 'type': 'bool'},
        'allow_team_mentions': {'key': 'allowTeamMentions', 'type': 'bool'},
        'allow_channel_mentions': {'key': 'allowChannelMentions', 'type': 'bool'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphTeamMessagingSettings, self).__init__(**kwargs)
        self.allow_user_edit_messages = kwargs.get('allow_user_edit_messages', None)
        self.allow_user_delete_messages = kwargs.get('allow_user_delete_messages', None)
        self.allow_owner_delete_messages = kwargs.get('allow_owner_delete_messages', None)
        self.allow_team_mentions = kwargs.get('allow_team_mentions', None)
        self.allow_channel_mentions = kwargs.get('allow_channel_mentions', None)


class MicrosoftGraphTeamsApp(MicrosoftGraphEntity):
    """teamsApp.

    :param id: Read-only.
    :type id: str
    :param external_id: The ID of the catalog provided by the app developer in the Microsoft Teams
     zip app package.
    :type external_id: str
    :param name:
    :type name: str
    :param display_name: The name of the catalog app provided by the app developer in the Microsoft
     Teams zip app package.
    :type display_name: str
    :param distribution_method: teamsAppDistributionMethod. Possible values include: "store",
     "organization", "sideloaded", "unknownFutureValue".
    :type distribution_method: str or ~users_user.models.MicrosoftGraphTeamsAppDistributionMethod
    :param app_definitions: The details for each version of the app.
    :type app_definitions: list[~users_user.models.MicrosoftGraphTeamsAppDefinition]
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'external_id': {'key': 'externalId', 'type': 'str'},
        'name': {'key': 'name', 'type': 'str'},
        'display_name': {'key': 'displayName', 'type': 'str'},
        'distribution_method': {'key': 'distributionMethod', 'type': 'str'},
        'app_definitions': {'key': 'appDefinitions', 'type': '[MicrosoftGraphTeamsAppDefinition]'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphTeamsApp, self).__init__(**kwargs)
        self.external_id = kwargs.get('external_id', None)
        self.name = kwargs.get('name', None)
        self.display_name = kwargs.get('display_name', None)
        self.distribution_method = kwargs.get('distribution_method', None)
        self.app_definitions = kwargs.get('app_definitions', None)


class MicrosoftGraphTeamsAppDefinition(MicrosoftGraphEntity):
    """teamsAppDefinition.

    :param id: Read-only.
    :type id: str
    :param teams_app_id: The id from the Teams App manifest.
    :type teams_app_id: str
    :param display_name: The name of the app provided by the app developer.
    :type display_name: str
    :param version: The version number of the application.
    :type version: str
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'teams_app_id': {'key': 'teamsAppId', 'type': 'str'},
        'display_name': {'key': 'displayName', 'type': 'str'},
        'version': {'key': 'version', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphTeamsAppDefinition, self).__init__(**kwargs)
        self.teams_app_id = kwargs.get('teams_app_id', None)
        self.display_name = kwargs.get('display_name', None)
        self.version = kwargs.get('version', None)


class MicrosoftGraphTeamsAppInstallation(MicrosoftGraphEntity):
    """teamsAppInstallation.

    :param id: Read-only.
    :type id: str
    :param teams_app: teamsApp.
    :type teams_app: ~users_user.models.MicrosoftGraphTeamsApp
    :param teams_app_definition: teamsAppDefinition.
    :type teams_app_definition: ~users_user.models.MicrosoftGraphTeamsAppDefinition
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'teams_app': {'key': 'teamsApp', 'type': 'MicrosoftGraphTeamsApp'},
        'teams_app_definition': {'key': 'teamsAppDefinition', 'type': 'MicrosoftGraphTeamsAppDefinition'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphTeamsAppInstallation, self).__init__(**kwargs)
        self.teams_app = kwargs.get('teams_app', None)
        self.teams_app_definition = kwargs.get('teams_app_definition', None)


class MicrosoftGraphTeamsAsyncOperation(MicrosoftGraphEntity):
    """teamsAsyncOperation.

    :param id: Read-only.
    :type id: str
    :param operation_type: teamsAsyncOperationType. Possible values include: "invalid",
     "cloneTeam", "archiveTeam", "unarchiveTeam", "createTeam", "unknownFutureValue".
    :type operation_type: str or ~users_user.models.MicrosoftGraphTeamsAsyncOperationType
    :param created_date_time:
    :type created_date_time: ~datetime.datetime
    :param status: teamsAsyncOperationStatus. Possible values include: "invalid", "notStarted",
     "inProgress", "succeeded", "failed", "unknownFutureValue".
    :type status: str or ~users_user.models.MicrosoftGraphTeamsAsyncOperationStatus
    :param last_action_date_time:
    :type last_action_date_time: ~datetime.datetime
    :param attempts_count:
    :type attempts_count: int
    :param target_resource_id:
    :type target_resource_id: str
    :param target_resource_location:
    :type target_resource_location: str
    :param error: operationError.
    :type error: ~users_user.models.MicrosoftGraphOperationError
    """

    _validation = {
        'attempts_count': {'maximum': 2147483647, 'minimum': -2147483648},
    }

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'operation_type': {'key': 'operationType', 'type': 'str'},
        'created_date_time': {'key': 'createdDateTime', 'type': 'iso-8601'},
        'status': {'key': 'status', 'type': 'str'},
        'last_action_date_time': {'key': 'lastActionDateTime', 'type': 'iso-8601'},
        'attempts_count': {'key': 'attemptsCount', 'type': 'int'},
        'target_resource_id': {'key': 'targetResourceId', 'type': 'str'},
        'target_resource_location': {'key': 'targetResourceLocation', 'type': 'str'},
        'error': {'key': 'error', 'type': 'MicrosoftGraphOperationError'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphTeamsAsyncOperation, self).__init__(**kwargs)
        self.operation_type = kwargs.get('operation_type', None)
        self.created_date_time = kwargs.get('created_date_time', None)
        self.status = kwargs.get('status', None)
        self.last_action_date_time = kwargs.get('last_action_date_time', None)
        self.attempts_count = kwargs.get('attempts_count', None)
        self.target_resource_id = kwargs.get('target_resource_id', None)
        self.target_resource_location = kwargs.get('target_resource_location', None)
        self.error = kwargs.get('error', None)


class MicrosoftGraphTeamsCatalogApp(MicrosoftGraphEntity):
    """teamsCatalogApp.

    :param id: Read-only.
    :type id: str
    :param external_id:
    :type external_id: str
    :param name:
    :type name: str
    :param distribution_method: teamsAppDistributionMethod. Possible values include: "store",
     "organization", "sideloaded", "unknownFutureValue".
    :type distribution_method: str or ~users_user.models.MicrosoftGraphTeamsAppDistributionMethod
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'external_id': {'key': 'externalId', 'type': 'str'},
        'name': {'key': 'name', 'type': 'str'},
        'distribution_method': {'key': 'distributionMethod', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphTeamsCatalogApp, self).__init__(**kwargs)
        self.external_id = kwargs.get('external_id', None)
        self.name = kwargs.get('name', None)
        self.distribution_method = kwargs.get('distribution_method', None)


class MicrosoftGraphTeamsTab(MicrosoftGraphEntity):
    """teamsTab.

    :param id: Read-only.
    :type id: str
    :param name:
    :type name: str
    :param display_name: Name of the tab.
    :type display_name: str
    :param teams_app_id:
    :type teams_app_id: str
    :param sort_order_index:
    :type sort_order_index: str
    :param message_id:
    :type message_id: str
    :param web_url: Deep link url of the tab instance. Read only.
    :type web_url: str
    :param configuration: teamsTabConfiguration.
    :type configuration: ~users_user.models.MicrosoftGraphTeamsTabConfiguration
    :param teams_app: teamsApp.
    :type teams_app: ~users_user.models.MicrosoftGraphTeamsApp
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'name': {'key': 'name', 'type': 'str'},
        'display_name': {'key': 'displayName', 'type': 'str'},
        'teams_app_id': {'key': 'teamsAppId', 'type': 'str'},
        'sort_order_index': {'key': 'sortOrderIndex', 'type': 'str'},
        'message_id': {'key': 'messageId', 'type': 'str'},
        'web_url': {'key': 'webUrl', 'type': 'str'},
        'configuration': {'key': 'configuration', 'type': 'MicrosoftGraphTeamsTabConfiguration'},
        'teams_app': {'key': 'teamsApp', 'type': 'MicrosoftGraphTeamsApp'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphTeamsTab, self).__init__(**kwargs)
        self.name = kwargs.get('name', None)
        self.display_name = kwargs.get('display_name', None)
        self.teams_app_id = kwargs.get('teams_app_id', None)
        self.sort_order_index = kwargs.get('sort_order_index', None)
        self.message_id = kwargs.get('message_id', None)
        self.web_url = kwargs.get('web_url', None)
        self.configuration = kwargs.get('configuration', None)
        self.teams_app = kwargs.get('teams_app', None)


class MicrosoftGraphTeamsTabConfiguration(msrest.serialization.Model):
    """teamsTabConfiguration.

    :param entity_id: Identifier for the entity hosted by the tab provider.
    :type entity_id: str
    :param content_url: Url used for rendering tab contents in Teams. Required.
    :type content_url: str
    :param remove_url: Url called by Teams client when a Tab is removed using the Teams Client.
    :type remove_url: str
    :param website_url: Url for showing tab contents outside of Teams.
    :type website_url: str
    """

    _attribute_map = {
        'entity_id': {'key': 'entityId', 'type': 'str'},
        'content_url': {'key': 'contentUrl', 'type': 'str'},
        'remove_url': {'key': 'removeUrl', 'type': 'str'},
        'website_url': {'key': 'websiteUrl', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphTeamsTabConfiguration, self).__init__(**kwargs)
        self.entity_id = kwargs.get('entity_id', None)
        self.content_url = kwargs.get('content_url', None)
        self.remove_url = kwargs.get('remove_url', None)
        self.website_url = kwargs.get('website_url', None)


class MicrosoftGraphTeamsTemplate(MicrosoftGraphEntity):
    """teamsTemplate.

    :param id: Read-only.
    :type id: str
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphTeamsTemplate, self).__init__(**kwargs)


class MicrosoftGraphTextColumn(msrest.serialization.Model):
    """textColumn.

    :param allow_multiple_lines: Whether to allow multiple lines of text.
    :type allow_multiple_lines: bool
    :param append_changes_to_existing_text: Whether updates to this column should replace existing
     text, or append to it.
    :type append_changes_to_existing_text: bool
    :param lines_for_editing: The size of the text box.
    :type lines_for_editing: int
    :param max_length: The maximum number of characters for the value.
    :type max_length: int
    :param text_type: The type of text being stored. Must be one of plain or richText.
    :type text_type: str
    """

    _validation = {
        'lines_for_editing': {'maximum': 2147483647, 'minimum': -2147483648},
        'max_length': {'maximum': 2147483647, 'minimum': -2147483648},
    }

    _attribute_map = {
        'allow_multiple_lines': {'key': 'allowMultipleLines', 'type': 'bool'},
        'append_changes_to_existing_text': {'key': 'appendChangesToExistingText', 'type': 'bool'},
        'lines_for_editing': {'key': 'linesForEditing', 'type': 'int'},
        'max_length': {'key': 'maxLength', 'type': 'int'},
        'text_type': {'key': 'textType', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphTextColumn, self).__init__(**kwargs)
        self.allow_multiple_lines = kwargs.get('allow_multiple_lines', None)
        self.append_changes_to_existing_text = kwargs.get('append_changes_to_existing_text', None)
        self.lines_for_editing = kwargs.get('lines_for_editing', None)
        self.max_length = kwargs.get('max_length', None)
        self.text_type = kwargs.get('text_type', None)


class MicrosoftGraphThreatAssessmentRequest(MicrosoftGraphEntity):
    """threatAssessmentRequest.

    :param id: Read-only.
    :type id: str
    :param created_date_time: The Timestamp type represents date and time information using ISO
     8601 format and is always in UTC time. For example, midnight UTC on Jan 1, 2014 would look like
     this: '2014-01-01T00:00:00Z'.
    :type created_date_time: ~datetime.datetime
    :param content_type: threatAssessmentContentType. Possible values include: "mail", "url",
     "file".
    :type content_type: str or ~users_user.models.MicrosoftGraphThreatAssessmentContentType
    :param expected_assessment: threatExpectedAssessment. Possible values include: "block",
     "unblock".
    :type expected_assessment: str or ~users_user.models.MicrosoftGraphThreatExpectedAssessment
    :param category: threatCategory. Possible values include: "undefined", "spam", "phishing",
     "malware", "unknownFutureValue".
    :type category: str or ~users_user.models.MicrosoftGraphThreatCategory
    :param status: threatAssessmentStatus. Possible values include: "pending", "completed".
    :type status: str or ~users_user.models.MicrosoftGraphThreatAssessmentStatus
    :param request_source: threatAssessmentRequestSource. Possible values include: "undefined",
     "user", "administrator".
    :type request_source: str or ~users_user.models.MicrosoftGraphThreatAssessmentRequestSource
    :param created_by: identitySet.
    :type created_by: ~users_user.models.MicrosoftGraphIdentitySet
    :param results: A collection of threat assessment results. Read-only. By default, a GET
     /threatAssessmentRequests/{id} does not return this property unless you apply $expand on it.
    :type results: list[~users_user.models.MicrosoftGraphThreatAssessmentResult]
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'created_date_time': {'key': 'createdDateTime', 'type': 'iso-8601'},
        'content_type': {'key': 'contentType', 'type': 'str'},
        'expected_assessment': {'key': 'expectedAssessment', 'type': 'str'},
        'category': {'key': 'category', 'type': 'str'},
        'status': {'key': 'status', 'type': 'str'},
        'request_source': {'key': 'requestSource', 'type': 'str'},
        'created_by': {'key': 'createdBy', 'type': 'MicrosoftGraphIdentitySet'},
        'results': {'key': 'results', 'type': '[MicrosoftGraphThreatAssessmentResult]'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphThreatAssessmentRequest, self).__init__(**kwargs)
        self.created_date_time = kwargs.get('created_date_time', None)
        self.content_type = kwargs.get('content_type', None)
        self.expected_assessment = kwargs.get('expected_assessment', None)
        self.category = kwargs.get('category', None)
        self.status = kwargs.get('status', None)
        self.request_source = kwargs.get('request_source', None)
        self.created_by = kwargs.get('created_by', None)
        self.results = kwargs.get('results', None)


class MicrosoftGraphThreatAssessmentResult(MicrosoftGraphEntity):
    """threatAssessmentResult.

    :param id: Read-only.
    :type id: str
    :param created_date_time: The Timestamp type represents date and time information using ISO
     8601 format and is always in UTC time. For example, midnight UTC on Jan 1, 2014 would look like
     this: '2014-01-01T00:00:00Z'.
    :type created_date_time: ~datetime.datetime
    :param result_type: threatAssessmentResultType. Possible values include: "checkPolicy",
     "rescan", "unknownFutureValue".
    :type result_type: str or ~users_user.models.MicrosoftGraphThreatAssessmentResultType
    :param message: The result message for each threat assessment.
    :type message: str
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'created_date_time': {'key': 'createdDateTime', 'type': 'iso-8601'},
        'result_type': {'key': 'resultType', 'type': 'str'},
        'message': {'key': 'message', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphThreatAssessmentResult, self).__init__(**kwargs)
        self.created_date_time = kwargs.get('created_date_time', None)
        self.result_type = kwargs.get('result_type', None)
        self.message = kwargs.get('message', None)


class MicrosoftGraphThumbnail(msrest.serialization.Model):
    """thumbnail.

    :param content: The content stream for the thumbnail.
    :type content: bytes
    :param height: The height of the thumbnail, in pixels.
    :type height: int
    :param source_item_id: The unique identifier of the item that provided the thumbnail. This is
     only available when a folder thumbnail is requested.
    :type source_item_id: str
    :param url: The URL used to fetch the thumbnail content.
    :type url: str
    :param width: The width of the thumbnail, in pixels.
    :type width: int
    """

    _validation = {
        'height': {'maximum': 2147483647, 'minimum': -2147483648},
        'width': {'maximum': 2147483647, 'minimum': -2147483648},
    }

    _attribute_map = {
        'content': {'key': 'content', 'type': 'base64'},
        'height': {'key': 'height', 'type': 'int'},
        'source_item_id': {'key': 'sourceItemId', 'type': 'str'},
        'url': {'key': 'url', 'type': 'str'},
        'width': {'key': 'width', 'type': 'int'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphThumbnail, self).__init__(**kwargs)
        self.content = kwargs.get('content', None)
        self.height = kwargs.get('height', None)
        self.source_item_id = kwargs.get('source_item_id', None)
        self.url = kwargs.get('url', None)
        self.width = kwargs.get('width', None)


class MicrosoftGraphThumbnailSet(MicrosoftGraphEntity):
    """thumbnailSet.

    :param id: Read-only.
    :type id: str
    :param large: thumbnail.
    :type large: ~users_user.models.MicrosoftGraphThumbnail
    :param medium: thumbnail.
    :type medium: ~users_user.models.MicrosoftGraphThumbnail
    :param small: thumbnail.
    :type small: ~users_user.models.MicrosoftGraphThumbnail
    :param source: thumbnail.
    :type source: ~users_user.models.MicrosoftGraphThumbnail
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'large': {'key': 'large', 'type': 'MicrosoftGraphThumbnail'},
        'medium': {'key': 'medium', 'type': 'MicrosoftGraphThumbnail'},
        'small': {'key': 'small', 'type': 'MicrosoftGraphThumbnail'},
        'source': {'key': 'source', 'type': 'MicrosoftGraphThumbnail'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphThumbnailSet, self).__init__(**kwargs)
        self.large = kwargs.get('large', None)
        self.medium = kwargs.get('medium', None)
        self.small = kwargs.get('small', None)
        self.source = kwargs.get('source', None)


class MicrosoftGraphTimeOff(MicrosoftGraphChangeTrackedEntity):
    """timeOff.

    :param id: Read-only.
    :type id: str
    :param created_date_time:
    :type created_date_time: ~datetime.datetime
    :param last_modified_date_time:
    :type last_modified_date_time: ~datetime.datetime
    :param last_modified_by: identitySet.
    :type last_modified_by: ~users_user.models.MicrosoftGraphIdentitySet
    :param shared_time_off: timeOffItem.
    :type shared_time_off: ~users_user.models.MicrosoftGraphTimeOffItem
    :param draft_time_off: timeOffItem.
    :type draft_time_off: ~users_user.models.MicrosoftGraphTimeOffItem
    :param user_id:
    :type user_id: str
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'created_date_time': {'key': 'createdDateTime', 'type': 'iso-8601'},
        'last_modified_date_time': {'key': 'lastModifiedDateTime', 'type': 'iso-8601'},
        'last_modified_by': {'key': 'lastModifiedBy', 'type': 'MicrosoftGraphIdentitySet'},
        'shared_time_off': {'key': 'sharedTimeOff', 'type': 'MicrosoftGraphTimeOffItem'},
        'draft_time_off': {'key': 'draftTimeOff', 'type': 'MicrosoftGraphTimeOffItem'},
        'user_id': {'key': 'userId', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphTimeOff, self).__init__(**kwargs)
        self.shared_time_off = kwargs.get('shared_time_off', None)
        self.draft_time_off = kwargs.get('draft_time_off', None)
        self.user_id = kwargs.get('user_id', None)


class MicrosoftGraphTimeOffItem(MicrosoftGraphScheduleEntity):
    """timeOffItem.

    :param start_date_time:
    :type start_date_time: ~datetime.datetime
    :param end_date_time:
    :type end_date_time: ~datetime.datetime
    :param theme: scheduleEntityTheme. Possible values include: "white", "blue", "green", "purple",
     "pink", "yellow", "gray", "darkBlue", "darkGreen", "darkPurple", "darkPink", "darkYellow",
     "unknownFutureValue".
    :type theme: str or ~users_user.models.MicrosoftGraphScheduleEntityTheme
    :param time_off_reason_id:
    :type time_off_reason_id: str
    """

    _attribute_map = {
        'start_date_time': {'key': 'startDateTime', 'type': 'iso-8601'},
        'end_date_time': {'key': 'endDateTime', 'type': 'iso-8601'},
        'theme': {'key': 'theme', 'type': 'str'},
        'time_off_reason_id': {'key': 'timeOffReasonId', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphTimeOffItem, self).__init__(**kwargs)
        self.time_off_reason_id = kwargs.get('time_off_reason_id', None)


class MicrosoftGraphTimeOffReason(MicrosoftGraphChangeTrackedEntity):
    """timeOffReason.

    :param id: Read-only.
    :type id: str
    :param created_date_time:
    :type created_date_time: ~datetime.datetime
    :param last_modified_date_time:
    :type last_modified_date_time: ~datetime.datetime
    :param last_modified_by: identitySet.
    :type last_modified_by: ~users_user.models.MicrosoftGraphIdentitySet
    :param display_name:
    :type display_name: str
    :param icon_type: timeOffReasonIconType. Possible values include: "none", "car", "calendar",
     "running", "plane", "firstAid", "doctor", "notWorking", "clock", "juryDuty", "globe", "cup",
     "phone", "weather", "umbrella", "piggyBank", "dog", "cake", "trafficCone", "pin", "sunny",
     "unknownFutureValue".
    :type icon_type: str or ~users_user.models.MicrosoftGraphTimeOffReasonIconType
    :param is_active:
    :type is_active: bool
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'created_date_time': {'key': 'createdDateTime', 'type': 'iso-8601'},
        'last_modified_date_time': {'key': 'lastModifiedDateTime', 'type': 'iso-8601'},
        'last_modified_by': {'key': 'lastModifiedBy', 'type': 'MicrosoftGraphIdentitySet'},
        'display_name': {'key': 'displayName', 'type': 'str'},
        'icon_type': {'key': 'iconType', 'type': 'str'},
        'is_active': {'key': 'isActive', 'type': 'bool'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphTimeOffReason, self).__init__(**kwargs)
        self.display_name = kwargs.get('display_name', None)
        self.icon_type = kwargs.get('icon_type', None)
        self.is_active = kwargs.get('is_active', None)


class MicrosoftGraphTimeOffRequest(MicrosoftGraphScheduleChangeRequest):
    """timeOffRequest.

    :param id: Read-only.
    :type id: str
    :param created_date_time:
    :type created_date_time: ~datetime.datetime
    :param last_modified_date_time:
    :type last_modified_date_time: ~datetime.datetime
    :param last_modified_by: identitySet.
    :type last_modified_by: ~users_user.models.MicrosoftGraphIdentitySet
    :param assigned_to: scheduleChangeRequestActor. Possible values include: "sender", "recipient",
     "manager", "system", "unknownFutureValue".
    :type assigned_to: str or ~users_user.models.MicrosoftGraphScheduleChangeRequestActor
    :param state: scheduleChangeState. Possible values include: "pending", "approved", "declined",
     "unknownFutureValue".
    :type state: str or ~users_user.models.MicrosoftGraphScheduleChangeState
    :param sender_message:
    :type sender_message: str
    :param sender_date_time:
    :type sender_date_time: ~datetime.datetime
    :param manager_action_message:
    :type manager_action_message: str
    :param manager_action_date_time:
    :type manager_action_date_time: ~datetime.datetime
    :param sender_user_id:
    :type sender_user_id: str
    :param manager_user_id:
    :type manager_user_id: str
    :param start_date_time:
    :type start_date_time: ~datetime.datetime
    :param end_date_time:
    :type end_date_time: ~datetime.datetime
    :param time_off_reason_id:
    :type time_off_reason_id: str
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'created_date_time': {'key': 'createdDateTime', 'type': 'iso-8601'},
        'last_modified_date_time': {'key': 'lastModifiedDateTime', 'type': 'iso-8601'},
        'last_modified_by': {'key': 'lastModifiedBy', 'type': 'MicrosoftGraphIdentitySet'},
        'assigned_to': {'key': 'assignedTo', 'type': 'str'},
        'state': {'key': 'state', 'type': 'str'},
        'sender_message': {'key': 'senderMessage', 'type': 'str'},
        'sender_date_time': {'key': 'senderDateTime', 'type': 'iso-8601'},
        'manager_action_message': {'key': 'managerActionMessage', 'type': 'str'},
        'manager_action_date_time': {'key': 'managerActionDateTime', 'type': 'iso-8601'},
        'sender_user_id': {'key': 'senderUserId', 'type': 'str'},
        'manager_user_id': {'key': 'managerUserId', 'type': 'str'},
        'start_date_time': {'key': 'startDateTime', 'type': 'iso-8601'},
        'end_date_time': {'key': 'endDateTime', 'type': 'iso-8601'},
        'time_off_reason_id': {'key': 'timeOffReasonId', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphTimeOffRequest, self).__init__(**kwargs)
        self.start_date_time = kwargs.get('start_date_time', None)
        self.end_date_time = kwargs.get('end_date_time', None)
        self.time_off_reason_id = kwargs.get('time_off_reason_id', None)


class MicrosoftGraphTimeRange(msrest.serialization.Model):
    """timeRange.

    :param start_time:
    :type start_time: ~datetime.time
    :param end_time:
    :type end_time: ~datetime.time
    """

    _attribute_map = {
        'start_time': {'key': 'startTime', 'type': 'time'},
        'end_time': {'key': 'endTime', 'type': 'time'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphTimeRange, self).__init__(**kwargs)
        self.start_time = kwargs.get('start_time', None)
        self.end_time = kwargs.get('end_time', None)


class MicrosoftGraphTimeSlot(msrest.serialization.Model):
    """timeSlot.

    :param start: dateTimeTimeZone.
    :type start: ~users_user.models.MicrosoftGraphDateTimeZone
    :param end: dateTimeTimeZone.
    :type end: ~users_user.models.MicrosoftGraphDateTimeZone
    """

    _attribute_map = {
        'start': {'key': 'start', 'type': 'MicrosoftGraphDateTimeZone'},
        'end': {'key': 'end', 'type': 'MicrosoftGraphDateTimeZone'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphTimeSlot, self).__init__(**kwargs)
        self.start = kwargs.get('start', None)
        self.end = kwargs.get('end', None)


class MicrosoftGraphTimeZoneBase(msrest.serialization.Model):
    """timeZoneBase.

    :param name: The name of a time zone. It can be a standard time zone name such as 'Hawaii-
     Aleutian Standard Time', or 'Customized Time Zone' for a custom time zone.
    :type name: str
    """

    _attribute_map = {
        'name': {'key': 'name', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphTimeZoneBase, self).__init__(**kwargs)
        self.name = kwargs.get('name', None)


class MicrosoftGraphTrending(MicrosoftGraphEntity):
    """trending.

    :param id: Read-only.
    :type id: str
    :param weight: Value indicating how much the document is currently trending. The larger the
     number, the more the document is currently trending around the user (the more relevant it is).
     Returned documents are sorted by this value.
    :type weight: float
    :param resource_visualization: resourceVisualization.
    :type resource_visualization: ~users_user.models.MicrosoftGraphResourceVisualization
    :param resource_reference: resourceReference.
    :type resource_reference: ~users_user.models.MicrosoftGraphResourceReference
    :param last_modified_date_time:
    :type last_modified_date_time: ~datetime.datetime
    :param resource: entity.
    :type resource: ~users_user.models.MicrosoftGraphEntity
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'weight': {'key': 'weight', 'type': 'float'},
        'resource_visualization': {'key': 'resourceVisualization', 'type': 'MicrosoftGraphResourceVisualization'},
        'resource_reference': {'key': 'resourceReference', 'type': 'MicrosoftGraphResourceReference'},
        'last_modified_date_time': {'key': 'lastModifiedDateTime', 'type': 'iso-8601'},
        'resource': {'key': 'resource', 'type': 'MicrosoftGraphEntity'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphTrending, self).__init__(**kwargs)
        self.weight = kwargs.get('weight', None)
        self.resource_visualization = kwargs.get('resource_visualization', None)
        self.resource_reference = kwargs.get('resource_reference', None)
        self.last_modified_date_time = kwargs.get('last_modified_date_time', None)
        self.resource = kwargs.get('resource', None)


class MicrosoftGraphTypedEmailAddress(MicrosoftGraphEmailAddress):
    """typedEmailAddress.

    :param name: The display name of the person or entity.
    :type name: str
    :param address: The email address of the person or entity.
    :type address: str
    :param type: emailType. Possible values include: "unknown", "work", "personal", "main",
     "other".
    :type type: str or ~users_user.models.MicrosoftGraphEmailType
    :param other_label:
    :type other_label: str
    """

    _attribute_map = {
        'name': {'key': 'name', 'type': 'str'},
        'address': {'key': 'address', 'type': 'str'},
        'type': {'key': 'type', 'type': 'str'},
        'other_label': {'key': 'otherLabel', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphTypedEmailAddress, self).__init__(**kwargs)
        self.type = kwargs.get('type', None)
        self.other_label = kwargs.get('other_label', None)


class MicrosoftGraphUsageDetails(msrest.serialization.Model):
    """usageDetails.

    :param last_accessed_date_time: The date and time the resource was last accessed by the user.
     The timestamp represents date and time information using ISO 8601 format and is always in UTC
     time. For example, midnight UTC on Jan 1, 2014 would look like this: 2014-01-01T00:00:00Z.
     Read-only.
    :type last_accessed_date_time: ~datetime.datetime
    :param last_modified_date_time: The date and time the resource was last modified by the user.
     The timestamp represents date and time information using ISO 8601 format and is always in UTC
     time. For example, midnight UTC on Jan 1, 2014 would look like this: 2014-01-01T00:00:00Z.
     Read-only.
    :type last_modified_date_time: ~datetime.datetime
    """

    _attribute_map = {
        'last_accessed_date_time': {'key': 'lastAccessedDateTime', 'type': 'iso-8601'},
        'last_modified_date_time': {'key': 'lastModifiedDateTime', 'type': 'iso-8601'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphUsageDetails, self).__init__(**kwargs)
        self.last_accessed_date_time = kwargs.get('last_accessed_date_time', None)
        self.last_modified_date_time = kwargs.get('last_modified_date_time', None)


class MicrosoftGraphUsedInsight(MicrosoftGraphEntity):
    """usedInsight.

    :param id: Read-only.
    :type id: str
    :param last_used: usageDetails.
    :type last_used: ~users_user.models.MicrosoftGraphUsageDetails
    :param resource_visualization: resourceVisualization.
    :type resource_visualization: ~users_user.models.MicrosoftGraphResourceVisualization
    :param resource_reference: resourceReference.
    :type resource_reference: ~users_user.models.MicrosoftGraphResourceReference
    :param resource: entity.
    :type resource: ~users_user.models.MicrosoftGraphEntity
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'last_used': {'key': 'lastUsed', 'type': 'MicrosoftGraphUsageDetails'},
        'resource_visualization': {'key': 'resourceVisualization', 'type': 'MicrosoftGraphResourceVisualization'},
        'resource_reference': {'key': 'resourceReference', 'type': 'MicrosoftGraphResourceReference'},
        'resource': {'key': 'resource', 'type': 'MicrosoftGraphEntity'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphUsedInsight, self).__init__(**kwargs)
        self.last_used = kwargs.get('last_used', None)
        self.resource_visualization = kwargs.get('resource_visualization', None)
        self.resource_reference = kwargs.get('resource_reference', None)
        self.resource = kwargs.get('resource', None)


class MicrosoftGraphUser(MicrosoftGraphDirectoryObject):
    """Represents an Azure Active Directory user object.

    :param id: Read-only.
    :type id: str
    :param deleted_date_time:
    :type deleted_date_time: ~datetime.datetime
    :param sign_in_activity: signInActivity.
    :type sign_in_activity: ~users_user.models.MicrosoftGraphSignInActivity
    :param account_enabled: true if the account is enabled; otherwise, false. This property is
     required when a user is created. Supports $filter.
    :type account_enabled: bool
    :param age_group: Sets the age group of the user. Allowed values: null, minor, notAdult and
     adult. Refer to the legal age group property definitions for further information.
    :type age_group: str
    :param assigned_licenses: The licenses that are assigned to the user. Not nullable.
    :type assigned_licenses: list[~users_user.models.MicrosoftGraphAssignedLicense]
    :param assigned_plans: The plans that are assigned to the user. Read-only. Not nullable.
    :type assigned_plans: list[~users_user.models.MicrosoftGraphAssignedPlan]
    :param business_phones: The telephone numbers for the user. NOTE: Although this is a string
     collection, only one number can be set for this property.
    :type business_phones: list[str]
    :param city: The city in which the user is located. Supports $filter.
    :type city: str
    :param company_name: The company name which the user is associated. This property can be useful
     for describing the company that an external user comes from.
    :type company_name: str
    :param consent_provided_for_minor: Sets whether consent has been obtained for minors. Allowed
     values: null, granted, denied and notRequired. Refer to the legal age group property
     definitions for further information.
    :type consent_provided_for_minor: str
    :param country: The country/region in which the user is located; for example, 'US' or 'UK'.
     Supports $filter.
    :type country: str
    :param created_date_time: The created date of the user object.
    :type created_date_time: ~datetime.datetime
    :param creation_type: Indicates whether the user account was created as a regular school or
     work account (null), an external account (Invitation), a local account for an Azure Active
     Directory B2C tenant (LocalAccount) or self-service sign-up using email verification
     (EmailVerified). Read-only.
    :type creation_type: str
    :param department: The name for the department in which the user works. Supports $filter.
    :type department: str
    :param device_keys:
    :type device_keys: list[~users_user.models.MicrosoftGraphDeviceKey]
    :param display_name: The name displayed in the address book for the user. This is usually the
     combination of the user's first name, middle initial and last name. This property is required
     when a user is created and it cannot be cleared during updates. Supports $filter and $orderby.
    :type display_name: str
    :param employee_id: The employee identifier assigned to the user by the organization. Supports
     $filter.
    :type employee_id: str
    :param fax_number: The fax number of the user.
    :type fax_number: str
    :param given_name: The given name (first name) of the user. Supports $filter.
    :type given_name: str
    :param identities: Represents the identities that can be used to sign in to this user account.
     An identity can be provided by Microsoft (also known as a local account), by organizations, or
     by social identity providers such as Facebook, Google, and Microsoft, and tied to a user
     account. May contain multiple items with the same signInType value. Supports $filter.
    :type identities: list[~users_user.models.MicrosoftGraphObjectIdentity]
    :param im_addresses: The instant message voice over IP (VOIP) session initiation protocol (SIP)
     addresses for the user. Read-only.
    :type im_addresses: list[str]
    :param is_resource_account: true if the user is a resource account; otherwise, false. Null
     value should be considered false.
    :type is_resource_account: bool
    :param job_title: The user’s job title. Supports $filter.
    :type job_title: str
    :param last_password_change_date_time: The time when this Azure AD user last changed their
     password. The date and time information uses ISO 8601 format and is always in UTC time. For
     example, midnight UTC on Jan 1, 2014 would look like this: '2014-01-01T00:00:00Z'.
    :type last_password_change_date_time: ~datetime.datetime
    :param legal_age_group_classification: Used by enterprise applications to determine the legal
     age group of the user. This property is read-only and calculated based on ageGroup and
     consentProvidedForMinor properties. Allowed values: null, minorWithOutParentalConsent,
     minorWithParentalConsent, minorNoParentalConsentRequired, notAdult and adult. Refer to the
     legal age group property definitions for further information.).
    :type legal_age_group_classification: str
    :param license_assignment_states: State of license assignments for this user. Read-only.
    :type license_assignment_states: list[~users_user.models.MicrosoftGraphLicenseAssignmentState]
    :param mail: The SMTP address for the user, for example, 'jeff@contoso.onmicrosoft.com'. Read-
     Only. Supports $filter.
    :type mail: str
    :param mail_nickname: The mail alias for the user. This property must be specified when a user
     is created. Supports $filter.
    :type mail_nickname: str
    :param mobile_phone: The primary cellular telephone number for the user.
    :type mobile_phone: str
    :param on_premises_distinguished_name: Contains the on-premises Active Directory distinguished
     name or DN. The property is only populated for customers who are synchronizing their on-
     premises directory to Azure Active Directory via Azure AD Connect. Read-only.
    :type on_premises_distinguished_name: str
    :param on_premises_extension_attributes: onPremisesExtensionAttributes.
    :type on_premises_extension_attributes:
     ~users_user.models.MicrosoftGraphOnPremisesExtensionAttributes
    :param on_premises_immutable_id: This property is used to associate an on-premises Active
     Directory user account to their Azure AD user object. This property must be specified when
     creating a new user account in the Graph if you are using a federated domain for the user’s
     userPrincipalName (UPN) property. Important: The $ and _ characters cannot be used when
     specifying this property. Supports $filter.
    :type on_premises_immutable_id: str
    :param on_premises_last_sync_date_time: Indicates the last time at which the object was synced
     with the on-premises directory; for example: '2013-02-16T03:04:54Z'. The Timestamp type
     represents date and time information using ISO 8601 format and is always in UTC time. For
     example, midnight UTC on Jan 1, 2014 would look like this: '2014-01-01T00:00:00Z'. Read-only.
    :type on_premises_last_sync_date_time: ~datetime.datetime
    :param on_premises_provisioning_errors: Errors when using Microsoft synchronization product
     during provisioning.
    :type on_premises_provisioning_errors:
     list[~users_user.models.MicrosoftGraphOnPremisesProvisioningError]
    :param on_premises_security_identifier: Contains the on-premises security identifier (SID) for
     the user that was synchronized from on-premises to the cloud. Read-only.
    :type on_premises_security_identifier: str
    :param on_premises_sync_enabled: true if this object is synced from an on-premises directory;
     false if this object was originally synced from an on-premises directory but is no longer
     synced; null if this object has never been synced from an on-premises directory (default).
     Read-only.
    :type on_premises_sync_enabled: bool
    :param on_premises_domain_name: Contains the on-premises domainFQDN, also called dnsDomainName
     synchronized from the on-premises directory. The property is only populated for customers who
     are synchronizing their on-premises directory to Azure Active Directory via Azure AD Connect.
     Read-only.
    :type on_premises_domain_name: str
    :param on_premises_sam_account_name: Contains the on-premises samAccountName synchronized from
     the on-premises directory. The property is only populated for customers who are synchronizing
     their on-premises directory to Azure Active Directory via Azure AD Connect. Read-only.
    :type on_premises_sam_account_name: str
    :param on_premises_user_principal_name: Contains the on-premises userPrincipalName synchronized
     from the on-premises directory. The property is only populated for customers who are
     synchronizing their on-premises directory to Azure Active Directory via Azure AD Connect. Read-
     only.
    :type on_premises_user_principal_name: str
    :param other_mails: A list of additional email addresses for the user; for example:
     ['bob@contoso.com', 'Robert@fabrikam.com']. Supports $filter.
    :type other_mails: list[str]
    :param password_policies: Specifies password policies for the user. This value is an
     enumeration with one possible value being 'DisableStrongPassword', which allows weaker
     passwords than the default policy to be specified. 'DisablePasswordExpiration' can also be
     specified. The two may be specified together; for example: 'DisablePasswordExpiration,
     DisableStrongPassword'.
    :type password_policies: str
    :param password_profile: passwordProfile.
    :type password_profile: ~users_user.models.MicrosoftGraphPasswordProfile
    :param office_location: The office location in the user's place of business.
    :type office_location: str
    :param postal_code: The postal code for the user's postal address. The postal code is specific
     to the user's country/region. In the United States of America, this attribute contains the ZIP
     code.
    :type postal_code: str
    :param preferred_data_location: The preferred data location for the user. For more information,
     see OneDrive Online Multi-Geo.
    :type preferred_data_location: str
    :param preferred_language: The preferred language for the user. Should follow ISO 639-1 Code;
     for example 'en-US'.
    :type preferred_language: str
    :param provisioned_plans: The plans that are provisioned for the user. Read-only. Not nullable.
    :type provisioned_plans: list[~users_user.models.MicrosoftGraphProvisionedPlan]
    :param proxy_addresses: For example: ['SMTP: bob@contoso.com', 'smtp: bob@sales.contoso.com']
     The any operator is required for filter expressions on multi-valued properties. Read-only, Not
     nullable. Supports $filter.
    :type proxy_addresses: list[str]
    :param refresh_tokens_valid_from_date_time: Any refresh tokens or sessions tokens (session
     cookies) issued before this time are invalid, and applications will get an error when using an
     invalid refresh or sessions token to acquire a delegated access token (to access APIs such as
     Microsoft Graph).  If this happens, the application will need to acquire a new refresh token by
     making a request to the authorize endpoint. Returned only on $select. Read-only.
    :type refresh_tokens_valid_from_date_time: ~datetime.datetime
    :param show_in_address_list: true if the Outlook global address list should contain this user,
     otherwise false. If not set, this will be treated as true. For users invited through the
     invitation manager, this property will be set to false.
    :type show_in_address_list: bool
    :param sign_in_sessions_valid_from_date_time: Any refresh tokens or sessions tokens (session
     cookies) issued before this time are invalid, and applications will get an error when using an
     invalid refresh or sessions token to acquire a delegated access token (to access APIs such as
     Microsoft Graph).  If this happens, the application will need to acquire a new refresh token by
     making a request to the authorize endpoint. Read-only. Use revokeSignInSessions to reset.
    :type sign_in_sessions_valid_from_date_time: ~datetime.datetime
    :param state: The state or province in the user's address. Supports $filter.
    :type state: str
    :param street_address: The street address of the user's place of business.
    :type street_address: str
    :param surname: The user's surname (family name or last name). Supports $filter.
    :type surname: str
    :param usage_location: A two letter country code (ISO standard 3166). Required for users that
     will be assigned licenses due to legal requirement to check for availability of services in
     countries.  Examples include: 'US', 'JP', and 'GB'. Not nullable. Supports $filter.
    :type usage_location: str
    :param user_principal_name: The user principal name (UPN) of the user. The UPN is an Internet-
     style login name for the user based on the Internet standard RFC 822. By convention, this
     should map to the user's email name. The general format is alias@domain, where domain must be
     present in the tenant’s collection of verified domains. This property is required when a user
     is created. The verified domains for the tenant can be accessed from the verifiedDomains
     property of organization. Supports $filter and $orderby.
    :type user_principal_name: str
    :param external_user_state:
    :type external_user_state: str
    :param external_user_state_change_date_time:
    :type external_user_state_change_date_time: str
    :param user_type: A string value that can be used to classify user types in your directory,
     such as 'Member' and 'Guest'. Supports $filter.
    :type user_type: str
    :param mailbox_settings: mailboxSettings.
    :type mailbox_settings: ~users_user.models.MicrosoftGraphMailboxSettings
    :param identity_user_risk: identityUserRisk.
    :type identity_user_risk: ~users_user.models.MicrosoftGraphIdentityUserRisk
    :param device_enrollment_limit: The limit on the maximum number of devices that the user is
     permitted to enroll. Allowed values are 5 or 1000.
    :type device_enrollment_limit: int
    :param about_me: A freeform text entry field for the user to describe themselves.
    :type about_me: str
    :param birthday: The birthday of the user. The Timestamp type represents date and time
     information using ISO 8601 format and is always in UTC time. For example, midnight UTC on Jan
     1, 2014 would look like this: '2014-01-01T00:00:00Z'.
    :type birthday: ~datetime.datetime
    :param hire_date: The hire date of the user. The Timestamp type represents date and time
     information using ISO 8601 format and is always in UTC time. For example, midnight UTC on Jan
     1, 2014 would look like this: '2014-01-01T00:00:00Z'.
    :type hire_date: ~datetime.datetime
    :param interests: A list for the user to describe their interests.
    :type interests: list[str]
    :param my_site: The URL for the user's personal site.
    :type my_site: str
    :param past_projects: A list for the user to enumerate their past projects.
    :type past_projects: list[str]
    :param preferred_name: The preferred name for the user.
    :type preferred_name: str
    :param responsibilities: A list for the user to enumerate their responsibilities.
    :type responsibilities: list[str]
    :param schools: A list for the user to enumerate the schools they have attended.
    :type schools: list[str]
    :param skills: A list for the user to enumerate their skills.
    :type skills: list[str]
    :param analytics: userAnalytics.
    :type analytics: ~users_user.models.MicrosoftGraphUserAnalytics
    :param information_protection: informationProtection.
    :type information_protection: ~users_user.models.MicrosoftGraphInformationProtection
    :param app_role_assignments:
    :type app_role_assignments: list[~users_user.models.MicrosoftGraphAppRoleAssignment]
    :param created_objects: Directory objects that were created by the user. Read-only. Nullable.
    :type created_objects: list[~users_user.models.MicrosoftGraphDirectoryObject]
    :param direct_reports: The users and contacts that report to the user. (The users and contacts
     that have their manager property set to this user.) Read-only. Nullable.
    :type direct_reports: list[~users_user.models.MicrosoftGraphDirectoryObject]
    :param license_details: A collection of this user's license details. Read-only.
    :type license_details: list[~users_user.models.MicrosoftGraphLicenseDetails]
    :param manager: Represents an Azure Active Directory object. The directoryObject type is the
     base type for many other directory entity types.
    :type manager: ~users_user.models.MicrosoftGraphDirectoryObject
    :param member_of: The groups and directory roles that the user is a member of. Read-only.
     Nullable.
    :type member_of: list[~users_user.models.MicrosoftGraphDirectoryObject]
    :param owned_devices: Devices that are owned by the user. Read-only. Nullable.
    :type owned_devices: list[~users_user.models.MicrosoftGraphDirectoryObject]
    :param owned_objects: Directory objects that are owned by the user. Read-only. Nullable.
    :type owned_objects: list[~users_user.models.MicrosoftGraphDirectoryObject]
    :param registered_devices: Devices that are registered for the user. Read-only. Nullable.
    :type registered_devices: list[~users_user.models.MicrosoftGraphDirectoryObject]
    :param scoped_role_member_of:
    :type scoped_role_member_of: list[~users_user.models.MicrosoftGraphScopedRoleMembership]
    :param transitive_member_of:
    :type transitive_member_of: list[~users_user.models.MicrosoftGraphDirectoryObject]
    :param outlook: outlookUser.
    :type outlook: ~users_user.models.MicrosoftGraphOutlookUser
    :param messages: The messages in a mailbox or folder. Read-only. Nullable.
    :type messages: list[~users_user.models.MicrosoftGraphMessage]
    :param joined_groups:
    :type joined_groups: list[~users_user.models.MicrosoftGraphGroup]
    :param mail_folders: The user's mail folders. Read-only. Nullable.
    :type mail_folders: list[~users_user.models.MicrosoftGraphMailFolder]
    :param calendar: calendar.
    :type calendar: ~users_user.models.MicrosoftGraphCalendar
    :param calendars: The user's calendars. Read-only. Nullable.
    :type calendars: list[~users_user.models.MicrosoftGraphCalendar]
    :param calendar_groups: The user's calendar groups. Read-only. Nullable.
    :type calendar_groups: list[~users_user.models.MicrosoftGraphCalendarGroup]
    :param calendar_view: The calendar view for the calendar. Read-only. Nullable.
    :type calendar_view: list[~users_user.models.MicrosoftGraphEvent]
    :param events: The user's events. Default is to show Events under the Default Calendar. Read-
     only. Nullable.
    :type events: list[~users_user.models.MicrosoftGraphEvent]
    :param people: People that are relevant to the user. Read-only. Nullable.
    :type people: list[~users_user.models.MicrosoftGraphPerson]
    :param contacts: The user's contacts. Read-only. Nullable.
    :type contacts: list[~users_user.models.MicrosoftGraphContact]
    :param contact_folders: The user's contacts folders. Read-only. Nullable.
    :type contact_folders: list[~users_user.models.MicrosoftGraphContactFolder]
    :param inference_classification: inferenceClassification.
    :type inference_classification: ~users_user.models.MicrosoftGraphInferenceClassification
    :param photo: profilePhoto.
    :type photo: ~users_user.models.MicrosoftGraphProfilePhoto
    :param photos:
    :type photos: list[~users_user.models.MicrosoftGraphProfilePhoto]
    :param drive: drive.
    :type drive: ~users_user.models.MicrosoftGraphDrive
    :param drives: A collection of drives available for this user. Read-only.
    :type drives: list[~users_user.models.MicrosoftGraphDrive]
    :param followed_sites:
    :type followed_sites: list[~users_user.models.MicrosoftGraphSite]
    :param extensions: The collection of open extensions defined for the user. Read-only. Nullable.
    :type extensions: list[~users_user.models.MicrosoftGraphEntity]
    :param approvals:
    :type approvals: list[~users_user.models.MicrosoftGraphApproval]
    :param app_consent_requests_for_approval:
    :type app_consent_requests_for_approval:
     list[~users_user.models.MicrosoftGraphAppConsentRequest]
    :param agreement_acceptances:
    :type agreement_acceptances: list[~users_user.models.MicrosoftGraphAgreementAcceptance]
    :param device_enrollment_configurations:
    :type device_enrollment_configurations:
     list[~users_user.models.MicrosoftGraphDeviceEnrollmentConfiguration]
    :param managed_devices: The managed devices associated with the user.
    :type managed_devices: list[~users_user.models.MicrosoftGraphManagedDevice]
    :param managed_app_registrations: Zero or more managed app registrations that belong to the
     user.
    :type managed_app_registrations: list[~users_user.models.MicrosoftGraphManagedAppRegistration]
    :param windows_information_protection_device_registrations: Zero or more WIP device
     registrations that belong to the user.
    :type windows_information_protection_device_registrations:
     list[~users_user.models.MicrosoftGraphWindowsInformationProtectionDeviceRegistration]
    :param device_management_troubleshooting_events: The list of troubleshooting events for this
     user.
    :type device_management_troubleshooting_events:
     list[~users_user.models.MicrosoftGraphDeviceManagementTroubleshootingEvent]
    :param mobile_app_intent_and_states: The list of troubleshooting events for this user.
    :type mobile_app_intent_and_states:
     list[~users_user.models.MicrosoftGraphMobileAppIntentAndState]
    :param mobile_app_troubleshooting_events: The list of mobile app troubleshooting events for
     this user.
    :type mobile_app_troubleshooting_events:
     list[~users_user.models.MicrosoftGraphMobileAppTroubleshootingEvent]
    :param notifications:
    :type notifications: list[~users_user.models.MicrosoftGraphNotification]
    :param planner: plannerUser.
    :type planner: ~users_user.models.MicrosoftGraphPlannerUser
    :param insights: officeGraphInsights.
    :type insights: ~users_user.models.MicrosoftGraphOfficeGraphInsights
    :param settings: userSettings.
    :type settings: ~users_user.models.MicrosoftGraphUserSettings
    :param onenote: onenote.
    :type onenote: ~users_user.models.MicrosoftGraphOnenote
    :param profile: profile.
    :type profile: ~users_user.models.MicrosoftGraphProfile
    :param activities: The user's activities across devices. Read-only. Nullable.
    :type activities: list[~users_user.models.MicrosoftGraphUserActivity]
    :param devices:
    :type devices: list[~users_user.models.MicrosoftGraphDevice]
    :param online_meetings:
    :type online_meetings: list[~users_user.models.MicrosoftGraphOnlineMeeting]
    :param presence: presence.
    :type presence: ~users_user.models.MicrosoftGraphPresence
    :param authentication: authentication.
    :type authentication: ~users_user.models.MicrosoftGraphAuthentication
    :param chats:
    :type chats: list[~users_user.models.MicrosoftGraphChat]
    :param joined_teams:
    :type joined_teams: list[~users_user.models.MicrosoftGraphTeam]
    :param teamwork: userTeamwork.
    :type teamwork: ~users_user.models.MicrosoftGraphUserTeamwork
    """

    _validation = {
        'device_enrollment_limit': {'maximum': 2147483647, 'minimum': -2147483648},
    }

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'deleted_date_time': {'key': 'deletedDateTime', 'type': 'iso-8601'},
        'sign_in_activity': {'key': 'signInActivity', 'type': 'MicrosoftGraphSignInActivity'},
        'account_enabled': {'key': 'accountEnabled', 'type': 'bool'},
        'age_group': {'key': 'ageGroup', 'type': 'str'},
        'assigned_licenses': {'key': 'assignedLicenses', 'type': '[MicrosoftGraphAssignedLicense]'},
        'assigned_plans': {'key': 'assignedPlans', 'type': '[MicrosoftGraphAssignedPlan]'},
        'business_phones': {'key': 'businessPhones', 'type': '[str]'},
        'city': {'key': 'city', 'type': 'str'},
        'company_name': {'key': 'companyName', 'type': 'str'},
        'consent_provided_for_minor': {'key': 'consentProvidedForMinor', 'type': 'str'},
        'country': {'key': 'country', 'type': 'str'},
        'created_date_time': {'key': 'createdDateTime', 'type': 'iso-8601'},
        'creation_type': {'key': 'creationType', 'type': 'str'},
        'department': {'key': 'department', 'type': 'str'},
        'device_keys': {'key': 'deviceKeys', 'type': '[MicrosoftGraphDeviceKey]'},
        'display_name': {'key': 'displayName', 'type': 'str'},
        'employee_id': {'key': 'employeeId', 'type': 'str'},
        'fax_number': {'key': 'faxNumber', 'type': 'str'},
        'given_name': {'key': 'givenName', 'type': 'str'},
        'identities': {'key': 'identities', 'type': '[MicrosoftGraphObjectIdentity]'},
        'im_addresses': {'key': 'imAddresses', 'type': '[str]'},
        'is_resource_account': {'key': 'isResourceAccount', 'type': 'bool'},
        'job_title': {'key': 'jobTitle', 'type': 'str'},
        'last_password_change_date_time': {'key': 'lastPasswordChangeDateTime', 'type': 'iso-8601'},
        'legal_age_group_classification': {'key': 'legalAgeGroupClassification', 'type': 'str'},
        'license_assignment_states': {'key': 'licenseAssignmentStates', 'type': '[MicrosoftGraphLicenseAssignmentState]'},
        'mail': {'key': 'mail', 'type': 'str'},
        'mail_nickname': {'key': 'mailNickname', 'type': 'str'},
        'mobile_phone': {'key': 'mobilePhone', 'type': 'str'},
        'on_premises_distinguished_name': {'key': 'onPremisesDistinguishedName', 'type': 'str'},
        'on_premises_extension_attributes': {'key': 'onPremisesExtensionAttributes', 'type': 'MicrosoftGraphOnPremisesExtensionAttributes'},
        'on_premises_immutable_id': {'key': 'onPremisesImmutableId', 'type': 'str'},
        'on_premises_last_sync_date_time': {'key': 'onPremisesLastSyncDateTime', 'type': 'iso-8601'},
        'on_premises_provisioning_errors': {'key': 'onPremisesProvisioningErrors', 'type': '[MicrosoftGraphOnPremisesProvisioningError]'},
        'on_premises_security_identifier': {'key': 'onPremisesSecurityIdentifier', 'type': 'str'},
        'on_premises_sync_enabled': {'key': 'onPremisesSyncEnabled', 'type': 'bool'},
        'on_premises_domain_name': {'key': 'onPremisesDomainName', 'type': 'str'},
        'on_premises_sam_account_name': {'key': 'onPremisesSamAccountName', 'type': 'str'},
        'on_premises_user_principal_name': {'key': 'onPremisesUserPrincipalName', 'type': 'str'},
        'other_mails': {'key': 'otherMails', 'type': '[str]'},
        'password_policies': {'key': 'passwordPolicies', 'type': 'str'},
        'password_profile': {'key': 'passwordProfile', 'type': 'MicrosoftGraphPasswordProfile'},
        'office_location': {'key': 'officeLocation', 'type': 'str'},
        'postal_code': {'key': 'postalCode', 'type': 'str'},
        'preferred_data_location': {'key': 'preferredDataLocation', 'type': 'str'},
        'preferred_language': {'key': 'preferredLanguage', 'type': 'str'},
        'provisioned_plans': {'key': 'provisionedPlans', 'type': '[MicrosoftGraphProvisionedPlan]'},
        'proxy_addresses': {'key': 'proxyAddresses', 'type': '[str]'},
        'refresh_tokens_valid_from_date_time': {'key': 'refreshTokensValidFromDateTime', 'type': 'iso-8601'},
        'show_in_address_list': {'key': 'showInAddressList', 'type': 'bool'},
        'sign_in_sessions_valid_from_date_time': {'key': 'signInSessionsValidFromDateTime', 'type': 'iso-8601'},
        'state': {'key': 'state', 'type': 'str'},
        'street_address': {'key': 'streetAddress', 'type': 'str'},
        'surname': {'key': 'surname', 'type': 'str'},
        'usage_location': {'key': 'usageLocation', 'type': 'str'},
        'user_principal_name': {'key': 'userPrincipalName', 'type': 'str'},
        'external_user_state': {'key': 'externalUserState', 'type': 'str'},
        'external_user_state_change_date_time': {'key': 'externalUserStateChangeDateTime', 'type': 'str'},
        'user_type': {'key': 'userType', 'type': 'str'},
        'mailbox_settings': {'key': 'mailboxSettings', 'type': 'MicrosoftGraphMailboxSettings'},
        'identity_user_risk': {'key': 'identityUserRisk', 'type': 'MicrosoftGraphIdentityUserRisk'},
        'device_enrollment_limit': {'key': 'deviceEnrollmentLimit', 'type': 'int'},
        'about_me': {'key': 'aboutMe', 'type': 'str'},
        'birthday': {'key': 'birthday', 'type': 'iso-8601'},
        'hire_date': {'key': 'hireDate', 'type': 'iso-8601'},
        'interests': {'key': 'interests', 'type': '[str]'},
        'my_site': {'key': 'mySite', 'type': 'str'},
        'past_projects': {'key': 'pastProjects', 'type': '[str]'},
        'preferred_name': {'key': 'preferredName', 'type': 'str'},
        'responsibilities': {'key': 'responsibilities', 'type': '[str]'},
        'schools': {'key': 'schools', 'type': '[str]'},
        'skills': {'key': 'skills', 'type': '[str]'},
        'analytics': {'key': 'analytics', 'type': 'MicrosoftGraphUserAnalytics'},
        'information_protection': {'key': 'informationProtection', 'type': 'MicrosoftGraphInformationProtection'},
        'app_role_assignments': {'key': 'appRoleAssignments', 'type': '[MicrosoftGraphAppRoleAssignment]'},
        'created_objects': {'key': 'createdObjects', 'type': '[MicrosoftGraphDirectoryObject]'},
        'direct_reports': {'key': 'directReports', 'type': '[MicrosoftGraphDirectoryObject]'},
        'license_details': {'key': 'licenseDetails', 'type': '[MicrosoftGraphLicenseDetails]'},
        'manager': {'key': 'manager', 'type': 'MicrosoftGraphDirectoryObject'},
        'member_of': {'key': 'memberOf', 'type': '[MicrosoftGraphDirectoryObject]'},
        'owned_devices': {'key': 'ownedDevices', 'type': '[MicrosoftGraphDirectoryObject]'},
        'owned_objects': {'key': 'ownedObjects', 'type': '[MicrosoftGraphDirectoryObject]'},
        'registered_devices': {'key': 'registeredDevices', 'type': '[MicrosoftGraphDirectoryObject]'},
        'scoped_role_member_of': {'key': 'scopedRoleMemberOf', 'type': '[MicrosoftGraphScopedRoleMembership]'},
        'transitive_member_of': {'key': 'transitiveMemberOf', 'type': '[MicrosoftGraphDirectoryObject]'},
        'outlook': {'key': 'outlook', 'type': 'MicrosoftGraphOutlookUser'},
        'messages': {'key': 'messages', 'type': '[MicrosoftGraphMessage]'},
        'joined_groups': {'key': 'joinedGroups', 'type': '[MicrosoftGraphGroup]'},
        'mail_folders': {'key': 'mailFolders', 'type': '[MicrosoftGraphMailFolder]'},
        'calendar': {'key': 'calendar', 'type': 'MicrosoftGraphCalendar'},
        'calendars': {'key': 'calendars', 'type': '[MicrosoftGraphCalendar]'},
        'calendar_groups': {'key': 'calendarGroups', 'type': '[MicrosoftGraphCalendarGroup]'},
        'calendar_view': {'key': 'calendarView', 'type': '[MicrosoftGraphEvent]'},
        'events': {'key': 'events', 'type': '[MicrosoftGraphEvent]'},
        'people': {'key': 'people', 'type': '[MicrosoftGraphPerson]'},
        'contacts': {'key': 'contacts', 'type': '[MicrosoftGraphContact]'},
        'contact_folders': {'key': 'contactFolders', 'type': '[MicrosoftGraphContactFolder]'},
        'inference_classification': {'key': 'inferenceClassification', 'type': 'MicrosoftGraphInferenceClassification'},
        'photo': {'key': 'photo', 'type': 'MicrosoftGraphProfilePhoto'},
        'photos': {'key': 'photos', 'type': '[MicrosoftGraphProfilePhoto]'},
        'drive': {'key': 'drive', 'type': 'MicrosoftGraphDrive'},
        'drives': {'key': 'drives', 'type': '[MicrosoftGraphDrive]'},
        'followed_sites': {'key': 'followedSites', 'type': '[MicrosoftGraphSite]'},
        'extensions': {'key': 'extensions', 'type': '[MicrosoftGraphEntity]'},
        'approvals': {'key': 'approvals', 'type': '[MicrosoftGraphApproval]'},
        'app_consent_requests_for_approval': {'key': 'appConsentRequestsForApproval', 'type': '[MicrosoftGraphAppConsentRequest]'},
        'agreement_acceptances': {'key': 'agreementAcceptances', 'type': '[MicrosoftGraphAgreementAcceptance]'},
        'device_enrollment_configurations': {'key': 'deviceEnrollmentConfigurations', 'type': '[MicrosoftGraphDeviceEnrollmentConfiguration]'},
        'managed_devices': {'key': 'managedDevices', 'type': '[MicrosoftGraphManagedDevice]'},
        'managed_app_registrations': {'key': 'managedAppRegistrations', 'type': '[MicrosoftGraphManagedAppRegistration]'},
        'windows_information_protection_device_registrations': {'key': 'windowsInformationProtectionDeviceRegistrations', 'type': '[MicrosoftGraphWindowsInformationProtectionDeviceRegistration]'},
        'device_management_troubleshooting_events': {'key': 'deviceManagementTroubleshootingEvents', 'type': '[MicrosoftGraphDeviceManagementTroubleshootingEvent]'},
        'mobile_app_intent_and_states': {'key': 'mobileAppIntentAndStates', 'type': '[MicrosoftGraphMobileAppIntentAndState]'},
        'mobile_app_troubleshooting_events': {'key': 'mobileAppTroubleshootingEvents', 'type': '[MicrosoftGraphMobileAppTroubleshootingEvent]'},
        'notifications': {'key': 'notifications', 'type': '[MicrosoftGraphNotification]'},
        'planner': {'key': 'planner', 'type': 'MicrosoftGraphPlannerUser'},
        'insights': {'key': 'insights', 'type': 'MicrosoftGraphOfficeGraphInsights'},
        'settings': {'key': 'settings', 'type': 'MicrosoftGraphUserSettings'},
        'onenote': {'key': 'onenote', 'type': 'MicrosoftGraphOnenote'},
        'profile': {'key': 'profile', 'type': 'MicrosoftGraphProfile'},
        'activities': {'key': 'activities', 'type': '[MicrosoftGraphUserActivity]'},
        'devices': {'key': 'devices', 'type': '[MicrosoftGraphDevice]'},
        'online_meetings': {'key': 'onlineMeetings', 'type': '[MicrosoftGraphOnlineMeeting]'},
        'presence': {'key': 'presence', 'type': 'MicrosoftGraphPresence'},
        'authentication': {'key': 'authentication', 'type': 'MicrosoftGraphAuthentication'},
        'chats': {'key': 'chats', 'type': '[MicrosoftGraphChat]'},
        'joined_teams': {'key': 'joinedTeams', 'type': '[MicrosoftGraphTeam]'},
        'teamwork': {'key': 'teamwork', 'type': 'MicrosoftGraphUserTeamwork'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphUser, self).__init__(**kwargs)
        self.sign_in_activity = kwargs.get('sign_in_activity', None)
        self.account_enabled = kwargs.get('account_enabled', None)
        self.age_group = kwargs.get('age_group', None)
        self.assigned_licenses = kwargs.get('assigned_licenses', None)
        self.assigned_plans = kwargs.get('assigned_plans', None)
        self.business_phones = kwargs.get('business_phones', None)
        self.city = kwargs.get('city', None)
        self.company_name = kwargs.get('company_name', None)
        self.consent_provided_for_minor = kwargs.get('consent_provided_for_minor', None)
        self.country = kwargs.get('country', None)
        self.created_date_time = kwargs.get('created_date_time', None)
        self.creation_type = kwargs.get('creation_type', None)
        self.department = kwargs.get('department', None)
        self.device_keys = kwargs.get('device_keys', None)
        self.display_name = kwargs.get('display_name', None)
        self.employee_id = kwargs.get('employee_id', None)
        self.fax_number = kwargs.get('fax_number', None)
        self.given_name = kwargs.get('given_name', None)
        self.identities = kwargs.get('identities', None)
        self.im_addresses = kwargs.get('im_addresses', None)
        self.is_resource_account = kwargs.get('is_resource_account', None)
        self.job_title = kwargs.get('job_title', None)
        self.last_password_change_date_time = kwargs.get('last_password_change_date_time', None)
        self.legal_age_group_classification = kwargs.get('legal_age_group_classification', None)
        self.license_assignment_states = kwargs.get('license_assignment_states', None)
        self.mail = kwargs.get('mail', None)
        self.mail_nickname = kwargs.get('mail_nickname', None)
        self.mobile_phone = kwargs.get('mobile_phone', None)
        self.on_premises_distinguished_name = kwargs.get('on_premises_distinguished_name', None)
        self.on_premises_extension_attributes = kwargs.get('on_premises_extension_attributes', None)
        self.on_premises_immutable_id = kwargs.get('on_premises_immutable_id', None)
        self.on_premises_last_sync_date_time = kwargs.get('on_premises_last_sync_date_time', None)
        self.on_premises_provisioning_errors = kwargs.get('on_premises_provisioning_errors', None)
        self.on_premises_security_identifier = kwargs.get('on_premises_security_identifier', None)
        self.on_premises_sync_enabled = kwargs.get('on_premises_sync_enabled', None)
        self.on_premises_domain_name = kwargs.get('on_premises_domain_name', None)
        self.on_premises_sam_account_name = kwargs.get('on_premises_sam_account_name', None)
        self.on_premises_user_principal_name = kwargs.get('on_premises_user_principal_name', None)
        self.other_mails = kwargs.get('other_mails', None)
        self.password_policies = kwargs.get('password_policies', None)
        self.password_profile = kwargs.get('password_profile', None)
        self.office_location = kwargs.get('office_location', None)
        self.postal_code = kwargs.get('postal_code', None)
        self.preferred_data_location = kwargs.get('preferred_data_location', None)
        self.preferred_language = kwargs.get('preferred_language', None)
        self.provisioned_plans = kwargs.get('provisioned_plans', None)
        self.proxy_addresses = kwargs.get('proxy_addresses', None)
        self.refresh_tokens_valid_from_date_time = kwargs.get('refresh_tokens_valid_from_date_time', None)
        self.show_in_address_list = kwargs.get('show_in_address_list', None)
        self.sign_in_sessions_valid_from_date_time = kwargs.get('sign_in_sessions_valid_from_date_time', None)
        self.state = kwargs.get('state', None)
        self.street_address = kwargs.get('street_address', None)
        self.surname = kwargs.get('surname', None)
        self.usage_location = kwargs.get('usage_location', None)
        self.user_principal_name = kwargs.get('user_principal_name', None)
        self.external_user_state = kwargs.get('external_user_state', None)
        self.external_user_state_change_date_time = kwargs.get('external_user_state_change_date_time', None)
        self.user_type = kwargs.get('user_type', None)
        self.mailbox_settings = kwargs.get('mailbox_settings', None)
        self.identity_user_risk = kwargs.get('identity_user_risk', None)
        self.device_enrollment_limit = kwargs.get('device_enrollment_limit', None)
        self.about_me = kwargs.get('about_me', None)
        self.birthday = kwargs.get('birthday', None)
        self.hire_date = kwargs.get('hire_date', None)
        self.interests = kwargs.get('interests', None)
        self.my_site = kwargs.get('my_site', None)
        self.past_projects = kwargs.get('past_projects', None)
        self.preferred_name = kwargs.get('preferred_name', None)
        self.responsibilities = kwargs.get('responsibilities', None)
        self.schools = kwargs.get('schools', None)
        self.skills = kwargs.get('skills', None)
        self.analytics = kwargs.get('analytics', None)
        self.information_protection = kwargs.get('information_protection', None)
        self.app_role_assignments = kwargs.get('app_role_assignments', None)
        self.created_objects = kwargs.get('created_objects', None)
        self.direct_reports = kwargs.get('direct_reports', None)
        self.license_details = kwargs.get('license_details', None)
        self.manager = kwargs.get('manager', None)
        self.member_of = kwargs.get('member_of', None)
        self.owned_devices = kwargs.get('owned_devices', None)
        self.owned_objects = kwargs.get('owned_objects', None)
        self.registered_devices = kwargs.get('registered_devices', None)
        self.scoped_role_member_of = kwargs.get('scoped_role_member_of', None)
        self.transitive_member_of = kwargs.get('transitive_member_of', None)
        self.outlook = kwargs.get('outlook', None)
        self.messages = kwargs.get('messages', None)
        self.joined_groups = kwargs.get('joined_groups', None)
        self.mail_folders = kwargs.get('mail_folders', None)
        self.calendar = kwargs.get('calendar', None)
        self.calendars = kwargs.get('calendars', None)
        self.calendar_groups = kwargs.get('calendar_groups', None)
        self.calendar_view = kwargs.get('calendar_view', None)
        self.events = kwargs.get('events', None)
        self.people = kwargs.get('people', None)
        self.contacts = kwargs.get('contacts', None)
        self.contact_folders = kwargs.get('contact_folders', None)
        self.inference_classification = kwargs.get('inference_classification', None)
        self.photo = kwargs.get('photo', None)
        self.photos = kwargs.get('photos', None)
        self.drive = kwargs.get('drive', None)
        self.drives = kwargs.get('drives', None)
        self.followed_sites = kwargs.get('followed_sites', None)
        self.extensions = kwargs.get('extensions', None)
        self.approvals = kwargs.get('approvals', None)
        self.app_consent_requests_for_approval = kwargs.get('app_consent_requests_for_approval', None)
        self.agreement_acceptances = kwargs.get('agreement_acceptances', None)
        self.device_enrollment_configurations = kwargs.get('device_enrollment_configurations', None)
        self.managed_devices = kwargs.get('managed_devices', None)
        self.managed_app_registrations = kwargs.get('managed_app_registrations', None)
        self.windows_information_protection_device_registrations = kwargs.get('windows_information_protection_device_registrations', None)
        self.device_management_troubleshooting_events = kwargs.get('device_management_troubleshooting_events', None)
        self.mobile_app_intent_and_states = kwargs.get('mobile_app_intent_and_states', None)
        self.mobile_app_troubleshooting_events = kwargs.get('mobile_app_troubleshooting_events', None)
        self.notifications = kwargs.get('notifications', None)
        self.planner = kwargs.get('planner', None)
        self.insights = kwargs.get('insights', None)
        self.settings = kwargs.get('settings', None)
        self.onenote = kwargs.get('onenote', None)
        self.profile = kwargs.get('profile', None)
        self.activities = kwargs.get('activities', None)
        self.devices = kwargs.get('devices', None)
        self.online_meetings = kwargs.get('online_meetings', None)
        self.presence = kwargs.get('presence', None)
        self.authentication = kwargs.get('authentication', None)
        self.chats = kwargs.get('chats', None)
        self.joined_teams = kwargs.get('joined_teams', None)
        self.teamwork = kwargs.get('teamwork', None)


class MicrosoftGraphUserAccountInformation(MicrosoftGraphItemFacet):
    """userAccountInformation.

    :param id: Read-only.
    :type id: str
    :param allowed_audiences: allowedAudiences. Possible values include: "me", "family",
     "contacts", "groupMembers", "organization", "federatedOrganizations", "everyone",
     "unknownFutureValue".
    :type allowed_audiences: str or ~users_user.models.MicrosoftGraphAllowedAudiences
    :param inference: inferenceData.
    :type inference: ~users_user.models.MicrosoftGraphInferenceData
    :param created_date_time:
    :type created_date_time: ~datetime.datetime
    :param created_by: identitySet.
    :type created_by: ~users_user.models.MicrosoftGraphIdentitySet
    :param last_modified_date_time:
    :type last_modified_date_time: ~datetime.datetime
    :param last_modified_by: identitySet.
    :type last_modified_by: ~users_user.models.MicrosoftGraphIdentitySet
    :param age_group:
    :type age_group: str
    :param country_code:
    :type country_code: str
    :param preferred_language_tag: localeInfo.
    :type preferred_language_tag: ~users_user.models.MicrosoftGraphLocaleInfo
    :param user_principal_name:
    :type user_principal_name: str
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'allowed_audiences': {'key': 'allowedAudiences', 'type': 'str'},
        'inference': {'key': 'inference', 'type': 'MicrosoftGraphInferenceData'},
        'created_date_time': {'key': 'createdDateTime', 'type': 'iso-8601'},
        'created_by': {'key': 'createdBy', 'type': 'MicrosoftGraphIdentitySet'},
        'last_modified_date_time': {'key': 'lastModifiedDateTime', 'type': 'iso-8601'},
        'last_modified_by': {'key': 'lastModifiedBy', 'type': 'MicrosoftGraphIdentitySet'},
        'age_group': {'key': 'ageGroup', 'type': 'str'},
        'country_code': {'key': 'countryCode', 'type': 'str'},
        'preferred_language_tag': {'key': 'preferredLanguageTag', 'type': 'MicrosoftGraphLocaleInfo'},
        'user_principal_name': {'key': 'userPrincipalName', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphUserAccountInformation, self).__init__(**kwargs)
        self.age_group = kwargs.get('age_group', None)
        self.country_code = kwargs.get('country_code', None)
        self.preferred_language_tag = kwargs.get('preferred_language_tag', None)
        self.user_principal_name = kwargs.get('user_principal_name', None)


class MicrosoftGraphUserActivity(MicrosoftGraphEntity):
    """userActivity.

    :param id: Read-only.
    :type id: str
    :param visual_elements: visualInfo.
    :type visual_elements: ~users_user.models.MicrosoftGraphVisualInfo
    :param activity_source_host: Required. URL for the domain representing the cross-platform
     identity mapping for the app. Mapping is stored either as a JSON file hosted on the domain or
     configurable via Windows Dev Center. The JSON file is named cross-platform-app-identifiers and
     is hosted at root of your HTTPS domain, either at the top level domain or include a sub domain.
     For example: https://contoso.com or https://myapp.contoso.com but NOT
     https://myapp.contoso.com/somepath. You must have a unique file and domain (or sub domain) per
     cross-platform app identity. For example, a separate file and domain is needed for Word vs.
     PowerPoint.
    :type activity_source_host: str
    :param activation_url: Required. URL used to launch the activity in the best native experience
     represented by the appId. Might launch a web-based app if no native app exists.
    :type activation_url: str
    :param app_activity_id: Required. The unique activity ID in the context of the app - supplied
     by caller and immutable thereafter.
    :type app_activity_id: str
    :param app_display_name: Optional. Short text description of the app used to generate the
     activity for use in cases when the app is not installed on the user’s local device.
    :type app_display_name: str
    :param content_url: Optional. Used in the event the content can be rendered outside of a native
     or web-based app experience (for example, a pointer to an item in an RSS feed).
    :type content_url: str
    :param created_date_time: Set by the server. DateTime in UTC when the object was created on the
     server.
    :type created_date_time: ~datetime.datetime
    :param expiration_date_time: Set by the server. DateTime in UTC when the object expired on the
     server.
    :type expiration_date_time: ~datetime.datetime
    :param fallback_url: Optional. URL used to launch the activity in a web-based app, if
     available.
    :type fallback_url: str
    :param last_modified_date_time: Set by the server. DateTime in UTC when the object was modified
     on the server.
    :type last_modified_date_time: ~datetime.datetime
    :param user_timezone: Optional. The timezone in which the user's device used to generate the
     activity was located at activity creation time; values supplied as Olson IDs in order to
     support cross-platform representation.
    :type user_timezone: str
    :param content_info: Any object.
    :type content_info: object
    :param status: status. Possible values include: "active", "updated", "deleted", "ignored",
     "unknownFutureValue".
    :type status: str or ~users_user.models.MicrosoftGraphStatus
    :param history_items: Optional. NavigationProperty/Containment; navigation property to the
     activity's historyItems.
    :type history_items: list[~users_user.models.MicrosoftGraphActivityHistoryItem]
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'visual_elements': {'key': 'visualElements', 'type': 'MicrosoftGraphVisualInfo'},
        'activity_source_host': {'key': 'activitySourceHost', 'type': 'str'},
        'activation_url': {'key': 'activationUrl', 'type': 'str'},
        'app_activity_id': {'key': 'appActivityId', 'type': 'str'},
        'app_display_name': {'key': 'appDisplayName', 'type': 'str'},
        'content_url': {'key': 'contentUrl', 'type': 'str'},
        'created_date_time': {'key': 'createdDateTime', 'type': 'iso-8601'},
        'expiration_date_time': {'key': 'expirationDateTime', 'type': 'iso-8601'},
        'fallback_url': {'key': 'fallbackUrl', 'type': 'str'},
        'last_modified_date_time': {'key': 'lastModifiedDateTime', 'type': 'iso-8601'},
        'user_timezone': {'key': 'userTimezone', 'type': 'str'},
        'content_info': {'key': 'contentInfo', 'type': 'object'},
        'status': {'key': 'status', 'type': 'str'},
        'history_items': {'key': 'historyItems', 'type': '[MicrosoftGraphActivityHistoryItem]'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphUserActivity, self).__init__(**kwargs)
        self.visual_elements = kwargs.get('visual_elements', None)
        self.activity_source_host = kwargs.get('activity_source_host', None)
        self.activation_url = kwargs.get('activation_url', None)
        self.app_activity_id = kwargs.get('app_activity_id', None)
        self.app_display_name = kwargs.get('app_display_name', None)
        self.content_url = kwargs.get('content_url', None)
        self.created_date_time = kwargs.get('created_date_time', None)
        self.expiration_date_time = kwargs.get('expiration_date_time', None)
        self.fallback_url = kwargs.get('fallback_url', None)
        self.last_modified_date_time = kwargs.get('last_modified_date_time', None)
        self.user_timezone = kwargs.get('user_timezone', None)
        self.content_info = kwargs.get('content_info', None)
        self.status = kwargs.get('status', None)
        self.history_items = kwargs.get('history_items', None)


class MicrosoftGraphUserAnalytics(MicrosoftGraphEntity):
    """userAnalytics.

    :param id: Read-only.
    :type id: str
    :param settings: settings.
    :type settings: ~users_user.models.MicrosoftGraphSettings
    :param activity_statistics:
    :type activity_statistics: list[~users_user.models.MicrosoftGraphActivityStatistics]
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'settings': {'key': 'settings', 'type': 'MicrosoftGraphSettings'},
        'activity_statistics': {'key': 'activityStatistics', 'type': '[MicrosoftGraphActivityStatistics]'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphUserAnalytics, self).__init__(**kwargs)
        self.settings = kwargs.get('settings', None)
        self.activity_statistics = kwargs.get('activity_statistics', None)


class MicrosoftGraphUserConfiguration(MicrosoftGraphEntity):
    """userConfiguration.

    :param id: Read-only.
    :type id: str
    :param binary_data:
    :type binary_data: bytes
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'binary_data': {'key': 'binaryData', 'type': 'base64'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphUserConfiguration, self).__init__(**kwargs)
        self.binary_data = kwargs.get('binary_data', None)


class MicrosoftGraphUserConsentRequest(MicrosoftGraphRequest):
    """userConsentRequest.

    :param id: Read-only.
    :type id: str
    :param approval: approval.
    :type approval: ~users_user.models.MicrosoftGraphApproval
    :param reason:
    :type reason: str
    :param created_by: userIdentity.
    :type created_by: ~users_user.models.MicrosoftGraphUserIdentity
    :param created_date_time:
    :type created_date_time: ~datetime.datetime
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'approval': {'key': 'approval', 'type': 'MicrosoftGraphApproval'},
        'reason': {'key': 'reason', 'type': 'str'},
        'created_by': {'key': 'createdBy', 'type': 'MicrosoftGraphUserIdentity'},
        'created_date_time': {'key': 'createdDateTime', 'type': 'iso-8601'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphUserConsentRequest, self).__init__(**kwargs)
        self.reason = kwargs.get('reason', None)
        self.created_by = kwargs.get('created_by', None)
        self.created_date_time = kwargs.get('created_date_time', None)


class MicrosoftGraphUserIdentity(MicrosoftGraphIdentity):
    """userIdentity.

    :param id: Unique identifier for the identity.
    :type id: str
    :param display_name: The identity's display name. Note that this may not always be available or
     up to date. For example, if a user changes their display name, the API may show the new value
     in a future response, but the items associated with the user won't show up as having changed
     when using delta.
    :type display_name: str
    :param ip_address: Indicates the client IP address used by user performing the activity (audit
     log only).
    :type ip_address: str
    :param user_principal_name: The userPrincipalName attribute of the user.
    :type user_principal_name: str
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'display_name': {'key': 'displayName', 'type': 'str'},
        'ip_address': {'key': 'ipAddress', 'type': 'str'},
        'user_principal_name': {'key': 'userPrincipalName', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphUserIdentity, self).__init__(**kwargs)
        self.ip_address = kwargs.get('ip_address', None)
        self.user_principal_name = kwargs.get('user_principal_name', None)


class MicrosoftGraphUserSettings(MicrosoftGraphEntity):
    """userSettings.

    :param id: Read-only.
    :type id: str
    :param contribution_to_content_discovery_disabled:
    :type contribution_to_content_discovery_disabled: bool
    :param contribution_to_content_discovery_as_organization_disabled:
    :type contribution_to_content_discovery_as_organization_disabled: bool
    :param shift_preferences: shiftPreferences.
    :type shift_preferences: ~users_user.models.MicrosoftGraphShiftPreferences
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'contribution_to_content_discovery_disabled': {'key': 'contributionToContentDiscoveryDisabled', 'type': 'bool'},
        'contribution_to_content_discovery_as_organization_disabled': {'key': 'contributionToContentDiscoveryAsOrganizationDisabled', 'type': 'bool'},
        'shift_preferences': {'key': 'shiftPreferences', 'type': 'MicrosoftGraphShiftPreferences'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphUserSettings, self).__init__(**kwargs)
        self.contribution_to_content_discovery_disabled = kwargs.get('contribution_to_content_discovery_disabled', None)
        self.contribution_to_content_discovery_as_organization_disabled = kwargs.get('contribution_to_content_discovery_as_organization_disabled', None)
        self.shift_preferences = kwargs.get('shift_preferences', None)


class MicrosoftGraphUserTeamwork(MicrosoftGraphEntity):
    """userTeamwork.

    :param id: Read-only.
    :type id: str
    :param installed_apps:
    :type installed_apps: list[~users_user.models.MicrosoftGraphTeamsAppInstallation]
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'installed_apps': {'key': 'installedApps', 'type': '[MicrosoftGraphTeamsAppInstallation]'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphUserTeamwork, self).__init__(**kwargs)
        self.installed_apps = kwargs.get('installed_apps', None)


class MicrosoftGraphVersionAction(msrest.serialization.Model):
    """versionAction.

    :param new_version: The name of the new version that was created by this action.
    :type new_version: str
    """

    _attribute_map = {
        'new_version': {'key': 'newVersion', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphVersionAction, self).__init__(**kwargs)
        self.new_version = kwargs.get('new_version', None)


class MicrosoftGraphVideo(msrest.serialization.Model):
    """video.

    :param audio_bits_per_sample: Number of audio bits per sample.
    :type audio_bits_per_sample: int
    :param audio_channels: Number of audio channels.
    :type audio_channels: int
    :param audio_format: Name of the audio format (AAC, MP3, etc.).
    :type audio_format: str
    :param audio_samples_per_second: Number of audio samples per second.
    :type audio_samples_per_second: int
    :param bitrate: Bit rate of the video in bits per second.
    :type bitrate: int
    :param duration: Duration of the file in milliseconds.
    :type duration: long
    :param four_cc: 'Four character code' name of the video format.
    :type four_cc: str
    :param frame_rate: Frame rate of the video.
    :type frame_rate: float
    :param height: Height of the video, in pixels.
    :type height: int
    :param width: Width of the video, in pixels.
    :type width: int
    """

    _validation = {
        'audio_bits_per_sample': {'maximum': 2147483647, 'minimum': -2147483648},
        'audio_channels': {'maximum': 2147483647, 'minimum': -2147483648},
        'audio_samples_per_second': {'maximum': 2147483647, 'minimum': -2147483648},
        'bitrate': {'maximum': 2147483647, 'minimum': -2147483648},
        'height': {'maximum': 2147483647, 'minimum': -2147483648},
        'width': {'maximum': 2147483647, 'minimum': -2147483648},
    }

    _attribute_map = {
        'audio_bits_per_sample': {'key': 'audioBitsPerSample', 'type': 'int'},
        'audio_channels': {'key': 'audioChannels', 'type': 'int'},
        'audio_format': {'key': 'audioFormat', 'type': 'str'},
        'audio_samples_per_second': {'key': 'audioSamplesPerSecond', 'type': 'int'},
        'bitrate': {'key': 'bitrate', 'type': 'int'},
        'duration': {'key': 'duration', 'type': 'long'},
        'four_cc': {'key': 'fourCC', 'type': 'str'},
        'frame_rate': {'key': 'frameRate', 'type': 'float'},
        'height': {'key': 'height', 'type': 'int'},
        'width': {'key': 'width', 'type': 'int'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphVideo, self).__init__(**kwargs)
        self.audio_bits_per_sample = kwargs.get('audio_bits_per_sample', None)
        self.audio_channels = kwargs.get('audio_channels', None)
        self.audio_format = kwargs.get('audio_format', None)
        self.audio_samples_per_second = kwargs.get('audio_samples_per_second', None)
        self.bitrate = kwargs.get('bitrate', None)
        self.duration = kwargs.get('duration', None)
        self.four_cc = kwargs.get('four_cc', None)
        self.frame_rate = kwargs.get('frame_rate', None)
        self.height = kwargs.get('height', None)
        self.width = kwargs.get('width', None)


class MicrosoftGraphVisualInfo(msrest.serialization.Model):
    """visualInfo.

    :param attribution: imageInfo.
    :type attribution: ~users_user.models.MicrosoftGraphImageInfo
    :param background_color: Optional. Background color used to render the activity in the UI -
     brand color for the application source of the activity. Must be a valid hex color.
    :type background_color: str
    :param description: Optional. Longer text description of the user's unique activity (example:
     document name, first sentence, and/or metadata).
    :type description: str
    :param display_text: Required. Short text description of the user's unique activity (for
     example, document name in cases where an activity refers to document creation).
    :type display_text: str
    :param content: Any object.
    :type content: object
    """

    _attribute_map = {
        'attribution': {'key': 'attribution', 'type': 'MicrosoftGraphImageInfo'},
        'background_color': {'key': 'backgroundColor', 'type': 'str'},
        'description': {'key': 'description', 'type': 'str'},
        'display_text': {'key': 'displayText', 'type': 'str'},
        'content': {'key': 'content', 'type': 'object'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphVisualInfo, self).__init__(**kwargs)
        self.attribution = kwargs.get('attribution', None)
        self.background_color = kwargs.get('background_color', None)
        self.description = kwargs.get('description', None)
        self.display_text = kwargs.get('display_text', None)
        self.content = kwargs.get('content', None)


class MicrosoftGraphVisualProperties(msrest.serialization.Model):
    """visualProperties.

    :param title:
    :type title: str
    :param body:
    :type body: str
    """

    _attribute_map = {
        'title': {'key': 'title', 'type': 'str'},
        'body': {'key': 'body', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphVisualProperties, self).__init__(**kwargs)
        self.title = kwargs.get('title', None)
        self.body = kwargs.get('body', None)


class MicrosoftGraphWebAccount(MicrosoftGraphItemFacet):
    """webAccount.

    :param id: Read-only.
    :type id: str
    :param allowed_audiences: allowedAudiences. Possible values include: "me", "family",
     "contacts", "groupMembers", "organization", "federatedOrganizations", "everyone",
     "unknownFutureValue".
    :type allowed_audiences: str or ~users_user.models.MicrosoftGraphAllowedAudiences
    :param inference: inferenceData.
    :type inference: ~users_user.models.MicrosoftGraphInferenceData
    :param created_date_time:
    :type created_date_time: ~datetime.datetime
    :param created_by: identitySet.
    :type created_by: ~users_user.models.MicrosoftGraphIdentitySet
    :param last_modified_date_time:
    :type last_modified_date_time: ~datetime.datetime
    :param last_modified_by: identitySet.
    :type last_modified_by: ~users_user.models.MicrosoftGraphIdentitySet
    :param description:
    :type description: str
    :param user_id:
    :type user_id: str
    :param service: serviceInformation.
    :type service: ~users_user.models.MicrosoftGraphServiceInformation
    :param status_message:
    :type status_message: str
    :param web_url:
    :type web_url: str
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'allowed_audiences': {'key': 'allowedAudiences', 'type': 'str'},
        'inference': {'key': 'inference', 'type': 'MicrosoftGraphInferenceData'},
        'created_date_time': {'key': 'createdDateTime', 'type': 'iso-8601'},
        'created_by': {'key': 'createdBy', 'type': 'MicrosoftGraphIdentitySet'},
        'last_modified_date_time': {'key': 'lastModifiedDateTime', 'type': 'iso-8601'},
        'last_modified_by': {'key': 'lastModifiedBy', 'type': 'MicrosoftGraphIdentitySet'},
        'description': {'key': 'description', 'type': 'str'},
        'user_id': {'key': 'userId', 'type': 'str'},
        'service': {'key': 'service', 'type': 'MicrosoftGraphServiceInformation'},
        'status_message': {'key': 'statusMessage', 'type': 'str'},
        'web_url': {'key': 'webUrl', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphWebAccount, self).__init__(**kwargs)
        self.description = kwargs.get('description', None)
        self.user_id = kwargs.get('user_id', None)
        self.service = kwargs.get('service', None)
        self.status_message = kwargs.get('status_message', None)
        self.web_url = kwargs.get('web_url', None)


class MicrosoftGraphWebPart(msrest.serialization.Model):
    """webPart.

    :param type:
    :type type: str
    :param data: Any object.
    :type data: object
    """

    _attribute_map = {
        'type': {'key': 'type', 'type': 'str'},
        'data': {'key': 'data', 'type': 'object'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphWebPart, self).__init__(**kwargs)
        self.type = kwargs.get('type', None)
        self.data = kwargs.get('data', None)


class MicrosoftGraphWebsite(msrest.serialization.Model):
    """website.

    :param type: websiteType. Possible values include: "other", "home", "work", "blog", "profile".
    :type type: str or ~users_user.models.MicrosoftGraphWebsiteType
    :param address: The URL of the website.
    :type address: str
    :param display_name: The display name of the web site.
    :type display_name: str
    """

    _attribute_map = {
        'type': {'key': 'type', 'type': 'str'},
        'address': {'key': 'address', 'type': 'str'},
        'display_name': {'key': 'displayName', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphWebsite, self).__init__(**kwargs)
        self.type = kwargs.get('type', None)
        self.address = kwargs.get('address', None)
        self.display_name = kwargs.get('display_name', None)


class MicrosoftGraphWindowsDeviceMalwareState(MicrosoftGraphEntity):
    """Malware detection entity.

    :param id: Read-only.
    :type id: str
    :param display_name: Malware name.
    :type display_name: str
    :param additional_information_url: Information URL to learn more about the malware.
    :type additional_information_url: str
    :param severity: windowsMalwareSeverity. Possible values include: "unknown", "low", "moderate",
     "high", "severe".
    :type severity: str or ~users_user.models.MicrosoftGraphWindowsMalwareSeverity
    :param catetgory: windowsMalwareCategory. Possible values include: "invalid", "adware",
     "spyware", "passwordStealer", "trojanDownloader", "worm", "backdoor", "remoteAccessTrojan",
     "trojan", "emailFlooder", "keylogger", "dialer", "monitoringSoftware", "browserModifier",
     "cookie", "browserPlugin", "aolExploit", "nuker", "securityDisabler", "jokeProgram",
     "hostileActiveXControl", "softwareBundler", "stealthNotifier", "settingsModifier", "toolBar",
     "remoteControlSoftware", "trojanFtp", "potentialUnwantedSoftware", "icqExploit",
     "trojanTelnet", "exploit", "filesharingProgram", "malwareCreationTool", "tool",
     "trojanDenialOfService", "trojanDropper", "trojanMassMailer", "trojanMonitoringSoftware",
     "trojanProxyServer", "virus", "known", "unknown", "spp", "behavior", "vulnerability", "policy",
     "enterpriseUnwantedSoftware", "ransom", "hipsRule".
    :type catetgory: str or ~users_user.models.MicrosoftGraphWindowsMalwareCategory
    :param execution_state: windowsMalwareExecutionState. Possible values include: "unknown",
     "blocked", "allowed", "running", "notRunning".
    :type execution_state: str or ~users_user.models.MicrosoftGraphWindowsMalwareExecutionState
    :param state: windowsMalwareState. Possible values include: "unknown", "detected", "cleaned",
     "quarantined", "removed", "allowed", "blocked", "cleanFailed", "quarantineFailed",
     "removeFailed", "allowFailed", "abandoned", "blockFailed".
    :type state: str or ~users_user.models.MicrosoftGraphWindowsMalwareState
    :param threat_state: windowsMalwareThreatState. Possible values include: "active",
     "actionFailed", "manualStepsRequired", "fullScanRequired", "rebootRequired",
     "remediatedWithNonCriticalFailures", "quarantined", "removed", "cleaned", "allowed",
     "noStatusCleared".
    :type threat_state: str or ~users_user.models.MicrosoftGraphWindowsMalwareThreatState
    :param initial_detection_date_time: Initial detection datetime of the malware.
    :type initial_detection_date_time: ~datetime.datetime
    :param last_state_change_date_time: The last time this particular threat was changed.
    :type last_state_change_date_time: ~datetime.datetime
    :param detection_count: Number of times the malware is detected.
    :type detection_count: int
    :param category: windowsMalwareCategory. Possible values include: "invalid", "adware",
     "spyware", "passwordStealer", "trojanDownloader", "worm", "backdoor", "remoteAccessTrojan",
     "trojan", "emailFlooder", "keylogger", "dialer", "monitoringSoftware", "browserModifier",
     "cookie", "browserPlugin", "aolExploit", "nuker", "securityDisabler", "jokeProgram",
     "hostileActiveXControl", "softwareBundler", "stealthNotifier", "settingsModifier", "toolBar",
     "remoteControlSoftware", "trojanFtp", "potentialUnwantedSoftware", "icqExploit",
     "trojanTelnet", "exploit", "filesharingProgram", "malwareCreationTool", "tool",
     "trojanDenialOfService", "trojanDropper", "trojanMassMailer", "trojanMonitoringSoftware",
     "trojanProxyServer", "virus", "known", "unknown", "spp", "behavior", "vulnerability", "policy",
     "enterpriseUnwantedSoftware", "ransom", "hipsRule".
    :type category: str or ~users_user.models.MicrosoftGraphWindowsMalwareCategory
    """

    _validation = {
        'detection_count': {'maximum': 2147483647, 'minimum': -2147483648},
    }

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'display_name': {'key': 'displayName', 'type': 'str'},
        'additional_information_url': {'key': 'additionalInformationUrl', 'type': 'str'},
        'severity': {'key': 'severity', 'type': 'str'},
        'catetgory': {'key': 'catetgory', 'type': 'str'},
        'execution_state': {'key': 'executionState', 'type': 'str'},
        'state': {'key': 'state', 'type': 'str'},
        'threat_state': {'key': 'threatState', 'type': 'str'},
        'initial_detection_date_time': {'key': 'initialDetectionDateTime', 'type': 'iso-8601'},
        'last_state_change_date_time': {'key': 'lastStateChangeDateTime', 'type': 'iso-8601'},
        'detection_count': {'key': 'detectionCount', 'type': 'int'},
        'category': {'key': 'category', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphWindowsDeviceMalwareState, self).__init__(**kwargs)
        self.display_name = kwargs.get('display_name', None)
        self.additional_information_url = kwargs.get('additional_information_url', None)
        self.severity = kwargs.get('severity', None)
        self.catetgory = kwargs.get('catetgory', None)
        self.execution_state = kwargs.get('execution_state', None)
        self.state = kwargs.get('state', None)
        self.threat_state = kwargs.get('threat_state', None)
        self.initial_detection_date_time = kwargs.get('initial_detection_date_time', None)
        self.last_state_change_date_time = kwargs.get('last_state_change_date_time', None)
        self.detection_count = kwargs.get('detection_count', None)
        self.category = kwargs.get('category', None)


class MicrosoftGraphWindowsInformationProtectionDeviceRegistration(MicrosoftGraphEntity):
    """Represents device registration records for Bring-Your-Own-Device(BYOD) Windows devices.

    :param id: Read-only.
    :type id: str
    :param user_id: UserId associated with this device registration record.
    :type user_id: str
    :param device_registration_id: Device identifier for this device registration record.
    :type device_registration_id: str
    :param device_name: Device name.
    :type device_name: str
    :param device_type: Device type, for example, Windows laptop VS Windows phone.
    :type device_type: str
    :param device_mac_address: Device Mac address.
    :type device_mac_address: str
    :param last_check_in_date_time: Last checkin time of the device.
    :type last_check_in_date_time: ~datetime.datetime
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'user_id': {'key': 'userId', 'type': 'str'},
        'device_registration_id': {'key': 'deviceRegistrationId', 'type': 'str'},
        'device_name': {'key': 'deviceName', 'type': 'str'},
        'device_type': {'key': 'deviceType', 'type': 'str'},
        'device_mac_address': {'key': 'deviceMacAddress', 'type': 'str'},
        'last_check_in_date_time': {'key': 'lastCheckInDateTime', 'type': 'iso-8601'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphWindowsInformationProtectionDeviceRegistration, self).__init__(**kwargs)
        self.user_id = kwargs.get('user_id', None)
        self.device_registration_id = kwargs.get('device_registration_id', None)
        self.device_name = kwargs.get('device_name', None)
        self.device_type = kwargs.get('device_type', None)
        self.device_mac_address = kwargs.get('device_mac_address', None)
        self.last_check_in_date_time = kwargs.get('last_check_in_date_time', None)


class MicrosoftGraphWindowsProtectionState(MicrosoftGraphEntity):
    """Device protection status entity.

    :param id: Read-only.
    :type id: str
    :param malware_protection_enabled: Anti malware is enabled or not.
    :type malware_protection_enabled: bool
    :param device_state: windowsDeviceHealthState. Possible values include: "clean",
     "fullScanPending", "rebootPending", "manualStepsPending", "offlineScanPending", "critical".
    :type device_state: str or ~users_user.models.MicrosoftGraphWindowsDeviceHealthState
    :param real_time_protection_enabled: Real time protection is enabled or not?.
    :type real_time_protection_enabled: bool
    :param network_inspection_system_enabled: Network inspection system enabled or not?.
    :type network_inspection_system_enabled: bool
    :param quick_scan_overdue: Quick scan overdue or not?.
    :type quick_scan_overdue: bool
    :param full_scan_overdue: Full scan overdue or not?.
    :type full_scan_overdue: bool
    :param signature_update_overdue: Signature out of date or not?.
    :type signature_update_overdue: bool
    :param reboot_required: Reboot required or not?.
    :type reboot_required: bool
    :param full_scan_required: Full scan required or not?.
    :type full_scan_required: bool
    :param engine_version: Current endpoint protection engine's version.
    :type engine_version: str
    :param signature_version: Current malware definitions version.
    :type signature_version: str
    :param anti_malware_version: Current anti malware version.
    :type anti_malware_version: str
    :param last_quick_scan_date_time: Last quick scan datetime.
    :type last_quick_scan_date_time: ~datetime.datetime
    :param last_full_scan_date_time: Last quick scan datetime.
    :type last_full_scan_date_time: ~datetime.datetime
    :param last_quick_scan_signature_version: Last quick scan signature version.
    :type last_quick_scan_signature_version: str
    :param last_full_scan_signature_version: Last full scan signature version.
    :type last_full_scan_signature_version: str
    :param last_reported_date_time: Last device health status reported time.
    :type last_reported_date_time: ~datetime.datetime
    :param detected_malware_state: Device malware list.
    :type detected_malware_state: list[~users_user.models.MicrosoftGraphWindowsDeviceMalwareState]
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'malware_protection_enabled': {'key': 'malwareProtectionEnabled', 'type': 'bool'},
        'device_state': {'key': 'deviceState', 'type': 'str'},
        'real_time_protection_enabled': {'key': 'realTimeProtectionEnabled', 'type': 'bool'},
        'network_inspection_system_enabled': {'key': 'networkInspectionSystemEnabled', 'type': 'bool'},
        'quick_scan_overdue': {'key': 'quickScanOverdue', 'type': 'bool'},
        'full_scan_overdue': {'key': 'fullScanOverdue', 'type': 'bool'},
        'signature_update_overdue': {'key': 'signatureUpdateOverdue', 'type': 'bool'},
        'reboot_required': {'key': 'rebootRequired', 'type': 'bool'},
        'full_scan_required': {'key': 'fullScanRequired', 'type': 'bool'},
        'engine_version': {'key': 'engineVersion', 'type': 'str'},
        'signature_version': {'key': 'signatureVersion', 'type': 'str'},
        'anti_malware_version': {'key': 'antiMalwareVersion', 'type': 'str'},
        'last_quick_scan_date_time': {'key': 'lastQuickScanDateTime', 'type': 'iso-8601'},
        'last_full_scan_date_time': {'key': 'lastFullScanDateTime', 'type': 'iso-8601'},
        'last_quick_scan_signature_version': {'key': 'lastQuickScanSignatureVersion', 'type': 'str'},
        'last_full_scan_signature_version': {'key': 'lastFullScanSignatureVersion', 'type': 'str'},
        'last_reported_date_time': {'key': 'lastReportedDateTime', 'type': 'iso-8601'},
        'detected_malware_state': {'key': 'detectedMalwareState', 'type': '[MicrosoftGraphWindowsDeviceMalwareState]'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphWindowsProtectionState, self).__init__(**kwargs)
        self.malware_protection_enabled = kwargs.get('malware_protection_enabled', None)
        self.device_state = kwargs.get('device_state', None)
        self.real_time_protection_enabled = kwargs.get('real_time_protection_enabled', None)
        self.network_inspection_system_enabled = kwargs.get('network_inspection_system_enabled', None)
        self.quick_scan_overdue = kwargs.get('quick_scan_overdue', None)
        self.full_scan_overdue = kwargs.get('full_scan_overdue', None)
        self.signature_update_overdue = kwargs.get('signature_update_overdue', None)
        self.reboot_required = kwargs.get('reboot_required', None)
        self.full_scan_required = kwargs.get('full_scan_required', None)
        self.engine_version = kwargs.get('engine_version', None)
        self.signature_version = kwargs.get('signature_version', None)
        self.anti_malware_version = kwargs.get('anti_malware_version', None)
        self.last_quick_scan_date_time = kwargs.get('last_quick_scan_date_time', None)
        self.last_full_scan_date_time = kwargs.get('last_full_scan_date_time', None)
        self.last_quick_scan_signature_version = kwargs.get('last_quick_scan_signature_version', None)
        self.last_full_scan_signature_version = kwargs.get('last_full_scan_signature_version', None)
        self.last_reported_date_time = kwargs.get('last_reported_date_time', None)
        self.detected_malware_state = kwargs.get('detected_malware_state', None)


class MicrosoftGraphWorkbook(MicrosoftGraphEntity):
    """workbook.

    :param id: Read-only.
    :type id: str
    :param application: workbookApplication.
    :type application: ~users_user.models.MicrosoftGraphWorkbookApplication
    :param names: Represents a collection of workbook scoped named items (named ranges and
     constants). Read-only.
    :type names: list[~users_user.models.MicrosoftGraphWorkbookNamedItem]
    :param tables: Represents a collection of tables associated with the workbook. Read-only.
    :type tables: list[~users_user.models.MicrosoftGraphWorkbookTable]
    :param worksheets: Represents a collection of worksheets associated with the workbook. Read-
     only.
    :type worksheets: list[~users_user.models.MicrosoftGraphWorkbookWorksheet]
    :param comments:
    :type comments: list[~users_user.models.MicrosoftGraphWorkbookComment]
    :param functions: entity.
    :type functions: ~users_user.models.MicrosoftGraphEntity
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'application': {'key': 'application', 'type': 'MicrosoftGraphWorkbookApplication'},
        'names': {'key': 'names', 'type': '[MicrosoftGraphWorkbookNamedItem]'},
        'tables': {'key': 'tables', 'type': '[MicrosoftGraphWorkbookTable]'},
        'worksheets': {'key': 'worksheets', 'type': '[MicrosoftGraphWorkbookWorksheet]'},
        'comments': {'key': 'comments', 'type': '[MicrosoftGraphWorkbookComment]'},
        'functions': {'key': 'functions', 'type': 'MicrosoftGraphEntity'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphWorkbook, self).__init__(**kwargs)
        self.application = kwargs.get('application', None)
        self.names = kwargs.get('names', None)
        self.tables = kwargs.get('tables', None)
        self.worksheets = kwargs.get('worksheets', None)
        self.comments = kwargs.get('comments', None)
        self.functions = kwargs.get('functions', None)


class MicrosoftGraphWorkbookApplication(MicrosoftGraphEntity):
    """workbookApplication.

    :param id: Read-only.
    :type id: str
    :param calculation_mode: Returns the calculation mode used in the workbook. Possible values
     are: Automatic, AutomaticExceptTables, Manual.
    :type calculation_mode: str
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'calculation_mode': {'key': 'calculationMode', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphWorkbookApplication, self).__init__(**kwargs)
        self.calculation_mode = kwargs.get('calculation_mode', None)


class MicrosoftGraphWorkbookChart(MicrosoftGraphEntity):
    """workbookChart.

    :param id: Read-only.
    :type id: str
    :param height: Represents the height, in points, of the chart object.
    :type height: float
    :param left: The distance, in points, from the left side of the chart to the worksheet origin.
    :type left: float
    :param name: Represents the name of a chart object.
    :type name: str
    :param top: Represents the distance, in points, from the top edge of the object to the top of
     row 1 (on a worksheet) or the top of the chart area (on a chart).
    :type top: float
    :param width: Represents the width, in points, of the chart object.
    :type width: float
    :param axes: workbookChartAxes.
    :type axes: ~users_user.models.MicrosoftGraphWorkbookChartAxes
    :param data_labels: workbookChartDataLabels.
    :type data_labels: ~users_user.models.MicrosoftGraphWorkbookChartDataLabels
    :param format: workbookChartAreaFormat.
    :type format: ~users_user.models.MicrosoftGraphWorkbookChartAreaFormat
    :param legend: workbookChartLegend.
    :type legend: ~users_user.models.MicrosoftGraphWorkbookChartLegend
    :param series: Represents either a single series or collection of series in the chart. Read-
     only.
    :type series: list[~users_user.models.MicrosoftGraphWorkbookChartSeries]
    :param title: workbookChartTitle.
    :type title: ~users_user.models.MicrosoftGraphWorkbookChartTitle
    :param worksheet: workbookWorksheet.
    :type worksheet: ~users_user.models.MicrosoftGraphWorkbookWorksheet
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'height': {'key': 'height', 'type': 'float'},
        'left': {'key': 'left', 'type': 'float'},
        'name': {'key': 'name', 'type': 'str'},
        'top': {'key': 'top', 'type': 'float'},
        'width': {'key': 'width', 'type': 'float'},
        'axes': {'key': 'axes', 'type': 'MicrosoftGraphWorkbookChartAxes'},
        'data_labels': {'key': 'dataLabels', 'type': 'MicrosoftGraphWorkbookChartDataLabels'},
        'format': {'key': 'format', 'type': 'MicrosoftGraphWorkbookChartAreaFormat'},
        'legend': {'key': 'legend', 'type': 'MicrosoftGraphWorkbookChartLegend'},
        'series': {'key': 'series', 'type': '[MicrosoftGraphWorkbookChartSeries]'},
        'title': {'key': 'title', 'type': 'MicrosoftGraphWorkbookChartTitle'},
        'worksheet': {'key': 'worksheet', 'type': 'MicrosoftGraphWorkbookWorksheet'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphWorkbookChart, self).__init__(**kwargs)
        self.height = kwargs.get('height', None)
        self.left = kwargs.get('left', None)
        self.name = kwargs.get('name', None)
        self.top = kwargs.get('top', None)
        self.width = kwargs.get('width', None)
        self.axes = kwargs.get('axes', None)
        self.data_labels = kwargs.get('data_labels', None)
        self.format = kwargs.get('format', None)
        self.legend = kwargs.get('legend', None)
        self.series = kwargs.get('series', None)
        self.title = kwargs.get('title', None)
        self.worksheet = kwargs.get('worksheet', None)


class MicrosoftGraphWorkbookChartAreaFormat(MicrosoftGraphEntity):
    """workbookChartAreaFormat.

    :param id: Read-only.
    :type id: str
    :param fill: entity.
    :type fill: ~users_user.models.MicrosoftGraphEntity
    :param font: workbookChartFont.
    :type font: ~users_user.models.MicrosoftGraphWorkbookChartFont
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'fill': {'key': 'fill', 'type': 'MicrosoftGraphEntity'},
        'font': {'key': 'font', 'type': 'MicrosoftGraphWorkbookChartFont'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphWorkbookChartAreaFormat, self).__init__(**kwargs)
        self.fill = kwargs.get('fill', None)
        self.font = kwargs.get('font', None)


class MicrosoftGraphWorkbookChartAxes(MicrosoftGraphEntity):
    """workbookChartAxes.

    :param id: Read-only.
    :type id: str
    :param category_axis: workbookChartAxis.
    :type category_axis: ~users_user.models.MicrosoftGraphWorkbookChartAxis
    :param series_axis: workbookChartAxis.
    :type series_axis: ~users_user.models.MicrosoftGraphWorkbookChartAxis
    :param value_axis: workbookChartAxis.
    :type value_axis: ~users_user.models.MicrosoftGraphWorkbookChartAxis
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'category_axis': {'key': 'categoryAxis', 'type': 'MicrosoftGraphWorkbookChartAxis'},
        'series_axis': {'key': 'seriesAxis', 'type': 'MicrosoftGraphWorkbookChartAxis'},
        'value_axis': {'key': 'valueAxis', 'type': 'MicrosoftGraphWorkbookChartAxis'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphWorkbookChartAxes, self).__init__(**kwargs)
        self.category_axis = kwargs.get('category_axis', None)
        self.series_axis = kwargs.get('series_axis', None)
        self.value_axis = kwargs.get('value_axis', None)


class MicrosoftGraphWorkbookChartAxis(MicrosoftGraphEntity):
    """workbookChartAxis.

    :param id: Read-only.
    :type id: str
    :param major_unit: Any object.
    :type major_unit: object
    :param maximum: Any object.
    :type maximum: object
    :param minimum: Any object.
    :type minimum: object
    :param minor_unit: Any object.
    :type minor_unit: object
    :param format: workbookChartAxisFormat.
    :type format: ~users_user.models.MicrosoftGraphWorkbookChartAxisFormat
    :param major_gridlines: workbookChartGridlines.
    :type major_gridlines: ~users_user.models.MicrosoftGraphWorkbookChartGridlines
    :param minor_gridlines: workbookChartGridlines.
    :type minor_gridlines: ~users_user.models.MicrosoftGraphWorkbookChartGridlines
    :param title: workbookChartAxisTitle.
    :type title: ~users_user.models.MicrosoftGraphWorkbookChartAxisTitle
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'major_unit': {'key': 'majorUnit', 'type': 'object'},
        'maximum': {'key': 'maximum', 'type': 'object'},
        'minimum': {'key': 'minimum', 'type': 'object'},
        'minor_unit': {'key': 'minorUnit', 'type': 'object'},
        'format': {'key': 'format', 'type': 'MicrosoftGraphWorkbookChartAxisFormat'},
        'major_gridlines': {'key': 'majorGridlines', 'type': 'MicrosoftGraphWorkbookChartGridlines'},
        'minor_gridlines': {'key': 'minorGridlines', 'type': 'MicrosoftGraphWorkbookChartGridlines'},
        'title': {'key': 'title', 'type': 'MicrosoftGraphWorkbookChartAxisTitle'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphWorkbookChartAxis, self).__init__(**kwargs)
        self.major_unit = kwargs.get('major_unit', None)
        self.maximum = kwargs.get('maximum', None)
        self.minimum = kwargs.get('minimum', None)
        self.minor_unit = kwargs.get('minor_unit', None)
        self.format = kwargs.get('format', None)
        self.major_gridlines = kwargs.get('major_gridlines', None)
        self.minor_gridlines = kwargs.get('minor_gridlines', None)
        self.title = kwargs.get('title', None)


class MicrosoftGraphWorkbookChartAxisFormat(MicrosoftGraphEntity):
    """workbookChartAxisFormat.

    :param id: Read-only.
    :type id: str
    :param font: workbookChartFont.
    :type font: ~users_user.models.MicrosoftGraphWorkbookChartFont
    :param line: workbookChartLineFormat.
    :type line: ~users_user.models.MicrosoftGraphWorkbookChartLineFormat
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'font': {'key': 'font', 'type': 'MicrosoftGraphWorkbookChartFont'},
        'line': {'key': 'line', 'type': 'MicrosoftGraphWorkbookChartLineFormat'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphWorkbookChartAxisFormat, self).__init__(**kwargs)
        self.font = kwargs.get('font', None)
        self.line = kwargs.get('line', None)


class MicrosoftGraphWorkbookChartAxisTitle(MicrosoftGraphEntity):
    """workbookChartAxisTitle.

    :param id: Read-only.
    :type id: str
    :param text: Represents the axis title.
    :type text: str
    :param visible: A boolean that specifies the visibility of an axis title.
    :type visible: bool
    :param format: workbookChartAxisTitleFormat.
    :type format: ~users_user.models.MicrosoftGraphWorkbookChartAxisTitleFormat
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'text': {'key': 'text', 'type': 'str'},
        'visible': {'key': 'visible', 'type': 'bool'},
        'format': {'key': 'format', 'type': 'MicrosoftGraphWorkbookChartAxisTitleFormat'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphWorkbookChartAxisTitle, self).__init__(**kwargs)
        self.text = kwargs.get('text', None)
        self.visible = kwargs.get('visible', None)
        self.format = kwargs.get('format', None)


class MicrosoftGraphWorkbookChartAxisTitleFormat(MicrosoftGraphEntity):
    """workbookChartAxisTitleFormat.

    :param id: Read-only.
    :type id: str
    :param font: workbookChartFont.
    :type font: ~users_user.models.MicrosoftGraphWorkbookChartFont
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'font': {'key': 'font', 'type': 'MicrosoftGraphWorkbookChartFont'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphWorkbookChartAxisTitleFormat, self).__init__(**kwargs)
        self.font = kwargs.get('font', None)


class MicrosoftGraphWorkbookChartDataLabelFormat(MicrosoftGraphEntity):
    """workbookChartDataLabelFormat.

    :param id: Read-only.
    :type id: str
    :param fill: entity.
    :type fill: ~users_user.models.MicrosoftGraphEntity
    :param font: workbookChartFont.
    :type font: ~users_user.models.MicrosoftGraphWorkbookChartFont
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'fill': {'key': 'fill', 'type': 'MicrosoftGraphEntity'},
        'font': {'key': 'font', 'type': 'MicrosoftGraphWorkbookChartFont'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphWorkbookChartDataLabelFormat, self).__init__(**kwargs)
        self.fill = kwargs.get('fill', None)
        self.font = kwargs.get('font', None)


class MicrosoftGraphWorkbookChartDataLabels(MicrosoftGraphEntity):
    """workbookChartDataLabels.

    :param id: Read-only.
    :type id: str
    :param position: DataLabelPosition value that represents the position of the data label. The
     possible values are: None, Center, InsideEnd, InsideBase, OutsideEnd, Left, Right, Top, Bottom,
     BestFit, Callout.
    :type position: str
    :param separator: String representing the separator used for the data labels on a chart.
    :type separator: str
    :param show_bubble_size: Boolean value representing if the data label bubble size is visible or
     not.
    :type show_bubble_size: bool
    :param show_category_name: Boolean value representing if the data label category name is
     visible or not.
    :type show_category_name: bool
    :param show_legend_key: Boolean value representing if the data label legend key is visible or
     not.
    :type show_legend_key: bool
    :param show_percentage: Boolean value representing if the data label percentage is visible or
     not.
    :type show_percentage: bool
    :param show_series_name: Boolean value representing if the data label series name is visible or
     not.
    :type show_series_name: bool
    :param show_value: Boolean value representing if the data label value is visible or not.
    :type show_value: bool
    :param format: workbookChartDataLabelFormat.
    :type format: ~users_user.models.MicrosoftGraphWorkbookChartDataLabelFormat
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'position': {'key': 'position', 'type': 'str'},
        'separator': {'key': 'separator', 'type': 'str'},
        'show_bubble_size': {'key': 'showBubbleSize', 'type': 'bool'},
        'show_category_name': {'key': 'showCategoryName', 'type': 'bool'},
        'show_legend_key': {'key': 'showLegendKey', 'type': 'bool'},
        'show_percentage': {'key': 'showPercentage', 'type': 'bool'},
        'show_series_name': {'key': 'showSeriesName', 'type': 'bool'},
        'show_value': {'key': 'showValue', 'type': 'bool'},
        'format': {'key': 'format', 'type': 'MicrosoftGraphWorkbookChartDataLabelFormat'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphWorkbookChartDataLabels, self).__init__(**kwargs)
        self.position = kwargs.get('position', None)
        self.separator = kwargs.get('separator', None)
        self.show_bubble_size = kwargs.get('show_bubble_size', None)
        self.show_category_name = kwargs.get('show_category_name', None)
        self.show_legend_key = kwargs.get('show_legend_key', None)
        self.show_percentage = kwargs.get('show_percentage', None)
        self.show_series_name = kwargs.get('show_series_name', None)
        self.show_value = kwargs.get('show_value', None)
        self.format = kwargs.get('format', None)


class MicrosoftGraphWorkbookChartFill(MicrosoftGraphEntity):
    """workbookChartFill.

    :param id: Read-only.
    :type id: str
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphWorkbookChartFill, self).__init__(**kwargs)


class MicrosoftGraphWorkbookChartFont(MicrosoftGraphEntity):
    """workbookChartFont.

    :param id: Read-only.
    :type id: str
    :param bold: Represents the bold status of font.
    :type bold: bool
    :param color: HTML color code representation of the text color. E.g. #FF0000 represents Red.
    :type color: str
    :param italic: Represents the italic status of the font.
    :type italic: bool
    :param name: Font name (e.g. 'Calibri').
    :type name: str
    :param size: Size of the font (e.g. 11).
    :type size: float
    :param underline: Type of underline applied to the font. The possible values are: None, Single.
    :type underline: str
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'bold': {'key': 'bold', 'type': 'bool'},
        'color': {'key': 'color', 'type': 'str'},
        'italic': {'key': 'italic', 'type': 'bool'},
        'name': {'key': 'name', 'type': 'str'},
        'size': {'key': 'size', 'type': 'float'},
        'underline': {'key': 'underline', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphWorkbookChartFont, self).__init__(**kwargs)
        self.bold = kwargs.get('bold', None)
        self.color = kwargs.get('color', None)
        self.italic = kwargs.get('italic', None)
        self.name = kwargs.get('name', None)
        self.size = kwargs.get('size', None)
        self.underline = kwargs.get('underline', None)


class MicrosoftGraphWorkbookChartGridlines(MicrosoftGraphEntity):
    """workbookChartGridlines.

    :param id: Read-only.
    :type id: str
    :param visible: Boolean value representing if the axis gridlines are visible or not.
    :type visible: bool
    :param format: workbookChartGridlinesFormat.
    :type format: ~users_user.models.MicrosoftGraphWorkbookChartGridlinesFormat
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'visible': {'key': 'visible', 'type': 'bool'},
        'format': {'key': 'format', 'type': 'MicrosoftGraphWorkbookChartGridlinesFormat'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphWorkbookChartGridlines, self).__init__(**kwargs)
        self.visible = kwargs.get('visible', None)
        self.format = kwargs.get('format', None)


class MicrosoftGraphWorkbookChartGridlinesFormat(MicrosoftGraphEntity):
    """workbookChartGridlinesFormat.

    :param id: Read-only.
    :type id: str
    :param line: workbookChartLineFormat.
    :type line: ~users_user.models.MicrosoftGraphWorkbookChartLineFormat
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'line': {'key': 'line', 'type': 'MicrosoftGraphWorkbookChartLineFormat'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphWorkbookChartGridlinesFormat, self).__init__(**kwargs)
        self.line = kwargs.get('line', None)


class MicrosoftGraphWorkbookChartLegend(MicrosoftGraphEntity):
    """workbookChartLegend.

    :param id: Read-only.
    :type id: str
    :param overlay: Boolean value for whether the chart legend should overlap with the main body of
     the chart.
    :type overlay: bool
    :param position: Represents the position of the legend on the chart. The possible values are:
     Top, Bottom, Left, Right, Corner, Custom.
    :type position: str
    :param visible: A boolean value the represents the visibility of a ChartLegend object.
    :type visible: bool
    :param format: workbookChartLegendFormat.
    :type format: ~users_user.models.MicrosoftGraphWorkbookChartLegendFormat
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'overlay': {'key': 'overlay', 'type': 'bool'},
        'position': {'key': 'position', 'type': 'str'},
        'visible': {'key': 'visible', 'type': 'bool'},
        'format': {'key': 'format', 'type': 'MicrosoftGraphWorkbookChartLegendFormat'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphWorkbookChartLegend, self).__init__(**kwargs)
        self.overlay = kwargs.get('overlay', None)
        self.position = kwargs.get('position', None)
        self.visible = kwargs.get('visible', None)
        self.format = kwargs.get('format', None)


class MicrosoftGraphWorkbookChartLegendFormat(MicrosoftGraphEntity):
    """workbookChartLegendFormat.

    :param id: Read-only.
    :type id: str
    :param fill: entity.
    :type fill: ~users_user.models.MicrosoftGraphEntity
    :param font: workbookChartFont.
    :type font: ~users_user.models.MicrosoftGraphWorkbookChartFont
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'fill': {'key': 'fill', 'type': 'MicrosoftGraphEntity'},
        'font': {'key': 'font', 'type': 'MicrosoftGraphWorkbookChartFont'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphWorkbookChartLegendFormat, self).__init__(**kwargs)
        self.fill = kwargs.get('fill', None)
        self.font = kwargs.get('font', None)


class MicrosoftGraphWorkbookChartLineFormat(MicrosoftGraphEntity):
    """workbookChartLineFormat.

    :param id: Read-only.
    :type id: str
    :param color: HTML color code representing the color of lines in the chart.
    :type color: str
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'color': {'key': 'color', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphWorkbookChartLineFormat, self).__init__(**kwargs)
        self.color = kwargs.get('color', None)


class MicrosoftGraphWorkbookChartPoint(MicrosoftGraphEntity):
    """workbookChartPoint.

    :param id: Read-only.
    :type id: str
    :param value: Any object.
    :type value: object
    :param format: workbookChartPointFormat.
    :type format: ~users_user.models.MicrosoftGraphWorkbookChartPointFormat
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'value': {'key': 'value', 'type': 'object'},
        'format': {'key': 'format', 'type': 'MicrosoftGraphWorkbookChartPointFormat'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphWorkbookChartPoint, self).__init__(**kwargs)
        self.value = kwargs.get('value', None)
        self.format = kwargs.get('format', None)


class MicrosoftGraphWorkbookChartPointFormat(MicrosoftGraphEntity):
    """workbookChartPointFormat.

    :param id: Read-only.
    :type id: str
    :param fill: entity.
    :type fill: ~users_user.models.MicrosoftGraphEntity
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'fill': {'key': 'fill', 'type': 'MicrosoftGraphEntity'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphWorkbookChartPointFormat, self).__init__(**kwargs)
        self.fill = kwargs.get('fill', None)


class MicrosoftGraphWorkbookChartSeries(MicrosoftGraphEntity):
    """workbookChartSeries.

    :param id: Read-only.
    :type id: str
    :param name: Represents the name of a series in a chart.
    :type name: str
    :param format: workbookChartSeriesFormat.
    :type format: ~users_user.models.MicrosoftGraphWorkbookChartSeriesFormat
    :param points: Represents a collection of all points in the series. Read-only.
    :type points: list[~users_user.models.MicrosoftGraphWorkbookChartPoint]
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'name': {'key': 'name', 'type': 'str'},
        'format': {'key': 'format', 'type': 'MicrosoftGraphWorkbookChartSeriesFormat'},
        'points': {'key': 'points', 'type': '[MicrosoftGraphWorkbookChartPoint]'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphWorkbookChartSeries, self).__init__(**kwargs)
        self.name = kwargs.get('name', None)
        self.format = kwargs.get('format', None)
        self.points = kwargs.get('points', None)


class MicrosoftGraphWorkbookChartSeriesFormat(MicrosoftGraphEntity):
    """workbookChartSeriesFormat.

    :param id: Read-only.
    :type id: str
    :param fill: entity.
    :type fill: ~users_user.models.MicrosoftGraphEntity
    :param line: workbookChartLineFormat.
    :type line: ~users_user.models.MicrosoftGraphWorkbookChartLineFormat
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'fill': {'key': 'fill', 'type': 'MicrosoftGraphEntity'},
        'line': {'key': 'line', 'type': 'MicrosoftGraphWorkbookChartLineFormat'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphWorkbookChartSeriesFormat, self).__init__(**kwargs)
        self.fill = kwargs.get('fill', None)
        self.line = kwargs.get('line', None)


class MicrosoftGraphWorkbookChartTitle(MicrosoftGraphEntity):
    """workbookChartTitle.

    :param id: Read-only.
    :type id: str
    :param overlay: Boolean value representing if the chart title will overlay the chart or not.
    :type overlay: bool
    :param text: Represents the title text of a chart.
    :type text: str
    :param visible: A boolean value the represents the visibility of a chart title object.
    :type visible: bool
    :param format: workbookChartTitleFormat.
    :type format: ~users_user.models.MicrosoftGraphWorkbookChartTitleFormat
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'overlay': {'key': 'overlay', 'type': 'bool'},
        'text': {'key': 'text', 'type': 'str'},
        'visible': {'key': 'visible', 'type': 'bool'},
        'format': {'key': 'format', 'type': 'MicrosoftGraphWorkbookChartTitleFormat'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphWorkbookChartTitle, self).__init__(**kwargs)
        self.overlay = kwargs.get('overlay', None)
        self.text = kwargs.get('text', None)
        self.visible = kwargs.get('visible', None)
        self.format = kwargs.get('format', None)


class MicrosoftGraphWorkbookChartTitleFormat(MicrosoftGraphEntity):
    """workbookChartTitleFormat.

    :param id: Read-only.
    :type id: str
    :param fill: entity.
    :type fill: ~users_user.models.MicrosoftGraphEntity
    :param font: workbookChartFont.
    :type font: ~users_user.models.MicrosoftGraphWorkbookChartFont
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'fill': {'key': 'fill', 'type': 'MicrosoftGraphEntity'},
        'font': {'key': 'font', 'type': 'MicrosoftGraphWorkbookChartFont'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphWorkbookChartTitleFormat, self).__init__(**kwargs)
        self.fill = kwargs.get('fill', None)
        self.font = kwargs.get('font', None)


class MicrosoftGraphWorkbookComment(MicrosoftGraphEntity):
    """workbookComment.

    :param id: Read-only.
    :type id: str
    :param content: The content of comment.
    :type content: str
    :param content_type: Indicates the type for the comment.
    :type content_type: str
    :param replies: Read-only. Nullable.
    :type replies: list[~users_user.models.MicrosoftGraphWorkbookCommentReply]
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'content': {'key': 'content', 'type': 'str'},
        'content_type': {'key': 'contentType', 'type': 'str'},
        'replies': {'key': 'replies', 'type': '[MicrosoftGraphWorkbookCommentReply]'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphWorkbookComment, self).__init__(**kwargs)
        self.content = kwargs.get('content', None)
        self.content_type = kwargs.get('content_type', None)
        self.replies = kwargs.get('replies', None)


class MicrosoftGraphWorkbookCommentReply(MicrosoftGraphEntity):
    """workbookCommentReply.

    :param id: Read-only.
    :type id: str
    :param content: The content of a comment reply.
    :type content: str
    :param content_type: Indicates the type for the comment reply.
    :type content_type: str
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'content': {'key': 'content', 'type': 'str'},
        'content_type': {'key': 'contentType', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphWorkbookCommentReply, self).__init__(**kwargs)
        self.content = kwargs.get('content', None)
        self.content_type = kwargs.get('content_type', None)


class MicrosoftGraphWorkbookFilter(MicrosoftGraphEntity):
    """workbookFilter.

    :param id: Read-only.
    :type id: str
    :param criteria: workbookFilterCriteria.
    :type criteria: ~users_user.models.MicrosoftGraphWorkbookFilterCriteria
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'criteria': {'key': 'criteria', 'type': 'MicrosoftGraphWorkbookFilterCriteria'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphWorkbookFilter, self).__init__(**kwargs)
        self.criteria = kwargs.get('criteria', None)


class MicrosoftGraphWorkbookFilterCriteria(msrest.serialization.Model):
    """workbookFilterCriteria.

    :param color:
    :type color: str
    :param criterion1:
    :type criterion1: str
    :param criterion2:
    :type criterion2: str
    :param dynamic_criteria:
    :type dynamic_criteria: str
    :param filter_on:
    :type filter_on: str
    :param icon: workbookIcon.
    :type icon: ~users_user.models.MicrosoftGraphWorkbookIcon
    :param operator:
    :type operator: str
    :param values: Any object.
    :type values: object
    """

    _attribute_map = {
        'color': {'key': 'color', 'type': 'str'},
        'criterion1': {'key': 'criterion1', 'type': 'str'},
        'criterion2': {'key': 'criterion2', 'type': 'str'},
        'dynamic_criteria': {'key': 'dynamicCriteria', 'type': 'str'},
        'filter_on': {'key': 'filterOn', 'type': 'str'},
        'icon': {'key': 'icon', 'type': 'MicrosoftGraphWorkbookIcon'},
        'operator': {'key': 'operator', 'type': 'str'},
        'values': {'key': 'values', 'type': 'object'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphWorkbookFilterCriteria, self).__init__(**kwargs)
        self.color = kwargs.get('color', None)
        self.criterion1 = kwargs.get('criterion1', None)
        self.criterion2 = kwargs.get('criterion2', None)
        self.dynamic_criteria = kwargs.get('dynamic_criteria', None)
        self.filter_on = kwargs.get('filter_on', None)
        self.icon = kwargs.get('icon', None)
        self.operator = kwargs.get('operator', None)
        self.values = kwargs.get('values', None)


class MicrosoftGraphWorkbookFunctions(MicrosoftGraphEntity):
    """workbookFunctions.

    :param id: Read-only.
    :type id: str
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphWorkbookFunctions, self).__init__(**kwargs)


class MicrosoftGraphWorkbookIcon(msrest.serialization.Model):
    """workbookIcon.

    :param index: Represents the index of the icon in the given set.
    :type index: int
    :param set: Represents the set that the icon is part of. The possible values are: Invalid,
     ThreeArrows, ThreeArrowsGray, ThreeFlags, ThreeTrafficLights1, ThreeTrafficLights2, ThreeSigns,
     ThreeSymbols, ThreeSymbols2, FourArrows, FourArrowsGray, FourRedToBlack, FourRating,
     FourTrafficLights, FiveArrows, FiveArrowsGray, FiveRating, FiveQuarters, ThreeStars,
     ThreeTriangles, FiveBoxes.
    :type set: str
    """

    _validation = {
        'index': {'maximum': 2147483647, 'minimum': -2147483648},
    }

    _attribute_map = {
        'index': {'key': 'index', 'type': 'int'},
        'set': {'key': 'set', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphWorkbookIcon, self).__init__(**kwargs)
        self.index = kwargs.get('index', None)
        self.set = kwargs.get('set', None)


class MicrosoftGraphWorkbookNamedItem(MicrosoftGraphEntity):
    """workbookNamedItem.

    :param id: Read-only.
    :type id: str
    :param comment: Represents the comment associated with this name.
    :type comment: str
    :param name: The name of the object. Read-only.
    :type name: str
    :param scope: Indicates whether the name is scoped to the workbook or to a specific worksheet.
     Read-only.
    :type scope: str
    :param type: Indicates what type of reference is associated with the name. The possible values
     are: String, Integer, Double, Boolean, Range. Read-only.
    :type type: str
    :param value: Any object.
    :type value: object
    :param visible: Specifies whether the object is visible or not.
    :type visible: bool
    :param worksheet: workbookWorksheet.
    :type worksheet: ~users_user.models.MicrosoftGraphWorkbookWorksheet
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'comment': {'key': 'comment', 'type': 'str'},
        'name': {'key': 'name', 'type': 'str'},
        'scope': {'key': 'scope', 'type': 'str'},
        'type': {'key': 'type', 'type': 'str'},
        'value': {'key': 'value', 'type': 'object'},
        'visible': {'key': 'visible', 'type': 'bool'},
        'worksheet': {'key': 'worksheet', 'type': 'MicrosoftGraphWorkbookWorksheet'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphWorkbookNamedItem, self).__init__(**kwargs)
        self.comment = kwargs.get('comment', None)
        self.name = kwargs.get('name', None)
        self.scope = kwargs.get('scope', None)
        self.type = kwargs.get('type', None)
        self.value = kwargs.get('value', None)
        self.visible = kwargs.get('visible', None)
        self.worksheet = kwargs.get('worksheet', None)


class MicrosoftGraphWorkbookPivotTable(MicrosoftGraphEntity):
    """workbookPivotTable.

    :param id: Read-only.
    :type id: str
    :param name: Name of the PivotTable.
    :type name: str
    :param worksheet: workbookWorksheet.
    :type worksheet: ~users_user.models.MicrosoftGraphWorkbookWorksheet
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'name': {'key': 'name', 'type': 'str'},
        'worksheet': {'key': 'worksheet', 'type': 'MicrosoftGraphWorkbookWorksheet'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphWorkbookPivotTable, self).__init__(**kwargs)
        self.name = kwargs.get('name', None)
        self.worksheet = kwargs.get('worksheet', None)


class MicrosoftGraphWorkbookSortField(msrest.serialization.Model):
    """workbookSortField.

    :param ascending: Represents whether the sorting is done in an ascending fashion.
    :type ascending: bool
    :param color: Represents the color that is the target of the condition if the sorting is on
     font or cell color.
    :type color: str
    :param data_option: Represents additional sorting options for this field. The possible values
     are: Normal, TextAsNumber.
    :type data_option: str
    :param icon: workbookIcon.
    :type icon: ~users_user.models.MicrosoftGraphWorkbookIcon
    :param key: Represents the column (or row, depending on the sort orientation) that the
     condition is on. Represented as an offset from the first column (or row).
    :type key: int
    :param sort_on: Represents the type of sorting of this condition. The possible values are:
     Value, CellColor, FontColor, Icon.
    :type sort_on: str
    """

    _validation = {
        'key': {'maximum': 2147483647, 'minimum': -2147483648},
    }

    _attribute_map = {
        'ascending': {'key': 'ascending', 'type': 'bool'},
        'color': {'key': 'color', 'type': 'str'},
        'data_option': {'key': 'dataOption', 'type': 'str'},
        'icon': {'key': 'icon', 'type': 'MicrosoftGraphWorkbookIcon'},
        'key': {'key': 'key', 'type': 'int'},
        'sort_on': {'key': 'sortOn', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphWorkbookSortField, self).__init__(**kwargs)
        self.ascending = kwargs.get('ascending', None)
        self.color = kwargs.get('color', None)
        self.data_option = kwargs.get('data_option', None)
        self.icon = kwargs.get('icon', None)
        self.key = kwargs.get('key', None)
        self.sort_on = kwargs.get('sort_on', None)


class MicrosoftGraphWorkbookTable(MicrosoftGraphEntity):
    """workbookTable.

    :param id: Read-only.
    :type id: str
    :param highlight_first_column: Indicates whether the first column contains special formatting.
    :type highlight_first_column: bool
    :param highlight_last_column: Indicates whether the last column contains special formatting.
    :type highlight_last_column: bool
    :param legacy_id: Legacy Id used in older Excle clients. The value of the identifier remains
     the same even when the table is renamed. This property should be interpreted as an opaque
     string value and should not be parsed to any other type. Read-only.
    :type legacy_id: str
    :param name: Name of the table.
    :type name: str
    :param show_banded_columns: Indicates whether the columns show banded formatting in which odd
     columns are highlighted differently from even ones to make reading the table easier.
    :type show_banded_columns: bool
    :param show_banded_rows: Indicates whether the rows show banded formatting in which odd rows
     are highlighted differently from even ones to make reading the table easier.
    :type show_banded_rows: bool
    :param show_filter_button: Indicates whether the filter buttons are visible at the top of each
     column header. Setting this is only allowed if the table contains a header row.
    :type show_filter_button: bool
    :param show_headers: Indicates whether the header row is visible or not. This value can be set
     to show or remove the header row.
    :type show_headers: bool
    :param show_totals: Indicates whether the total row is visible or not. This value can be set to
     show or remove the total row.
    :type show_totals: bool
    :param style: Constant value that represents the Table style. The possible values are:
     TableStyleLight1 thru TableStyleLight21, TableStyleMedium1 thru TableStyleMedium28,
     TableStyleStyleDark1 thru TableStyleStyleDark11. A custom user-defined style present in the
     workbook can also be specified.
    :type style: str
    :param columns: Represents a collection of all the columns in the table. Read-only.
    :type columns: list[~users_user.models.MicrosoftGraphWorkbookTableColumn]
    :param rows: Represents a collection of all the rows in the table. Read-only.
    :type rows: list[~users_user.models.MicrosoftGraphWorkbookTableRow]
    :param sort: workbookTableSort.
    :type sort: ~users_user.models.MicrosoftGraphWorkbookTableSort
    :param worksheet: workbookWorksheet.
    :type worksheet: ~users_user.models.MicrosoftGraphWorkbookWorksheet
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'highlight_first_column': {'key': 'highlightFirstColumn', 'type': 'bool'},
        'highlight_last_column': {'key': 'highlightLastColumn', 'type': 'bool'},
        'legacy_id': {'key': 'legacyId', 'type': 'str'},
        'name': {'key': 'name', 'type': 'str'},
        'show_banded_columns': {'key': 'showBandedColumns', 'type': 'bool'},
        'show_banded_rows': {'key': 'showBandedRows', 'type': 'bool'},
        'show_filter_button': {'key': 'showFilterButton', 'type': 'bool'},
        'show_headers': {'key': 'showHeaders', 'type': 'bool'},
        'show_totals': {'key': 'showTotals', 'type': 'bool'},
        'style': {'key': 'style', 'type': 'str'},
        'columns': {'key': 'columns', 'type': '[MicrosoftGraphWorkbookTableColumn]'},
        'rows': {'key': 'rows', 'type': '[MicrosoftGraphWorkbookTableRow]'},
        'sort': {'key': 'sort', 'type': 'MicrosoftGraphWorkbookTableSort'},
        'worksheet': {'key': 'worksheet', 'type': 'MicrosoftGraphWorkbookWorksheet'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphWorkbookTable, self).__init__(**kwargs)
        self.highlight_first_column = kwargs.get('highlight_first_column', None)
        self.highlight_last_column = kwargs.get('highlight_last_column', None)
        self.legacy_id = kwargs.get('legacy_id', None)
        self.name = kwargs.get('name', None)
        self.show_banded_columns = kwargs.get('show_banded_columns', None)
        self.show_banded_rows = kwargs.get('show_banded_rows', None)
        self.show_filter_button = kwargs.get('show_filter_button', None)
        self.show_headers = kwargs.get('show_headers', None)
        self.show_totals = kwargs.get('show_totals', None)
        self.style = kwargs.get('style', None)
        self.columns = kwargs.get('columns', None)
        self.rows = kwargs.get('rows', None)
        self.sort = kwargs.get('sort', None)
        self.worksheet = kwargs.get('worksheet', None)


class MicrosoftGraphWorkbookTableColumn(MicrosoftGraphEntity):
    """workbookTableColumn.

    :param id: Read-only.
    :type id: str
    :param index: Returns the index number of the column within the columns collection of the
     table. Zero-indexed. Read-only.
    :type index: int
    :param name: Returns the name of the table column.
    :type name: str
    :param values: Any object.
    :type values: object
    :param filter: workbookFilter.
    :type filter: ~users_user.models.MicrosoftGraphWorkbookFilter
    """

    _validation = {
        'index': {'maximum': 2147483647, 'minimum': -2147483648},
    }

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'index': {'key': 'index', 'type': 'int'},
        'name': {'key': 'name', 'type': 'str'},
        'values': {'key': 'values', 'type': 'object'},
        'filter': {'key': 'filter', 'type': 'MicrosoftGraphWorkbookFilter'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphWorkbookTableColumn, self).__init__(**kwargs)
        self.index = kwargs.get('index', None)
        self.name = kwargs.get('name', None)
        self.values = kwargs.get('values', None)
        self.filter = kwargs.get('filter', None)


class MicrosoftGraphWorkbookTableRow(MicrosoftGraphEntity):
    """workbookTableRow.

    :param id: Read-only.
    :type id: str
    :param index: Returns the index number of the row within the rows collection of the table.
     Zero-indexed. Read-only.
    :type index: int
    :param values: Any object.
    :type values: object
    """

    _validation = {
        'index': {'maximum': 2147483647, 'minimum': -2147483648},
    }

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'index': {'key': 'index', 'type': 'int'},
        'values': {'key': 'values', 'type': 'object'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphWorkbookTableRow, self).__init__(**kwargs)
        self.index = kwargs.get('index', None)
        self.values = kwargs.get('values', None)


class MicrosoftGraphWorkbookTableSort(MicrosoftGraphEntity):
    """workbookTableSort.

    :param id: Read-only.
    :type id: str
    :param fields: Represents the current conditions used to last sort the table. Read-only.
    :type fields: list[~users_user.models.MicrosoftGraphWorkbookSortField]
    :param match_case: Represents whether the casing impacted the last sort of the table. Read-
     only.
    :type match_case: bool
    :param method: Represents Chinese character ordering method last used to sort the table. The
     possible values are: PinYin, StrokeCount. Read-only.
    :type method: str
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'fields': {'key': 'fields', 'type': '[MicrosoftGraphWorkbookSortField]'},
        'match_case': {'key': 'matchCase', 'type': 'bool'},
        'method': {'key': 'method', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphWorkbookTableSort, self).__init__(**kwargs)
        self.fields = kwargs.get('fields', None)
        self.match_case = kwargs.get('match_case', None)
        self.method = kwargs.get('method', None)


class MicrosoftGraphWorkbookWorksheet(MicrosoftGraphEntity):
    """workbookWorksheet.

    :param id: Read-only.
    :type id: str
    :param name: The display name of the worksheet.
    :type name: str
    :param position: The zero-based position of the worksheet within the workbook.
    :type position: int
    :param visibility: The Visibility of the worksheet. The possible values are: Visible, Hidden,
     VeryHidden.
    :type visibility: str
    :param charts: Returns collection of charts that are part of the worksheet. Read-only.
    :type charts: list[~users_user.models.MicrosoftGraphWorkbookChart]
    :param names: Returns collection of names that are associated with the worksheet. Read-only.
    :type names: list[~users_user.models.MicrosoftGraphWorkbookNamedItem]
    :param pivot_tables: Collection of PivotTables that are part of the worksheet.
    :type pivot_tables: list[~users_user.models.MicrosoftGraphWorkbookPivotTable]
    :param protection: workbookWorksheetProtection.
    :type protection: ~users_user.models.MicrosoftGraphWorkbookWorksheetProtection
    :param tables: Collection of tables that are part of the worksheet. Read-only.
    :type tables: list[~users_user.models.MicrosoftGraphWorkbookTable]
    """

    _validation = {
        'position': {'maximum': 2147483647, 'minimum': -2147483648},
    }

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'name': {'key': 'name', 'type': 'str'},
        'position': {'key': 'position', 'type': 'int'},
        'visibility': {'key': 'visibility', 'type': 'str'},
        'charts': {'key': 'charts', 'type': '[MicrosoftGraphWorkbookChart]'},
        'names': {'key': 'names', 'type': '[MicrosoftGraphWorkbookNamedItem]'},
        'pivot_tables': {'key': 'pivotTables', 'type': '[MicrosoftGraphWorkbookPivotTable]'},
        'protection': {'key': 'protection', 'type': 'MicrosoftGraphWorkbookWorksheetProtection'},
        'tables': {'key': 'tables', 'type': '[MicrosoftGraphWorkbookTable]'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphWorkbookWorksheet, self).__init__(**kwargs)
        self.name = kwargs.get('name', None)
        self.position = kwargs.get('position', None)
        self.visibility = kwargs.get('visibility', None)
        self.charts = kwargs.get('charts', None)
        self.names = kwargs.get('names', None)
        self.pivot_tables = kwargs.get('pivot_tables', None)
        self.protection = kwargs.get('protection', None)
        self.tables = kwargs.get('tables', None)


class MicrosoftGraphWorkbookWorksheetProtection(MicrosoftGraphEntity):
    """workbookWorksheetProtection.

    :param id: Read-only.
    :type id: str
    :param options: workbookWorksheetProtectionOptions.
    :type options: ~users_user.models.MicrosoftGraphWorkbookWorksheetProtectionOptions
    :param protected: Indicates if the worksheet is protected.  Read-only.
    :type protected: bool
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'options': {'key': 'options', 'type': 'MicrosoftGraphWorkbookWorksheetProtectionOptions'},
        'protected': {'key': 'protected', 'type': 'bool'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphWorkbookWorksheetProtection, self).__init__(**kwargs)
        self.options = kwargs.get('options', None)
        self.protected = kwargs.get('protected', None)


class MicrosoftGraphWorkbookWorksheetProtectionOptions(msrest.serialization.Model):
    """workbookWorksheetProtectionOptions.

    :param allow_auto_filter: Represents the worksheet protection option of allowing using auto
     filter feature.
    :type allow_auto_filter: bool
    :param allow_delete_columns: Represents the worksheet protection option of allowing deleting
     columns.
    :type allow_delete_columns: bool
    :param allow_delete_rows: Represents the worksheet protection option of allowing deleting rows.
    :type allow_delete_rows: bool
    :param allow_format_cells: Represents the worksheet protection option of allowing formatting
     cells.
    :type allow_format_cells: bool
    :param allow_format_columns: Represents the worksheet protection option of allowing formatting
     columns.
    :type allow_format_columns: bool
    :param allow_format_rows: Represents the worksheet protection option of allowing formatting
     rows.
    :type allow_format_rows: bool
    :param allow_insert_columns: Represents the worksheet protection option of allowing inserting
     columns.
    :type allow_insert_columns: bool
    :param allow_insert_hyperlinks: Represents the worksheet protection option of allowing
     inserting hyperlinks.
    :type allow_insert_hyperlinks: bool
    :param allow_insert_rows: Represents the worksheet protection option of allowing inserting
     rows.
    :type allow_insert_rows: bool
    :param allow_pivot_tables: Represents the worksheet protection option of allowing using pivot
     table feature.
    :type allow_pivot_tables: bool
    :param allow_sort: Represents the worksheet protection option of allowing using sort feature.
    :type allow_sort: bool
    """

    _attribute_map = {
        'allow_auto_filter': {'key': 'allowAutoFilter', 'type': 'bool'},
        'allow_delete_columns': {'key': 'allowDeleteColumns', 'type': 'bool'},
        'allow_delete_rows': {'key': 'allowDeleteRows', 'type': 'bool'},
        'allow_format_cells': {'key': 'allowFormatCells', 'type': 'bool'},
        'allow_format_columns': {'key': 'allowFormatColumns', 'type': 'bool'},
        'allow_format_rows': {'key': 'allowFormatRows', 'type': 'bool'},
        'allow_insert_columns': {'key': 'allowInsertColumns', 'type': 'bool'},
        'allow_insert_hyperlinks': {'key': 'allowInsertHyperlinks', 'type': 'bool'},
        'allow_insert_rows': {'key': 'allowInsertRows', 'type': 'bool'},
        'allow_pivot_tables': {'key': 'allowPivotTables', 'type': 'bool'},
        'allow_sort': {'key': 'allowSort', 'type': 'bool'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphWorkbookWorksheetProtectionOptions, self).__init__(**kwargs)
        self.allow_auto_filter = kwargs.get('allow_auto_filter', None)
        self.allow_delete_columns = kwargs.get('allow_delete_columns', None)
        self.allow_delete_rows = kwargs.get('allow_delete_rows', None)
        self.allow_format_cells = kwargs.get('allow_format_cells', None)
        self.allow_format_columns = kwargs.get('allow_format_columns', None)
        self.allow_format_rows = kwargs.get('allow_format_rows', None)
        self.allow_insert_columns = kwargs.get('allow_insert_columns', None)
        self.allow_insert_hyperlinks = kwargs.get('allow_insert_hyperlinks', None)
        self.allow_insert_rows = kwargs.get('allow_insert_rows', None)
        self.allow_pivot_tables = kwargs.get('allow_pivot_tables', None)
        self.allow_sort = kwargs.get('allow_sort', None)


class MicrosoftGraphWorkingHours(msrest.serialization.Model):
    """workingHours.

    :param days_of_week: The days of the week on which the user works.
    :type days_of_week: list[str or ~users_user.models.MicrosoftGraphDayOfWeek]
    :param start_time: The time of the day that the user starts working.
    :type start_time: ~datetime.time
    :param end_time: The time of the day that the user stops working.
    :type end_time: ~datetime.time
    :param time_zone: timeZoneBase.
    :type time_zone: ~users_user.models.MicrosoftGraphTimeZoneBase
    """

    _attribute_map = {
        'days_of_week': {'key': 'daysOfWeek', 'type': '[str]'},
        'start_time': {'key': 'startTime', 'type': 'time'},
        'end_time': {'key': 'endTime', 'type': 'time'},
        'time_zone': {'key': 'timeZone', 'type': 'MicrosoftGraphTimeZoneBase'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphWorkingHours, self).__init__(**kwargs)
        self.days_of_week = kwargs.get('days_of_week', None)
        self.start_time = kwargs.get('start_time', None)
        self.end_time = kwargs.get('end_time', None)
        self.time_zone = kwargs.get('time_zone', None)


class MicrosoftGraphWorkPosition(MicrosoftGraphItemFacet):
    """workPosition.

    :param id: Read-only.
    :type id: str
    :param allowed_audiences: allowedAudiences. Possible values include: "me", "family",
     "contacts", "groupMembers", "organization", "federatedOrganizations", "everyone",
     "unknownFutureValue".
    :type allowed_audiences: str or ~users_user.models.MicrosoftGraphAllowedAudiences
    :param inference: inferenceData.
    :type inference: ~users_user.models.MicrosoftGraphInferenceData
    :param created_date_time:
    :type created_date_time: ~datetime.datetime
    :param created_by: identitySet.
    :type created_by: ~users_user.models.MicrosoftGraphIdentitySet
    :param last_modified_date_time:
    :type last_modified_date_time: ~datetime.datetime
    :param last_modified_by: identitySet.
    :type last_modified_by: ~users_user.models.MicrosoftGraphIdentitySet
    :param categories:
    :type categories: list[str]
    :param detail: positionDetail.
    :type detail: ~users_user.models.MicrosoftGraphPositionDetail
    """

    _attribute_map = {
        'id': {'key': 'id', 'type': 'str'},
        'allowed_audiences': {'key': 'allowedAudiences', 'type': 'str'},
        'inference': {'key': 'inference', 'type': 'MicrosoftGraphInferenceData'},
        'created_date_time': {'key': 'createdDateTime', 'type': 'iso-8601'},
        'created_by': {'key': 'createdBy', 'type': 'MicrosoftGraphIdentitySet'},
        'last_modified_date_time': {'key': 'lastModifiedDateTime', 'type': 'iso-8601'},
        'last_modified_by': {'key': 'lastModifiedBy', 'type': 'MicrosoftGraphIdentitySet'},
        'categories': {'key': 'categories', 'type': '[str]'},
        'detail': {'key': 'detail', 'type': 'MicrosoftGraphPositionDetail'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphWorkPosition, self).__init__(**kwargs)
        self.categories = kwargs.get('categories', None)
        self.detail = kwargs.get('detail', None)


class MicrosoftGraphYomiPersonName(msrest.serialization.Model):
    """yomiPersonName.

    :param display_name:
    :type display_name: str
    :param first:
    :type first: str
    :param maiden:
    :type maiden: str
    :param middle:
    :type middle: str
    :param last:
    :type last: str
    """

    _attribute_map = {
        'display_name': {'key': 'displayName', 'type': 'str'},
        'first': {'key': 'first', 'type': 'str'},
        'maiden': {'key': 'maiden', 'type': 'str'},
        'middle': {'key': 'middle', 'type': 'str'},
        'last': {'key': 'last', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(MicrosoftGraphYomiPersonName, self).__init__(**kwargs)
        self.display_name = kwargs.get('display_name', None)
        self.first = kwargs.get('first', None)
        self.maiden = kwargs.get('maiden', None)
        self.middle = kwargs.get('middle', None)
        self.last = kwargs.get('last', None)


class OdataError(msrest.serialization.Model):
    """OdataError.

    All required parameters must be populated in order to send to Azure.

    :param error: Required.
    :type error: ~users_user.models.OdataErrorMain
    """

    _validation = {
        'error': {'required': True},
    }

    _attribute_map = {
        'error': {'key': 'error', 'type': 'OdataErrorMain'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(OdataError, self).__init__(**kwargs)
        self.error = kwargs['error']


class OdataErrorDetail(msrest.serialization.Model):
    """OdataErrorDetail.

    All required parameters must be populated in order to send to Azure.

    :param code: Required.
    :type code: str
    :param message: Required.
    :type message: str
    :param target:
    :type target: str
    """

    _validation = {
        'code': {'required': True},
        'message': {'required': True},
    }

    _attribute_map = {
        'code': {'key': 'code', 'type': 'str'},
        'message': {'key': 'message', 'type': 'str'},
        'target': {'key': 'target', 'type': 'str'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(OdataErrorDetail, self).__init__(**kwargs)
        self.code = kwargs['code']
        self.message = kwargs['message']
        self.target = kwargs.get('target', None)


class OdataErrorMain(msrest.serialization.Model):
    """OdataErrorMain.

    All required parameters must be populated in order to send to Azure.

    :param code: Required.
    :type code: str
    :param message: Required.
    :type message: str
    :param target:
    :type target: str
    :param details:
    :type details: list[~users_user.models.OdataErrorDetail]
    :param innererror: The structure of this object is service-specific.
    :type innererror: object
    """

    _validation = {
        'code': {'required': True},
        'message': {'required': True},
    }

    _attribute_map = {
        'code': {'key': 'code', 'type': 'str'},
        'message': {'key': 'message', 'type': 'str'},
        'target': {'key': 'target', 'type': 'str'},
        'details': {'key': 'details', 'type': '[OdataErrorDetail]'},
        'innererror': {'key': 'innererror', 'type': 'object'},
    }

    def __init__(
        self,
        **kwargs
    ):
        super(OdataErrorMain, self).__init__(**kwargs)
        self.code = kwargs['code']
        self.message = kwargs['message']
        self.target = kwargs.get('target', None)
        self.details = kwargs.get('details', None)
        self.innererror = kwargs.get('innererror', None)
